@@ -6,6 +6,7 @@
 
 #include <errno.h>
 
+#include <atomic>
 #include <new>
 
 #include "base/atomicops.h"
@@ -35,10 +36,9 @@
 
 namespace {
 
-using namespace base;
-
-subtle::AtomicWord g_chain_head = reinterpret_cast<subtle::AtomicWord>(
-    &allocator::AllocatorDispatch::default_dispatch);
+base::subtle::AtomicWord g_chain_head =
+    reinterpret_cast<base::subtle::AtomicWord>(
+        &base::allocator::AllocatorDispatch::default_dispatch);
 
 bool g_call_new_handler_on_malloc_failure = false;
 
@@ -65,15 +65,15 @@ bool CallNewHandler(size_t size) {
 #endif
 }
 
-inline const allocator::AllocatorDispatch* GetChainHead() {
+inline const base::allocator::AllocatorDispatch* GetChainHead() {
   // TODO(primiano): Just use NoBarrier_Load once crbug.com/593344 is fixed.
   // Unfortunately due to that bug NoBarrier_Load() is mistakenly fully
   // barriered on Linux+Clang, and that causes visible perf regressons.
-  return reinterpret_cast<const allocator::AllocatorDispatch*>(
+  return reinterpret_cast<const base::allocator::AllocatorDispatch*>(
 #if defined(OS_LINUX) && defined(__clang__)
-      *static_cast<const volatile subtle::AtomicWord*>(&g_chain_head)
+      *static_cast<const volatile base::subtle::AtomicWord*>(&g_chain_head)
 #else
-      subtle::NoBarrier_Load(&g_chain_head)
+      base::subtle::NoBarrier_Load(&g_chain_head)
 #endif
   );
 }
@@ -101,11 +101,11 @@ void InsertAllocatorDispatch(AllocatorDispatch* dispatch) {
 
     // This function guarantees to be thread-safe w.r.t. concurrent
     // insertions. It also has to guarantee that all the threads always
-    // see a consistent chain, hence the MemoryBarrier() below.
+    // see a consistent chain, hence the atomic_thread_fence() below.
     // InsertAllocatorDispatch() is NOT a fastpath, as opposite to malloc(), so
     // we don't really want this to be a release-store with a corresponding
     // acquire-load during malloc().
-    subtle::MemoryBarrier();
+    std::atomic_thread_fence(std::memory_order_seq_cst);
     subtle::AtomicWord old_value =
         reinterpret_cast<subtle::AtomicWord>(chain_head);
     // Set the chain head to the new dispatch atomically. If we lose the race,
@@ -150,7 +150,7 @@ extern "C" {
 //     - Assume it did succeed if it returns, in which case reattempt the alloc.
 
 ALWAYS_INLINE void* ShimCppNew(size_t size) {
-  const allocator::AllocatorDispatch* const chain_head = GetChainHead();
+  const base::allocator::AllocatorDispatch* const chain_head = GetChainHead();
   void* ptr;
   do {
     void* context = nullptr;
@@ -163,7 +163,7 @@ ALWAYS_INLINE void* ShimCppNew(size_t size) {
 }
 
 ALWAYS_INLINE void* ShimCppAlignedNew(size_t size, size_t alignment) {
-  const allocator::AllocatorDispatch* const chain_head = GetChainHead();
+  const base::allocator::AllocatorDispatch* const chain_head = GetChainHead();
   void* ptr;
   do {
     void* context = nullptr;
@@ -181,12 +181,12 @@ ALWAYS_INLINE void ShimCppDelete(void* address) {
 #if defined(OS_MACOSX)
   context = malloc_default_zone();
 #endif
-  const allocator::AllocatorDispatch* const chain_head = GetChainHead();
+  const base::allocator::AllocatorDispatch* const chain_head = GetChainHead();
   return chain_head->free_function(chain_head, address, context);
 }
 
 ALWAYS_INLINE void* ShimMalloc(size_t size, void* context) {
-  const allocator::AllocatorDispatch* const chain_head = GetChainHead();
+  const base::allocator::AllocatorDispatch* const chain_head = GetChainHead();
   void* ptr;
   do {
     ptr = chain_head->alloc_function(chain_head, size, context);
@@ -196,7 +196,7 @@ ALWAYS_INLINE void* ShimMalloc(size_t size, void* context) {
 }
 
 ALWAYS_INLINE void* ShimCalloc(size_t n, size_t size, void* context) {
-  const allocator::AllocatorDispatch* const chain_head = GetChainHead();
+  const base::allocator::AllocatorDispatch* const chain_head = GetChainHead();
   void* ptr;
   do {
     ptr = chain_head->alloc_zero_initialized_function(chain_head, n, size,
@@ -209,7 +209,7 @@ ALWAYS_INLINE void* ShimCalloc(size_t n, size_t size, void* context) {
 ALWAYS_INLINE void* ShimRealloc(void* address, size_t size, void* context) {
   // realloc(size == 0) means free() and might return a nullptr. We should
   // not call the std::new_handler in that case, though.
-  const allocator::AllocatorDispatch* const chain_head = GetChainHead();
+  const base::allocator::AllocatorDispatch* const chain_head = GetChainHead();
   void* ptr;
   do {
     ptr = chain_head->realloc_function(chain_head, address, size, context);
@@ -219,7 +219,7 @@ ALWAYS_INLINE void* ShimRealloc(void* address, size_t size, void* context) {
 }
 
 ALWAYS_INLINE void* ShimMemalign(size_t alignment, size_t size, void* context) {
-  const allocator::AllocatorDispatch* const chain_head = GetChainHead();
+  const base::allocator::AllocatorDispatch* const chain_head = GetChainHead();
   void* ptr;
   do {
     ptr = chain_head->alloc_aligned_function(chain_head, alignment, size,
@@ -258,12 +258,12 @@ ALWAYS_INLINE void* ShimPvalloc(size_t size) {
 }
 
 ALWAYS_INLINE void ShimFree(void* address, void* context) {
-  const allocator::AllocatorDispatch* const chain_head = GetChainHead();
+  const base::allocator::AllocatorDispatch* const chain_head = GetChainHead();
   return chain_head->free_function(chain_head, address, context);
 }
 
 ALWAYS_INLINE size_t ShimGetSizeEstimate(const void* address, void* context) {
-  const allocator::AllocatorDispatch* const chain_head = GetChainHead();
+  const base::allocator::AllocatorDispatch* const chain_head = GetChainHead();
   return chain_head->get_size_estimate_function(
       chain_head, const_cast<void*>(address), context);
 }
@@ -272,7 +272,7 @@ ALWAYS_INLINE unsigned ShimBatchMalloc(size_t size,
                                        void** results,
                                        unsigned num_requested,
                                        void* context) {
-  const allocator::AllocatorDispatch* const chain_head = GetChainHead();
+  const base::allocator::AllocatorDispatch* const chain_head = GetChainHead();
   return chain_head->batch_malloc_function(chain_head, size, results,
                                            num_requested, context);
 }
@@ -280,13 +280,13 @@ ALWAYS_INLINE unsigned ShimBatchMalloc(size_t size,
 ALWAYS_INLINE void ShimBatchFree(void** to_be_freed,
                                  unsigned num_to_be_freed,
                                  void* context) {
-  const allocator::AllocatorDispatch* const chain_head = GetChainHead();
+  const base::allocator::AllocatorDispatch* const chain_head = GetChainHead();
   return chain_head->batch_free_function(chain_head, to_be_freed,
                                          num_to_be_freed, context);
 }
 
 ALWAYS_INLINE void ShimFreeDefiniteSize(void* ptr, size_t size, void* context) {
-  const allocator::AllocatorDispatch* const chain_head = GetChainHead();
+  const base::allocator::AllocatorDispatch* const chain_head = GetChainHead();
   return chain_head->free_definite_size_function(chain_head, ptr, size,
                                                  context);
 }
@@ -294,7 +294,7 @@ ALWAYS_INLINE void ShimFreeDefiniteSize(void* ptr, size_t size, void* context) {
 ALWAYS_INLINE void* ShimAlignedMalloc(size_t size,
                                       size_t alignment,
                                       void* context) {
-  const allocator::AllocatorDispatch* const chain_head = GetChainHead();
+  const base::allocator::AllocatorDispatch* const chain_head = GetChainHead();
   void* ptr = nullptr;
   do {
     ptr = chain_head->aligned_malloc_function(chain_head, size, alignment,
@@ -310,7 +310,7 @@ ALWAYS_INLINE void* ShimAlignedRealloc(void* address,
                                        void* context) {
   // _aligned_realloc(size == 0) means _aligned_free() and might return a
   // nullptr. We should not call the std::new_handler in that case, though.
-  const allocator::AllocatorDispatch* const chain_head = GetChainHead();
+  const base::allocator::AllocatorDispatch* const chain_head = GetChainHead();
   void* ptr = nullptr;
   do {
     ptr = chain_head->aligned_realloc_function(chain_head, address, size,
@@ -321,7 +321,7 @@ ALWAYS_INLINE void* ShimAlignedRealloc(void* address,
 }
 
 ALWAYS_INLINE void ShimAlignedFree(void* address, void* context) {
-  const allocator::AllocatorDispatch* const chain_head = GetChainHead();
+  const base::allocator::AllocatorDispatch* const chain_head = GetChainHead();
   return chain_head->aligned_free_function(chain_head, address, context);
 }
 
