@@ -23,7 +23,6 @@ class TestRefCounted : public RefCountedThreadSafe<TestRefCounted> {
  private:
   friend class RefCountedThreadSafe<TestRefCounted>;
   ~TestRefCounted() = default;
-  ;
 };
 
 void Increment(int* count) { (*count)++; }
@@ -39,10 +38,10 @@ void OnMoveOnlyReceived(int* value, std::unique_ptr<int> result) {
 //  - After Cancel(), Run() completes but has no effect.
 TEST(CancelableCallbackTest, Cancel) {
   int count = 0;
-  CancelableClosure cancelable(
-      base::Bind(&Increment, base::Unretained(&count)));
+  CancelableRepeatingClosure cancelable(
+      base::BindRepeating(&Increment, base::Unretained(&count)));
 
-  base::Closure callback = cancelable.callback();
+  base::RepeatingClosure callback = cancelable.callback();
   callback.Run();
   EXPECT_EQ(1, count);
 
@@ -60,11 +59,11 @@ TEST(CancelableCallbackTest, Cancel) {
 //  - After Cancel(), callback() returns a null callback.
 TEST(CancelableCallbackTest, MultipleCancel) {
   int count = 0;
-  CancelableClosure cancelable(
-      base::Bind(&Increment, base::Unretained(&count)));
+  CancelableRepeatingClosure cancelable(
+      base::BindRepeating(&Increment, base::Unretained(&count)));
 
-  base::Closure callback1 = cancelable.callback();
-  base::Closure callback2 = cancelable.callback();
+  base::RepeatingClosure callback1 = cancelable.callback();
+  base::RepeatingClosure callback2 = cancelable.callback();
   cancelable.Cancel();
 
   callback1.Run();
@@ -77,7 +76,7 @@ TEST(CancelableCallbackTest, MultipleCancel) {
   cancelable.Cancel();
 
   // callback() of a cancelled callback is null.
-  base::Closure callback3 = cancelable.callback();
+  base::RepeatingClosure callback3 = cancelable.callback();
   EXPECT_TRUE(callback3.is_null());
 }
 
@@ -85,11 +84,11 @@ TEST(CancelableCallbackTest, MultipleCancel) {
 //  - Destruction of CancelableCallback cancels outstanding callbacks.
 TEST(CancelableCallbackTest, CallbackCanceledOnDestruction) {
   int count = 0;
-  base::Closure callback;
+  base::RepeatingClosure callback;
 
   {
-    CancelableClosure cancelable(
-        base::Bind(&Increment, base::Unretained(&count)));
+    CancelableRepeatingClosure cancelable(
+        base::BindRepeating(&Increment, base::Unretained(&count)));
 
     callback = cancelable.callback();
     callback.Run();
@@ -106,7 +105,8 @@ TEST(CancelableCallbackTest, CancelDropsCallback) {
   scoped_refptr<TestRefCounted> ref_counted = new TestRefCounted;
   EXPECT_TRUE(ref_counted->HasOneRef());
 
-  CancelableClosure cancelable(base::Bind(RefCountedParam, ref_counted));
+  CancelableOnceClosure cancelable(
+      base::BindOnce(RefCountedParam, ref_counted));
   EXPECT_FALSE(cancelable.IsCancelled());
   EXPECT_TRUE(ref_counted.get());
   EXPECT_FALSE(ref_counted->HasOneRef());
@@ -123,10 +123,10 @@ TEST(CancelableCallbackTest, CancelDropsCallback) {
 //  - Reset() deactivates outstanding callbacks.
 TEST(CancelableCallbackTest, Reset) {
   int count = 0;
-  CancelableClosure cancelable(
-      base::Bind(&Increment, base::Unretained(&count)));
+  CancelableRepeatingClosure cancelable(
+      base::BindRepeating(&Increment, base::Unretained(&count)));
 
-  base::Closure callback = cancelable.callback();
+  base::RepeatingClosure callback = cancelable.callback();
   callback.Run();
   EXPECT_EQ(1, count);
 
@@ -134,7 +134,7 @@ TEST(CancelableCallbackTest, Reset) {
   EXPECT_EQ(2, count);
 
   cancelable.Reset(
-      base::Bind(&IncrementBy, base::Unretained(&count), 3));
+      base::BindRepeating(&IncrementBy, base::Unretained(&count), 3));
   EXPECT_FALSE(cancelable.IsCancelled());
 
   // The stale copy of the cancelable callback is non-null.
@@ -144,7 +144,7 @@ TEST(CancelableCallbackTest, Reset) {
   callback.Run();
   EXPECT_EQ(2, count);
 
-  base::Closure callback2 = cancelable.callback();
+  base::RepeatingClosure callback2 = cancelable.callback();
   ASSERT_FALSE(callback2.is_null());
 
   callback2.Run();
@@ -154,12 +154,11 @@ TEST(CancelableCallbackTest, Reset) {
 // IsCanceled().
 //  - Cancel() transforms the CancelableCallback into a cancelled state.
 TEST(CancelableCallbackTest, IsNull) {
-  CancelableClosure cancelable;
+  CancelableOnceClosure cancelable;
   EXPECT_TRUE(cancelable.IsCancelled());
 
   int count = 0;
-  cancelable.Reset(base::Bind(&Increment,
-                              base::Unretained(&count)));
+  cancelable.Reset(base::BindOnce(&Increment, base::Unretained(&count)));
   EXPECT_FALSE(cancelable.IsCancelled());
 
   cancelable.Cancel();
@@ -172,8 +171,8 @@ TEST(CancelableCallbackTest, PostTask) {
   test::ScopedTaskEnvironment scoped_task_environment;
 
   int count = 0;
-  CancelableClosure cancelable(base::Bind(&Increment,
-                                           base::Unretained(&count)));
+  CancelableRepeatingClosure cancelable(
+      base::BindRepeating(&Increment, base::Unretained(&count)));
 
   ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE, cancelable.callback());
   RunLoop().RunUntilIdle();
@@ -195,8 +194,8 @@ TEST(CancelableCallbackTest, MoveOnlyType) {
   const int kExpectedResult = 42;
 
   int result = 0;
-  CancelableCallback<void(std::unique_ptr<int>)> cb(
-      base::Bind(&OnMoveOnlyReceived, base::Unretained(&result)));
+  CancelableRepeatingCallback<void(std::unique_ptr<int>)> cb(
+      base::BindRepeating(&OnMoveOnlyReceived, base::Unretained(&result)));
   cb.callback().Run(std::make_unique<int>(kExpectedResult));
 
   EXPECT_EQ(kExpectedResult, result);
