@@ -16,6 +16,7 @@
 #include "base/macros.h"
 #include "base/memory/ptr_util.h"
 #include "base/pending_task.h"
+#include "base/task/sequence_manager/enqueue_order_generator.h"
 #include "base/task/sequence_manager/task_queue_impl.h"
 #include "base/task/sequence_manager/test/mock_time_domain.h"
 #include "base/task/sequence_manager/work_queue.h"
@@ -46,29 +47,31 @@ class MockObserver : public TaskQueueSelector::Observer {
 
 class TaskQueueSelectorForTest : public TaskQueueSelector {
  public:
-  using TaskQueueSelector::ChooseOldestWithPriority;
+  using TaskQueueSelector::ChooseWithPriority;
   using TaskQueueSelector::delayed_work_queue_sets;
   using TaskQueueSelector::immediate_work_queue_sets;
   using TaskQueueSelector::kMaxHighPriorityStarvationScore;
   using TaskQueueSelector::kMaxLowPriorityStarvationScore;
   using TaskQueueSelector::kMaxNormalPriorityStarvationScore;
   using TaskQueueSelector::SetImmediateStarvationCountForTest;
+  using TaskQueueSelector::SetOperationOldest;
   using TaskQueueSelector::SmallPriorityQueue;
 
-  TaskQueueSelectorForTest(scoped_refptr<AssociatedThreadId> associated_thread)
-      : TaskQueueSelector(associated_thread) {}
+  TaskQueueSelectorForTest(scoped_refptr<AssociatedThreadId> associated_thread,
+                           const SequenceManager::Settings& settings)
+      : TaskQueueSelector(associated_thread, settings) {}
 };
 
-class TaskQueueSelectorTest : public testing::Test {
+class TaskQueueSelectorTestBase : public testing::Test {
  public:
-  TaskQueueSelectorTest()
-      : test_closure_(BindRepeating(&TaskQueueSelectorTest::TestFunction)),
+  explicit TaskQueueSelectorTestBase(const SequenceManager::Settings& settings)
+      : test_closure_(BindRepeating(&TaskQueueSelectorTestBase::TestFunction)),
         associated_thread_(AssociatedThreadId::CreateBound()),
-        selector_(associated_thread_) {}
-  ~TaskQueueSelectorTest() override = default;
+        selector_(associated_thread_, settings) {}
+  ~TaskQueueSelectorTestBase() override = default;
 
   void PushTasks(const size_t queue_indices[], size_t num_tasks) {
-    EnqueueOrder::Generator enqueue_order_generator;
+    EnqueueOrderGenerator enqueue_order_generator;
     for (size_t i = 0; i < num_tasks; i++) {
       task_queues_[queue_indices[i]]->immediate_work_queue()->Push(
           Task(PostedTask(test_closure_, FROM_HERE), TimeTicks(),
@@ -100,7 +103,8 @@ class TaskQueueSelectorTest : public testing::Test {
           queue_to_index_map_.find(chosen_work_queue->task_queue())->second;
       order.push_back(chosen_queue_index);
       chosen_work_queue->PopTaskForTesting();
-      chosen_work_queue->work_queue_sets()->OnPopQueue(chosen_work_queue);
+      chosen_work_queue->work_queue_sets()->OnPopMinQueueInSet(
+          chosen_work_queue);
     }
     return order;
   }
@@ -149,6 +153,15 @@ class TaskQueueSelectorTest : public testing::Test {
   std::map<TaskQueueImpl*, size_t> queue_to_index_map_;
 };
 
+class TaskQueueSelectorTest : public TaskQueueSelectorTestBase {
+ public:
+  TaskQueueSelectorTest()
+      : TaskQueueSelectorTestBase(
+            SequenceManager::Settings::Builder()
+                .SetAntiStarvationLogicForPrioritiesDisabled(false)
+                .Build()) {}
+};
+
 TEST_F(TaskQueueSelectorTest, TestDefaultPriority) {
   size_t queue_order[] = {4, 3, 2, 1, 0};
   PushTasks(queue_order, 5);
@@ -204,12 +217,12 @@ TEST_F(TaskQueueSelectorTest, TestControlPriority) {
 }
 
 TEST_F(TaskQueueSelectorTest, TestObserverWithEnabledQueue) {
-  task_queues_[1]->SetQueueEnabledForTest(false);
+  task_queues_[1]->SetQueueEnabled(false);
   selector_.DisableQueue(task_queues_[1].get());
   MockObserver mock_observer;
   selector_.SetTaskQueueSelectorObserver(&mock_observer);
   EXPECT_CALL(mock_observer, OnTaskQueueEnabled(_)).Times(1);
-  task_queues_[1]->SetQueueEnabledForTest(true);
+  task_queues_[1]->SetQueueEnabled(true);
   selector_.EnableQueue(task_queues_[1].get());
 }
 
@@ -229,9 +242,9 @@ TEST_F(TaskQueueSelectorTest, TestDisableEnable) {
 
   size_t queue_order[] = {0, 1, 2, 3, 4};
   PushTasks(queue_order, 5);
-  task_queues_[2]->SetQueueEnabledForTest(false);
+  task_queues_[2]->SetQueueEnabled(false);
   selector_.DisableQueue(task_queues_[2].get());
-  task_queues_[4]->SetQueueEnabledForTest(false);
+  task_queues_[4]->SetQueueEnabled(false);
   selector_.DisableQueue(task_queues_[4].get());
   // Disabling a queue should not affect its priority.
   EXPECT_EQ(TaskQueue::kNormalPriority, task_queues_[2]->GetQueuePriority());
@@ -239,12 +252,12 @@ TEST_F(TaskQueueSelectorTest, TestDisableEnable) {
   EXPECT_THAT(PopTasksAndReturnQueueIndices(), ElementsAre(0, 1, 3));
 
   EXPECT_CALL(mock_observer, OnTaskQueueEnabled(_)).Times(2);
-  task_queues_[2]->SetQueueEnabledForTest(true);
+  task_queues_[2]->SetQueueEnabled(true);
   selector_.EnableQueue(task_queues_[2].get());
   selector_.SetQueuePriority(task_queues_[2].get(),
                              TaskQueue::kBestEffortPriority);
   EXPECT_THAT(PopTasksAndReturnQueueIndices(), ElementsAre(2));
-  task_queues_[4]->SetQueueEnabledForTest(true);
+  task_queues_[4]->SetQueueEnabled(true);
   selector_.EnableQueue(task_queues_[4].get());
   EXPECT_THAT(PopTasksAndReturnQueueIndices(), ElementsAre(4));
 }
@@ -253,7 +266,7 @@ TEST_F(TaskQueueSelectorTest, TestDisableChangePriorityThenEnable) {
   EXPECT_TRUE(task_queues_[2]->delayed_work_queue()->Empty());
   EXPECT_TRUE(task_queues_[2]->immediate_work_queue()->Empty());
 
-  task_queues_[2]->SetQueueEnabledForTest(false);
+  task_queues_[2]->SetQueueEnabled(false);
   selector_.SetQueuePriority(task_queues_[2].get(),
                              TaskQueue::kHighestPriority);
 
@@ -262,7 +275,7 @@ TEST_F(TaskQueueSelectorTest, TestDisableChangePriorityThenEnable) {
 
   EXPECT_TRUE(task_queues_[2]->delayed_work_queue()->Empty());
   EXPECT_FALSE(task_queues_[2]->immediate_work_queue()->Empty());
-  task_queues_[2]->SetQueueEnabledForTest(true);
+  task_queues_[2]->SetQueueEnabled(true);
 
   EXPECT_EQ(TaskQueue::kHighestPriority, task_queues_[2]->GetQueuePriority());
   EXPECT_THAT(PopTasksAndReturnQueueIndices(), ElementsAre(2, 0, 1, 3, 4));
@@ -274,14 +287,14 @@ TEST_F(TaskQueueSelectorTest, TestEmptyQueues) {
   // Test only disabled queues.
   size_t queue_order[] = {0};
   PushTasks(queue_order, 1);
-  task_queues_[0]->SetQueueEnabledForTest(false);
+  task_queues_[0]->SetQueueEnabled(false);
   selector_.DisableQueue(task_queues_[0].get());
   EXPECT_EQ(nullptr, selector_.SelectWorkQueueToService());
 
   // These tests are unusual since there's no TQM. To avoid a later DCHECK when
   // deleting the task queue, we re-enable the queue here so the selector
   // doesn't get out of sync.
-  task_queues_[0]->SetQueueEnabledForTest(true);
+  task_queues_[0]->SetQueueEnabled(true);
   selector_.EnableQueue(task_queues_[0].get());
 }
 
@@ -328,7 +341,7 @@ TEST_F(TaskQueueSelectorTest, ControlTasksDontTriggerAntiStarvationLogic) {
   // Simulate the control queue becoming empty.
   WorkQueue* chosen_work_queue = selector_.SelectWorkQueueToService();
   chosen_work_queue->PopTaskForTesting();
-  chosen_work_queue->work_queue_sets()->OnPopQueue(chosen_work_queue);
+  chosen_work_queue->work_queue_sets()->OnPopMinQueueInSet(chosen_work_queue);
 
   // Simulate posting a normal priority task.
   PushTask(2, 2);
@@ -624,60 +637,64 @@ TEST_F(TaskQueueSelectorTest,
   EXPECT_NE(chosen_work_queue->task_queue(), task_queues_[3].get());
 }
 
-TEST_F(TaskQueueSelectorTest, AllEnabledWorkQueuesAreEmpty) {
-  EXPECT_TRUE(selector_.AllEnabledWorkQueuesAreEmpty());
+TEST_F(TaskQueueSelectorTest, GetHighestPendingPriority) {
+  EXPECT_FALSE(selector_.GetHighestPendingPriority().has_value());
   size_t queue_order[] = {0, 1};
   PushTasks(queue_order, 2);
 
-  EXPECT_FALSE(selector_.AllEnabledWorkQueuesAreEmpty());
+  selector_.SetQueuePriority(task_queues_[1].get(), TaskQueue::kHighPriority);
+
+  EXPECT_EQ(TaskQueue::kHighPriority, *selector_.GetHighestPendingPriority());
   PopTasksAndReturnQueueIndices();
-  EXPECT_TRUE(selector_.AllEnabledWorkQueuesAreEmpty());
-}
+  EXPECT_FALSE(selector_.GetHighestPendingPriority().has_value());
 
-TEST_F(TaskQueueSelectorTest, AllEnabledWorkQueuesAreEmpty_ControlPriority) {
-  size_t queue_order[] = {0};
   PushTasks(queue_order, 1);
-
-  selector_.SetQueuePriority(task_queues_[0].get(),
-                             TaskQueue::kControlPriority);
-
-  EXPECT_FALSE(selector_.AllEnabledWorkQueuesAreEmpty());
+  EXPECT_EQ(TaskQueue::kNormalPriority, *selector_.GetHighestPendingPriority());
+  PopTasksAndReturnQueueIndices();
+  EXPECT_FALSE(selector_.GetHighestPendingPriority().has_value());
 }
 
-TEST_F(TaskQueueSelectorTest, ChooseOldestWithPriority_Empty) {
+TEST_F(TaskQueueSelectorTest, ChooseWithPriority_Empty) {
   bool chose_delayed_over_immediate = false;
-  EXPECT_EQ(nullptr,
-            selector_.ChooseOldestWithPriority(TaskQueue::kNormalPriority,
-                                               &chose_delayed_over_immediate));
+  EXPECT_EQ(
+      nullptr,
+      selector_
+          .ChooseWithPriority<TaskQueueSelectorForTest::SetOperationOldest>(
+              TaskQueue::kNormalPriority, &chose_delayed_over_immediate));
   EXPECT_FALSE(chose_delayed_over_immediate);
 }
 
-TEST_F(TaskQueueSelectorTest, ChooseOldestWithPriority_OnlyDelayed) {
+TEST_F(TaskQueueSelectorTest, ChooseWithPriority_OnlyDelayed) {
   task_queues_[0]->delayed_work_queue()->Push(
       Task(PostedTask(test_closure_, FROM_HERE), TimeTicks(), EnqueueOrder(),
            EnqueueOrder::FromIntForTesting(2)));
 
   bool chose_delayed_over_immediate = false;
-  EXPECT_EQ(task_queues_[0]->delayed_work_queue(),
-            selector_.ChooseOldestWithPriority(TaskQueue::kNormalPriority,
-                                               &chose_delayed_over_immediate));
+  EXPECT_EQ(
+      task_queues_[0]->delayed_work_queue(),
+      selector_
+          .ChooseWithPriority<TaskQueueSelectorForTest::SetOperationOldest>(
+              TaskQueue::kNormalPriority, &chose_delayed_over_immediate));
   EXPECT_FALSE(chose_delayed_over_immediate);
 }
 
-TEST_F(TaskQueueSelectorTest, ChooseOldestWithPriority_OnlyImmediate) {
+TEST_F(TaskQueueSelectorTest, ChooseWithPriority_OnlyImmediate) {
   task_queues_[0]->immediate_work_queue()->Push(
       Task(PostedTask(test_closure_, FROM_HERE), TimeTicks(), EnqueueOrder(),
            EnqueueOrder::FromIntForTesting(2)));
 
   bool chose_delayed_over_immediate = false;
-  EXPECT_EQ(task_queues_[0]->immediate_work_queue(),
-            selector_.ChooseOldestWithPriority(TaskQueue::kNormalPriority,
-                                               &chose_delayed_over_immediate));
+  EXPECT_EQ(
+      task_queues_[0]->immediate_work_queue(),
+      selector_
+          .ChooseWithPriority<TaskQueueSelectorForTest::SetOperationOldest>(
+              TaskQueue::kNormalPriority, &chose_delayed_over_immediate));
   EXPECT_FALSE(chose_delayed_over_immediate);
 }
 
 TEST_F(TaskQueueSelectorTest, TestObserverWithOneBlockedQueue) {
-  TaskQueueSelectorForTest selector(associated_thread_);
+  TaskQueueSelectorForTest selector(associated_thread_,
+                                    SequenceManager::Settings());
   MockObserver mock_observer;
   selector.SetTaskQueueSelectorObserver(&mock_observer);
 
@@ -686,7 +703,7 @@ TEST_F(TaskQueueSelectorTest, TestObserverWithOneBlockedQueue) {
   std::unique_ptr<TaskQueueImpl> task_queue(NewTaskQueueWithBlockReporting());
   selector.AddQueue(task_queue.get());
 
-  task_queue->SetQueueEnabledForTest(false);
+  task_queue->SetQueueEnabled(false);
   selector.DisableQueue(task_queue.get());
 
   Task task(PostedTask(test_closure_, FROM_HERE), TimeTicks(), EnqueueOrder(),
@@ -695,14 +712,15 @@ TEST_F(TaskQueueSelectorTest, TestObserverWithOneBlockedQueue) {
 
   EXPECT_EQ(nullptr, selector.SelectWorkQueueToService());
 
-  task_queue->SetQueueEnabledForTest(true);
+  task_queue->SetQueueEnabled(true);
   selector.EnableQueue(task_queue.get());
   selector.RemoveQueue(task_queue.get());
   task_queue->UnregisterTaskQueue();
 }
 
 TEST_F(TaskQueueSelectorTest, TestObserverWithTwoBlockedQueues) {
-  TaskQueueSelectorForTest selector(associated_thread_);
+  TaskQueueSelectorForTest selector(associated_thread_,
+                                    SequenceManager::Settings());
   MockObserver mock_observer;
   selector.SetTaskQueueSelectorObserver(&mock_observer);
 
@@ -711,8 +729,8 @@ TEST_F(TaskQueueSelectorTest, TestObserverWithTwoBlockedQueues) {
   selector.AddQueue(task_queue.get());
   selector.AddQueue(task_queue2.get());
 
-  task_queue->SetQueueEnabledForTest(false);
-  task_queue2->SetQueueEnabledForTest(false);
+  task_queue->SetQueueEnabled(false);
+  task_queue2->SetQueueEnabled(false);
   selector.DisableQueue(task_queue.get());
   selector.DisableQueue(task_queue2.get());
 
@@ -731,20 +749,76 @@ TEST_F(TaskQueueSelectorTest, TestObserverWithTwoBlockedQueues) {
 
   EXPECT_CALL(mock_observer, OnTaskQueueEnabled(_)).Times(2);
 
-  task_queue->SetQueueEnabledForTest(true);
+  task_queue->SetQueueEnabled(true);
   selector.EnableQueue(task_queue.get());
 
   selector.RemoveQueue(task_queue.get());
   task_queue->UnregisterTaskQueue();
   EXPECT_EQ(nullptr, selector.SelectWorkQueueToService());
 
-  task_queue2->SetQueueEnabledForTest(true);
+  task_queue2->SetQueueEnabled(true);
   selector.EnableQueue(task_queue2.get());
   selector.RemoveQueue(task_queue2.get());
   task_queue2->UnregisterTaskQueue();
 }
 
-struct ChooseOldestWithPriorityTestParam {
+class DisabledAntiStarvationLogicTaskQueueSelectorTest
+    : public TaskQueueSelectorTestBase,
+      public testing::WithParamInterface<TaskQueue::QueuePriority> {
+ public:
+  DisabledAntiStarvationLogicTaskQueueSelectorTest()
+      : TaskQueueSelectorTestBase(
+            SequenceManager::Settings::Builder()
+                .SetAntiStarvationLogicForPrioritiesDisabled(true)
+                .Build()) {}
+};
+
+TEST_P(DisabledAntiStarvationLogicTaskQueueSelectorTest,
+       TestStarvedByHigherPriorities) {
+  TaskQueue::QueuePriority priority_to_test = GetParam();
+  size_t queue_order[] = {0, 1};
+  PushTasks(queue_order, 2);
+
+  // Setting the queue priority to its current value causes a check to fail.
+  if (task_queues_[0]->GetQueuePriority() != priority_to_test) {
+    selector_.SetQueuePriority(task_queues_[0].get(), priority_to_test);
+  }
+
+  // Test that |priority_to_test| is starved by all higher priorities.
+  for (int higher_priority = static_cast<int>(priority_to_test) - 1;
+       higher_priority >= 0; higher_priority--) {
+    // Setting the queue priority to its current value causes a check to fail.
+    if (task_queues_[1]->GetQueuePriority() != higher_priority) {
+      selector_.SetQueuePriority(
+          task_queues_[1].get(),
+          static_cast<TaskQueue::QueuePriority>(higher_priority));
+    }
+
+    for (int i = 0; i < 100; i++) {
+      WorkQueue* chosen_work_queue = selector_.SelectWorkQueueToService();
+      ASSERT_THAT(chosen_work_queue, NotNull());
+      EXPECT_EQ(task_queues_[1].get(), chosen_work_queue->task_queue());
+      // Don't remove task from queue to simulate all queues still being full.
+    }
+  }
+}
+
+std::string GetPriorityTestNameSuffix(
+    const testing::TestParamInfo<TaskQueue::QueuePriority>& info) {
+  return TaskQueue::PriorityToString(info.param);
+}
+
+INSTANTIATE_TEST_SUITE_P(,
+                         DisabledAntiStarvationLogicTaskQueueSelectorTest,
+                         testing::Values(TaskQueue::kHighestPriority,
+                                         TaskQueue::kVeryHighPriority,
+                                         TaskQueue::kHighPriority,
+                                         TaskQueue::kNormalPriority,
+                                         TaskQueue::kLowPriority,
+                                         TaskQueue::kBestEffortPriority),
+                         GetPriorityTestNameSuffix);
+
+struct ChooseWithPriorityTestParam {
   int delayed_task_enqueue_order;
   int immediate_task_enqueue_order;
   int immediate_starvation_count;
@@ -752,19 +826,18 @@ struct ChooseOldestWithPriorityTestParam {
   bool expected_did_starve_immediate_queue;
 };
 
-static const ChooseOldestWithPriorityTestParam
-    kChooseOldestWithPriorityTestCases[] = {
-        {1, 2, 0, "delayed", true},    {1, 2, 1, "delayed", true},
-        {1, 2, 2, "delayed", true},    {1, 2, 3, "immediate", false},
-        {1, 2, 4, "immediate", false}, {2, 1, 4, "immediate", false},
-        {2, 1, 4, "immediate", false},
+static const ChooseWithPriorityTestParam kChooseWithPriorityTestCases[] = {
+    {1, 2, 0, "delayed", true},    {1, 2, 1, "delayed", true},
+    {1, 2, 2, "delayed", true},    {1, 2, 3, "immediate", false},
+    {1, 2, 4, "immediate", false}, {2, 1, 4, "immediate", false},
+    {2, 1, 4, "immediate", false},
 };
 
-class ChooseOldestWithPriorityTest
+class ChooseWithPriorityTest
     : public TaskQueueSelectorTest,
-      public testing::WithParamInterface<ChooseOldestWithPriorityTestParam> {};
+      public testing::WithParamInterface<ChooseWithPriorityTestParam> {};
 
-TEST_P(ChooseOldestWithPriorityTest, RoundRobinTest) {
+TEST_P(ChooseWithPriorityTest, RoundRobinTest) {
   task_queues_[0]->immediate_work_queue()->Push(Task(
       PostedTask(test_closure_, FROM_HERE), TimeTicks(),
       EnqueueOrder::FromIntForTesting(GetParam().immediate_task_enqueue_order),
@@ -780,17 +853,19 @@ TEST_P(ChooseOldestWithPriorityTest, RoundRobinTest) {
       GetParam().immediate_starvation_count);
 
   bool chose_delayed_over_immediate = false;
-  WorkQueue* chosen_work_queue = selector_.ChooseOldestWithPriority(
-      TaskQueue::kNormalPriority, &chose_delayed_over_immediate);
+  WorkQueue* chosen_work_queue =
+      selector_
+          .ChooseWithPriority<TaskQueueSelectorForTest::SetOperationOldest>(
+              TaskQueue::kNormalPriority, &chose_delayed_over_immediate);
   EXPECT_EQ(chosen_work_queue->task_queue(), task_queues_[0].get());
   EXPECT_STREQ(chosen_work_queue->name(), GetParam().expected_work_queue_name);
   EXPECT_EQ(chose_delayed_over_immediate,
             GetParam().expected_did_starve_immediate_queue);
 }
 
-INSTANTIATE_TEST_CASE_P(ChooseOldestWithPriorityTest,
-                        ChooseOldestWithPriorityTest,
-                        testing::ValuesIn(kChooseOldestWithPriorityTestCases));
+INSTANTIATE_TEST_SUITE_P(ChooseWithPriorityTest,
+                         ChooseWithPriorityTest,
+                         testing::ValuesIn(kChooseWithPriorityTestCases));
 
 class SmallPriorityQueueTest : public testing::Test {
  public:
