@@ -9,12 +9,14 @@
 #include "base/lazy_instance.h"
 #include "base/location.h"
 #include "base/logging.h"
+#include "base/memory/ptr_util.h"
 #include "base/run_loop.h"
 #include "base/synchronization/waitable_event.h"
 #include "base/third_party/dynamic_annotations/dynamic_annotations.h"
 #include "base/threading/thread_id_name_manager.h"
 #include "base/threading/thread_local.h"
 #include "base/threading/thread_restrictions.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "build/build_config.h"
 
 #if defined(OS_POSIX) && !defined(OS_NACL)
@@ -43,7 +45,7 @@ Thread::Options::Options() = default;
 Thread::Options::Options(MessageLoop::Type type, size_t size)
     : message_loop_type(type), stack_size(size) {}
 
-Thread::Options::Options(const Options& other) = default;
+Thread::Options::Options(Options&& other) = default;
 
 Thread::Options::~Options() = default;
 
@@ -77,7 +79,7 @@ bool Thread::Start() {
 
 bool Thread::StartWithOptions(const Options& options) {
   DCHECK(owning_sequence_checker_.CalledOnValidSequence());
-  DCHECK(!message_loop_);
+  DCHECK(!task_environment_);
   DCHECK(!IsRunning());
   DCHECK(!stopping_) << "Starting a non-joinable thread a second time? That's "
                      << "not allowed!";
@@ -92,14 +94,19 @@ bool Thread::StartWithOptions(const Options& options) {
 
   SetThreadWasQuitProperly(false);
 
-  MessageLoop::Type type = options.message_loop_type;
-  if (!options.message_pump_factory.is_null())
-    type = MessageLoop::TYPE_CUSTOM;
-
   timer_slack_ = options.timer_slack;
-  std::unique_ptr<MessageLoop> message_loop_owned =
-      MessageLoop::CreateUnbound(type, options.message_pump_factory);
-  message_loop_ = message_loop_owned.get();
+
+  if (options.task_environment) {
+    DCHECK(!options.message_pump_factory);
+    task_environment_ = WrapUnique(options.task_environment);
+  } else if (options.message_pump_factory) {
+    task_environment_ = std::make_unique<internal::MessageLoopTaskEnvironment>(
+        MessageLoop::CreateUnbound(options.message_pump_factory.Run()));
+  } else {
+    task_environment_ = std::make_unique<internal::MessageLoopTaskEnvironment>(
+        MessageLoop::CreateUnbound(options.message_loop_type));
+  }
+
   start_event_.Reset();
 
   // Hold |thread_lock_| while starting the new thread to synchronize with
@@ -115,18 +122,12 @@ bool Thread::StartWithOptions(const Options& options) {
                   options.stack_size, this, options.priority);
     if (!success) {
       DLOG(ERROR) << "failed to create thread";
-      message_loop_ = nullptr;
       return false;
     }
   }
 
   joinable_ = options.joinable;
 
-  // The ownership of |message_loop_| is managed by the newly created thread
-  // within the ThreadMain.
-  ignore_result(message_loop_owned.release());
-
-  DCHECK(message_loop_);
   return true;
 }
 
@@ -141,7 +142,7 @@ bool Thread::StartAndWaitForTesting() {
 
 bool Thread::WaitUntilThreadStarted() const {
   DCHECK(owning_sequence_checker_.CalledOnValidSequence());
-  if (!message_loop_)
+  if (!task_environment_)
     return false;
   // https://crbug.com/918039
   base::ScopedAllowBaseSyncPrimitivesOutsideBlockingScope allow_wait;
@@ -151,7 +152,7 @@ bool Thread::WaitUntilThreadStarted() const {
 
 void Thread::FlushForTesting() {
   DCHECK(owning_sequence_checker_.CalledOnValidSequence());
-  if (!message_loop_)
+  if (!task_environment_)
     return;
 
   WaitableEvent done(WaitableEvent::ResetPolicy::AUTOMATIC,
@@ -178,15 +179,14 @@ void Thread::Stop() {
 
   // Wait for the thread to exit.
   //
-  // TODO(darin): Unfortunately, we need to keep |message_loop_| around until
-  // the thread exits.  Some consumers are abusing the API.  Make them stop.
-  //
+  // TODO(darin): Unfortunately, we need to keep |task_environment_| around
+  // until the thread exits. Some consumers are abusing the API. Make them stop.
   PlatformThread::Join(thread_);
   thread_ = base::PlatformThreadHandle();
 
-  // The thread should nullify |message_loop_| on exit (note: Join() adds an
-  // implicit memory barrier and no lock is thus required for this check).
-  DCHECK(!message_loop_);
+  // The thread should release |task_environment_| on exit (note: Join() adds
+  // an implicit memory barrier and no lock is thus required for this check).
+  DCHECK(!task_environment_);
 
   stopping_ = false;
 }
@@ -196,20 +196,11 @@ void Thread::StopSoon() {
   // enable this check.
   // DCHECK(owning_sequence_checker_.CalledOnValidSequence());
 
-  if (stopping_ || !message_loop_)
+  if (stopping_ || !task_environment_)
     return;
 
   stopping_ = true;
 
-  if (using_external_message_loop_) {
-    // Setting |stopping_| to true above should have been sufficient for this
-    // thread to be considered "stopped" per it having never set its |running_|
-    // bit by lack of its own ThreadMain.
-    DCHECK(!IsRunning());
-    message_loop_ = nullptr;
-    return;
-  }
-
   task_runner()->PostTask(
       FROM_HERE, base::BindOnce(&Thread::ThreadQuitHelper, Unretained(this)));
 }
@@ -231,11 +222,11 @@ bool Thread::IsRunning() const {
   // enable this check.
   // DCHECK(owning_sequence_checker_.CalledOnValidSequence());
 
-  // If the thread's already started (i.e. |message_loop_| is non-null) and not
-  // yet requested to stop (i.e. |stopping_| is false) we can just return true.
-  // (Note that |stopping_| is touched only on the same sequence that starts /
-  // started the new thread so we need no locking here.)
-  if (message_loop_ && !stopping_)
+  // If the thread's already started (i.e. |task_environment_| is non-null) and
+  // not yet requested to stop (i.e. |stopping_| is false) we can just return
+  // true. (Note that |stopping_| is touched only on the same sequence that
+  // starts / started the new thread so we need no locking here.)
+  if (task_environment_ && !stopping_)
     return true;
   // Otherwise check the |running_| flag, which is set to true by the new thread
   // only while it is inside Run().
@@ -259,25 +250,12 @@ void Thread::SetThreadWasQuitProperly(bool flag) {
 // static
 bool Thread::GetThreadWasQuitProperly() {
   bool quit_properly = true;
-#ifndef NDEBUG
+#if DCHECK_IS_ON()
   quit_properly = lazy_tls_bool.Pointer()->Get();
 #endif
   return quit_properly;
 }
 
-void Thread::SetMessageLoop(MessageLoop* message_loop) {
-  DCHECK(owning_sequence_checker_.CalledOnValidSequence());
-  DCHECK(message_loop);
-
-  // Setting |message_loop_| should suffice for this thread to be considered
-  // as "running", until Stop() is invoked.
-  DCHECK(!IsRunning());
-  message_loop_ = message_loop;
-  DCHECK(IsRunning());
-
-  using_external_message_loop_ = true;
-}
-
 void Thread::ThreadMain() {
   // First, make GetThreadId() available to avoid deadlocks. It could be called
   // any place in the following thread initialization code.
@@ -295,17 +273,18 @@ void Thread::ThreadMain() {
   ANNOTATE_THREAD_NAME(name_.c_str());  // Tell the name to race detector.
 
   // Lazily initialize the |message_loop| so that it can run on this thread.
-  DCHECK(message_loop_);
-  std::unique_ptr<MessageLoop> message_loop(message_loop_);
-  message_loop_->BindToCurrentThread();
-  message_loop_->SetTimerSlack(timer_slack_);
+  DCHECK(task_environment_);
+  // This binds MessageLoopCurrent and ThreadTaskRunnerHandle.
+  task_environment_->BindToCurrentThread(timer_slack_);
+  DCHECK(MessageLoopCurrent::Get());
+  DCHECK(ThreadTaskRunnerHandle::IsSet());
 
 #if defined(OS_POSIX) && !defined(OS_NACL)
   // Allow threads running a MessageLoopForIO to use FileDescriptorWatcher API.
   std::unique_ptr<FileDescriptorWatcher> file_descriptor_watcher;
   if (MessageLoopCurrentForIO::IsSet()) {
     file_descriptor_watcher.reset(
-        new FileDescriptorWatcher(message_loop_->task_runner()));
+        new FileDescriptorWatcher(task_environment_->GetDefaultTaskRunner()));
   }
 #endif
 
@@ -344,16 +323,11 @@ void Thread::ThreadMain() {
   com_initializer.reset();
 #endif
 
-  if (message_loop->type() != MessageLoop::TYPE_CUSTOM) {
-    // Assert that RunLoop::QuitWhenIdle was called by ThreadQuitHelper. Don't
-    // check for custom message pumps, because their shutdown might not allow
-    // this.
-    DCHECK(GetThreadWasQuitProperly());
-  }
+  DCHECK(GetThreadWasQuitProperly());
 
   // We can't receive messages anymore.
   // (The message loop is destructed at the end of this block)
-  message_loop_ = nullptr;
+  task_environment_.reset();
   run_loop_ = nullptr;
 }
 
@@ -363,4 +337,24 @@ void Thread::ThreadQuitHelper() {
   SetThreadWasQuitProperly(true);
 }
 
+namespace internal {
+
+MessageLoopTaskEnvironment::MessageLoopTaskEnvironment(
+    std::unique_ptr<MessageLoop> message_loop)
+    : message_loop_(std::move(message_loop)) {}
+
+MessageLoopTaskEnvironment::~MessageLoopTaskEnvironment() {}
+
+scoped_refptr<SingleThreadTaskRunner>
+MessageLoopTaskEnvironment::GetDefaultTaskRunner() {
+  return message_loop_->task_runner();
+}
+
+void MessageLoopTaskEnvironment::BindToCurrentThread(TimerSlack timer_slack) {
+  message_loop_->BindToCurrentThread();
+  message_loop_->SetTimerSlack(timer_slack);
+}
+
+}  // namespace internal
+
 }  // namespace base
