@@ -6,7 +6,6 @@
 
 #include <unordered_map>
 
-#include "base/memory/ptr_util.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "cc/animation/animation_host.h"
 #include "cc/input/scrollbar_animation_controller.h"
@@ -17,7 +16,6 @@
 #include "cc/layers/scrollbar_layer_interface.h"
 #include "cc/layers/solid_color_scrollbar_layer.h"
 #include "cc/layers/solid_color_scrollbar_layer_impl.h"
-#include "cc/quads/solid_color_draw_quad.h"
 #include "cc/resources/ui_resource_manager.h"
 #include "cc/test/fake_impl_task_runner_provider.h"
 #include "cc/test/fake_layer_tree_host.h"
@@ -30,21 +28,22 @@
 #include "cc/test/layer_tree_test.h"
 #include "cc/test/mock_occlusion_tracker.h"
 #include "cc/test/stub_layer_tree_host_single_thread_client.h"
-#include "cc/test/test_context_provider.h"
 #include "cc/test/test_task_graph_runner.h"
-#include "cc/test/test_web_graphics_context_3d.h"
 #include "cc/trees/effect_node.h"
 #include "cc/trees/layer_tree_host.h"
+#include "cc/trees/layer_tree_host_common.h"
 #include "cc/trees/layer_tree_impl.h"
 #include "cc/trees/occlusion_tracker.h"
 #include "cc/trees/scroll_node.h"
 #include "cc/trees/single_thread_proxy.h"
 #include "cc/trees/tree_synchronizer.h"
+#include "components/viz/common/quads/solid_color_draw_quad.h"
+#include "components/viz/test/test_context_provider.h"
+#include "components/viz/test/test_gles2_interface.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 namespace cc {
-namespace {
 
 class FakeResourceTrackingUIResourceManager : public UIResourceManager {
  public:
@@ -64,7 +63,7 @@ class FakeResourceTrackingUIResourceManager : public UIResourceManager {
 
   // Deletes a UI resource.  May safely be called more than once.
   void DeleteUIResource(UIResourceId id) override {
-    UIResourceBitmapMap::iterator iter = ui_resource_bitmap_map_.find(id);
+    auto iter = ui_resource_bitmap_map_.find(id);
     if (iter != ui_resource_bitmap_map_.end()) {
       ui_resource_bitmap_map_.erase(iter);
       total_ui_resource_deleted_++;
@@ -76,14 +75,14 @@ class FakeResourceTrackingUIResourceManager : public UIResourceManager {
   int TotalUIResourceCreated() { return total_ui_resource_created_; }
 
   gfx::Size ui_resource_size(UIResourceId id) {
-    UIResourceBitmapMap::iterator iter = ui_resource_bitmap_map_.find(id);
+    auto iter = ui_resource_bitmap_map_.find(id);
     if (iter != ui_resource_bitmap_map_.end())
       return iter->second.GetSize();
     return gfx::Size();
   }
 
   UIResourceBitmap* ui_resource_bitmap(UIResourceId id) {
-    UIResourceBitmapMap::iterator iter = ui_resource_bitmap_map_.find(id);
+    auto iter = ui_resource_bitmap_map_.find(id);
     if (iter != ui_resource_bitmap_map_.end())
       return &iter->second;
     return nullptr;
@@ -124,11 +123,11 @@ class BaseScrollbarLayerTest : public testing::Test {
 
     std::unique_ptr<FakeResourceTrackingUIResourceManager>
         fake_ui_resource_manager =
-            base::MakeUnique<FakeResourceTrackingUIResourceManager>();
+            std::make_unique<FakeResourceTrackingUIResourceManager>();
     fake_ui_resource_manager_ = fake_ui_resource_manager.get();
 
-    layer_tree_host_.reset(new FakeLayerTreeHost(
-        &fake_client_, &params, CompositorMode::SINGLE_THREADED));
+    layer_tree_host_ = std::make_unique<FakeLayerTreeHost>(
+        &fake_client_, std::move(params), CompositorMode::SINGLE_THREADED);
     layer_tree_host_->SetUIResourceManagerForTesting(
         std::move(fake_ui_resource_manager));
     layer_tree_host_->InitializeSingleThreaded(
@@ -221,7 +220,6 @@ TEST_F(ScrollbarLayerTest, RepaintOverlayWhenResourceDisposed) {
     scrollbar_layer->SetBounds(gfx::Size(100, 100));
     layer_tree_root->SetBounds(gfx::Size(100, 200));
     content_layer->SetBounds(gfx::Size(100, 200));
-    scrollbar_layer->set_visible_layer_rect(gfx::Rect(0, 0, 100, 200));
   }
 
   // First call to update should create a resource. The scrollbar itself thinks
@@ -255,47 +253,79 @@ TEST_F(ScrollbarLayerTest, RepaintOverlayWhenResourceDisposed) {
   }
 }
 
-TEST_F(ScrollbarLayerTest, ShouldScrollNonOverlayOnMainThread) {
-  // Create and attach a non-overlay scrollbar.
-  std::unique_ptr<Scrollbar> scrollbar(new FakeScrollbar);
-  LayerImpl* layer_impl_tree_root = LayerImplForScrollAreaAndScrollbar(
-      layer_tree_host_.get(), std::move(scrollbar), false, false, 0, 0);
-  PaintedScrollbarLayerImpl* scrollbar_layer_impl =
-      static_cast<PaintedScrollbarLayerImpl*>(
+class FakeNinePatchScrollbar : public FakeScrollbar {
+ public:
+  bool UsesNinePatchThumbResource() const override { return true; }
+};
+
+TEST_F(ScrollbarLayerTest, ScrollElementIdPushedAcrossCommit) {
+  std::unique_ptr<Scrollbar> scrollbar1(new FakeScrollbar);
+  std::unique_ptr<Scrollbar> scrollbar2(new FakeNinePatchScrollbar);
+  scoped_refptr<Layer> layer_tree_root = Layer::Create();
+  scoped_refptr<Layer> layer_a = Layer::Create();
+  scoped_refptr<Layer> layer_b = Layer::Create();
+  layer_a->SetElementId(LayerIdToElementIdForTesting(layer_a->id()));
+  layer_b->SetElementId(LayerIdToElementIdForTesting(layer_b->id()));
+
+  scoped_refptr<PaintedScrollbarLayer> painted_scrollbar_layer =
+      PaintedScrollbarLayer::Create(std::move(scrollbar1),
+                                    layer_a->element_id());
+  scoped_refptr<PaintedOverlayScrollbarLayer> painted_overlay_scrollbar_layer =
+      PaintedOverlayScrollbarLayer::Create(std::move(scrollbar2),
+                                           layer_a->element_id());
+  scoped_refptr<SolidColorScrollbarLayer> solid_color_scrollbar_layer =
+      SolidColorScrollbarLayer::Create(VERTICAL, 1, 1, false,
+                                       layer_a->element_id());
+
+  layer_tree_host_->SetRootLayer(layer_tree_root);
+  layer_tree_root->AddChild(layer_a);
+  layer_tree_root->AddChild(layer_b);
+  layer_tree_root->AddChild(painted_scrollbar_layer);
+  layer_tree_root->AddChild(painted_overlay_scrollbar_layer);
+  layer_tree_root->AddChild(solid_color_scrollbar_layer);
+
+  layer_tree_host_->UpdateLayers();
+  LayerImpl* layer_impl_tree_root =
+      layer_tree_host_->CommitAndCreateLayerImplTree();
+
+  ScrollbarLayerImplBase* painted_scrollbar_layer_impl =
+      static_cast<ScrollbarLayerImplBase*>(
           layer_impl_tree_root->layer_tree_impl()->LayerById(
-              scrollbar_layer_id_));
-  ScrollTree& scroll_tree =
-      layer_impl_tree_root->layer_tree_impl()->property_trees()->scroll_tree;
-  ScrollNode* scroll_node =
-      scroll_tree.Node(scrollbar_layer_impl->scroll_tree_index());
+              painted_scrollbar_layer->id()));
+  ScrollbarLayerImplBase* painted_overlay_scrollbar_layer_impl =
+      static_cast<ScrollbarLayerImplBase*>(
+          layer_impl_tree_root->layer_tree_impl()->LayerById(
+              painted_overlay_scrollbar_layer->id()));
+  ScrollbarLayerImplBase* solid_color_scrollbar_layer_impl =
+      static_cast<ScrollbarLayerImplBase*>(
+          layer_impl_tree_root->layer_tree_impl()->LayerById(
+              solid_color_scrollbar_layer->id()));
 
-  // When the scrollbar is not an overlay scrollbar, the scroll should be
-  // responded to on the main thread as the compositor does not yet implement
-  // scrollbar scrolling.
-  InputHandler::ScrollStatus status = layer_tree_host_->host_impl()->TryScroll(
-      gfx::PointF(), InputHandler::TOUCHSCREEN, scroll_tree, scroll_node);
-  EXPECT_EQ(InputHandler::SCROLL_ON_MAIN_THREAD, status.thread);
-  EXPECT_EQ(MainThreadScrollingReason::kScrollbarScrolling,
-            status.main_thread_scrolling_reasons);
+  ASSERT_EQ(painted_scrollbar_layer_impl->scroll_element_id_,
+            layer_a->element_id());
+  ASSERT_EQ(painted_overlay_scrollbar_layer_impl->scroll_element_id_,
+            layer_a->element_id());
+  ASSERT_EQ(solid_color_scrollbar_layer_impl->scroll_element_id_,
+            layer_a->element_id());
 
-  // Create and attach an overlay scrollbar.
-  scrollbar.reset(new FakeScrollbar(false, false, true));
+  painted_scrollbar_layer->SetScrollElementId(layer_b->element_id());
+  painted_overlay_scrollbar_layer->SetScrollElementId(layer_b->element_id());
+  solid_color_scrollbar_layer->SetScrollElementId(layer_b->element_id());
 
-  layer_impl_tree_root = LayerImplForScrollAreaAndScrollbar(
-      layer_tree_host_.get(), std::move(scrollbar), false, false, 0, 0);
-  scrollbar_layer_impl = static_cast<PaintedScrollbarLayerImpl*>(
-      layer_impl_tree_root->layer_tree_impl()->LayerById(scrollbar_layer_id_));
-  scroll_tree =
-      layer_impl_tree_root->layer_tree_impl()->property_trees()->scroll_tree;
-  scroll_node = scroll_tree.Node(scrollbar_layer_impl->scroll_tree_index());
+  ASSERT_TRUE(layer_tree_host_->needs_commit());
 
-  // The user shouldn't be able to drag an overlay scrollbar and the scroll
-  // may be handled in the compositor.
-  status = layer_tree_host_->host_impl()->TryScroll(
-      gfx::PointF(), InputHandler::TOUCHSCREEN, scroll_tree, scroll_node);
-  EXPECT_EQ(InputHandler::SCROLL_IGNORED, status.thread);
-  EXPECT_EQ(MainThreadScrollingReason::kNotScrollable,
-            status.main_thread_scrolling_reasons);
+  {
+    DebugScopedSetImplThread scoped_impl_thread(
+        layer_tree_host_->GetTaskRunnerProvider());
+    layer_tree_host_->FinishCommitOnImplThread(layer_tree_host_->host_impl());
+  }
+
+  EXPECT_EQ(painted_scrollbar_layer_impl->scroll_element_id_,
+            layer_b->element_id());
+  EXPECT_EQ(painted_overlay_scrollbar_layer_impl->scroll_element_id_,
+            layer_b->element_id());
+  EXPECT_EQ(solid_color_scrollbar_layer_impl->scroll_element_id_,
+            layer_b->element_id());
 }
 
 TEST_F(ScrollbarLayerTest, ScrollOffsetSynchronization) {
@@ -385,10 +415,13 @@ TEST_F(ScrollbarLayerTest, UpdatePropertiesOfScrollBarWhenThumbRemoved) {
   root_layer->SetScrollOffset(gfx::ScrollOffset(0, 0));
   scrollbar_layer->SetBounds(gfx::Size(70, 10));
   scrollbar_layer->SetScrollElementId(root_layer->element_id());
+
+  // The track_rect should be relative to the scrollbar's origin.
   scrollbar_layer->fake_scrollbar()->set_location(gfx::Point(20, 10));
-  scrollbar_layer->fake_scrollbar()->set_track_rect(gfx::Rect(30, 10, 50, 10));
+  scrollbar_layer->fake_scrollbar()->set_track_rect(gfx::Rect(10, 10, 50, 10));
   scrollbar_layer->fake_scrollbar()->set_thumb_thickness(10);
   scrollbar_layer->fake_scrollbar()->set_thumb_length(4);
+
   LayerImpl* root_layer_impl = nullptr;
   PaintedScrollbarLayerImpl* scrollbar_layer_impl = nullptr;
 
@@ -423,10 +456,13 @@ TEST_F(ScrollbarLayerTest, ThumbRect) {
   root_layer->SetScrollOffset(gfx::ScrollOffset(0, 0));
   scrollbar_layer->SetBounds(gfx::Size(70, 10));
   scrollbar_layer->SetScrollElementId(root_layer->element_id());
+
+  // The track_rect should be relative to the scrollbar's origin.
   scrollbar_layer->fake_scrollbar()->set_location(gfx::Point(20, 10));
-  scrollbar_layer->fake_scrollbar()->set_track_rect(gfx::Rect(30, 10, 50, 10));
+  scrollbar_layer->fake_scrollbar()->set_track_rect(gfx::Rect(10, 10, 50, 10));
   scrollbar_layer->fake_scrollbar()->set_thumb_thickness(10);
   scrollbar_layer->fake_scrollbar()->set_thumb_length(4);
+
   layer_tree_host_->UpdateLayers();
   LayerImpl* root_layer_impl = nullptr;
   PaintedScrollbarLayerImpl* scrollbar_layer_impl = nullptr;
@@ -464,7 +500,7 @@ TEST_F(ScrollbarLayerTest, ThumbRect) {
   // Shrink the scrollbar layer to cover only the track.
   scrollbar_layer->SetBounds(gfx::Size(50, 10));
   scrollbar_layer->fake_scrollbar()->set_location(gfx::Point(30, 10));
-  scrollbar_layer->fake_scrollbar()->set_track_rect(gfx::Rect(30, 10, 50, 10));
+  scrollbar_layer->fake_scrollbar()->set_track_rect(gfx::Rect(0, 10, 50, 10));
 
   UPDATE_AND_EXTRACT_LAYER_POINTERS();
   EXPECT_EQ(gfx::Rect(44, 0, 6, 4).ToString(),
@@ -473,7 +509,7 @@ TEST_F(ScrollbarLayerTest, ThumbRect) {
   // Shrink the track in the non-scrolling dimension so that it only covers the
   // middle third of the scrollbar layer (this does not affect the thumb
   // position).
-  scrollbar_layer->fake_scrollbar()->set_track_rect(gfx::Rect(30, 12, 50, 6));
+  scrollbar_layer->fake_scrollbar()->set_track_rect(gfx::Rect(0, 12, 50, 6));
 
   UPDATE_AND_EXTRACT_LAYER_POINTERS();
   EXPECT_EQ(gfx::Rect(44, 0, 6, 4).ToString(),
@@ -547,13 +583,13 @@ TEST_F(ScrollbarLayerTest, SolidColorDrawQuads) {
 
   // Thickness should be overridden to 3.
   {
-    std::unique_ptr<RenderPass> render_pass = RenderPass::Create();
+    std::unique_ptr<viz::RenderPass> render_pass = viz::RenderPass::Create();
     AppendQuadsData data;
     scrollbar_layer_impl->AppendQuads(render_pass.get(), &data);
 
-    const QuadList& quads = render_pass->quad_list;
+    const auto& quads = render_pass->quad_list;
     ASSERT_EQ(1u, quads.size());
-    EXPECT_EQ(DrawQuad::SOLID_COLOR, quads.front()->material);
+    EXPECT_EQ(viz::DrawQuad::Material::kSolidColor, quads.front()->material);
     EXPECT_EQ(gfx::Rect(6, 0, 39, 3), quads.front()->rect);
   }
 
@@ -562,13 +598,13 @@ TEST_F(ScrollbarLayerTest, SolidColorDrawQuads) {
   scrollbar_layer_impl->SetClipLayerLength(25.f);
   scrollbar_layer_impl->SetScrollLayerLength(125.f);
   {
-    std::unique_ptr<RenderPass> render_pass = RenderPass::Create();
+    std::unique_ptr<viz::RenderPass> render_pass = viz::RenderPass::Create();
     AppendQuadsData data;
     scrollbar_layer_impl->AppendQuads(render_pass.get(), &data);
 
-    const QuadList& quads = render_pass->quad_list;
+    const auto& quads = render_pass->quad_list;
     ASSERT_EQ(1u, quads.size());
-    EXPECT_EQ(DrawQuad::SOLID_COLOR, quads.front()->material);
+    EXPECT_EQ(viz::DrawQuad::Material::kSolidColor, quads.front()->material);
     EXPECT_EQ(gfx::Rect(8, 0, 19, 3), quads.front()->rect);
   }
 
@@ -577,13 +613,13 @@ TEST_F(ScrollbarLayerTest, SolidColorDrawQuads) {
   scrollbar_layer_impl->SetClipLayerLength(125.f);
   scrollbar_layer_impl->SetScrollLayerLength(125.f);
   {
-    std::unique_ptr<RenderPass> render_pass = RenderPass::Create();
+    std::unique_ptr<viz::RenderPass> render_pass = viz::RenderPass::Create();
     AppendQuadsData data;
     scrollbar_layer_impl->AppendQuads(render_pass.get(), &data);
 
-    const QuadList& quads = render_pass->quad_list;
+    const auto& quads = render_pass->quad_list;
     ASSERT_EQ(1u, quads.size());
-    EXPECT_EQ(DrawQuad::SOLID_COLOR, quads.front()->material);
+    EXPECT_EQ(viz::DrawQuad::Material::kSolidColor, quads.front()->material);
     EXPECT_EQ(gfx::Rect(1, 0, 98, 3), quads.front()->rect);
   }
 }
@@ -633,14 +669,14 @@ TEST_F(ScrollbarLayerTest, LayerDrivenSolidColorDrawQuads) {
   layer_tree_host_->active_tree()->UpdateScrollbarGeometries();
 
   {
-    std::unique_ptr<RenderPass> render_pass = RenderPass::Create();
+    std::unique_ptr<viz::RenderPass> render_pass = viz::RenderPass::Create();
 
     AppendQuadsData data;
     scrollbar_layer_impl->AppendQuads(render_pass.get(), &data);
 
-    const QuadList& quads = render_pass->quad_list;
+    const auto& quads = render_pass->quad_list;
     ASSERT_EQ(1u, quads.size());
-    EXPECT_EQ(DrawQuad::SOLID_COLOR, quads.front()->material);
+    EXPECT_EQ(viz::DrawQuad::Material::kSolidColor, quads.front()->material);
     EXPECT_EQ(gfx::Rect(3, 0, 3, 3), quads.front()->rect);
   }
 }
@@ -808,8 +844,6 @@ TEST_F(ScrollbarLayerTest, SubPixelCanScrollOrientation) {
 }
 
 TEST_F(ScrollbarLayerTest, LayerChangesAffectingScrollbarGeometries) {
-  gfx::Size viewport_size(980, 980);
-
   LayerTestCommon::LayerImplTest impl;
 
   LayerImpl* clip_layer = impl.AddChildToRoot<LayerImpl>();
@@ -894,10 +928,6 @@ TEST_F(AuraScrollbarLayerTest, ScrollbarLayerCreateAfterSetScrollable) {
   layer_tree_host_->CommitAndCreatePendingTree();
   host_impl->ActivateSyncTree();
 
-  LayerImpl* scroll_layer_impl =
-      host_impl->active_tree()->LayerByElementId(scroll_layer->element_id());
-  EXPECT_TRUE(scroll_layer_impl->needs_show_scrollbars());
-
   std::unique_ptr<Scrollbar> scrollbar(new FakeScrollbar(false, true, true));
   scoped_refptr<Layer> scrollbar_layer = SolidColorScrollbarLayer::Create(
       scrollbar->Orientation(), kThumbThickness, kTrackStart,
@@ -1047,17 +1077,19 @@ class ScrollbarLayerTestResourceCreationAndRelease : public ScrollbarLayerTest {
 
     layer_tree_host_->SetRootLayer(layer_tree_root);
 
+    LayerTreeHostCommon::CalcDrawPropsMainInputsForTesting inputs(
+        layer_tree_root.get(), layer_tree_host_->device_viewport_size());
+    LayerTreeHostCommon::CalculateDrawPropertiesForTesting(&inputs);
+
     scrollbar_layer->SetIsDrawable(true);
     scrollbar_layer->SetBounds(gfx::Size(100, 100));
     layer_tree_root->SetScrollable(gfx::Size(100, 200));
     layer_tree_root->SetScrollOffset(gfx::ScrollOffset(10, 20));
     layer_tree_root->SetBounds(gfx::Size(100, 200));
     content_layer->SetBounds(gfx::Size(100, 200));
-    scrollbar_layer->set_visible_layer_rect(gfx::Rect(0, 0, 100, 200));
 
     testing::Mock::VerifyAndClearExpectations(layer_tree_host_.get());
-    EXPECT_EQ(scrollbar_layer->GetLayerTreeHostForTesting(),
-              layer_tree_host_.get());
+    EXPECT_EQ(scrollbar_layer->layer_tree_host(), layer_tree_host_.get());
 
     for (int update_counter = 0; update_counter < num_updates; update_counter++)
       scrollbar_layer->Update();
@@ -1107,16 +1139,18 @@ TEST_F(ScrollbarLayerTestResourceCreationAndRelease, TestResourceUpdate) {
 
   layer_tree_host_->SetRootLayer(layer_tree_root);
 
+  LayerTreeHostCommon::CalcDrawPropsMainInputsForTesting inputs(
+      layer_tree_root.get(), layer_tree_host_->device_viewport_size());
+  LayerTreeHostCommon::CalculateDrawPropertiesForTesting(&inputs);
+
   scrollbar_layer->SetIsDrawable(true);
   scrollbar_layer->SetBounds(gfx::Size(100, 15));
   scrollbar_layer->SetPosition(gfx::PointF(scrollbar_location));
   layer_tree_root->SetBounds(gfx::Size(100, 200));
   content_layer->SetBounds(gfx::Size(100, 200));
-  scrollbar_layer->set_visible_layer_rect(gfx::Rect(0, 0, 100, 200));
 
   testing::Mock::VerifyAndClearExpectations(layer_tree_host_.get());
-  EXPECT_EQ(scrollbar_layer->GetLayerTreeHostForTesting(),
-            layer_tree_host_.get());
+  EXPECT_EQ(scrollbar_layer->layer_tree_host(), layer_tree_host_.get());
 
   size_t resource_count;
   int expected_created, expected_deleted;
@@ -1239,7 +1273,10 @@ TEST_F(ScrollbarLayerTestResourceCreationAndRelease, TestResourceUpdate) {
   EXPECT_EQ(gfx::Size(90, 15), fake_ui_resource_manager_->ui_resource_size(
                                    scrollbar_layer->track_resource_id()));
 
-  scrollbar_layer->ResetNeedsDisplayForTesting();
+  // Simulate commit to compositor thread.
+  scrollbar_layer->PushPropertiesTo(
+      scrollbar_layer->CreateLayerImpl(layer_tree_host_->active_tree()).get());
+
   EXPECT_FALSE(scrollbar_layer->Update());
   EXPECT_NE(0, scrollbar_layer->track_resource_id());
   EXPECT_EQ(0, scrollbar_layer->thumb_resource_id());
@@ -1272,13 +1309,17 @@ class ScaledScrollbarLayerTestResourceCreation : public ScrollbarLayerTest {
     scrollbar_layer->SetPosition(gfx::PointF(scrollbar_location));
     layer_tree_root->SetBounds(gfx::Size(100, 200));
     content_layer->SetBounds(gfx::Size(100, 200));
-    scrollbar_layer->set_visible_layer_rect(
-        gfx::Rect(scrollbar_location, scrollbar_layer->bounds()));
 
-    EXPECT_EQ(scrollbar_layer->GetLayerTreeHostForTesting(),
-              layer_tree_host_.get());
+    EXPECT_EQ(scrollbar_layer->layer_tree_host(), layer_tree_host_.get());
 
-    layer_tree_host_->SetDeviceScaleFactor(test_scale);
+    LayerTreeHostCommon::CalcDrawPropsMainInputsForTesting inputs(
+        layer_tree_root.get(), layer_tree_host_->device_viewport_size());
+    inputs.device_scale_factor = test_scale;
+    LayerTreeHostCommon::CalculateDrawPropertiesForTesting(&inputs);
+
+    layer_tree_host_->SetViewportSizeAndScale(
+        layer_tree_host_->device_viewport_size(), test_scale,
+        layer_tree_host_->local_surface_id_allocation_from_parent());
 
     scrollbar_layer->Update();
 
@@ -1310,10 +1351,11 @@ TEST_F(ScaledScrollbarLayerTestResourceCreation, ScaledResourceUpload) {
 
   // Try something extreme to be larger than max texture size, and make it a
   // non-integer for funsies.
-  scoped_refptr<TestContextProvider> context = TestContextProvider::Create();
+  scoped_refptr<viz::TestContextProvider> context =
+      viz::TestContextProvider::Create();
   // Keep the max texture size reasonable so we don't OOM on low end devices
   // (crbug.com/642333).
-  context->UnboundTestContext3d()->set_max_texture_size(512);
+  context->UnboundTestContextGL()->set_max_texture_size(512);
   context->BindToCurrentThread();
   int max_texture_size = 0;
   context->ContextGL()->GetIntegerv(GL_MAX_TEXTURE_SIZE, &max_texture_size);
@@ -1339,11 +1381,14 @@ class ScaledScrollbarLayerTestScaledRasterization : public ScrollbarLayerTest {
     scrollbar_layer->SetPosition(gfx::PointF(scrollbar_rect.origin()));
     scrollbar_layer->fake_scrollbar()->set_location(scrollbar_rect.origin());
     scrollbar_layer->fake_scrollbar()->set_track_rect(scrollbar_rect);
-    scrollbar_layer->set_visible_layer_rect(scrollbar_rect);
 
-    layer_tree_host_->SetDeviceScaleFactor(test_scale);
-
-    gfx::Rect screen_space_clip_rect;
+    LayerTreeHostCommon::CalcDrawPropsMainInputsForTesting inputs(
+        layer_tree_root.get(), layer_tree_host_->device_viewport_size());
+    inputs.device_scale_factor = test_scale;
+    LayerTreeHostCommon::CalculateDrawPropertiesForTesting(&inputs);
+    layer_tree_host_->SetViewportSizeAndScale(
+        layer_tree_host_->device_viewport_size(), test_scale,
+        layer_tree_host_->local_surface_id_allocation_from_parent());
 
     scrollbar_layer->Update();
 
@@ -1395,5 +1440,4 @@ TEST_F(ScaledScrollbarLayerTestScaledRasterization, TestLostPrecisionInClip) {
   TestScale(gfx::Rect(0, 1240, 677, 15), 2.46677136f);
 }
 
-}  // namespace
 }  // namespace cc
