@@ -6,12 +6,13 @@
 
 #include <stddef.h>
 
+#include <algorithm>
+#include <memory>
 #include <utility>
 #include <vector>
 
 #include "base/i18n/rtl.h"
 #include "base/lazy_instance.h"
-#include "base/memory/ptr_util.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/time/time.h"
@@ -24,14 +25,14 @@
 #include "chrome/browser/search/search.h"
 #include "chrome/browser/sessions/session_restore.h"
 #include "chrome/browser/sessions/tab_restore_service_factory.h"
-#include "chrome/browser/sync/profile_sync_service_factory.h"
+#include "chrome/browser/sync/session_sync_service_factory.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/browser_finder.h"
 #include "chrome/browser/ui/browser_live_tab_context.h"
 #include "chrome/browser/ui/tabs/tab_strip_model.h"
-#include "components/browser_sync/profile_sync_service.h"
 #include "components/sessions/content/content_live_tab.h"
 #include "components/sync_sessions/open_tabs_ui_delegate.h"
+#include "components/sync_sessions/session_sync_service.h"
 #include "components/sync_sessions/synced_session.h"
 #include "components/url_formatter/url_formatter.h"
 #include "content/public/browser/web_contents.h"
@@ -43,13 +44,13 @@
 
 namespace extensions {
 
+namespace windows = api::windows;
+
 namespace {
 
 namespace GetRecentlyClosed = api::sessions::GetRecentlyClosed;
 namespace GetDevices = api::sessions::GetDevices;
 namespace Restore = api::sessions::Restore;
-namespace tabs = api::tabs;
-namespace windows = api::windows;
 
 const char kNoRecentlyClosedSessionsError[] =
     "There are no recently closed sessions.";
@@ -74,14 +75,14 @@ bool SortTabsByRecency(const sessions::SessionTab* t1,
   return t1->timestamp > t2->timestamp;
 }
 
-tabs::Tab CreateTabModelHelper(
+api::tabs::Tab CreateTabModelHelper(
     const sessions::SerializedNavigationEntry& current_navigation,
     const std::string& session_id,
     int index,
     bool pinned,
     bool active,
     const Extension* extension) {
-  tabs::Tab tab_struct;
+  api::tabs::Tab tab_struct;
 
   const GURL& url = current_navigation.virtual_url();
   std::string title = base::UTF16ToUTF8(current_navigation.title());
@@ -103,12 +104,12 @@ tabs::Tab CreateTabModelHelper(
   return tab_struct;
 }
 
-std::unique_ptr<windows::Window> CreateWindowModelHelper(
-    std::unique_ptr<std::vector<tabs::Tab>> tabs,
+std::unique_ptr<api::windows::Window> CreateWindowModelHelper(
+    std::unique_ptr<std::vector<api::tabs::Tab>> tabs,
     const std::string& session_id,
-    const windows::WindowType& type,
-    const windows::WindowState& state) {
-  std::unique_ptr<windows::Window> window_struct(new windows::Window);
+    const api::windows::WindowType& type,
+    const api::windows::WindowState& state) {
+  std::unique_ptr<api::windows::Window> window_struct(new api::windows::Window);
   window_struct->tabs = std::move(tabs);
   window_struct->session_id.reset(new std::string(session_id));
   window_struct->incognito = false;
@@ -121,8 +122,8 @@ std::unique_ptr<windows::Window> CreateWindowModelHelper(
 
 std::unique_ptr<api::sessions::Session> CreateSessionModelHelper(
     int last_modified,
-    std::unique_ptr<tabs::Tab> tab,
-    std::unique_ptr<windows::Window> window) {
+    std::unique_ptr<api::tabs::Tab> tab,
+    std::unique_ptr<api::windows::Window> window) {
   std::unique_ptr<api::sessions::Session> session_struct(
       new api::sessions::Session());
   session_struct->last_modified = last_modified;
@@ -141,37 +142,38 @@ bool is_window_entry(const sessions::TabRestoreService::Entry& entry) {
 
 }  // namespace
 
-tabs::Tab SessionsGetRecentlyClosedFunction::CreateTabModel(
+api::tabs::Tab SessionsGetRecentlyClosedFunction::CreateTabModel(
     const sessions::TabRestoreService::Tab& tab,
     bool active) {
   return CreateTabModelHelper(tab.navigations[tab.current_navigation_index],
-                              base::IntToString(tab.id), tab.tabstrip_index,
-                              tab.pinned, active, extension());
+                              base::NumberToString(tab.id.id()),
+                              tab.tabstrip_index, tab.pinned, active,
+                              extension());
 }
 
-std::unique_ptr<windows::Window>
+std::unique_ptr<api::windows::Window>
 SessionsGetRecentlyClosedFunction::CreateWindowModel(
     const sessions::TabRestoreService::Window& window) {
   DCHECK(!window.tabs.empty());
 
-  auto tabs = base::MakeUnique<std::vector<tabs::Tab>>();
+  auto tabs = std::make_unique<std::vector<api::tabs::Tab>>();
   for (const auto& tab : window.tabs)
     tabs->push_back(
         CreateTabModel(*tab, tab->tabstrip_index == window.selected_tab_index));
 
-  return CreateWindowModelHelper(std::move(tabs), base::IntToString(window.id),
-                                 windows::WINDOW_TYPE_NORMAL,
-                                 windows::WINDOW_STATE_NORMAL);
+  return CreateWindowModelHelper(
+      std::move(tabs), base::NumberToString(window.id.id()),
+      api::windows::WINDOW_TYPE_NORMAL, api::windows::WINDOW_STATE_NORMAL);
 }
 
 std::unique_ptr<api::sessions::Session>
 SessionsGetRecentlyClosedFunction::CreateSessionModel(
     const sessions::TabRestoreService::Entry& entry) {
-  std::unique_ptr<tabs::Tab> tab;
-  std::unique_ptr<windows::Window> window;
+  std::unique_ptr<api::tabs::Tab> tab;
+  std::unique_ptr<api::windows::Window> window;
   switch (entry.type) {
     case sessions::TabRestoreService::TAB:
-      tab.reset(new tabs::Tab(CreateTabModel(
+      tab.reset(new api::tabs::Tab(CreateTabModel(
           static_cast<const sessions::TabRestoreService::Tab&>(entry), false)));
       break;
     case sessions::TabRestoreService::WINDOW:
@@ -218,7 +220,7 @@ ExtensionFunction::ResponseAction SessionsGetRecentlyClosedFunction::Run() {
   return RespondNow(ArgumentList(GetRecentlyClosed::Results::Create(result)));
 }
 
-tabs::Tab SessionsGetDevicesFunction::CreateTabModel(
+api::tabs::Tab SessionsGetDevicesFunction::CreateTabModel(
     const std::string& session_tag,
     const sessions::SessionTab& tab,
     int tab_index,
@@ -229,7 +231,8 @@ tabs::Tab SessionsGetDevicesFunction::CreateTabModel(
       tab.pinned, active, extension());
 }
 
-std::unique_ptr<windows::Window> SessionsGetDevicesFunction::CreateWindowModel(
+std::unique_ptr<api::windows::Window>
+SessionsGetDevicesFunction::CreateWindowModel(
     const sessions::SessionWindow& window,
     const std::string& session_tag) {
   DCHECK(!window.tabs.empty());
@@ -243,17 +246,19 @@ std::unique_ptr<windows::Window> SessionsGetDevicesFunction::CreateWindowModel(
       continue;
     const sessions::SerializedNavigationEntry& current_navigation =
         tab->navigations.at(tab->normalized_navigation_index());
-    if (search::IsNTPURL(current_navigation.virtual_url(),
-                         Profile::FromBrowserContext(browser_context()))) {
+    if (search::IsNTPOrRelatedURL(
+            current_navigation.virtual_url(),
+            Profile::FromBrowserContext(browser_context()))) {
       continue;
     }
     tabs_in_window.push_back(tab);
   }
   if (tabs_in_window.empty())
-    return std::unique_ptr<windows::Window>();
+    return nullptr;
   std::sort(tabs_in_window.begin(), tabs_in_window.end(), SortTabsByRecency);
 
-  std::unique_ptr<std::vector<tabs::Tab>> tabs(new std::vector<tabs::Tab>());
+  std::unique_ptr<std::vector<api::tabs::Tab>> tabs(
+      new std::vector<api::tabs::Tab>());
   for (size_t i = 0; i < tabs_in_window.size(); ++i) {
     tabs->push_back(CreateTabModel(session_tag, *tabs_in_window[i], i,
                                    window.selected_tab_index == (int)i));
@@ -262,29 +267,29 @@ std::unique_ptr<windows::Window> SessionsGetDevicesFunction::CreateWindowModel(
   std::string session_id =
       SessionId(session_tag, window.window_id.id()).ToString();
 
-  windows::WindowType type = windows::WINDOW_TYPE_NONE;
+  api::windows::WindowType type = api::windows::WINDOW_TYPE_NONE;
   switch (window.type) {
     case sessions::SessionWindow::TYPE_TABBED:
-      type = windows::WINDOW_TYPE_NORMAL;
+      type = api::windows::WINDOW_TYPE_NORMAL;
       break;
     case sessions::SessionWindow::TYPE_POPUP:
-      type = windows::WINDOW_TYPE_POPUP;
+      type = api::windows::WINDOW_TYPE_POPUP;
       break;
   }
 
-  windows::WindowState state = windows::WINDOW_STATE_NONE;
+  api::windows::WindowState state = api::windows::WINDOW_STATE_NONE;
   switch (window.show_state) {
     case ui::SHOW_STATE_NORMAL:
-      state = windows::WINDOW_STATE_NORMAL;
+      state = api::windows::WINDOW_STATE_NORMAL;
       break;
     case ui::SHOW_STATE_MINIMIZED:
-      state = windows::WINDOW_STATE_MINIMIZED;
+      state = api::windows::WINDOW_STATE_MINIMIZED;
       break;
     case ui::SHOW_STATE_MAXIMIZED:
-      state = windows::WINDOW_STATE_MAXIMIZED;
+      state = api::windows::WINDOW_STATE_MAXIMIZED;
       break;
     case ui::SHOW_STATE_FULLSCREEN:
-      state = windows::WINDOW_STATE_FULLSCREEN;
+      state = api::windows::WINDOW_STATE_FULLSCREEN;
       break;
     case ui::SHOW_STATE_DEFAULT:
     case ui::SHOW_STATE_INACTIVE:
@@ -292,7 +297,7 @@ std::unique_ptr<windows::Window> SessionsGetDevicesFunction::CreateWindowModel(
       break;
   }
 
-  std::unique_ptr<windows::Window> window_struct(
+  std::unique_ptr<api::windows::Window> window_struct(
       CreateWindowModelHelper(std::move(tabs), session_id, type, state));
   // TODO(dwankri): Dig deeper to resolve bounds not being optional, so closed
   // windows in GetRecentlyClosed can have set values in Window helper.
@@ -308,14 +313,15 @@ std::unique_ptr<api::sessions::Session>
 SessionsGetDevicesFunction::CreateSessionModel(
     const sessions::SessionWindow& window,
     const std::string& session_tag) {
-  std::unique_ptr<windows::Window> window_model(
+  std::unique_ptr<api::windows::Window> window_model(
       CreateWindowModel(window, session_tag));
   // There is a chance that after pruning uninteresting tabs the window will be
   // empty.
-  return !window_model ? std::unique_ptr<api::sessions::Session>()
-                       : CreateSessionModelHelper(window.timestamp.ToTimeT(),
-                                                  std::unique_ptr<tabs::Tab>(),
-                                                  std::move(window_model));
+  return !window_model
+             ? nullptr
+             : CreateSessionModelHelper(window.timestamp.ToTimeT(),
+                                        std::unique_ptr<api::tabs::Tab>(),
+                                        std::move(window_model));
 }
 
 api::sessions::Device SessionsGetDevicesFunction::CreateDeviceModel(
@@ -344,18 +350,15 @@ api::sessions::Device SessionsGetDevicesFunction::CreateDeviceModel(
 }
 
 ExtensionFunction::ResponseAction SessionsGetDevicesFunction::Run() {
-  browser_sync::ProfileSyncService* service =
-      ProfileSyncServiceFactory::GetInstance()->GetForProfile(
+  sync_sessions::SessionSyncService* service =
+      SessionSyncServiceFactory::GetInstance()->GetForProfile(
           Profile::FromBrowserContext(browser_context()));
-  if (!(service && service->GetPreferredDataTypes().Has(syncer::SESSIONS))) {
-    // Sync not enabled.
-    return RespondNow(ArgumentList(
-        GetDevices::Results::Create(std::vector<api::sessions::Device>())));
-  }
+  DCHECK(service);
 
   sync_sessions::OpenTabsUIDelegate* open_tabs =
       service->GetOpenTabsUIDelegate();
   std::vector<const sync_sessions::SyncedSession*> sessions;
+  // If the user has disabled tab sync, GetOpenTabsUIDelegate() returns null.
   if (!(open_tabs && open_tabs->GetAllForeignSessions(&sessions))) {
     return RespondNow(ArgumentList(
         GetDevices::Results::Create(std::vector<api::sessions::Device>())));
@@ -380,28 +383,29 @@ ExtensionFunction::ResponseAction SessionsGetDevicesFunction::Run() {
 
 ExtensionFunction::ResponseValue SessionsRestoreFunction::GetRestoredTabResult(
     content::WebContents* contents) {
-  std::unique_ptr<tabs::Tab> tab(
-      ExtensionTabUtil::CreateTabObject(contents, extension()));
+  std::unique_ptr<api::tabs::Tab> tab(ExtensionTabUtil::CreateTabObject(
+      contents, ExtensionTabUtil::kScrubTab, extension()));
   std::unique_ptr<api::sessions::Session> restored_session(
       CreateSessionModelHelper(base::Time::Now().ToTimeT(), std::move(tab),
-                               std::unique_ptr<windows::Window>()));
+                               std::unique_ptr<api::windows::Window>()));
   return ArgumentList(Restore::Results::Create(*restored_session));
 }
 
 ExtensionFunction::ResponseValue
 SessionsRestoreFunction::GetRestoredWindowResult(int window_id) {
-  WindowController* controller = NULL;
+  Browser* browser = nullptr;
   std::string error;
-  if (!windows_util::GetWindowFromWindowID(this, window_id, 0, &controller,
-                                           &error)) {
+  if (!windows_util::GetBrowserFromWindowID(this, window_id, 0, &browser,
+                                            &error)) {
     return Error(error);
   }
   std::unique_ptr<base::DictionaryValue> window_value(
-      controller->CreateWindowValueWithTabs(extension()));
-  std::unique_ptr<windows::Window> window(
-      windows::Window::FromValue(*window_value));
+      ExtensionTabUtil::CreateWindowValueForExtension(
+          *browser, extension(), ExtensionTabUtil::kPopulateTabs));
+  std::unique_ptr<api::windows::Window> window(
+      api::windows::Window::FromValue(*window_value));
   return ArgumentList(Restore::Results::Create(*CreateSessionModelHelper(
-      base::Time::Now().ToTimeT(), std::unique_ptr<tabs::Tab>(),
+      base::Time::Now().ToTimeT(), std::unique_ptr<api::tabs::Tab>(),
       std::move(window))));
 }
 
@@ -449,7 +453,7 @@ ExtensionFunction::ResponseValue SessionsRestoreFunction::RestoreLocalSession(
   // Check if the recently closed list contains an entry with the provided id.
   bool is_window = false;
   for (const auto& entry : entries) {
-    if (entry->id == session_id.id()) {
+    if (entry->id.id() == session_id.id()) {
       // A full window is being restored only if the entry ID
       // matches the provided ID and the entry type is Window.
       is_window = is_window_entry(*entry);
@@ -461,8 +465,9 @@ ExtensionFunction::ResponseValue SessionsRestoreFunction::RestoreLocalSession(
       BrowserLiveTabContext::FindContextForWebContents(
           browser->tab_strip_model()->GetActiveWebContents());
   std::vector<sessions::LiveTab*> restored_tabs =
-      tab_restore_service->RestoreEntryById(context, session_id.id(),
-                                            WindowOpenDisposition::UNKNOWN);
+      tab_restore_service->RestoreEntryById(
+          context, SessionID::FromSerializedValue(session_id.id()),
+          WindowOpenDisposition::UNKNOWN);
   // If the ID is invalid, restored_tabs will be empty.
   if (restored_tabs.empty())
     return Error(kInvalidSessionIdError, session_id.ToString());
@@ -483,18 +488,20 @@ ExtensionFunction::ResponseValue SessionsRestoreFunction::RestoreForeignSession(
     const SessionId& session_id,
     Browser* browser) {
   Profile* profile = Profile::FromBrowserContext(browser_context());
-  browser_sync::ProfileSyncService* service =
-      ProfileSyncServiceFactory::GetInstance()->GetForProfile(profile);
-  if (!(service && service->GetPreferredDataTypes().Has(syncer::SESSIONS)))
-    return Error(kSessionSyncError);
+  sync_sessions::SessionSyncService* service =
+      SessionSyncServiceFactory::GetInstance()->GetForProfile(
+          Profile::FromBrowserContext(browser_context()));
+  DCHECK(service);
+
   sync_sessions::OpenTabsUIDelegate* open_tabs =
       service->GetOpenTabsUIDelegate();
+  // If the user has disabled tab sync, GetOpenTabsUIDelegate() returns null.
   if (!open_tabs)
     return Error(kSessionSyncError);
 
   const sessions::SessionTab* tab = NULL;
   if (open_tabs->GetForeignTab(session_id.session_tag(),
-                               session_id.id(),
+                               SessionID::FromSerializedValue(session_id.id()),
                                &tab)) {
     TabStripModel* tab_strip = browser->tab_strip_model();
     content::WebContents* contents = tab_strip->GetActiveWebContents();
@@ -570,7 +577,7 @@ SessionsEventRouter::~SessionsEventRouter() {
 void SessionsEventRouter::TabRestoreServiceChanged(
     sessions::TabRestoreService* service) {
   std::unique_ptr<base::ListValue> args(new base::ListValue());
-  EventRouter::Get(profile_)->BroadcastEvent(base::MakeUnique<Event>(
+  EventRouter::Get(profile_)->BroadcastEvent(std::make_unique<Event>(
       events::SESSIONS_ON_CHANGED, api::sessions::OnChanged::kEventName,
       std::move(args)));
 }
@@ -593,13 +600,12 @@ void SessionsAPI::Shutdown() {
   EventRouter::Get(browser_context_)->UnregisterObserver(this);
 }
 
-static base::LazyInstance<
-    BrowserContextKeyedAPIFactory<SessionsAPI>>::DestructorAtExit g_factory =
-    LAZY_INSTANCE_INITIALIZER;
+static base::LazyInstance<BrowserContextKeyedAPIFactory<SessionsAPI>>::
+    DestructorAtExit g_sessions_api_factory = LAZY_INSTANCE_INITIALIZER;
 
 BrowserContextKeyedAPIFactory<SessionsAPI>*
 SessionsAPI::GetFactoryInstance() {
-  return g_factory.Pointer();
+  return g_sessions_api_factory.Pointer();
 }
 
 void SessionsAPI::OnListenerAdded(const EventListenerInfo& details) {
