@@ -15,52 +15,34 @@
 cr.define('local_discovery', function() {
   'use strict';
 
-  // Histogram buckets for UMA tracking.
-  /** @const */ var DEVICES_PAGE_EVENTS = {
-    OPENED: 0,
-    LOG_IN_STARTED_FROM_REGISTER_PROMO: 1,
-    LOG_IN_STARTED_FROM_DEVICE_LIST_PROMO: 2,
-    ADD_PRINTER_CLICKED: 3,
-    REGISTER_CLICKED: 4,
-    REGISTER_CONFIRMED: 5,
-    REGISTER_SUCCESS: 6,
-    REGISTER_CANCEL: 7,
-    REGISTER_FAILURE: 8,
-    MANAGE_CLICKED: 9,
-    REGISTER_CANCEL_ON_PRINTER: 10,
-    REGISTER_TIMEOUT: 11,
-    LOG_IN_STARTED_FROM_REGISTER_OVERLAY_PROMO: 12,
-    MAX_EVENT: 13,
-  };
-
   /**
    * Map of service names to corresponding service objects.
    * @type {Object<string,Service>}
    */
-  var devices = {};
+  const devices = {};
 
   /**
    * Whether or not the user is currently logged in.
    * @type bool
    */
-  var isUserLoggedIn = true;
+  let isUserLoggedIn = true;
 
   /**
    * Whether or not the user is supervised or off the record.
    * @type bool
    */
-  var isUserSupervisedOrOffTheRecord = false;
+  let isUserSupervisedOrOffTheRecord = false;
 
   /**
    * Whether or not the path-based dialog has been shown.
    * @type bool
    */
-  var dialogFromPathHasBeenShown = false;
+  let dialogFromPathHasBeenShown = false;
 
   /**
    * Focus manager for page.
    */
-  var focusManager = null;
+  let focusManager = null;
 
   /**
    * Object that represents a device in the device list.
@@ -104,40 +86,32 @@ cr.define('local_discovery', function() {
 
       this.registerButton = fillDeviceDescription(
           this.domElement, this.info.display_name, this.info.description,
-          this.info.type, loadTimeData.getString('serviceRegister'),
-          this.showRegister.bind(this, this.info.type));
+          loadTimeData.getString('serviceRegister'),
+          this.showRegister.bind(this));
 
       this.setRegisterEnabled(this.registerEnabled);
     },
 
     /**
      * Return the correct container for the device.
-     * @param {boolean} is_mine Whether or not the device is in the 'Registered'
-     *    section.
      */
     deviceContainer: function() {
       return $('register-device-list');
     },
+
     /**
      * Register the device.
      */
     register: function() {
-      recordUmaEvent(DEVICES_PAGE_EVENTS.REGISTER_CONFIRMED);
       chrome.send('registerDevice', [this.info.service_name]);
-      setRegisterPage(
-          isPrinter(this.info.type) ? 'register-printer-page-adding1' :
-                                      'register-device-page-adding1');
+      setRegisterPage('register-printer-page-adding1');
     },
     /**
      * Show registrtation UI for device.
      */
     showRegister: function() {
-      recordUmaEvent(DEVICES_PAGE_EVENTS.REGISTER_CLICKED);
-      $('register-message').textContent = loadTimeData.getStringF(
-          isPrinter(this.info.type) ? 'registerPrinterConfirmMessage' :
-                                      'registerDeviceConfirmMessage',
-          this.info.display_name);
-      $('register-continue-button').onclick = this.register.bind(this);
+      $('register-continue').onclick = this.register.bind(this);
+
       showRegisterOverlay();
     },
     /**
@@ -186,39 +160,37 @@ cr.define('local_discovery', function() {
 
   /**
    * Fill device element with the description of a device.
-   * @param {HTMLElement} device_dom_element Element to be filled.
+   * @param {HTMLElement} deviceDomElement Element to be filled.
    * @param {string} name Name of device.
    * @param {string} description Description of device.
-   * @param {string} type Type of device.
-   * @param {string} button_text Text to appear on button.
-   * @param {function()?} button_action Action for button.
+   * @param {string} buttonText Text to appear on button.
+   * @param {function()?} buttonAction Action for button.
    * @return {HTMLElement} The button (for enabling/disabling/rebinding)
    */
   function fillDeviceDescription(
-      device_dom_element, name, description, type, button_text, button_action) {
-    device_dom_element.classList.add('device');
-    if (isPrinter(type))
-      device_dom_element.classList.add('printer');
+      deviceDomElement, name, description, buttonText, buttonAction) {
+    deviceDomElement.classList.add('device');
 
-    var deviceInfo = document.createElement('div');
+    const deviceInfo = document.createElement('div');
     deviceInfo.className = 'device-info';
-    device_dom_element.appendChild(deviceInfo);
+    deviceDomElement.appendChild(deviceInfo);
 
-    var deviceName = document.createElement('h3');
+    const deviceName = document.createElement('h3');
     deviceName.className = 'device-name';
     deviceName.textContent = name;
     deviceInfo.appendChild(deviceName);
 
-    var deviceDescription = document.createElement('div');
+    const deviceDescription = document.createElement('div');
     deviceDescription.className = 'device-subline';
     deviceDescription.textContent = description;
     deviceInfo.appendChild(deviceDescription);
 
-    if (button_action) {
-      var button = document.createElement('button');
-      button.textContent = button_text;
-      button.addEventListener('click', button_action);
-      device_dom_element.appendChild(button);
+    let button;
+    if (buttonAction) {
+      button = document.createElement('button');
+      button.textContent = buttonText;
+      button.addEventListener('click', buttonAction);
+      deviceDomElement.appendChild(button);
     }
 
     return button;
@@ -228,9 +200,7 @@ cr.define('local_discovery', function() {
    * Show the register overlay.
    */
   function showRegisterOverlay() {
-    recordUmaEvent(DEVICES_PAGE_EVENTS.ADD_PRINTER_CLICKED);
-
-    var registerOverlay = $('register-overlay');
+    const registerOverlay = $('register-overlay');
     registerOverlay.classList.add('showing');
     registerOverlay.focus();
 
@@ -263,7 +233,6 @@ cr.define('local_discovery', function() {
     $('error-message').textContent =
         loadTimeData.getString('addingErrorMessage');
     setRegisterPage('register-page-error');
-    recordUmaEvent(DEVICES_PAGE_EVENTS.REGISTER_FAILURE);
   }
 
   /**
@@ -273,7 +242,6 @@ cr.define('local_discovery', function() {
     $('error-message').textContent =
         loadTimeData.getString('addingCanceledMessage');
     setRegisterPage('register-page-error');
-    recordUmaEvent(DEVICES_PAGE_EVENTS.REGISTER_CANCEL_ON_PRINTER);
   }
 
   /**
@@ -283,7 +251,6 @@ cr.define('local_discovery', function() {
     $('error-message').textContent =
         loadTimeData.getString('addingTimeoutMessage');
     setRegisterPage('register-page-error');
-    recordUmaEvent(DEVICES_PAGE_EVENTS.REGISTER_TIMEOUT);
   }
 
   /**
@@ -293,15 +260,6 @@ cr.define('local_discovery', function() {
     setRegisterPage('register-printer-page-adding2');
   }
 
-  /**
-   * Shows UI to confirm security code.
-   * @param {string} code The security code to confirm.
-   */
-  function onRegistrationConfirmDeviceCode(code) {
-    setRegisterPage('register-device-page-adding2');
-    $('register-device-page-code').textContent = code;
-  }
-
   /**
    * Update device unregistered device list, and update related strings to
    * reflect the number of devices available to register.
@@ -334,42 +292,34 @@ cr.define('local_discovery', function() {
 
   /**
    * Create the DOM for a cloud device described by the device section.
-   * @param {Array<Object>} devices_list List of devices.
+   * @param {Object} device The device to create the DOM for.
    */
   function createCloudDeviceDOM(device) {
-    var devicesDomElement = document.createElement('div');
+    const devicesDomElement = document.createElement('div');
 
-    var description;
-    if (device.description == '') {
-      if (isPrinter(device.type))
-        description = loadTimeData.getString('noDescriptionPrinter');
-      else
-        description = loadTimeData.getString('noDescriptionDevice');
-    } else {
-      description = device.description;
-    }
+    const description =
+        device.description || loadTimeData.getString('noDescriptionPrinter');
 
     fillDeviceDescription(
-        devicesDomElement, device.display_name, description, device.type,
+        devicesDomElement, device.display_name, description,
         loadTimeData.getString('manageDevice'),
-        isPrinter(device.type) ? manageCloudDevice.bind(null, device.id) :
-                                 null);
+        manageCloudDevice.bind(null, device.id));
     return devicesDomElement;
   }
 
   /**
    * Handle a list of cloud devices available to the user globally.
-   * @param {Array<Object>} devices_list List of devices.
+   * @param {Array<Object>} devicesList List of devices.
    */
-  function onCloudDeviceListAvailable(devices_list) {
-    var devicesListLength = devices_list.length;
-    var devicesContainer = $('cloud-devices');
+  function onCloudDeviceListAvailable(devicesList) {
+    const devicesListLength = devicesList.length;
+    const devicesContainer = $('cloud-devices');
 
     clearElement(devicesContainer);
     $('cloud-devices-loading').hidden = true;
 
-    for (var i = 0; i < devicesListLength; i++) {
-      devicesContainer.appendChild(createCloudDeviceDOM(devices_list[i]));
+    for (let i = 0; i < devicesListLength; i++) {
+      devicesContainer.appendChild(createCloudDeviceDOM(devicesList[i]));
     }
   }
 
@@ -387,7 +337,7 @@ cr.define('local_discovery', function() {
    * Handle the case where the cache for local devices has been flushed..
    */
   function onDeviceCacheFlushed() {
-    for (var deviceName in devices) {
+    for (const deviceName in devices) {
       devices[deviceName].removeDevice();
       delete devices[deviceName];
     }
@@ -399,7 +349,7 @@ cr.define('local_discovery', function() {
    * Update UI strings to reflect the number of local devices.
    */
   function updateUIToReflectState() {
-    var numberPrinters = $('register-device-list').children.length;
+    const numberPrinters = $('register-device-list').children.length;
     if (numberPrinters == 0) {
       $('no-printers-message').hidden = false;
 
@@ -419,38 +369,31 @@ cr.define('local_discovery', function() {
 
   /**
    * Announce that a registration succeeeded.
+   * @param {!Object} deviceData data describing the device that was registered.
    */
-  function onRegistrationSuccess(device_data) {
+  function onRegistrationSuccess(deviceData) {
     hideRegisterOverlay();
 
-    if (device_data.service_name == getOverlayIDFromPath()) {
+    if (deviceData.service_name == getOverlayIDFromPath()) {
       window.close();
     }
 
-    var deviceDOM = createCloudDeviceDOM(device_data);
+    const deviceDOM = createCloudDeviceDOM(deviceData);
     $('cloud-devices').insertBefore(deviceDOM, $('cloud-devices').firstChild);
-    recordUmaEvent(DEVICES_PAGE_EVENTS.REGISTER_SUCCESS);
-  }
-
-  /**
-   * Update visibility status for page.
-   */
-  function updateVisibility() {
-    chrome.send('isVisible', [!document.hidden]);
   }
 
   /**
    * Set the page that the register wizard is on.
-   * @param {string} page_id ID string for page.
+   * @param {string} pageId ID string for page.
    */
-  function setRegisterPage(page_id) {
-    var pages = $('register-overlay').querySelectorAll('.register-page');
-    var pagesLength = pages.length;
-    for (var i = 0; i < pagesLength; i++) {
+  function setRegisterPage(pageId) {
+    const pages = $('register-overlay').querySelectorAll('.register-page');
+    const pagesLength = pages.length;
+    for (let i = 0; i < pagesLength; i++) {
       pages[i].hidden = true;
     }
 
-    $(page_id).hidden = false;
+    $(pageId).hidden = false;
   }
 
   /**
@@ -468,22 +411,10 @@ cr.define('local_discovery', function() {
 
   /**
    * Go to management page for a cloud device.
-   * @param {string} device_id ID of device.
+   * @param {string} deviceId ID of device.
    */
-  function manageCloudDevice(device_id) {
-    recordUmaEvent(DEVICES_PAGE_EVENTS.MANAGE_CLICKED);
-    chrome.send('openCloudPrintURL', [device_id]);
-  }
-
-  /**
-   * Record an event in the UMA histogram.
-   * @param {number} eventId The id of the event to be recorded.
-   * @private
-   */
-  function recordUmaEvent(eventId) {
-    chrome.send(
-        'metricsHandler:recordInHistogram',
-        ['LocalDiscovery.DevicesPage', eventId, DEVICES_PAGE_EVENTS.MAX_EVENT]);
+  function manageCloudDevice(deviceId) {
+    chrome.send('openCloudPrintURL', [deviceId]);
   }
 
   /**
@@ -492,15 +423,6 @@ cr.define('local_discovery', function() {
   function cancelRegistration() {
     hideRegisterOverlay();
     chrome.send('cancelRegistration');
-    recordUmaEvent(DEVICES_PAGE_EVENTS.REGISTER_CANCEL);
-  }
-
-  /**
-   * Confirms device code.
-   */
-  function confirmCode() {
-    chrome.send('confirmCode');
-    setRegisterPage('register-device-page-adding1');
   }
 
   /**
@@ -521,7 +443,7 @@ cr.define('local_discovery', function() {
         isUserLoggedIn || isUserSupervisedOrOffTheRecord;
     $('register-overlay-login-promo').hidden =
         isUserLoggedIn || isUserSupervisedOrOffTheRecord;
-    $('register-continue-button').disabled =
+    $('register-continue').disabled =
         !isUserLoggedIn || isUserSupervisedOrOffTheRecord;
 
     $('my-devices-container').hidden = userSupervisedOrOffTheRecord;
@@ -542,7 +464,7 @@ cr.define('local_discovery', function() {
 
     updateUIToReflectState();
 
-    for (var device in devices) {
+    for (const device in devices) {
       devices[device].setRegisterEnabled(isUserLoggedIn);
     }
   }
@@ -552,18 +474,14 @@ cr.define('local_discovery', function() {
   }
 
   function registerLoginButtonClicked() {
-    recordUmaEvent(DEVICES_PAGE_EVENTS.LOG_IN_STARTED_FROM_REGISTER_PROMO);
     openSignInPage();
   }
 
   function registerOverlayLoginButtonClicked() {
-    recordUmaEvent(
-        DEVICES_PAGE_EVENTS.LOG_IN_STARTED_FROM_REGISTER_OVERLAY_PROMO);
     openSignInPage();
   }
 
   function cloudDevicesLoginButtonClicked() {
-    recordUmaEvent(DEVICES_PAGE_EVENTS.LOG_IN_STARTED_FROM_DEVICE_LIST_PROMO);
     openSignInPage();
   }
 
@@ -602,19 +520,11 @@ cr.define('local_discovery', function() {
 
   function getOverlayIDFromPath() {
     if (document.location.pathname == '/register') {
-      var params = parseQueryParams(document.location);
+      const params = parseQueryParams(document.location);
       return params['id'] || null;
     }
   }
 
-  /**
-   * Returns true of device is printer.
-   * @param {string} type Type of printer.
-   */
-  function isPrinter(type) {
-    return type == 'printer';
-  }
-
   document.addEventListener('DOMContentLoaded', function() {
     cr.ui.overlay.setupOverlay($('overlay'));
     cr.ui.overlay.globalInitialization();
@@ -625,11 +535,6 @@ cr.define('local_discovery', function() {
           button.addEventListener('click', cancelRegistration);
         });
 
-    [].forEach.call(
-        document.querySelectorAll('.confirm-code'), function(button) {
-          button.addEventListener('click', confirmCode);
-        });
-
     $('register-error-exit').addEventListener('click', cancelRegistration);
 
 
@@ -645,21 +550,10 @@ cr.define('local_discovery', function() {
     $('register-overlay-login-button')
         .addEventListener('click', registerOverlayLoginButtonClicked);
 
-    if (loadTimeData.valueExists('backButtonURL')) {
-      $('back-link').hidden = false;
-      $('back-link').addEventListener('click', function() {
-        window.location.href = loadTimeData.getString('backButtonURL');
-      });
-    }
-
-    updateVisibility();
-    document.addEventListener('visibilitychange', updateVisibility, false);
-
     focusManager = new LocalDiscoveryFocusManager();
     focusManager.initialize();
 
     chrome.send('start');
-    recordUmaEvent(DEVICES_PAGE_EVENTS.OPENED);
   });
 
   return {
@@ -667,7 +561,6 @@ cr.define('local_discovery', function() {
     onRegistrationFailed: onRegistrationFailed,
     onUnregisteredDeviceUpdate: onUnregisteredDeviceUpdate,
     onRegistrationConfirmedOnPrinter: onRegistrationConfirmedOnPrinter,
-    onRegistrationConfirmDeviceCode: onRegistrationConfirmDeviceCode,
     onCloudDeviceListAvailable: onCloudDeviceListAvailable,
     onCloudDeviceListUnavailable: onCloudDeviceListUnavailable,
     onDeviceCacheFlushed: onDeviceCacheFlushed,
