@@ -13,38 +13,25 @@ Polymer({
     /** @type {!Array<!SearchEngine>} */
     engines: Array,
 
-    /** Whether column headers should be displayed */
-    hideHeaders: Boolean,
-
     /**
      * The scroll target that this list should use.
      * @type {?HTMLElement}
      */
-    scrollTarget: {
-      type: Element,
-      value: null,  // Required to populate class.
-    },
+    scrollTarget: Object,
 
     /** Used to fix scrolling glitch when list is not top most element. */
     scrollOffset: Number,
 
     /** @private {Object}*/
     lastFocused_: Object,
-  },
 
-  /**
-   * Fix height of list if no scrollTarget is present.
-   * @return {string}
-   */
-  getHeightClass: function() {
-    return this.scrollTarget ? '' : 'fixed-height-container';
-  },
+    /** @private */
+    listBlurred_: Boolean,
 
-  /**
-   * Only scroll when no target is set.
-   * @return {boolean}
-   */
-  isScrollable: function() {
-    return !this.scrollTarget;
+    fixedHeight: {
+      type: Boolean,
+      value: false,
+      reflectToAttribute: true,
+    },
   },
 });
