@@ -9,23 +9,19 @@
 #include <utility>
 
 #include "base/bind.h"
-#include "base/memory/ptr_util.h"
-#include "base/message_loop/message_loop.h"
 #include "base/metrics/user_metrics.h"
+#include "base/no_destructor.h"
 #include "base/stl_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/values.h"
 #include "build/build_config.h"
 #include "chrome/browser/local_discovery/service_discovery_shared_client.h"
-#include "chrome/browser/printing/cloud_print/cloud_print_proxy_service.h"
-#include "chrome/browser/printing/cloud_print/cloud_print_proxy_service_factory.h"
 #include "chrome/browser/printing/cloud_print/privet_confirm_api_flow.h"
 #include "chrome/browser/printing/cloud_print/privet_constants.h"
 #include "chrome/browser/printing/cloud_print/privet_device_lister_impl.h"
 #include "chrome/browser/printing/cloud_print/privet_http_asynchronous_factory.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/signin/profile_oauth2_token_service_factory.h"
-#include "chrome/browser/signin/signin_manager_factory.h"
+#include "chrome/browser/signin/identity_manager_factory.h"
 #include "chrome/browser/ui/browser_finder.h"
 #include "chrome/browser/ui/browser_tabstrip.h"
 #include "chrome/browser/ui/chrome_pages.h"
@@ -33,15 +29,20 @@
 #include "chrome/grit/generated_resources.h"
 #include "components/cloud_devices/common/cloud_devices_urls.h"
 #include "components/prefs/pref_service.h"
-#include "components/signin/core/browser/profile_oauth2_token_service.h"
 #include "content/public/browser/web_ui.h"
-#include "printing/features/features.h"
+#include "printing/buildflags/buildflags.h"
+#include "services/network/public/cpp/shared_url_loader_factory.h"
 #include "ui/base/l10n/l10n_util.h"
 
 #if BUILDFLAG(ENABLE_PRINT_PREVIEW) && !defined(OS_CHROMEOS)
 #define CLOUD_PRINT_CONNECTOR_UI_AVAILABLE
 #endif
 
+#if defined(CLOUD_PRINT_CONNECTOR_UI_AVAILABLE)
+#include "chrome/browser/printing/cloud_print/cloud_print_proxy_service.h"
+#include "chrome/browser/printing/cloud_print/cloud_print_proxy_service_factory.h"
+#endif
+
 using cloud_print::CloudPrintPrinterList;
 using cloud_print::DeviceDescription;
 using cloud_print::GCDApiFlow;
@@ -54,8 +55,6 @@ namespace {
 const char kDictionaryKeyServiceName[] = "service_name";
 const char kDictionaryKeyDisplayName[] = "display_name";
 const char kDictionaryKeyDescription[] = "description";
-const char kDictionaryKeyType[] = "type";
-const char kDictionaryKeyIsWifi[] = "is_wifi";
 const char kDictionaryKeyID[] = "id";
 
 const char kKeyPrefixMDns[] = "MDns:";
@@ -72,7 +71,6 @@ std::unique_ptr<base::DictionaryValue> CreateDeviceInfo(
   return_value->SetString(kDictionaryKeyID, description.id);
   return_value->SetString(kDictionaryKeyDisplayName, description.display_name);
   return_value->SetString(kDictionaryKeyDescription, description.description);
-  return_value->SetString(kDictionaryKeyType, "printer");
 
   return return_value;
 }
@@ -81,34 +79,52 @@ void ReadDevicesList(const CloudPrintPrinterList::DeviceList& devices,
                      const std::set<std::string>& local_ids,
                      base::ListValue* devices_list) {
   for (const auto& i : devices) {
-    if (base::ContainsKey(local_ids, i.id)) {
+    if (base::Contains(local_ids, i.id)) {
       devices_list->Append(CreateDeviceInfo(i));
     }
   }
 
   for (const auto& i : devices) {
-    if (!base::ContainsKey(local_ids, i.id)) {
+    if (!base::Contains(local_ids, i.id)) {
       devices_list->Append(CreateDeviceInfo(i));
     }
   }
 }
 
+scoped_refptr<network::SharedURLLoaderFactory>&
+GetURLLoaderFactoryForTesting() {
+  static base::NoDestructor<scoped_refptr<network::SharedURLLoaderFactory>>
+      instance;
+  return *instance;
+}
+
 }  // namespace
 
+LocalDiscoveryUIHandler::SetURLLoaderFactoryForTesting::
+    SetURLLoaderFactoryForTesting(
+        scoped_refptr<network::SharedURLLoaderFactory> url_loader_factory) {
+  DCHECK(!GetURLLoaderFactoryForTesting());
+  GetURLLoaderFactoryForTesting() = url_loader_factory;
+}
+
+LocalDiscoveryUIHandler::SetURLLoaderFactoryForTesting::
+    ~SetURLLoaderFactoryForTesting() {
+  GetURLLoaderFactoryForTesting() = nullptr;
+}
+
 LocalDiscoveryUIHandler::LocalDiscoveryUIHandler()
-    : is_visible_(false),
-      failed_list_count_(0),
-      succeded_list_count_(0) {
+    : failed_list_count_(0), succeded_list_count_(0) {
+  g_num_visible++;
 }
 
 LocalDiscoveryUIHandler::~LocalDiscoveryUIHandler() {
   Profile* profile = Profile::FromWebUI(web_ui());
-  SigninManagerBase* signin_manager =
-      SigninManagerFactory::GetInstance()->GetForProfile(profile);
-  if (signin_manager)
-    signin_manager->RemoveObserver(this);
+  signin::IdentityManager* identity_manager =
+      IdentityManagerFactory::GetForProfile(profile);
+  if (identity_manager)
+    identity_manager->RemoveObserver(this);
   ResetCurrentRegistration();
-  SetIsVisible(false);
+  g_num_visible--;
 }
 
 // static
@@ -117,39 +133,43 @@ bool LocalDiscoveryUIHandler::GetHasVisible() {
 }
 
 void LocalDiscoveryUIHandler::RegisterMessages() {
-  web_ui()->RegisterMessageCallback("start", base::Bind(
-      &LocalDiscoveryUIHandler::HandleStart,
-      base::Unretained(this)));
-  web_ui()->RegisterMessageCallback("isVisible", base::Bind(
-      &LocalDiscoveryUIHandler::HandleIsVisible,
-      base::Unretained(this)));
-  web_ui()->RegisterMessageCallback("registerDevice", base::Bind(
-      &LocalDiscoveryUIHandler::HandleRegisterDevice,
-      base::Unretained(this)));
-  web_ui()->RegisterMessageCallback("cancelRegistration", base::Bind(
-      &LocalDiscoveryUIHandler::HandleCancelRegistration,
-      base::Unretained(this)));
+  web_ui()->RegisterMessageCallback(
+      "start", base::BindRepeating(&LocalDiscoveryUIHandler::HandleStart,
+                                   base::Unretained(this)));
+  web_ui()->RegisterMessageCallback(
+      "registerDevice",
+      base::BindRepeating(&LocalDiscoveryUIHandler::HandleRegisterDevice,
+                          base::Unretained(this)));
+  web_ui()->RegisterMessageCallback(
+      "cancelRegistration",
+      base::BindRepeating(&LocalDiscoveryUIHandler::HandleCancelRegistration,
+                          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       "requestDeviceList",
-      base::Bind(&LocalDiscoveryUIHandler::HandleRequestDeviceList,
-                 base::Unretained(this)));
-  web_ui()->RegisterMessageCallback("openCloudPrintURL", base::Bind(
-      &LocalDiscoveryUIHandler::HandleOpenCloudPrintURL,
-      base::Unretained(this)));
-  web_ui()->RegisterMessageCallback("showSyncUI", base::Bind(
-      &LocalDiscoveryUIHandler::HandleShowSyncUI,
-      base::Unretained(this)));
+      base::BindRepeating(&LocalDiscoveryUIHandler::HandleRequestDeviceList,
+                          base::Unretained(this)));
+  web_ui()->RegisterMessageCallback(
+      "openCloudPrintURL",
+      base::BindRepeating(&LocalDiscoveryUIHandler::HandleOpenCloudPrintURL,
+                          base::Unretained(this)));
+#if !defined(OS_CHROMEOS)
+  web_ui()->RegisterMessageCallback(
+      "showSyncUI",
+      base::BindRepeating(&LocalDiscoveryUIHandler::HandleShowSyncUI,
+                          base::Unretained(this)));
+#endif
 
   // Cloud print connector related messages
 #if defined(CLOUD_PRINT_CONNECTOR_UI_AVAILABLE)
   web_ui()->RegisterMessageCallback(
       "showCloudPrintSetupDialog",
-      base::Bind(&LocalDiscoveryUIHandler::ShowCloudPrintSetupDialog,
-                 base::Unretained(this)));
+      base::BindRepeating(&LocalDiscoveryUIHandler::ShowCloudPrintSetupDialog,
+                          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       "disableCloudPrintConnector",
-      base::Bind(&LocalDiscoveryUIHandler::HandleDisableCloudPrintConnector,
-                 base::Unretained(this)));
+      base::BindRepeating(
+          &LocalDiscoveryUIHandler::HandleDisableCloudPrintConnector,
+          base::Unretained(this)));
 #endif  // defined(CLOUD_PRINT_CONNECTOR_UI_AVAILABLE)
 }
 
@@ -163,14 +183,20 @@ void LocalDiscoveryUIHandler::HandleStart(const base::ListValue* args) {
     privet_lister_.reset(
         new cloud_print::PrivetDeviceListerImpl(service_discovery_client_.get(),
                                                 this));
+
+    scoped_refptr<network::SharedURLLoaderFactory> url_loader_factory =
+        GetURLLoaderFactoryForTesting();
+    if (!url_loader_factory)
+      url_loader_factory = profile->GetURLLoaderFactory();
+
     privet_http_factory_ =
         cloud_print::PrivetHTTPAsynchronousFactory::CreateInstance(
-            profile->GetRequestContext());
+            url_loader_factory);
 
-    SigninManagerBase* signin_manager =
-        SigninManagerFactory::GetInstance()->GetForProfile(profile);
-    if (signin_manager)
-      signin_manager->AddObserver(this);
+    signin::IdentityManager* identity_manager =
+        IdentityManagerFactory::GetForProfile(profile);
+    if (identity_manager)
+      identity_manager->AddObserver(this);
   }
 
   privet_lister_->Start();
@@ -183,20 +209,13 @@ void LocalDiscoveryUIHandler::HandleStart(const base::ListValue* args) {
   CheckUserLoggedIn();
 }
 
-void LocalDiscoveryUIHandler::HandleIsVisible(const base::ListValue* args) {
-  bool is_visible = false;
-  bool rv = args->GetBoolean(0, &is_visible);
-  DCHECK(rv);
-  SetIsVisible(is_visible);
-}
-
 void LocalDiscoveryUIHandler::HandleRegisterDevice(
     const base::ListValue* args) {
   std::string device;
   bool rv = args->GetString(0, &device);
   DCHECK(rv);
 
-  DeviceDescriptionMap::iterator it = device_descriptions_.find(device);
+  auto it = device_descriptions_.find(device);
   if (it == device_descriptions_.end()) {
     OnSetupError();
     return;
@@ -228,7 +247,7 @@ void LocalDiscoveryUIHandler::HandleRequestDeviceList(
 
   if (cloud_print_printer_list_) {
     cloud_print_printer_list_->Start(
-        base::MakeUnique<CloudPrintPrinterList>(this));
+        std::make_unique<CloudPrintPrinterList>(this));
   }
 
   CheckListingDone();
@@ -249,6 +268,7 @@ void LocalDiscoveryUIHandler::HandleOpenCloudPrintURL(
                                 ui::PAGE_TRANSITION_FROM_API);
 }
 
+#if !defined(OS_CHROMEOS)
 void LocalDiscoveryUIHandler::HandleShowSyncUI(
     const base::ListValue* args) {
   Browser* browser = chrome::FindBrowserWithWebContents(
@@ -257,6 +277,7 @@ void LocalDiscoveryUIHandler::HandleShowSyncUI(
   chrome::ShowBrowserSignin(
       browser, signin_metrics::AccessPoint::ACCESS_POINT_DEVICES_PAGE);
 }
+#endif
 
 void LocalDiscoveryUIHandler::StartRegisterHTTP(
     std::unique_ptr<cloud_print::PrivetHTTPClient> http_client) {
@@ -279,8 +300,7 @@ void LocalDiscoveryUIHandler::OnPrivetRegisterClaimToken(
     const GURL& url) {
   web_ui()->CallJavascriptFunctionUnsafe(
       "local_discovery.onRegistrationConfirmedOnPrinter");
-  if (!base::ContainsKey(device_descriptions_,
-                         current_http_client_->GetName())) {
+  if (!base::Contains(device_descriptions_, current_http_client_->GetName())) {
     SendRegisterError();
     return;
   }
@@ -291,7 +311,7 @@ void LocalDiscoveryUIHandler::OnPrivetRegisterClaimToken(
     return;
   }
   confirm_api_call_flow_->Start(
-      base::MakeUnique<cloud_print::PrivetConfirmApiCallFlow>(
+      std::make_unique<cloud_print::PrivetConfirmApiCallFlow>(
           token, base::Bind(&LocalDiscoveryUIHandler::OnConfirmDone,
                             base::Unretained(this))));
 }
@@ -356,13 +376,11 @@ void LocalDiscoveryUIHandler::DeviceChanged(
     info.SetString(kDictionaryKeyServiceName, name);
     info.SetString(kDictionaryKeyDisplayName, description.name);
     info.SetString(kDictionaryKeyDescription, description.description);
-    info.SetString(kDictionaryKeyType, description.type);
-    info.SetBoolean(kDictionaryKeyIsWifi, false);
 
     web_ui()->CallJavascriptFunctionUnsafe(
         "local_discovery.onUnregisteredDeviceUpdate", service_key, info);
   } else {
-    auto null_value = base::MakeUnique<base::Value>();
+    auto null_value = std::make_unique<base::Value>();
 
     web_ui()->CallJavascriptFunctionUnsafe(
         "local_discovery.onUnregisteredDeviceUpdate", service_key, *null_value);
@@ -371,7 +389,7 @@ void LocalDiscoveryUIHandler::DeviceChanged(
 
 void LocalDiscoveryUIHandler::DeviceRemoved(const std::string& name) {
   device_descriptions_.erase(name);
-  auto null_value = base::MakeUnique<base::Value>();
+  auto null_value = std::make_unique<base::Value>();
   base::Value name_value(kKeyPrefixMDns + name);
 
   web_ui()->CallJavascriptFunctionUnsafe(
@@ -396,14 +414,13 @@ void LocalDiscoveryUIHandler::OnDeviceListUnavailable() {
   CheckListingDone();
 }
 
-void LocalDiscoveryUIHandler::GoogleSigninSucceeded(
-    const std::string& account_id,
-    const std::string& username) {
+void LocalDiscoveryUIHandler::OnPrimaryAccountSet(
+    const CoreAccountInfo& primary_account_info) {
   CheckUserLoggedIn();
 }
 
-void LocalDiscoveryUIHandler::GoogleSignedOut(const std::string& account_id,
-                                              const std::string& username) {
+void LocalDiscoveryUIHandler::OnPrimaryAccountCleared(
+    const CoreAccountInfo& previous_primary_account_info) {
   CheckUserLoggedIn();
 }
 
@@ -418,7 +435,7 @@ void LocalDiscoveryUIHandler::SendRegisterDone(
   // block the printer's announcement.
   privet_lister_->DiscoverNewDevices();
 
-  DeviceDescriptionMap::iterator it = device_descriptions_.find(service_name);
+  auto it = device_descriptions_.find(service_name);
 
   if (it == device_descriptions_.end()) {
     // TODO(noamsml): Handle the case where a printer's record is not present at
@@ -430,7 +447,6 @@ void LocalDiscoveryUIHandler::SendRegisterDone(
   const DeviceDescription& device = it->second;
   base::DictionaryValue device_value;
 
-  device_value.SetString(kDictionaryKeyType, device.type);
   device_value.SetString(kDictionaryKeyID, device.id);
   device_value.SetString(kDictionaryKeyDisplayName, device.name);
   device_value.SetString(kDictionaryKeyDescription, device.description);
@@ -440,22 +456,14 @@ void LocalDiscoveryUIHandler::SendRegisterDone(
       "local_discovery.onRegistrationSuccess", device_value);
 }
 
-void LocalDiscoveryUIHandler::SetIsVisible(bool visible) {
-  if (visible == is_visible_)
-    return;
-
-  g_num_visible += visible ? 1 : -1;
-  is_visible_ = visible;
-}
-
 std::string LocalDiscoveryUIHandler::GetSyncAccount() const {
   Profile* profile = Profile::FromWebUI(web_ui());
-  SigninManagerBase* signin_manager =
-      SigninManagerFactory::GetForProfileIfExists(profile);
+  signin::IdentityManager* identity_manager =
+      IdentityManagerFactory::GetForProfile(profile);
 
   std::string email;
-  if (signin_manager)
-    email = signin_manager->GetAuthenticatedAccountInfo().email;
+  if (identity_manager && identity_manager->HasPrimaryAccount())
+    email = identity_manager->GetPrimaryAccountInfo().email;
   return email;
 }
 
@@ -507,19 +515,17 @@ std::unique_ptr<GCDApiFlow> LocalDiscoveryUIHandler::CreateApiFlow() {
   if (!profile)
     return std::unique_ptr<GCDApiFlow>();
 
-  ProfileOAuth2TokenService* token_service =
-      ProfileOAuth2TokenServiceFactory::GetForProfile(profile);
-  if (!token_service)
+  signin::IdentityManager* identity_manager =
+      IdentityManagerFactory::GetForProfile(profile);
+  if (!(identity_manager && identity_manager->HasPrimaryAccount()))
     return std::unique_ptr<GCDApiFlow>();
 
-  SigninManagerBase* signin_manager =
-      SigninManagerFactory::GetInstance()->GetForProfile(profile);
-  if (!signin_manager)
-    return std::unique_ptr<GCDApiFlow>();
+  scoped_refptr<network::SharedURLLoaderFactory> url_loader_factory =
+      GetURLLoaderFactoryForTesting();
+  if (!url_loader_factory)
+    url_loader_factory = profile->GetURLLoaderFactory();
 
-  return GCDApiFlow::Create(profile->GetRequestContext(),
-                            token_service,
-                            signin_manager->GetAuthenticatedAccountId());
+  return GCDApiFlow::Create(url_loader_factory, identity_manager);
 }
 
 bool LocalDiscoveryUIHandler::IsUserSupervisedOrOffTheRecord() {
@@ -593,7 +599,7 @@ void LocalDiscoveryUIHandler::SetupCloudPrintConnectorSection() {
         l10n_util::GetStringUTF16(IDS_GOOGLE_CLOUD_PRINT));
   } else {
     label_str = l10n_util::GetStringFUTF16(
-        IDS_OPTIONS_CLOUD_PRINT_CONNECTOR_ENABLED_LABEL,
+        IDS_CLOUD_PRINT_CONNECTOR_ENABLED_LABEL,
         l10n_util::GetStringUTF16(IDS_GOOGLE_CLOUD_PRINT),
         base::UTF8ToUTF16(email));
   }
