@@ -7,21 +7,20 @@
 #include <iterator>
 #include <utility>
 
-#include "ash/new_window_controller.h"
-#include "ash/public/interfaces/constants.mojom.h"
-#include "ash/public/interfaces/wallpaper.mojom.h"
-#include "ash/shell.h"
+#include "ash/public/cpp/new_window_delegate.h"
+#include "ash/public/cpp/wallpaper_controller.h"
 #include "base/logging.h"
 #include "base/memory/singleton.h"
 #include "base/memory/weak_ptr.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/stl_util.h"
 #include "base/strings/string_util.h"
-#include "components/arc/arc_bridge_service.h"
 #include "components/arc/arc_browser_context_keyed_service_factory_base.h"
 #include "components/arc/arc_service_manager.h"
 #include "components/arc/audio/arc_audio_bridge.h"
+#include "components/arc/intent_helper/factory_reset_delegate.h"
 #include "components/arc/intent_helper/open_url_delegate.h"
+#include "components/arc/session/arc_bridge_service.h"
 #include "components/url_formatter/url_fixer.h"
 #include "content/public/common/service_manager_connection.h"
 #include "services/service_manager/public/cpp/connector.h"
@@ -31,53 +30,14 @@
 namespace arc {
 namespace {
 
-constexpr std::pair<mojom::ChromePage, const char*> kMapping[] = {
-    {mojom::ChromePage::MULTIDEVICE, "multidevice"},
-    {mojom::ChromePage::MAIN, ""},
-    {mojom::ChromePage::POWER, "power"},
-    {mojom::ChromePage::BLUETOOTH, "bluetoothDevices"},
-    {mojom::ChromePage::DATETIME, "dateTime"},
-    {mojom::ChromePage::DISPLAY, "display"},
-    {mojom::ChromePage::WIFI, "networks/?type=WiFi"},
-    {mojom::ChromePage::PRIVACY, "privacy"},
-    {mojom::ChromePage::HELP, "help"},
-    {mojom::ChromePage::ACCOUNTS, "accounts"},
-    {mojom::ChromePage::APPEARANCE, "appearance"},
-    {mojom::ChromePage::AUTOFILL, "autofill"},
-    {mojom::ChromePage::BLUETOOTHDEVICES, "bluetoothDevices"},
-    {mojom::ChromePage::CHANGEPICTURE, "changePicture"},
-    {mojom::ChromePage::CLEARBROWSERDATA, "clearBrowserData"},
-    {mojom::ChromePage::CLOUDPRINTERS, "cloudPrinters"},
-    {mojom::ChromePage::CUPSPRINTERS, "cupsPrinters"},
-    {mojom::ChromePage::DOWNLOADS, "downloads"},
-    {mojom::ChromePage::KEYBOARDOVERLAY, "keyboard-overlay"},
-    {mojom::ChromePage::LANGUAGES, "languages"},
-    {mojom::ChromePage::LOCKSCREEN, "lockScreen"},
-    {mojom::ChromePage::MANAGEACCESSIBILITY, "manageAccessibility"},
-    {mojom::ChromePage::NETWORKSTYPEVPN, "networks?type=VPN"},
-    {mojom::ChromePage::ONSTARTUP, "onStartup"},
-    {mojom::ChromePage::PASSWORDS, "passwords"},
-    {mojom::ChromePage::POINTEROVERLAY, "pointer-overlay"},
-    {mojom::ChromePage::RESET, "reset"},
-    {mojom::ChromePage::SEARCH, "search"},
-    {mojom::ChromePage::STORAGE, "storage"},
-    {mojom::ChromePage::SYNCSETUP, "syncSetup"},
-    {mojom::ChromePage::ABOUTBLANK, url::kAboutBlankURL},
-    {mojom::ChromePage::ABOUTDOWNLOADS, "about:downloads"},
-    {mojom::ChromePage::ABOUTHISTORY, "about:history"}};
-
 constexpr const char* kArcSchemes[] = {url::kHttpScheme, url::kHttpsScheme,
                                        url::kContentScheme, url::kFileScheme,
                                        url::kMailToScheme};
 
-// mojom::ChromePage::LAST returns the ammout of valid entries - 1.
-static_assert(base::size(kMapping) ==
-                  static_cast<size_t>(mojom::ChromePage::LAST) + 1,
-              "kMapping is out of sync");
-
 // Not owned. Must outlive all ArcIntentHelperBridge instances. Typically this
 // is ChromeNewWindowClient in the browser.
 OpenUrlDelegate* g_open_url_delegate = nullptr;
+FactoryResetDelegate* g_factory_reset_delegate = nullptr;
 
 // Singleton factory for ArcIntentHelperBridge.
 class ArcIntentHelperBridgeFactory
@@ -99,8 +59,23 @@ class ArcIntentHelperBridgeFactory
   ~ArcIntentHelperBridgeFactory() override = default;
 };
 
-// Base URL for the Chrome settings pages.
-constexpr char kSettingsPageBaseUrl[] = "chrome://settings";
+// Keep in sync with ArcIntentHelperOpenType enum in
+// //tools/metrics/histograms/enums.xml.
+enum class ArcIntentHelperOpenType {
+  DOWNLOADS = 0,
+  URL = 1,
+  CUSTOM_TAB = 2,
+  WALLPAPER_PICKER = 3,
+  VOLUME_CONTROL = 4,
+  CHROME_PAGE = 5,
+  WEB_APP = 6,
+  kMaxValue = WEB_APP,
+};
+
+// Records Arc.IntentHelper.OpenType UMA histogram.
+void RecordOpenType(ArcIntentHelperOpenType type) {
+  UMA_HISTOGRAM_ENUMERATION("Arc.IntentHelper.OpenType", type);
+}
 
 }  // namespace
 
@@ -130,11 +105,17 @@ void ArcIntentHelperBridge::SetOpenUrlDelegate(OpenUrlDelegate* delegate) {
   g_open_url_delegate = delegate;
 }
 
+// static
+void ArcIntentHelperBridge::SetFactoryResetDelegate(
+    FactoryResetDelegate* delegate) {
+  g_factory_reset_delegate = delegate;
+}
+
 ArcIntentHelperBridge::ArcIntentHelperBridge(content::BrowserContext* context,
                                              ArcBridgeService* bridge_service)
     : context_(context),
       arc_bridge_service_(bridge_service),
-      allowed_chrome_pages_map_(std::cbegin(kMapping), std::cend(kMapping)),
+      camera_intent_id_(0),
       allowed_arc_schemes_(std::cbegin(kArcSchemes), std::cend(kArcSchemes)) {
   arc_bridge_service_->intent_helper()->SetHost(this);
 }
@@ -151,17 +132,18 @@ void ArcIntentHelperBridge::OnIconInvalidated(const std::string& package_name) {
 
 void ArcIntentHelperBridge::OnOpenDownloads() {
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
+  RecordOpenType(ArcIntentHelperOpenType::DOWNLOADS);
   // TODO(607411): If the FileManager is not yet open this will open to
   // downloads by default, which is what we want.  However if it is open it will
   // simply be brought to the forgeground without forcibly being navigated to
   // downloads, which is probably not ideal.
   // TODO(mash): Support this functionality without ash::Shell access in Chrome.
-  if (ash::Shell::HasInstance())
-    ash::Shell::Get()->new_window_controller()->OpenFileManager();
+  ash::NewWindowDelegate::GetInstance()->OpenFileManager();
 }
 
 void ArcIntentHelperBridge::OnOpenUrl(const std::string& url) {
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
+  RecordOpenType(ArcIntentHelperOpenType::URL);
   // Converts |url| to a fixed-up one and checks validity.
   const GURL gurl(url_formatter::FixupURL(url, /*desired_tld=*/std::string()));
   if (!gurl.is_valid())
@@ -171,31 +153,39 @@ void ArcIntentHelperBridge::OnOpenUrl(const std::string& url) {
     g_open_url_delegate->OpenUrlFromArc(gurl);
 }
 
-void ArcIntentHelperBridge::OnOpenChromePage(mojom::ChromePage page) {
+void ArcIntentHelperBridge::OnOpenCustomTab(const std::string& url,
+                                            int32_t task_id,
+                                            int32_t surface_id,
+                                            int32_t top_margin,
+                                            OnOpenCustomTabCallback callback) {
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
-  auto it = allowed_chrome_pages_map_.find(page);
-  if (it == allowed_chrome_pages_map_.end()) {
-    LOG(WARNING) << "The requested ChromePage is invalid: "
-                 << static_cast<int>(page);
+  RecordOpenType(ArcIntentHelperOpenType::CUSTOM_TAB);
+  // Converts |url| to a fixed-up one and checks validity.
+  const GURL gurl(url_formatter::FixupURL(url, /*desired_tld=*/std::string()));
+  if (!gurl.is_valid() ||
+      allowed_arc_schemes_.find(gurl.scheme()) == allowed_arc_schemes_.end()) {
+    std::move(callback).Run(nullptr);
     return;
   }
+  g_open_url_delegate->OpenArcCustomTab(gurl, task_id, surface_id, top_margin,
+                                        std::move(callback));
+}
 
-  GURL page_gurl(it->second);
-  if (page_gurl.SchemeIs(url::kAboutScheme)) {
-    g_open_url_delegate->OpenUrlFromArc(page_gurl);
-  } else {
-    g_open_url_delegate->OpenUrlFromArc(
-        GURL(kSettingsPageBaseUrl).Resolve(it->second));
-  }
+void ArcIntentHelperBridge::OnOpenChromePage(mojom::ChromePage page) {
+  DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
+  RecordOpenType(ArcIntentHelperOpenType::CHROME_PAGE);
+
+  g_open_url_delegate->OpenChromePageFromArc(page);
+}
+
+void ArcIntentHelperBridge::FactoryResetArc() {
+  g_factory_reset_delegate->ResetArc();
 }
 
 void ArcIntentHelperBridge::OpenWallpaperPicker() {
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
-  ash::mojom::WallpaperControllerPtr wallpaper_controller_ptr;
-  content::ServiceManagerConnection::GetForProcess()
-      ->GetConnector()
-      ->BindInterface(ash::mojom::kServiceName, &wallpaper_controller_ptr);
-  wallpaper_controller_ptr->OpenWallpaperPickerIfAllowed();
+  RecordOpenType(ArcIntentHelperOpenType::WALLPAPER_PICKER);
+  ash::WallpaperController::Get()->OpenWallpaperPickerIfAllowed();
 }
 
 void ArcIntentHelperBridge::SetWallpaperDeprecated(
@@ -205,6 +195,8 @@ void ArcIntentHelperBridge::SetWallpaperDeprecated(
 }
 
 void ArcIntentHelperBridge::OpenVolumeControl() {
+  DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
+  RecordOpenType(ArcIntentHelperOpenType::VOLUME_CONTROL);
   auto* audio = ArcAudioBridge::GetForBrowserContext(context_);
   DCHECK(audio);
   audio->ShowVolumeControls();
@@ -212,6 +204,7 @@ void ArcIntentHelperBridge::OpenVolumeControl() {
 
 void ArcIntentHelperBridge::OnOpenWebApp(const std::string& url) {
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
+  RecordOpenType(ArcIntentHelperOpenType::WEB_APP);
   // Converts |url| to a fixed-up one and checks validity.
   const GURL gurl(url_formatter::FixupURL(url, /*desired_tld=*/std::string()));
   if (!gurl.is_valid())
@@ -229,6 +222,30 @@ void ArcIntentHelperBridge::RecordShareFilesMetrics(mojom::ShareFiles flag) {
   UMA_HISTOGRAM_ENUMERATION("Arc.ShareFilesOnExit", flag);
 }
 
+void ArcIntentHelperBridge::LaunchCameraApp(arc::mojom::CameraIntentMode mode,
+                                            bool should_handle_result,
+                                            bool should_down_scale,
+                                            bool is_secure,
+                                            LaunchCameraAppCallback callback) {
+  DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
+
+  launch_camera_app_callback_map_.emplace(camera_intent_id_,
+                                          std::move(callback));
+
+  base::DictionaryValue intent_info;
+  std::string mode_str =
+      mode == arc::mojom::CameraIntentMode::PHOTO ? "photo" : "video";
+
+  std::stringstream queries;
+  queries << "?intentId=" << camera_intent_id_ << "&mode=" << mode_str
+          << "&shouldHandleResult=" << should_handle_result
+          << "&shouldDownScale=" << should_down_scale
+          << "&isSecure=" << is_secure;
+  ash::NewWindowDelegate::GetInstance()->LaunchCameraApp(queries.str());
+
+  camera_intent_id_++;
+}
+
 ArcIntentHelperBridge::GetResult ArcIntentHelperBridge::GetActivityIcons(
     const std::vector<ActivityName>& activities,
     OnIconsReadyCallback callback) {
@@ -266,6 +283,16 @@ bool ArcIntentHelperBridge::HasObserver(
   return observer_list_.HasObserver(observer);
 }
 
+void ArcIntentHelperBridge::OnCameraIntentHandled(
+    uint32_t intent_id,
+    bool is_success,
+    const std::vector<uint8_t>& captured_data) {
+  CHECK(launch_camera_app_callback_map_.find(intent_id) !=
+        launch_camera_app_callback_map_.end());
+  std::move(launch_camera_app_callback_map_[intent_id])
+      .Run(is_success, captured_data);
+}
+
 // static
 bool ArcIntentHelperBridge::IsIntentHelperPackage(
     const std::string& package_name) {
