@@ -6,7 +6,8 @@
 
 #include "base/strings/utf_string_conversions.h"
 #include "components/autofill/core/browser/autofill_metrics.h"
-#include "components/autofill/core/browser/local_card_migration_manager.h"
+#include "components/autofill/core/browser/autofill_test_utils.h"
+#include "components/autofill/core/browser/payments/local_card_migration_manager.h"
 #include "components/autofill/core/browser/webdata/autofill_webdata_service.h"
 
 namespace autofill {
@@ -14,8 +15,7 @@ namespace autofill {
 TestAutofillClient::TestAutofillClient()
     : form_origin_(GURL("https://example.test")), source_id_(-1) {}
 
-TestAutofillClient::~TestAutofillClient() {
-}
+TestAutofillClient::~TestAutofillClient() {}
 
 PersonalDataManager* TestAutofillClient::GetPersonalDataManager() {
   return &test_personal_data_manager_;
@@ -34,7 +34,7 @@ syncer::SyncService* TestAutofillClient::GetSyncService() {
   return test_sync_service_;
 }
 
-identity::IdentityManager* TestAutofillClient::GetIdentityManager() {
+signin::IdentityManager* TestAutofillClient::GetIdentityManager() {
   return identity_test_env_.identity_manager();
 }
 
@@ -46,22 +46,18 @@ payments::PaymentsClient* TestAutofillClient::GetPaymentsClient() {
   return payments_client_.get();
 }
 
-LegacyStrikeDatabase* TestAutofillClient::GetLegacyStrikeDatabase() {
-  return test_legacy_strike_database_.get();
-}
-
 StrikeDatabase* TestAutofillClient::GetStrikeDatabase() {
   return test_strike_database_.get();
 }
 
 ukm::UkmRecorder* TestAutofillClient::GetUkmRecorder() {
-  return ukm::UkmRecorder::Get();
+  return &test_ukm_recorder_;
 }
 
 ukm::SourceId TestAutofillClient::GetUkmSourceId() {
   if (source_id_ == -1) {
     source_id_ = ukm::UkmRecorder::GetNewSourceID();
-    UpdateSourceURL(GetUkmRecorder(), source_id_, form_origin_);
+    test_ukm_recorder_.UpdateSourceURL(source_id_, form_origin_);
   }
   return source_id_;
 }
@@ -121,10 +117,10 @@ void TestAutofillClient::ConfirmSaveAutofillProfile(
 
 void TestAutofillClient::ConfirmSaveCreditCardLocally(
     const CreditCard& card,
-    bool show_prompt,
+    SaveCreditCardOptions options,
     LocalSaveCardPromptCallback callback) {
   confirm_save_credit_card_locally_called_ = true;
-  offer_to_save_credit_card_bubble_was_shown_ = show_prompt;
+  offer_to_save_credit_card_bubble_was_shown_ = options.show_prompt;
   std::move(callback).Run(AutofillClient::ACCEPTED);
 }
 
@@ -134,19 +130,29 @@ void TestAutofillClient::ConfirmAccountNameFixFlow(
   credit_card_name_fix_flow_bubble_was_shown_ = true;
   std::move(callback).Run(base::string16(base::ASCIIToUTF16("Gaia Name")));
 }
+
+void TestAutofillClient::ConfirmExpirationDateFixFlow(
+    const CreditCard& card,
+    base::OnceCallback<void(const base::string16&, const base::string16&)>
+        callback) {
+  credit_card_name_fix_flow_bubble_was_shown_ = true;
+  std::move(callback).Run(
+      base::string16(base::ASCIIToUTF16("03")),
+      base::string16(base::ASCIIToUTF16(test::NextYear().c_str())));
+}
 #endif  // defined(OS_ANDROID)
 
 void TestAutofillClient::ConfirmSaveCreditCardToCloud(
     const CreditCard& card,
     std::unique_ptr<base::DictionaryValue> legal_message,
-    bool should_request_name_from_user,
-    bool should_request_expiration_date_from_user,
-    bool show_prompt,
+    SaveCreditCardOptions options,
     UploadSaveCardPromptCallback callback) {
-  offer_to_save_credit_card_bubble_was_shown_ = show_prompt;
+  offer_to_save_credit_card_bubble_was_shown_ = options.show_prompt;
   std::move(callback).Run(AutofillClient::ACCEPTED, {});
 }
 
+void TestAutofillClient::CreditCardUploadCompleted(bool card_saved) {}
+
 void TestAutofillClient::ConfirmCreditCardFillAssist(
     const CreditCard& card,
     base::OnceClosure callback) {
@@ -166,6 +172,7 @@ void TestAutofillClient::ShowAutofillPopup(
     base::i18n::TextDirection text_direction,
     const std::vector<Suggestion>& suggestions,
     bool autoselect_first_suggestion,
+    PopupType popup_type,
     base::WeakPtr<AutofillPopupDelegate> delegate) {}
 
 void TestAutofillClient::UpdateAutofillPopupDataListValues(
@@ -211,23 +218,18 @@ void TestAutofillClient::LoadRiskData(
 }
 
 void TestAutofillClient::InitializeUKMSources() {
-  UpdateSourceURL(GetUkmRecorder(), source_id_, form_origin_);
+  test_ukm_recorder_.UpdateSourceURL(source_id_, form_origin_);
 }
 
 void TestAutofillClient::set_form_origin(const GURL& url) {
   form_origin_ = url;
   // Also reset source_id_.
   source_id_ = ukm::UkmRecorder::GetNewSourceID();
-  UpdateSourceURL(GetUkmRecorder(), source_id_, form_origin_);
+  test_ukm_recorder_.UpdateSourceURL(source_id_, form_origin_);
 }
 
-// static
-void TestAutofillClient::UpdateSourceURL(ukm::UkmRecorder* ukm_recorder,
-                                         ukm::SourceId source_id,
-                                         GURL url) {
-  if (ukm_recorder) {
-    ukm_recorder->UpdateSourceURL(source_id, url);
-  }
+ukm::TestUkmRecorder* TestAutofillClient::GetTestUkmRecorder() {
+  return &test_ukm_recorder_;
 }
 
 }  // namespace autofill
