@@ -19,6 +19,10 @@ void TestPersonalDataManager::OnSyncServiceInitialized(
   sync_service_initialized_ = true;
 }
 
+AutofillSyncSigninState TestPersonalDataManager::GetSyncSigninState() const {
+  return sync_and_signin_state_;
+}
+
 void TestPersonalDataManager::RecordUseOf(const AutofillDataModel& data_model) {
   CreditCard* credit_card = GetCreditCardWithGUID(data_model.guid().c_str());
   if (credit_card)
@@ -46,7 +50,7 @@ void TestPersonalDataManager::AddProfile(const AutofillProfile& profile) {
   std::unique_ptr<AutofillProfile> profile_ptr =
       std::make_unique<AutofillProfile>(profile);
   web_profiles_.push_back(std::move(profile_ptr));
-  NotifyPersonalDataChanged();
+  NotifyPersonalDataObserver();
 }
 
 void TestPersonalDataManager::UpdateProfile(const AutofillProfile& profile) {
@@ -82,7 +86,7 @@ void TestPersonalDataManager::AddCreditCard(const CreditCard& credit_card) {
   std::unique_ptr<CreditCard> local_credit_card =
       std::make_unique<CreditCard>(credit_card);
   local_credit_cards_.push_back(std::move(local_credit_card));
-  NotifyPersonalDataChanged();
+  NotifyPersonalDataObserver();
 }
 
 void TestPersonalDataManager::DeleteLocalCreditCards(
@@ -90,7 +94,7 @@ void TestPersonalDataManager::DeleteLocalCreditCards(
   for (const auto& card : cards)
     RemoveByGUID(card.guid());
 
-  NotifyPersonalDataChanged();
+  NotifyPersonalDataObserver();
 }
 
 void TestPersonalDataManager::UpdateCreditCard(const CreditCard& credit_card) {
@@ -248,7 +252,8 @@ bool TestPersonalDataManager::IsSyncFeatureEnabled() const {
   return sync_feature_enabled_;
 }
 
-AccountInfo TestPersonalDataManager::GetAccountInfoForPaymentsServer() const {
+CoreAccountInfo TestPersonalDataManager::GetAccountInfoForPaymentsServer()
+    const {
   return account_info_;
 }
 
@@ -282,7 +287,7 @@ void TestPersonalDataManager::AddServerCreditCard(
   std::unique_ptr<CreditCard> server_credit_card =
       std::make_unique<CreditCard>(credit_card);
   server_credit_cards_.push_back(std::move(server_credit_card));
-  NotifyPersonalDataChanged();
+  NotifyPersonalDataObserver();
 }
 
 }  // namespace autofill
