@@ -21,7 +21,26 @@ struct PasswordAndRealm {
 // struct are only set when the password's realm differs from the realm of the
 // form that we are filling.
 struct PasswordFormFillData {
-  typedef std::map<base::string16, PasswordAndRealm> LoginCollection;
+  using LoginCollection = std::map<base::string16, PasswordAndRealm>;
+
+  PasswordFormFillData();
+
+  // Create a FillData structure in preparation for autofilling a form, from
+  // basic_data identifying which form to fill, and a collection of matching
+  // stored logins to use as username/password values. |preferred_match| should
+  // equal (address) one of matches. |wait_for_username| is true if we should
+  // not autofill anything until the user typed in a valid username and blurred
+  // the field. If |enable_possible_usernames| is true, we will populate
+  // possible_usernames.
+  PasswordFormFillData(
+      const PasswordForm& form_on_page,
+      const std::map<base::string16, const PasswordForm*>& matches,
+      const PasswordForm& preferred_match,
+      bool wait_for_username);
+
+  PasswordFormFillData(const PasswordFormFillData& other);
+
+  ~PasswordFormFillData();
 
   // If |has_renderer_ids| == true then |form_renderer_id| contains the unique
   // renderer form id. No special values for |has_renderer_ids| == false case
@@ -32,7 +51,7 @@ struct PasswordFormFillData {
   // Username and Password elements renderer ids are in
   // |username_field.unique_renderer_id| and |password_field.unique_renderer_id|
   // correspondingly.
-  uint32_t form_renderer_id;
+  uint32_t form_renderer_id = FormData::kNotSetFormRendererId;
 
   // The name of the form.
   base::string16 name;
@@ -64,33 +83,15 @@ struct PasswordFormFillData {
   // PasswordManager determined there is an additional risk associated with this
   // form. This can happen, for example, if action URI's of the observed form
   // and our saved representation don't match up.
-  bool wait_for_username;
+  bool wait_for_username = false;
 
   // True if renderer ids for form, username and password fields are present.
   // TODO(https://crbug.com/831123): Remove this field when old parsing is
   // removed and filling by renderer ids is by default.
   bool has_renderer_ids = false;
 
-  PasswordFormFillData();
-  PasswordFormFillData(const PasswordFormFillData& other);
-  ~PasswordFormFillData();
 };
 
-// Create a FillData structure in preparation for autofilling a form,
-// from basic_data identifying which form to fill, and a collection of
-// matching stored logins to use as username/password values.
-// |preferred_match| should equal (address) one of matches.
-// |wait_for_username_before_autofill| is true if we should not autofill
-// anything until the user typed in a valid username and blurred the field.
-// If |enable_possible_usernames| is true, we will populate possible_usernames
-// in |result|.
-void InitPasswordFormFillData(
-    const PasswordForm& form_on_page,
-    const std::map<base::string16, const PasswordForm*>& matches,
-    const PasswordForm* const preferred_match,
-    bool wait_for_username_before_autofill,
-    PasswordFormFillData* result);
-
 // If |data.wait_for_username| is set, the renderer does not need to receive
 // passwords, yet, and this function clears the password values from |data|.
 PasswordFormFillData MaybeClearPasswordValues(const PasswordFormFillData& data);
