@@ -8,14 +8,13 @@
 #include "components/autofill/core/browser/form_structure.h"
 #include "components/autofill/ios/browser/autofill_driver_ios_bridge.h"
 #include "components/autofill/ios/browser/autofill_driver_ios_webframe.h"
-#include "components/autofill/ios/browser/autofill_driver_ios_webstate.h"
-#include "components/autofill/ios/browser/autofill_switches.h"
+#import "ios/web/common/origin_util.h"
 #include "ios/web/public/browser_state.h"
-#import "ios/web/public/origin_util.h"
-#import "ios/web/public/web_state/web_frame_util.h"
+#include "ios/web/public/js_messaging/web_frame_util.h"
 #import "ios/web/public/web_state/web_state.h"
 #include "services/network/public/cpp/shared_url_loader_factory.h"
 #include "services/network/public/cpp/weak_wrapper_shared_url_loader_factory.h"
+#include "ui/accessibility/ax_tree_id.h"
 #include "ui/gfx/geometry/rect_f.h"
 
 #if !defined(__has_feature) || !__has_feature(objc_arc)
@@ -31,29 +30,20 @@ void AutofillDriverIOS::PrepareForWebStateWebFrameAndDelegate(
     id<AutofillDriverIOSBridge> bridge,
     const std::string& app_locale,
     AutofillManager::AutofillDownloadManagerState enable_download_manager) {
-  if (autofill::switches::IsAutofillIFrameMessagingEnabled()) {
-    AutofillDriverIOSWebFrameFactory::CreateForWebStateAndDelegate(
-        web_state, client, bridge, app_locale, enable_download_manager);
-    // By the time this method is called, no web_frame is available. This method
-    // only prepare the factory and the AutofillDriverIOS will be created in the
-    // first call to FromWebStateAndWebFrame.
-  } else {
-    AutofillDriverIOSWebState::CreateForWebStateAndDelegate(
-        web_state, client, bridge, app_locale, enable_download_manager);
-  }
+  // By the time this method is called, no web_frame is available. This method
+  // only prepares the factory and the AutofillDriverIOS will be created in the
+  // first call to FromWebStateAndWebFrame.
+  AutofillDriverIOSWebFrameFactory::CreateForWebStateAndDelegate(
+      web_state, client, bridge, app_locale, enable_download_manager);
 }
 
 // static
 AutofillDriverIOS* AutofillDriverIOS::FromWebStateAndWebFrame(
     web::WebState* web_state,
     web::WebFrame* web_frame) {
-  if (autofill::switches::IsAutofillIFrameMessagingEnabled()) {
     return AutofillDriverIOSWebFrameFactory::FromWebState(web_state)
         ->AutofillDriverIOSFromWebFrame(web_frame)
         ->driver();
-  } else {
-    return AutofillDriverIOSWebState::FromWebState(web_state);
-  }
 }
 
 AutofillDriverIOS::AutofillDriverIOS(
@@ -82,6 +72,11 @@ bool AutofillDriverIOS::IsInMainFrame() const {
   return web_frame ? web_frame->IsMainFrame() : true;
 }
 
+ui::AXTreeID AutofillDriverIOS::GetAxTreeId() const {
+  NOTIMPLEMENTED() << "See https://crbug.com/985933";
+  return ui::AXTreeIDUnknown();
+}
+
 net::URLRequestContextGetter* AutofillDriverIOS::GetURLRequestContext() {
   return web_state_->GetBrowserState()->GetRequestContext();
 }
@@ -101,17 +96,23 @@ void AutofillDriverIOS::SendFormDataToRenderer(
     RendererFormDataAction action,
     const FormData& data) {
   web::WebFrame* web_frame = web::GetWebFrameWithId(web_state_, web_frame_id_);
+  if (!web_frame) {
+    return;
+  }
   [bridge_ fillFormData:data inFrame:web_frame];
 }
 
 void AutofillDriverIOS::PropagateAutofillPredictions(
     const std::vector<autofill::FormStructure*>& forms) {
   autofill_manager_.client()->PropagateAutofillPredictions(nullptr, forms);
-};
+}
 
 void AutofillDriverIOS::SendAutofillTypePredictionsToRenderer(
     const std::vector<FormStructure*>& forms) {
   web::WebFrame* web_frame = web::GetWebFrameWithId(web_state_, web_frame_id_);
+  if (!web_frame) {
+    return;
+  }
   [bridge_ fillFormDataPredictions:FormStructure::GetFieldTypePredictions(forms)
                            inFrame:web_frame];
 }
@@ -133,6 +134,9 @@ void AutofillDriverIOS::RendererShouldPreviewFieldWithValue(
     const base::string16& value) {
 }
 
+void AutofillDriverIOS::RendererShouldSetSuggestionAvailability(
+    bool available) {}
+
 void AutofillDriverIOS::PopupHidden() {
 }
 
