@@ -5,6 +5,7 @@
 #ifndef COMPONENTS_AUTOFILL_ASSISTANT_BROWSER_WEB_CONTROLLER_H_
 #define COMPONENTS_AUTOFILL_ASSISTANT_BROWSER_WEB_CONTROLLER_H_
 
+#include <map>
 #include <memory>
 #include <string>
 #include <vector>
@@ -12,7 +13,9 @@
 #include "base/callback_forward.h"
 #include "base/macros.h"
 #include "base/memory/weak_ptr.h"
+#include "components/autofill_assistant/browser/actions/click_action.h"
 #include "components/autofill_assistant/browser/batch_element_checker.h"
+#include "components/autofill_assistant/browser/client_status.h"
 #include "components/autofill_assistant/browser/devtools/devtools/domains/types_dom.h"
 #include "components/autofill_assistant/browser/devtools/devtools/domains/types_input.h"
 #include "components/autofill_assistant/browser/devtools/devtools/domains/types_network.h"
@@ -20,6 +23,9 @@
 #include "components/autofill_assistant/browser/devtools/devtools_client.h"
 #include "components/autofill_assistant/browser/rectf.h"
 #include "components/autofill_assistant/browser/selector.h"
+#include "components/autofill_assistant/browser/top_padding.h"
+#include "third_party/icu/source/common/unicode/umachine.h"
+#include "url/gurl.h"
 
 namespace autofill {
 class AutofillProfile;
@@ -37,6 +43,7 @@ struct FormFieldData;
 }
 
 namespace autofill_assistant {
+struct ClientSettings;
 
 // Controller to interact with the web pages.
 //
@@ -49,117 +56,62 @@ namespace autofill_assistant {
 // multiple operations, whether in sequence or in parallel.
 class WebController {
  public:
-  // Create web controller for a given |web_contents|.
+  // Create web controller for a given |web_contents|. |settings| must be valid
+  // for the lifetime of the controller.
   static std::unique_ptr<WebController> CreateForWebContents(
-      content::WebContents* web_contents);
+      content::WebContents* web_contents,
+      const ClientSettings* settings);
 
-  // |web_contents| must outlive this web controller.
+  // |web_contents| and |settings| must outlive this web controller.
   WebController(content::WebContents* web_contents,
-                std::unique_ptr<DevtoolsClient> devtools_client);
+                std::unique_ptr<DevtoolsClient> devtools_client,
+                const ClientSettings* settings);
   virtual ~WebController();
 
-  // Returns the last committed URL of the associated |web_contents_|.
-  virtual const GURL& GetUrl();
-
   // Load |url| in the current tab. Returns immediately, before the new page has
   // been loaded.
   virtual void LoadURL(const GURL& url);
 
   // Perform a mouse left button click or a touch tap on the element given by
   // |selector| and return the result through callback.
-  virtual void ClickOrTapElement(const Selector& selector,
-                                 base::OnceCallback<void(bool)> callback);
+  virtual void ClickOrTapElement(
+      const Selector& selector,
+      ClickAction::ClickType click_type,
+      base::OnceCallback<void(const ClientStatus&)> callback);
 
   // Fill the address form given by |selector| with the given address
   // |profile|.
-  virtual void FillAddressForm(const autofill::AutofillProfile* profile,
-                               const Selector& selector,
-                               base::OnceCallback<void(bool)> callback);
+  virtual void FillAddressForm(
+      const autofill::AutofillProfile* profile,
+      const Selector& selector,
+      base::OnceCallback<void(const ClientStatus&)> callback);
 
   // Fill the card form given by |selector| with the given |card| and its
   // |cvc|.
-  virtual void FillCardForm(std::unique_ptr<autofill::CreditCard> card,
-                            const base::string16& cvc,
-                            const Selector& selector,
-                            base::OnceCallback<void(bool)> callback);
+  virtual void FillCardForm(
+      std::unique_ptr<autofill::CreditCard> card,
+      const base::string16& cvc,
+      const Selector& selector,
+      base::OnceCallback<void(const ClientStatus&)> callback);
 
   // Select the option given by |selector| and the value of the option to be
   // picked.
-  virtual void SelectOption(const Selector& selector,
-                            const std::string& selected_option,
-                            base::OnceCallback<void(bool)> callback);
+  virtual void SelectOption(
+      const Selector& selector,
+      const std::string& selected_option,
+      base::OnceCallback<void(const ClientStatus&)> callback);
 
   // Highlight an element given by |selector|.
-  virtual void HighlightElement(const Selector& selector,
-                                base::OnceCallback<void(bool)> callback);
-
-  // Focus on element given by |selector|.
-  virtual void FocusElement(const Selector& selector,
-                            base::OnceCallback<void(bool)> callback);
-
-  // Set the |value| of field |selector| and return the result through
-  // |callback|. If |simulate_key_presses| is true, the value will be set by
-  // clicking the field and then simulating key presses, otherwise the `value`
-  // attribute will be set directly.
-  virtual void SetFieldValue(const Selector& selector,
-                             const std::string& value,
-                             bool simulate_key_presses,
-                             base::OnceCallback<void(bool)> callback);
-
-  // Set the |value| of the |attribute| of the element given by |selector|.
-  virtual void SetAttribute(const Selector& selector,
-                            const std::vector<std::string>& attribute,
-                            const std::string& value,
-                            base::OnceCallback<void(bool)> callback);
-
-  // Sets the keyboard focus to |selector| and inputs the specified UTF-8
-  // characters in the specified order.
-  // Returns the result through |callback|.
-  virtual void SendKeyboardInput(const Selector& selector,
-                                 const std::vector<std::string>& utf8_chars,
-                                 base::OnceCallback<void(bool)> callback);
-
-  // Return the outerHTML of |selector|.
-  virtual void GetOuterHtml(
+  virtual void HighlightElement(
       const Selector& selector,
-      base::OnceCallback<void(bool, const std::string&)> callback);
+      base::OnceCallback<void(const ClientStatus&)> callback);
 
-  // Create a helper for checking element existence and field value.
-  virtual std::unique_ptr<BatchElementChecker> CreateBatchElementChecker();
-
-  // Gets the position of the element identified by the selector.
-  //
-  // If unsuccessful, the callback gets (false, 0, 0, 0, 0).
-  //
-  // If successful, the callback gets (true, left, top, right, bottom), with
-  // coordinates expressed as numbers between 0 and 1, relative to the width or
-  // height of the visible viewport.
-  virtual void GetElementPosition(
+  // Focus on element given by |selector|. |top_padding| specifies the padding
+  // between focused element and the top.
+  virtual void FocusElement(
       const Selector& selector,
-      base::OnceCallback<void(bool, const RectF&)> callback);
-
-  // Functions to set, get and expire the Autofill Assistant cookie used to
-  // detect when Autofill Assistant has been used on a domain before.
-  virtual void SetCookie(const std::string& domain,
-                         base::OnceCallback<void(bool)> callback);
-  virtual void HasCookie(base::OnceCallback<void(bool)> callback);
-  virtual void ClearCookie();
-
- protected:
-  friend class BatchElementChecker;
-
-  // Checks an element for:
-  //
-  // kExistenceCheck: Checks whether at least one element given by |selector|
-  // exists on the web page.
-  //
-  // kVisibilityCheck: Checks whether at least on element given by |selector|
-  // is visible on the web page.
-  //
-  // Normally done through BatchElementChecker.
-  virtual void ElementCheck(ElementCheckType type,
-                            const Selector& selector,
-                            base::OnceCallback<void(bool)> callback);
+      const TopPadding& top_padding,
+      base::OnceCallback<void(const ClientStatus&)> callback);
 
   // Get the value of |selector| and return the result through |callback|. The
   // returned value might be false, if the element cannot be found, true and the
@@ -170,6 +122,85 @@ class WebController {
       const Selector& selector,
       base::OnceCallback<void(bool, const std::string&)> callback);
 
+  // Set the |value| of field |selector| and return the result through
+  // |callback|. If |simulate_key_presses| is true, the value will be set by
+  // clicking the field and then simulating key presses, otherwise the `value`
+  // attribute will be set directly.
+  virtual void SetFieldValue(
+      const Selector& selector,
+      const std::string& value,
+      bool simulate_key_presses,
+      int key_press_delay_in_millisecond,
+      base::OnceCallback<void(const ClientStatus&)> callback);
+
+  // Set the |value| of the |attribute| of the element given by |selector|.
+  virtual void SetAttribute(
+      const Selector& selector,
+      const std::vector<std::string>& attribute,
+      const std::string& value,
+      base::OnceCallback<void(const ClientStatus&)> callback);
+
+  // Sets the keyboard focus to |selector| and inputs |codepoints|, one
+  // character at a time. Key presses will have a delay of |delay_in_milli|
+  // between them.
+  // Returns the result through |callback|.
+  virtual void SendKeyboardInput(
+      const Selector& selector,
+      const std::vector<UChar32>& codepoints,
+      int delay_in_milli,
+      base::OnceCallback<void(const ClientStatus&)> callback);
+
+  // Return the outerHTML of |selector|.
+  virtual void GetOuterHtml(
+      const Selector& selector,
+      base::OnceCallback<void(const ClientStatus&, const std::string&)>
+          callback);
+
+  // Gets the visual viewport coordinates and size.
+  //
+  // The rectangle is expressed in absolute CSS coordinates.
+  virtual void GetVisualViewport(
+      base::OnceCallback<void(bool, const RectF&)> callback);
+
+  // Gets the position of the element identified by the selector.
+  //
+  // If unsuccessful, the callback gets (false, 0, 0, 0, 0).
+  //
+  // If successful, the callback gets (true, left, top, right, bottom), with
+  // coordinates expressed in absolute CSS coordinates.
+  virtual void GetElementPosition(
+      const Selector& selector,
+      base::OnceCallback<void(bool, const RectF&)> callback);
+
+  // Checks whether an element matches the given selector.
+  //
+  // If strict, there must be exactly one matching element for the check to
+  // pass. Otherwise, there must be at least one.
+  //
+  // To check multiple elements, use a BatchElementChecker.
+  virtual void ElementCheck(const Selector& selector,
+                            bool strict,
+                            base::OnceCallback<void(bool)> callback);
+
+  // Calls the callback once the main document window has been resized.
+  virtual void WaitForWindowHeightChange(
+      base::OnceCallback<void(const ClientStatus&)> callback);
+
+  // Gets the value of document.readyState for |optional_frame| or, if it is
+  // empty, in the main document.
+  virtual void GetDocumentReadyState(
+      const Selector& optional_frame,
+      base::OnceCallback<void(const ClientStatus&,
+                              DocumentReadyState end_state)> callback);
+
+  // Waits for the value of Document.readyState to satisfy |min_ready_state| in
+  // |optional_frame| or, if it is empty, in the main document.
+  virtual void WaitForDocumentReadyState(
+      const Selector& optional_frame,
+      DocumentReadyState min_ready_state,
+      base::OnceCallback<void(const ClientStatus&,
+                              DocumentReadyState end_state)> callback);
+
  private:
   friend class WebControllerBrowserTest;
 
@@ -181,57 +212,18 @@ class WebController {
   // of the element in viewport coordinates.
   using ElementPositionCallback = base::OnceCallback<void(bool, int, int)>;
 
-  // Helper class to get element's position in viewport coordinates when is
+  // Superclass for workers that execute complex operation and keep a pointer to
+  // this controller or the devtools client. Workers are owned by
+  // pending_workers_ and are removed once the operation is finished.
+  class Worker;
+
+  // Worker class to get element's position in viewport coordinates when is
   // stable and the frame it belongs finished visual update.
-  class ElementPositionGetter {
-   public:
-    ElementPositionGetter();
-    ~ElementPositionGetter();
+  class ElementPositionGetter;
 
-    // |devtools_client| must outlive this class which is guarantteed by the
-    // owner of this class.
-    void Start(content::RenderFrameHost* frame_host,
-               DevtoolsClient* devtools_client,
-               std::string element_object_id,
-               ElementPositionCallback callback);
-
-   private:
-    void OnVisualStateUpdatedCallback(bool success);
-    void GetAndWaitBoxModelStable();
-    void OnGetBoxModelForStableCheck(
-        std::unique_ptr<dom::GetBoxModelResult> result);
-    void OnScrollIntoView(
-        std::unique_ptr<runtime::CallFunctionOnResult> result);
-    void OnResult(int x, int y);
-    void OnError();
-
-    DevtoolsClient* devtools_client_ = nullptr;
-    std::string object_id_;
-    int remaining_rounds_ = 0;
-    ElementPositionCallback callback_;
-    bool visual_state_updated_ = false;
-
-    // If |has_point_| is true, |point_x_| and |point_y_| contain the last
-    // computed center of the element, in viewport coordinates. Note that
-    // negative coordinates are valid, in case the element is above or to the
-    // left of the viewport.
-    bool has_point_ = false;
-    int point_x_ = 0;
-    int point_y_ = 0;
-
-    base::WeakPtrFactory<ElementPositionGetter> weak_ptr_factory_;
-    DISALLOW_COPY_AND_ASSIGN(ElementPositionGetter);
-  };
-
-  // Perform a mouse left button click on the element given by |selector| and
-  // return the result through callback.
-  void ClickElement(const Selector& selector,
-                    base::OnceCallback<void(bool)> callback);
-
-  // Perform a touch tap on the element given by |selector| and return the
-  // result through callback.
-  void TapElement(const Selector& selector,
-                  base::OnceCallback<void(bool)> callback);
+  // Worker class to find element(s) matching a selector. Returns
+  // FindElementResult.
+  class ElementFinder;
 
   struct FindElementResult {
     FindElementResult() = default;
@@ -250,7 +242,8 @@ class WebController {
     std::string object_id;
   };
   using FindElementCallback =
-      base::OnceCallback<void(std::unique_ptr<FindElementResult>)>;
+      base::OnceCallback<void(const ClientStatus&,
+                              std::unique_ptr<FindElementResult>)>;
 
   struct FillFormInputData {
     FillFormInputData();
@@ -264,44 +257,57 @@ class WebController {
     base::string16 cvc;
   };
 
-  void OnFindElementForClickOrTap(base::OnceCallback<void(bool)> callback,
-                                  bool is_a_click,
-                                  std::unique_ptr<FindElementResult> result);
-  void OnFindElementForTap(base::OnceCallback<void(bool)> callback,
-                           std::unique_ptr<FindElementResult> result);
-  void ClickOrTapElement(std::unique_ptr<FindElementResult> target_element,
-                         bool is_a_click,
-                         base::OnceCallback<void(bool)> callback);
+  void OnFindElementForClickOrTap(
+      base::OnceCallback<void(const ClientStatus&)> callback,
+      ClickAction::ClickType click_type,
+      const ClientStatus& status,
+      std::unique_ptr<FindElementResult> result);
+  void OnWaitDocumentToBecomeInteractiveForClickOrTap(
+      base::OnceCallback<void(const ClientStatus&)> callback,
+      ClickAction::ClickType click_type,
+      std::unique_ptr<FindElementResult> target_element,
+      bool result);
+  void OnFindElementForTap(
+      base::OnceCallback<void(const ClientStatus&)> callback,
+      const ClientStatus& status,
+      std::unique_ptr<FindElementResult> result);
+  void ClickOrTapElement(
+      std::unique_ptr<FindElementResult> target_element,
+      ClickAction::ClickType click_type,
+      base::OnceCallback<void(const ClientStatus&)> callback);
+  void OnClickJS(base::OnceCallback<void(const ClientStatus&)> callback,
+                 std::unique_ptr<runtime::CallFunctionOnResult> result);
   void OnScrollIntoView(std::unique_ptr<FindElementResult> target_element,
-                        base::OnceCallback<void(bool)> callback,
-                        bool is_a_click,
+                        base::OnceCallback<void(const ClientStatus&)> callback,
+                        ClickAction::ClickType click_type,
                         std::unique_ptr<runtime::CallFunctionOnResult> result);
   void TapOrClickOnCoordinates(
-      std::unique_ptr<ElementPositionGetter> element_position_getter,
-      base::OnceCallback<void(bool)> callback,
-      bool is_a_click,
+      ElementPositionGetter* getter_to_release,
+      base::OnceCallback<void(const ClientStatus&)> callback,
+      ClickAction::ClickType click_type,
       bool has_coordinates,
       int x,
       int y);
   void OnDispatchPressMouseEvent(
-      base::OnceCallback<void(bool)> callback,
+      base::OnceCallback<void(const ClientStatus&)> callback,
       int x,
       int y,
       std::unique_ptr<input::DispatchMouseEventResult> result);
   void OnDispatchReleaseMouseEvent(
-      base::OnceCallback<void(bool)> callback,
+      base::OnceCallback<void(const ClientStatus&)> callback,
       std::unique_ptr<input::DispatchMouseEventResult> result);
   void OnDispatchTouchEventStart(
-      base::OnceCallback<void(bool)> callback,
+      base::OnceCallback<void(const ClientStatus&)> callback,
       std::unique_ptr<input::DispatchTouchEventResult> result);
   void OnDispatchTouchEventEnd(
-      base::OnceCallback<void(bool)> callback,
+      base::OnceCallback<void(const ClientStatus&)> callback,
       std::unique_ptr<input::DispatchTouchEventResult> result);
-  void OnFindElementForCheck(ElementCheckType check_type,
-                             base::OnceCallback<void(bool)> callback,
+  void OnFindElementForCheck(base::OnceCallback<void(bool)> callback,
+                             const ClientStatus& status,
                              std::unique_ptr<FindElementResult> result);
-  void OnGetBoxModelForVisible(base::OnceCallback<void(bool)> callback,
-                               std::unique_ptr<dom::GetBoxModelResult> result);
+  void OnWaitForWindowHeightChange(
+      base::OnceCallback<void(const ClientStatus&)> callback,
+      std::unique_ptr<runtime::EvaluateResult> result);
 
   // Find the element given by |selector|. If multiple elements match
   // |selector| and if |strict_mode| is false, return the first one that is
@@ -309,162 +315,164 @@ class WebController {
   void FindElement(const Selector& selector,
                    bool strict_mode,
                    FindElementCallback callback);
-  void OnGetDocumentElement(const Selector& selector,
-                            bool strict_mode,
-                            FindElementCallback callback,
-                            std::unique_ptr<runtime::EvaluateResult> result);
-  void RecursiveFindElement(const std::string& object_id,
-                            size_t index,
-                            const Selector& selector,
-                            bool strict_mode,
-                            std::unique_ptr<FindElementResult> element_result,
-                            FindElementCallback callback);
-  void OnQuerySelectorAll(
-      size_t index,
-      const Selector& selector,
-      bool strict_mode,
-      std::unique_ptr<FindElementResult> element_result,
-      FindElementCallback callback,
-      std::unique_ptr<runtime::CallFunctionOnResult> result);
-  void OnDescribeNodeForPseudoElement(
-      dom::PseudoType pseudo_type,
-      std::unique_ptr<FindElementResult> element_result,
-      FindElementCallback callback,
-      std::unique_ptr<dom::DescribeNodeResult> result);
-  void OnResolveNodeForPseudoElement(
-      std::unique_ptr<FindElementResult> element_result,
-      FindElementCallback callback,
-      std::unique_ptr<dom::ResolveNodeResult> result);
-  void OnDescribeNode(const std::string& object_id,
-                      size_t index,
-                      const Selector& selector,
-                      bool strict_mode,
-                      std::unique_ptr<FindElementResult> element_result,
-                      FindElementCallback callback,
-                      std::unique_ptr<dom::DescribeNodeResult> result);
-  void OnResolveNode(size_t index,
-                     const Selector& selector,
-                     bool strict_mode,
-                     std::unique_ptr<FindElementResult> element_result,
-                     FindElementCallback callback,
-                     std::unique_ptr<dom::ResolveNodeResult> result);
-  content::RenderFrameHost* FindCorrespondingRenderFrameHost(
-      std::string name,
-      std::string document_url);
-  void OnResult(bool result, base::OnceCallback<void(bool)> callback);
-  void OnResult(bool exists,
-                const std::string& result,
-                base::OnceCallback<void(bool, const std::string&)> callback);
+  void OnFindElementResult(ElementFinder* finder_to_release,
+                           FindElementCallback callback,
+                           const ClientStatus& status,
+                           std::unique_ptr<FindElementResult> result);
   void OnFindElementForFillingForm(
       std::unique_ptr<FillFormInputData> data_to_autofill,
       const Selector& selector,
-      base::OnceCallback<void(bool)> callback,
+      base::OnceCallback<void(const ClientStatus&)> callback,
+      const ClientStatus& status,
       std::unique_ptr<FindElementResult> element_result);
   void OnGetFormAndFieldDataForFillingForm(
       std::unique_ptr<FillFormInputData> data_to_autofill,
-      base::OnceCallback<void(bool)> callback,
+      base::OnceCallback<void(const ClientStatus&)> callback,
       content::RenderFrameHost* container_frame_host,
       const autofill::FormData& form_data,
       const autofill::FormFieldData& form_field);
   void OnFindElementForFocusElement(
-      base::OnceCallback<void(bool)> callback,
+      const TopPadding& top_padding,
+      base::OnceCallback<void(const ClientStatus&)> callback,
+      const ClientStatus& status,
       std::unique_ptr<FindElementResult> element_result);
-  void OnFocusElement(base::OnceCallback<void(bool)> callback,
+  void OnWaitDocumentToBecomeInteractiveForFocusElement(
+      const TopPadding& top_padding,
+      base::OnceCallback<void(const ClientStatus&)> callback,
+      std::unique_ptr<FindElementResult> target_element,
+      bool result);
+  void OnFocusElement(base::OnceCallback<void(const ClientStatus&)> callback,
                       std::unique_ptr<runtime::CallFunctionOnResult> result);
   void OnFindElementForSelectOption(
       const std::string& selected_option,
-      base::OnceCallback<void(bool)> callback,
+      base::OnceCallback<void(const ClientStatus&)> callback,
+      const ClientStatus& status,
       std::unique_ptr<FindElementResult> element_result);
-  void OnSelectOption(base::OnceCallback<void(bool)> callback,
+  void OnSelectOption(base::OnceCallback<void(const ClientStatus&)> callback,
                       std::unique_ptr<runtime::CallFunctionOnResult> result);
   void OnFindElementForHighlightElement(
-      base::OnceCallback<void(bool)> callback,
+      base::OnceCallback<void(const ClientStatus&)> callback,
+      const ClientStatus& status,
       std::unique_ptr<FindElementResult> element_result);
   void OnHighlightElement(
-      base::OnceCallback<void(bool)> callback,
+      base::OnceCallback<void(const ClientStatus&)> callback,
       std::unique_ptr<runtime::CallFunctionOnResult> result);
   void OnFindElementForGetFieldValue(
       base::OnceCallback<void(bool, const std::string&)> callback,
+      const ClientStatus& status,
       std::unique_ptr<FindElementResult> element_result);
   void OnGetValueAttribute(
       base::OnceCallback<void(bool, const std::string&)> callback,
       std::unique_ptr<runtime::CallFunctionOnResult> result);
-  void InternalSetFieldValue(const Selector& selector,
-                             const std::string& value,
-                             base::OnceCallback<void(bool)> callback);
+  void InternalSetFieldValue(
+      const Selector& selector,
+      const std::string& value,
+      base::OnceCallback<void(const ClientStatus&)> callback);
   void OnClearFieldForSendKeyboardInput(
       const Selector& selector,
-      const std::vector<std::string>& utf8_chars,
-      base::OnceCallback<void(bool)> callback,
-      bool clear_status);
+      const std::vector<UChar32>& codepoints,
+      int key_press_delay_in_millisecond,
+      base::OnceCallback<void(const ClientStatus&)> callback,
+      const ClientStatus& status);
   void OnClickElementForSendKeyboardInput(
-      const std::vector<std::string>& utf8_chars,
-      base::OnceCallback<void(bool)> callback,
-      bool click_status);
-  void DispatchKeyboardTextDownEvent(const std::vector<std::string>& utf8_chars,
-                                     size_t index,
-                                     base::OnceCallback<void(bool)> callback);
-  void DispatchKeyboardTextUpEvent(const std::vector<std::string>& utf8_chars,
-                                   size_t index,
-                                   base::OnceCallback<void(bool)> callback);
+      const std::vector<UChar32>& codepoints,
+      int delay_in_milli,
+      base::OnceCallback<void(const ClientStatus&)> callback,
+      const ClientStatus& click_status);
+  void DispatchKeyboardTextDownEvent(
+      const std::vector<UChar32>& codepoints,
+      size_t index,
+      bool delay,
+      int delay_in_milli,
+      base::OnceCallback<void(const ClientStatus&)> callback);
+  void DispatchKeyboardTextUpEvent(
+      const std::vector<UChar32>& codepoints,
+      size_t index,
+      int delay_in_milli,
+      base::OnceCallback<void(const ClientStatus&)> callback);
   void OnFindElementForSetAttribute(
       const std::vector<std::string>& attribute,
       const std::string& value,
-      base::OnceCallback<void(bool)> callback,
+      base::OnceCallback<void(const ClientStatus&)> callback,
+      const ClientStatus& status,
       std::unique_ptr<FindElementResult> element_result);
-  void OnSetAttribute(base::OnceCallback<void(bool)> callback,
+  void OnSetAttribute(base::OnceCallback<void(const ClientStatus&)> callback,
                       std::unique_ptr<runtime::CallFunctionOnResult> result);
   void OnFindElementForSendKeyboardInput(
       const Selector& selector,
-      const std::vector<std::string>& utf8_chars,
-      base::OnceCallback<void(bool)> callback,
+      const std::vector<UChar32>& codepoints,
+      int delay_in_milli,
+      base::OnceCallback<void(const ClientStatus&)> callback,
+      const ClientStatus& status,
       std::unique_ptr<FindElementResult> element_result);
-  void OnPressKeyboard(int key_code,
-                       base::OnceCallback<void(bool)> callback,
-                       std::unique_ptr<runtime::CallFunctionOnResult> result);
   void OnFindElementForSetFieldValue(
       const std::string& value,
-      base::OnceCallback<void(bool)> callback,
+      base::OnceCallback<void(const ClientStatus&)> callback,
+      const ClientStatus& status,
       std::unique_ptr<FindElementResult> element_result);
   void OnSetValueAttribute(
-      base::OnceCallback<void(bool)> callback,
+      base::OnceCallback<void(const ClientStatus&)> callback,
       std::unique_ptr<runtime::CallFunctionOnResult> result);
   void OnFindElementForGetOuterHtml(
-      base::OnceCallback<void(bool, const std::string&)> callback,
+      base::OnceCallback<void(const ClientStatus&, const std::string&)>
+          callback,
+      const ClientStatus& status,
       std::unique_ptr<FindElementResult> element_result);
-  void OnGetOuterHtml(
-      base::OnceCallback<void(bool, const std::string&)> callback,
-      std::unique_ptr<runtime::CallFunctionOnResult> result);
-
+  void OnGetOuterHtml(base::OnceCallback<void(const ClientStatus&,
+                                              const std::string&)> callback,
+                      std::unique_ptr<runtime::CallFunctionOnResult> result);
   void OnFindElementForPosition(
       base::OnceCallback<void(bool, const RectF&)> callback,
+      const ClientStatus& status,
       std::unique_ptr<FindElementResult> result);
-
+  void OnGetVisualViewport(
+      base::OnceCallback<void(bool, const RectF&)> callback,
+      std::unique_ptr<runtime::EvaluateResult> result);
   void OnGetElementPositionResult(
       base::OnceCallback<void(bool, const RectF&)> callback,
       std::unique_ptr<runtime::CallFunctionOnResult> result);
 
   // Creates a new instance of DispatchKeyEventParams for the specified type and
-  // text.
+  // unicode codepoint.
   using DispatchKeyEventParamsPtr =
       std::unique_ptr<autofill_assistant::input::DispatchKeyEventParams>;
-  static DispatchKeyEventParamsPtr CreateKeyEventParamsFromText(
+  static DispatchKeyEventParamsPtr CreateKeyEventParamsForCharacter(
       autofill_assistant::input::DispatchKeyEventType type,
-      const std::string& text);
+      const UChar32 codepoint);
 
-  void OnSetCookie(base::OnceCallback<void(bool)> callback,
-                   std::unique_ptr<network::SetCookieResult> result);
-  void OnHasCookie(base::OnceCallback<void(bool)> callback,
-                   std::unique_ptr<network::GetCookiesResult> result);
+  // Waits for the document.readyState to be 'interactive' or 'complete'.
+  void WaitForDocumentToBecomeInteractive(
+      int remaining_rounds,
+      std::string object_id,
+      base::OnceCallback<void(bool)> callback);
+  void OnWaitForDocumentToBecomeInteractive(
+      int remaining_rounds,
+      std::string object_id,
+      base::OnceCallback<void(bool)> callback,
+      std::unique_ptr<runtime::CallFunctionOnResult> result);
+  void OnFindElementForDocumentReadyState(
+      base::OnceCallback<void(const ClientStatus&, const std::string&)>
+          callback,
+      const ClientStatus& status,
+      std::unique_ptr<FindElementResult> element);
+  void OnFindElementForWaitForDocumentReadyState(
+      DocumentReadyState min_ready_state,
+      base::OnceCallback<void(const ClientStatus&, DocumentReadyState)>
+          callback,
+      const ClientStatus& status,
+      std::unique_ptr<FindElementResult> element);
 
   // Weak pointer is fine here since it must outlive this web controller, which
   // is guaranteed by the owner of this object.
   content::WebContents* web_contents_;
   std::unique_ptr<DevtoolsClient> devtools_client_;
+  const ClientSettings* const settings_;
+
+  // Workers currently running and using |devtools_client_|.
+  std::map<Worker*, std::unique_ptr<Worker>> pending_workers_;
 
   base::WeakPtrFactory<WebController> weak_ptr_factory_;
   DISALLOW_COPY_AND_ASSIGN(WebController);
 };
-
 }  // namespace autofill_assistant
 #endif  // COMPONENTS_AUTOFILL_ASSISTANT_BROWSER_WEB_CONTROLLER_H_
