@@ -44,8 +44,7 @@ BookmarkExpandedStateTrackerTest::BookmarkExpandedStateTrackerTest() {}
 BookmarkExpandedStateTrackerTest::~BookmarkExpandedStateTrackerTest() {}
 
 void BookmarkExpandedStateTrackerTest::SetUp() {
-  prefs_.registry()->RegisterListPref(prefs::kBookmarkEditorExpandedNodes,
-                                      std::make_unique<base::ListValue>());
+  prefs_.registry()->RegisterListPref(prefs::kBookmarkEditorExpandedNodes);
   prefs_.registry()->RegisterListPref(prefs::kManagedBookmarks);
   model_.reset(new BookmarkModel(std::make_unique<TestBookmarkClient>()));
   model_->Load(&prefs_, base::FilePath(),
@@ -79,7 +78,7 @@ TEST_F(BookmarkExpandedStateTrackerTest, SetExpandedNodes) {
   EXPECT_EQ(nodes, tracker->GetExpandedNodes());
 
   // Remove the folder, which should remove it from the list of expanded nodes.
-  model_->Remove(model_->bookmark_bar_node()->GetChild(0));
+  model_->Remove(model_->bookmark_bar_node()->children().front().get());
   nodes.erase(n1);
   n1 = nullptr;
   EXPECT_EQ(nodes, tracker->GetExpandedNodes());
