@@ -7,6 +7,7 @@
 #include <string>
 #include <utility>
 
+#include "base/bind.h"
 #include "components/cast_channel/cast_channel_enum.h"
 #include "components/cast_channel/cast_socket.h"
 #include "components/cast_channel/logger.h"
@@ -31,8 +32,7 @@ KeepAliveDelegate::KeepAliveDelegate(
       liveness_timeout_(liveness_timeout),
       ping_interval_(ping_interval),
       ping_message_(CreateKeepAlivePingMessage()),
-      pong_message_(CreateKeepAlivePongMessage()),
-      weak_factory_(this) {
+      pong_message_(CreateKeepAlivePongMessage()) {
   DCHECK(ping_interval_ < liveness_timeout_);
   DCHECK(inner_delegate_);
   DCHECK(socket_);
@@ -92,8 +92,8 @@ void KeepAliveDelegate::SendKeepAliveMessage(const CastMessage& message,
   DVLOG(2) << "Sending " << ToString(message_type);
 
   socket_->transport()->SendMessage(
-      message, base::Bind(&KeepAliveDelegate::SendKeepAliveMessageComplete,
-                          weak_factory_.GetWeakPtr(), message_type));
+      message, base::BindOnce(&KeepAliveDelegate::SendKeepAliveMessageComplete,
+                              weak_factory_.GetWeakPtr(), message_type));
 }
 
 void KeepAliveDelegate::SendKeepAliveMessageComplete(
