@@ -11,7 +11,6 @@
 
 #include "base/command_line.h"
 #include "base/macros.h"
-#include "base/md5.h"
 #include "base/metrics/field_trial.h"
 #include "base/metrics/histogram_samples.h"
 #include "base/test/metrics/histogram_tester.h"
@@ -32,6 +31,7 @@
 #include "components/data_reduction_proxy/core/common/data_reduction_proxy_switches.h"
 #include "components/prefs/pref_registry_simple.h"
 #include "net/base/proxy_server.h"
+#include "net/http/http_util.h"
 #include "services/network/public/cpp/weak_wrapper_shared_url_loader_factory.h"
 #include "services/network/test/test_url_loader_factory.h"
 #include "testing/gmock/include/gmock/gmock.h"
@@ -55,62 +55,8 @@ class DataReductionProxySettingsTest
     settings_->MaybeActivateDataReductionProxy(false);
     test_context_->RunUntilIdle();
   }
-
-  void InitPrefMembers() {
-    settings_->set_data_reduction_proxy_enabled_pref_name_for_test(
-        test_context_->GetDataReductionProxyEnabledPrefName());
-    settings_->InitPrefMembers();
-  }
 };
 
-TEST_F(DataReductionProxySettingsTest, TestIsProxyEnabledOrManaged) {
-  InitPrefMembers();
-  NetworkPropertiesManager network_properties_manager(
-      base::DefaultClock::GetInstance(), test_context_->pref_service(),
-      test_context_->task_runner());
-  test_context_->config()->SetNetworkPropertiesManagerForTesting(
-      &network_properties_manager);
-
-  // The proxy is disabled initially.
-  test_context_->config()->UpdateConfigForTesting(false, true, true);
-
-  EXPECT_FALSE(settings_->IsDataReductionProxyEnabled());
-  EXPECT_FALSE(settings_->IsDataReductionProxyManaged());
-
-  CheckOnPrefChange(true, true, false);
-  EXPECT_TRUE(settings_->IsDataReductionProxyEnabled());
-  EXPECT_FALSE(settings_->IsDataReductionProxyManaged());
-
-  CheckOnPrefChange(true, true, true);
-  EXPECT_TRUE(settings_->IsDataReductionProxyEnabled());
-  EXPECT_TRUE(settings_->IsDataReductionProxyManaged());
-
-  test_context_->RunUntilIdle();
-}
-
-TEST_F(DataReductionProxySettingsTest, TestCanUseDataReductionProxy) {
-  InitPrefMembers();
-  NetworkPropertiesManager network_properties_manager(
-      base::DefaultClock::GetInstance(), test_context_->pref_service(),
-      test_context_->task_runner());
-  test_context_->config()->SetNetworkPropertiesManagerForTesting(
-      &network_properties_manager);
-
-  // The proxy is disabled initially.
-  test_context_->config()->UpdateConfigForTesting(false, true, true);
-
-  GURL http_gurl("http://url.com/");
-  EXPECT_FALSE(settings_->CanUseDataReductionProxy(http_gurl));
-
-  CheckOnPrefChange(true, true, false);
-  EXPECT_TRUE(settings_->CanUseDataReductionProxy(http_gurl));
-
-  GURL https_gurl("https://url.com/");
-  EXPECT_FALSE(settings_->CanUseDataReductionProxy(https_gurl));
-
-  test_context_->RunUntilIdle();
-}
-
 TEST_F(DataReductionProxySettingsTest, TestResetDataReductionStatistics) {
   int64_t original_content_length;
   int64_t received_content_length;
@@ -235,8 +181,9 @@ TEST(DataReductionProxySettingsStandaloneTest, TestEndToEndSecureProxyCheck) {
 
     network::ResourceResponseHead resource_response_head;
     std::string headers(test_case.response_headers);
-    resource_response_head.headers = new net::HttpResponseHeaders(
-        net::HttpUtil::AssembleRawHeaders(headers.c_str(), headers.size()));
+    resource_response_head.headers =
+        base::MakeRefCounted<net::HttpResponseHeaders>(
+            net::HttpUtil::AssembleRawHeaders(headers));
     test_url_loader_factory.SimulateResponseWithoutRemovingFromPendingList(
         test_url_loader_factory.GetPendingRequest(0), resource_response_head,
         test_case.response_body,
@@ -285,6 +232,88 @@ TEST(DataReductionProxySettingsStandaloneTest, TestOnProxyEnabledPrefChange) {
   drp_test_context->SetDataReductionProxyEnabled(true);
 }
 
+TEST(DataReductionProxySettingsStandaloneTest, TestIsProxyEnabledOrManaged) {
+  base::test::ScopedTaskEnvironment task_environment{
+      base::test::ScopedTaskEnvironment::MainThreadType::IO};
+  std::unique_ptr<DataReductionProxyTestContext> drp_test_context =
+      DataReductionProxyTestContext::Builder()
+          .WithMockConfig()
+          .WithMockDataReductionProxyService()
+          .SkipSettingsInitialization()
+          .Build();
+
+  NetworkPropertiesManager network_properties_manager(
+      base::DefaultClock::GetInstance(), drp_test_context->pref_service(),
+      drp_test_context->task_runner());
+  drp_test_context->config()->SetNetworkPropertiesManagerForTesting(
+      &network_properties_manager);
+  drp_test_context->InitSettings();
+
+  DataReductionProxySettings* settings = drp_test_context->settings();
+
+  drp_test_context->SetDataReductionProxyEnabled(true);
+  EXPECT_TRUE(settings->IsDataReductionProxyEnabled());
+  EXPECT_FALSE(settings->IsDataReductionProxyManaged());
+
+  drp_test_context->SetDataReductionProxyEnabled(false);
+  EXPECT_FALSE(settings->IsDataReductionProxyEnabled());
+  EXPECT_FALSE(settings->IsDataReductionProxyManaged());
+
+  drp_test_context->SetDataReductionProxyEnabled(false);
+  drp_test_context->pref_service()->SetManagedPref(
+      prefs::kDataSaverEnabled, std::make_unique<base::Value>(false));
+  EXPECT_FALSE(settings->IsDataReductionProxyEnabled());
+  EXPECT_TRUE(settings->IsDataReductionProxyManaged());
+
+  drp_test_context->SetDataReductionProxyEnabled(true);
+  drp_test_context->pref_service()->SetManagedPref(
+      prefs::kDataSaverEnabled, std::make_unique<base::Value>(true));
+  EXPECT_TRUE(settings->IsDataReductionProxyEnabled());
+  EXPECT_TRUE(settings->IsDataReductionProxyManaged());
+
+  drp_test_context->RunUntilIdle();
+}
+
+TEST(DataReductionProxySettingsStandaloneTest, TestCanUseDataReductionProxy) {
+  base::test::ScopedTaskEnvironment task_environment{
+      base::test::ScopedTaskEnvironment::MainThreadType::IO};
+  std::unique_ptr<DataReductionProxyTestContext> drp_test_context =
+      DataReductionProxyTestContext::Builder()
+          .WithMockConfig()
+          .WithMockDataReductionProxyService()
+          .SkipSettingsInitialization()
+          .Build();
+
+  NetworkPropertiesManager network_properties_manager(
+      base::DefaultClock::GetInstance(), drp_test_context->pref_service(),
+      drp_test_context->task_runner());
+  drp_test_context->config()->SetNetworkPropertiesManagerForTesting(
+      &network_properties_manager);
+  drp_test_context->InitSettings();
+
+  MockDataReductionProxyService* mock_service =
+      static_cast<MockDataReductionProxyService*>(
+          drp_test_context->data_reduction_proxy_service());
+
+  DataReductionProxySettings* settings = drp_test_context->settings();
+  GURL http_gurl("http://url.com/");
+  GURL https_gurl("https://url.com/");
+
+  // The pref is disabled, so correspondingly should be the proxy.
+  EXPECT_CALL(*mock_service, SetProxyPrefs(false, false));
+  drp_test_context->SetDataReductionProxyEnabled(false);
+  EXPECT_FALSE(settings->CanUseDataReductionProxy(http_gurl));
+  EXPECT_FALSE(settings->CanUseDataReductionProxy(https_gurl));
+
+  // The pref is enabled, so correspondingly should be the proxy.
+  EXPECT_CALL(*mock_service, SetProxyPrefs(true, false));
+  drp_test_context->SetDataReductionProxyEnabled(true);
+  EXPECT_TRUE(settings->CanUseDataReductionProxy(http_gurl));
+  EXPECT_FALSE(settings->CanUseDataReductionProxy(https_gurl));
+
+  drp_test_context->RunUntilIdle();
+}
+
 TEST_F(DataReductionProxySettingsTest, TestMaybeActivateDataReductionProxy) {
   // Initialize the pref member in |settings_| without the usual callback
   // so it won't trigger MaybeActivateDataReductionProxy when the pref value
@@ -295,10 +324,6 @@ TEST_F(DataReductionProxySettingsTest, TestMaybeActivateDataReductionProxy) {
   test_context_->config()->SetNetworkPropertiesManagerForTesting(
       &network_properties_manager);
 
-  settings_->spdy_proxy_auth_enabled_.Init(
-      test_context_->GetDataReductionProxyEnabledPrefName(),
-      settings_->GetOriginalProfilePrefs());
-
   // TODO(bengr): Test enabling/disabling while a secure proxy check is
   // outstanding.
   // The proxy is enabled and unrestricted initially.
@@ -349,13 +374,12 @@ TEST_F(DataReductionProxySettingsTest, TestSetDataReductionProxyEnabled) {
   test_context_->SetDataReductionProxyEnabled(true);
   InitDataReductionProxy(true);
 
-  ExpectSetProxyPrefs(false, false);
-  settings_->SetDataReductionProxyEnabled(false);
+  test_context_->SetDataReductionProxyEnabled(false);
   test_context_->RunUntilIdle();
   CheckDataReductionProxySyntheticTrial(false);
 
-  ExpectSetProxyPrefs(true, false);
-  settings->SetDataReductionProxyEnabled(true);
+  test_context_->SetDataReductionProxyEnabled(true);
+  test_context_->RunUntilIdle();
   CheckDataReductionProxySyntheticTrial(true);
 }
 
@@ -363,7 +387,6 @@ TEST_F(DataReductionProxySettingsTest, TestSettingsEnabledStateHistograms) {
   const char kUMAEnabledState[] = "DataReductionProxy.EnabledState";
   base::HistogramTester histogram_tester;
 
-  InitPrefMembers();
   settings_->data_reduction_proxy_service_->SetIOData(
       test_context_->io_data()->GetWeakPtr());
 
@@ -371,14 +394,16 @@ TEST_F(DataReductionProxySettingsTest, TestSettingsEnabledStateHistograms) {
   test_context_->RunUntilIdle();
   histogram_tester.ExpectTotalCount(kUMAEnabledState, 0);
 
-  settings_->SetDataReductionProxyEnabled(true);
+  test_context_->SetDataReductionProxyEnabled(true);
+  settings_->MaybeActivateDataReductionProxy(false);
   test_context_->RunUntilIdle();
   histogram_tester.ExpectBucketCount(
       kUMAEnabledState, DATA_REDUCTION_SETTINGS_ACTION_OFF_TO_ON, 1);
   histogram_tester.ExpectBucketCount(
       kUMAEnabledState, DATA_REDUCTION_SETTINGS_ACTION_ON_TO_OFF, 0);
 
-  settings_->SetDataReductionProxyEnabled(false);
+  test_context_->SetDataReductionProxyEnabled(false);
+  settings_->MaybeActivateDataReductionProxy(false);
   test_context_->RunUntilIdle();
   histogram_tester.ExpectBucketCount(
       kUMAEnabledState, DATA_REDUCTION_SETTINGS_ACTION_OFF_TO_ON, 1);
@@ -396,7 +421,6 @@ TEST_F(DataReductionProxySettingsTest, TestDaysSinceEnabledWithTestClock) {
 
   base::Time last_enabled_time = clock.Now();
 
-  InitPrefMembers();
   {
     base::HistogramTester histogram_tester;
     settings_->data_reduction_proxy_service_->SetIOData(
@@ -406,7 +430,8 @@ TEST_F(DataReductionProxySettingsTest, TestDaysSinceEnabledWithTestClock) {
     histogram_tester.ExpectTotalCount(kUMAEnabledState, 0);
 
     // Enable data reduction proxy. The metric should be recorded.
-    settings_->SetDataReductionProxyEnabled(true /* enabled */);
+    test_context_->SetDataReductionProxyEnabled(true);
+    settings_->MaybeActivateDataReductionProxy(false);
     test_context_->RunUntilIdle();
 
     last_enabled_time = clock.Now();
@@ -421,13 +446,14 @@ TEST_F(DataReductionProxySettingsTest, TestDaysSinceEnabledWithTestClock) {
   {
     // Simulate turning off and on of data reduction proxy while Chromium is
     // running.
-    settings_->SetDataReductionProxyEnabled(false /* enabled */);
+    test_context_->SetDataReductionProxyEnabled(false);
+    settings_->MaybeActivateDataReductionProxy(false);
     clock.Advance(base::TimeDelta::FromDays(1));
-    base::HistogramTester histogram_tester;
     last_enabled_time = clock.Now();
 
-    settings_->spdy_proxy_auth_enabled_.SetValue(true);
-    settings_->MaybeActivateDataReductionProxy(false);
+    test_context_->SetDataReductionProxyEnabled(true);
+    base::HistogramTester histogram_tester;
+    settings_->MaybeActivateDataReductionProxy(false /* at_startup */);
     test_context_->RunUntilIdle();
     histogram_tester.ExpectUniqueSample(kUMAEnabledState, 0, 1);
     EXPECT_EQ(
@@ -443,7 +469,7 @@ TEST_F(DataReductionProxySettingsTest, TestDaysSinceEnabledWithTestClock) {
     base::HistogramTester histogram_tester;
     // Simulate Chromium start up. Data reduction proxy was enabled
     // |advance_clock_days| ago.
-    settings_->MaybeActivateDataReductionProxy(true);
+    settings_->MaybeActivateDataReductionProxy(true /* at_startup */);
     test_context_->RunUntilIdle();
     histogram_tester.ExpectUniqueSample(kUMAEnabledState, advance_clock_days,
                                         1);
@@ -456,19 +482,35 @@ TEST_F(DataReductionProxySettingsTest, TestDaysSinceEnabledWithTestClock) {
 
 // Verify that the pref and the UMA metric are not recorded for existing users
 // that already have data reduction proxy on.
-TEST_F(DataReductionProxySettingsTest, TestDaysSinceEnabledExistingUser) {
-  InitPrefMembers();
+TEST(DataReductionProxySettingsStandaloneTest,
+     TestDaysSinceEnabledExistingUser) {
+  base::test::ScopedTaskEnvironment task_environment{
+      base::test::ScopedTaskEnvironment::MainThreadType::IO};
+  std::unique_ptr<DataReductionProxyTestContext> drp_test_context =
+      DataReductionProxyTestContext::Builder()
+          .WithMockConfig()
+          .WithMockDataReductionProxyService()
+          .SkipSettingsInitialization()
+          .Build();
+
+  NetworkPropertiesManager network_properties_manager(
+      base::DefaultClock::GetInstance(), drp_test_context->pref_service(),
+      drp_test_context->task_runner());
+  drp_test_context->config()->SetNetworkPropertiesManagerForTesting(
+      &network_properties_manager);
+
+  // The proxy is enabled initially.
+  drp_test_context->config()->UpdateConfigForTesting(true, true, true);
+  drp_test_context->InitSettings();
+
   base::HistogramTester histogram_tester;
-  settings_->data_reduction_proxy_service_->SetIOData(
-      test_context_->io_data()->GetWeakPtr());
-  test_context_->RunUntilIdle();
 
   // Simulate Chromium startup with data reduction proxy already enabled.
-  settings_->spdy_proxy_auth_enabled_.SetValue(true);
-  settings_->MaybeActivateDataReductionProxy(true /* at_startup */);
-  test_context_->RunUntilIdle();
+  drp_test_context->settings()->MaybeActivateDataReductionProxy(
+      true /* at_startup */);
+  drp_test_context->RunUntilIdle();
   histogram_tester.ExpectTotalCount("DataReductionProxy.DaysSinceEnabled", 0);
-  EXPECT_EQ(0, test_context_->pref_service()->GetInt64(
+  EXPECT_EQ(0, drp_test_context->pref_service()->GetInt64(
                    prefs::kDataReductionProxyLastEnabledTime));
 }
 
@@ -477,7 +519,6 @@ TEST_F(DataReductionProxySettingsTest, TestDaysSinceSavingsCleared) {
   clock.Advance(base::TimeDelta::FromDays(1));
   ResetSettings(&clock);
 
-  InitPrefMembers();
   base::HistogramTester histogram_tester;
   test_context_->pref_service()->SetInt64(
       prefs::kDataReductionProxySavingsClearedNegativeSystemClock,
@@ -490,7 +531,7 @@ TEST_F(DataReductionProxySettingsTest, TestDaysSinceSavingsCleared) {
   clock.Advance(base::TimeDelta::FromDays(100));
 
   // Simulate Chromium startup with data reduction proxy already enabled.
-  settings_->spdy_proxy_auth_enabled_.SetValue(true);
+  test_context_->SetDataReductionProxyEnabled(true);
   settings_->MaybeActivateDataReductionProxy(true /* at_startup */);
   test_context_->RunUntilIdle();
   histogram_tester.ExpectUniqueSample(
