@@ -8,6 +8,7 @@
 #include <memory>
 #include <vector>
 
+#include "base/bind.h"
 #include "base/bind_helpers.h"
 #include "base/macros.h"
 #include "base/run_loop.h"
@@ -37,10 +38,8 @@ namespace {
 
 class WarmupURLFetcherTest : public WarmupURLFetcher {
  public:
-  WarmupURLFetcherTest(
-      scoped_refptr<network::SharedURLLoaderFactory> url_loader_factory)
+  WarmupURLFetcherTest(network::mojom::URLLoaderFactory* url_loader_factory)
       : WarmupURLFetcher(
-            std::move(url_loader_factory),
             base::BindRepeating(
                 [](const std::vector<DataReductionProxyServer>&) {
                   return network::mojom::CustomProxyConfig::New();
@@ -50,7 +49,9 @@ class WarmupURLFetcherTest : public WarmupURLFetcher {
                 base::Unretained(this)),
             base::BindRepeating(&WarmupURLFetcherTest::GetHttpRttEstimate,
                                 base::Unretained(this)),
-            base::ThreadTaskRunnerHandle::Get()) {}
+            base::ThreadTaskRunnerHandle::Get(),
+            std::string() /*user_agent*/),
+        url_loader_factory_(url_loader_factory) {}
   ~WarmupURLFetcherTest() override {}
 
   size_t callback_received_count() const { return callback_received_count_; }
@@ -59,29 +60,6 @@ class WarmupURLFetcherTest : public WarmupURLFetcher {
   }
   FetchResult success_response_last() const { return success_response_last_; }
 
-  static void InitExperimentWithTimeout(
-      base::test::ScopedFeatureList* scoped_feature_list) {
-    std::map<std::string, std::string> params;
-    params["warmup_url_fetch_min_timeout_seconds"] = "10";
-    params["warmup_url_fetch_max_timeout_seconds"] = "60";
-    params["warmup_url_fetch_init_http_rtt_multiplier"] = "12";
-    scoped_feature_list->InitAndEnableFeatureWithParameters(
-        features::kDataReductionProxyRobustConnection, params);
-  }
-
-  static void InitExperimentWithFetchWaitTimeParams(
-      base::test::ScopedFeatureList* scoped_feature_list,
-      base::TimeDelta first_retry,
-      base::TimeDelta second_retry) {
-    std::map<std::string, std::string> params;
-    params["warmup_url_fetch_wait_timer_first_retry_seconds"] =
-        base::IntToString(first_retry.InSeconds());
-    params["warmup_url_fetch_wait_timer_second_retry_seconds"] =
-        base::IntToString(second_retry.InSeconds());
-    scoped_feature_list->InitAndEnableFeatureWithParameters(
-        features::kDataReductionProxyRobustConnection, params);
-  }
-
   base::TimeDelta GetFetchWaitTime() const override {
     if (!fetch_wait_time_)
       return WarmupURLFetcher::GetFetchWaitTime();
@@ -119,6 +97,11 @@ class WarmupURLFetcherTest : public WarmupURLFetcher {
     http_rtt_override_ = http_rtt;
   }
 
+  network::mojom::URLLoaderFactory* GetNetworkServiceURLLoaderFactory(
+      const DataReductionProxyServer& proxy_server) override {
+    return url_loader_factory_;
+  }
+
  private:
   base::Optional<base::TimeDelta> GetHttpRttEstimate() const {
     if (http_rtt_override_)
@@ -138,6 +121,7 @@ class WarmupURLFetcherTest : public WarmupURLFetcher {
   FetchResult success_response_last_ = FetchResult::kFailed;
   base::Optional<base::TimeDelta> fetch_timeout_;
   base::Optional<base::TimeDelta> http_rtt_override_;
+  network::mojom::URLLoaderFactory* url_loader_factory_;
   DISALLOW_COPY_AND_ASSIGN(WarmupURLFetcherTest);
 };
 
@@ -145,11 +129,7 @@ class WarmupURLFetcherTest : public WarmupURLFetcher {
 TEST(WarmupURLFetcherTest, TestGetWarmupURLWithQueryParam) {
   base::test::ScopedTaskEnvironment scoped_task_environment;
   network::TestURLLoaderFactory test_url_loader_factory;
-  auto test_shared_url_loader_factory =
-      base::MakeRefCounted<network::WeakWrapperSharedURLLoaderFactory>(
-          &test_url_loader_factory);
-
-  WarmupURLFetcherTest warmup_url_fetcher(test_shared_url_loader_factory);
+  WarmupURLFetcherTest warmup_url_fetcher(&test_url_loader_factory);
 
   GURL gurl_original;
   warmup_url_fetcher.GetWarmupURLWithQueryParam(&gurl_original);
@@ -180,17 +160,13 @@ TEST(WarmupURLFetcherTest, TestGetWarmupURLWithQueryParam) {
 TEST(WarmupURLFetcherTest, TestSuccessfulFetchWarmupURLNoViaHeader) {
   base::test::ScopedTaskEnvironment scoped_task_environment;
   network::TestURLLoaderFactory test_url_loader_factory;
-  auto test_shared_url_loader_factory =
-      base::MakeRefCounted<network::WeakWrapperSharedURLLoaderFactory>(
-          &test_url_loader_factory);
 
   base::HistogramTester histogram_tester;
 
   auto proxy_server = net::ProxyServer::Direct();
-  WarmupURLFetcherTest warmup_url_fetcher(test_shared_url_loader_factory);
+  WarmupURLFetcherTest warmup_url_fetcher(&test_url_loader_factory);
   EXPECT_FALSE(warmup_url_fetcher.IsFetchInFlight());
-  warmup_url_fetcher.FetchWarmupURL(
-      0, DataReductionProxyServer(proxy_server, ProxyServer::UNSPECIFIED_TYPE));
+  warmup_url_fetcher.FetchWarmupURL(0, DataReductionProxyServer(proxy_server));
   EXPECT_TRUE(warmup_url_fetcher.IsFetchInFlight());
   scoped_task_environment.RunUntilIdle();
 
@@ -230,17 +206,13 @@ TEST(WarmupURLFetcherTest, TestSuccessfulFetchWarmupURLNoViaHeader) {
 TEST(WarmupURLFetcherTest, TestSuccessfulFetchWarmupURLWithViaHeader) {
   base::test::ScopedTaskEnvironment scoped_task_environment;
   network::TestURLLoaderFactory test_url_loader_factory;
-  auto test_shared_url_loader_factory =
-      base::MakeRefCounted<network::WeakWrapperSharedURLLoaderFactory>(
-          &test_url_loader_factory);
 
   base::HistogramTester histogram_tester;
 
   auto proxy_server = net::ProxyServer::Direct();
-  WarmupURLFetcherTest warmup_url_fetcher(test_shared_url_loader_factory);
+  WarmupURLFetcherTest warmup_url_fetcher(&test_url_loader_factory);
   EXPECT_FALSE(warmup_url_fetcher.IsFetchInFlight());
-  warmup_url_fetcher.FetchWarmupURL(
-      0, DataReductionProxyServer(proxy_server, ProxyServer::UNSPECIFIED_TYPE));
+  warmup_url_fetcher.FetchWarmupURL(0, DataReductionProxyServer(proxy_server));
   EXPECT_TRUE(warmup_url_fetcher.IsFetchInFlight());
   scoped_task_environment.RunUntilIdle();
 
@@ -286,16 +258,12 @@ TEST(WarmupURLFetcherTest,
      TestSuccessfulFetchWarmupURLWithViaHeaderExperimentNotEnabled) {
   base::test::ScopedTaskEnvironment scoped_task_environment;
   network::TestURLLoaderFactory test_url_loader_factory;
-  auto test_shared_url_loader_factory =
-      base::MakeRefCounted<network::WeakWrapperSharedURLLoaderFactory>(
-          &test_url_loader_factory);
 
   base::HistogramTester histogram_tester;
 
   auto proxy_server = net::ProxyServer::Direct();
-  WarmupURLFetcherTest warmup_url_fetcher(test_shared_url_loader_factory);
-  warmup_url_fetcher.FetchWarmupURL(
-      0, DataReductionProxyServer(proxy_server, ProxyServer::UNSPECIFIED_TYPE));
+  WarmupURLFetcherTest warmup_url_fetcher(&test_url_loader_factory);
+  warmup_url_fetcher.FetchWarmupURL(0, DataReductionProxyServer(proxy_server));
   base::RunLoop().RunUntilIdle();
 
   auto resource_response_head =
@@ -330,17 +298,13 @@ TEST(WarmupURLFetcherTest,
 TEST(WarmupURLFetcherTest, TestConnectionResetFetchWarmupURL) {
   base::test::ScopedTaskEnvironment scoped_task_environment;
   network::TestURLLoaderFactory test_url_loader_factory;
-  auto test_shared_url_loader_factory =
-      base::MakeRefCounted<network::WeakWrapperSharedURLLoaderFactory>(
-          &test_url_loader_factory);
 
   base::HistogramTester histogram_tester;
 
-  WarmupURLFetcherTest warmup_url_fetcher(test_shared_url_loader_factory);
+  WarmupURLFetcherTest warmup_url_fetcher(&test_url_loader_factory);
   EXPECT_FALSE(warmup_url_fetcher.IsFetchInFlight());
   warmup_url_fetcher.FetchWarmupURL(
-      0, DataReductionProxyServer(net::ProxyServer::Direct(),
-                                  ProxyServer::UNSPECIFIED_TYPE));
+      0, DataReductionProxyServer(net::ProxyServer::Direct()));
   EXPECT_TRUE(warmup_url_fetcher.IsFetchInFlight());
   base::RunLoop().RunUntilIdle();
 
@@ -376,20 +340,16 @@ TEST(WarmupURLFetcherTest, TestConnectionResetFetchWarmupURL) {
 TEST(WarmupURLFetcherTest, TestFetchTimesout) {
   base::test::ScopedTaskEnvironment scoped_task_environment;
   network::TestURLLoaderFactory test_url_loader_factory;
-  auto test_shared_url_loader_factory =
-      base::MakeRefCounted<network::WeakWrapperSharedURLLoaderFactory>(
-          &test_url_loader_factory);
 
   base::HistogramTester histogram_tester;
 
-  WarmupURLFetcherTest warmup_url_fetcher(test_shared_url_loader_factory);
+  WarmupURLFetcherTest warmup_url_fetcher(&test_url_loader_factory);
   // Set the timeout to a very low value. This should cause warmup URL fetcher
   // to run the callback with appropriate error code.
   warmup_url_fetcher.SetFetchTimeout(base::TimeDelta::FromSeconds(0));
   EXPECT_FALSE(warmup_url_fetcher.IsFetchInFlight());
   warmup_url_fetcher.FetchWarmupURL(
-      0, DataReductionProxyServer(net::ProxyServer::Direct(),
-                                  ProxyServer::UNSPECIFIED_TYPE));
+      0, DataReductionProxyServer(net::ProxyServer::Direct()));
   EXPECT_TRUE(warmup_url_fetcher.IsFetchInFlight());
   base::RunLoop().RunUntilIdle();
   EXPECT_FALSE(warmup_url_fetcher.IsFetchInFlight());
@@ -414,20 +374,16 @@ TEST(WarmupURLFetcherTest, TestFetchTimesout) {
 
 TEST(WarmupURLFetcherTest, TestSuccessfulFetchWarmupURLWithDelay) {
   base::test::ScopedTaskEnvironment scoped_task_environment(
-      base::test::ScopedTaskEnvironment::MainThreadType::MOCK_TIME);
+      base::test::ScopedTaskEnvironment::TimeSource::MOCK_TIME);
   network::TestURLLoaderFactory test_url_loader_factory;
-  auto test_shared_url_loader_factory =
-      base::MakeRefCounted<network::WeakWrapperSharedURLLoaderFactory>(
-          &test_url_loader_factory);
 
   base::HistogramTester histogram_tester;
 
   auto proxy_server = net::ProxyServer::Direct();
-  WarmupURLFetcherTest warmup_url_fetcher(test_shared_url_loader_factory);
+  WarmupURLFetcherTest warmup_url_fetcher(&test_url_loader_factory);
   EXPECT_FALSE(warmup_url_fetcher.IsFetchInFlight());
   warmup_url_fetcher.SetFetchWaitTime(base::TimeDelta::FromMilliseconds(1));
-  warmup_url_fetcher.FetchWarmupURL(
-      1, DataReductionProxyServer(proxy_server, ProxyServer::UNSPECIFIED_TYPE));
+  warmup_url_fetcher.FetchWarmupURL(1, DataReductionProxyServer(proxy_server));
   scoped_task_environment.FastForwardBy(base::TimeDelta::FromMilliseconds(2));
 
   auto resource_response_head =
@@ -477,13 +433,9 @@ TEST(WarmupURLFetcherTest, TestFetchTimeoutIncreasing) {
 
   base::test::ScopedTaskEnvironment scoped_task_environment;
   network::TestURLLoaderFactory test_url_loader_factory;
-  auto test_shared_url_loader_factory =
-      base::MakeRefCounted<network::WeakWrapperSharedURLLoaderFactory>(
-          &test_url_loader_factory);
 
-  DataReductionProxyServer proxy_server(net::ProxyServer::Direct(),
-                                        ProxyServer::UNSPECIFIED_TYPE);
-  WarmupURLFetcherTest warmup_url_fetcher(test_shared_url_loader_factory);
+  DataReductionProxyServer proxy_server(net::ProxyServer::Direct());
+  WarmupURLFetcherTest warmup_url_fetcher(&test_url_loader_factory);
   EXPECT_FALSE(warmup_url_fetcher.IsFetchInFlight());
 
   EXPECT_EQ(kMinTimeout, warmup_url_fetcher.GetFetchTimeout());
@@ -511,13 +463,9 @@ TEST(WarmupURLFetcherTest, TestFetchWaitTime) {
 
   base::test::ScopedTaskEnvironment scoped_task_environment;
   network::TestURLLoaderFactory test_url_loader_factory;
-  auto test_shared_url_loader_factory =
-      base::MakeRefCounted<network::WeakWrapperSharedURLLoaderFactory>(
-          &test_url_loader_factory);
 
-  DataReductionProxyServer proxy_server(net::ProxyServer::Direct(),
-                                        ProxyServer::UNSPECIFIED_TYPE);
-  WarmupURLFetcherTest warmup_url_fetcher(test_shared_url_loader_factory);
+  DataReductionProxyServer proxy_server(net::ProxyServer::Direct());
+  WarmupURLFetcherTest warmup_url_fetcher(&test_url_loader_factory);
   EXPECT_FALSE(warmup_url_fetcher.IsFetchInFlight());
 
   warmup_url_fetcher.FetchWarmupURL(1, proxy_server);
@@ -533,77 +481,6 @@ TEST(WarmupURLFetcherTest, TestFetchWaitTime) {
             warmup_url_fetcher.GetFetchWaitTime());
 }
 
-TEST(WarmupURLFetcherTest, TestFetchWaitTimeWithFieldTrial) {
-  base::test::ScopedFeatureList scoped_feature_list;
-  base::TimeDelta first_retry(base::TimeDelta::FromSeconds(30));
-  base::TimeDelta second_retry(base::TimeDelta::FromSeconds(60));
-  WarmupURLFetcherTest::InitExperimentWithFetchWaitTimeParams(
-      &scoped_feature_list, first_retry, second_retry);
-
-  base::HistogramTester histogram_tester;
-
-  base::test::ScopedTaskEnvironment scoped_task_environment;
-  network::TestURLLoaderFactory test_url_loader_factory;
-  auto test_shared_url_loader_factory =
-      base::MakeRefCounted<network::WeakWrapperSharedURLLoaderFactory>(
-          &test_url_loader_factory);
-
-  DataReductionProxyServer proxy_server(net::ProxyServer::Direct(),
-                                        ProxyServer::UNSPECIFIED_TYPE);
-  WarmupURLFetcherTest warmup_url_fetcher(test_shared_url_loader_factory);
-  EXPECT_FALSE(warmup_url_fetcher.IsFetchInFlight());
-
-  warmup_url_fetcher.FetchWarmupURL(1, proxy_server);
-  EXPECT_EQ(first_retry, warmup_url_fetcher.GetFetchWaitTime());
-
-  warmup_url_fetcher.FetchWarmupURL(2, proxy_server);
-  EXPECT_EQ(second_retry, warmup_url_fetcher.GetFetchWaitTime());
-
-  warmup_url_fetcher.FetchWarmupURL(1, proxy_server);
-  EXPECT_EQ(first_retry, warmup_url_fetcher.GetFetchWaitTime());
-}
-
-TEST(WarmupURLFetcherTest, TestFetchTimeoutIncreasingWithFieldTrial) {
-  base::test::ScopedFeatureList scoped_feature_list;
-  WarmupURLFetcherTest::InitExperimentWithTimeout(&scoped_feature_list);
-
-  // Must remain in sync with InitExperimentWithTimeout().
-  constexpr base::TimeDelta kMinTimeout = base::TimeDelta::FromSeconds(10);
-  constexpr base::TimeDelta kMaxTimeout = base::TimeDelta::FromSeconds(60);
-
-  base::HistogramTester histogram_tester;
-
-  base::test::ScopedTaskEnvironment scoped_task_environment;
-  network::TestURLLoaderFactory test_url_loader_factory;
-  auto test_shared_url_loader_factory =
-      base::MakeRefCounted<network::WeakWrapperSharedURLLoaderFactory>(
-          &test_url_loader_factory);
-
-  DataReductionProxyServer proxy_server(net::ProxyServer::Direct(),
-                                        ProxyServer::UNSPECIFIED_TYPE);
-  WarmupURLFetcherTest warmup_url_fetcher(test_shared_url_loader_factory);
-  EXPECT_FALSE(warmup_url_fetcher.IsFetchInFlight());
-
-  EXPECT_EQ(kMinTimeout, warmup_url_fetcher.GetFetchTimeout());
-
-  base::TimeDelta http_rtt = base::TimeDelta::FromSeconds(1);
-  warmup_url_fetcher.SetHttpRttOverride(http_rtt);
-  EXPECT_EQ(http_rtt * 12, warmup_url_fetcher.GetFetchTimeout());
-
-  warmup_url_fetcher.FetchWarmupURL(1, proxy_server);
-  EXPECT_EQ(http_rtt * 24, warmup_url_fetcher.GetFetchTimeout());
-
-  warmup_url_fetcher.FetchWarmupURL(2, proxy_server);
-  EXPECT_EQ(http_rtt * 48, warmup_url_fetcher.GetFetchTimeout());
-
-  http_rtt = base::TimeDelta::FromSeconds(5);
-  warmup_url_fetcher.SetHttpRttOverride(http_rtt);
-  EXPECT_EQ(kMaxTimeout, warmup_url_fetcher.GetFetchTimeout());
-
-  warmup_url_fetcher.FetchWarmupURL(0, proxy_server);
-  EXPECT_EQ(http_rtt * 12, warmup_url_fetcher.GetFetchTimeout());
-}
-
 }  // namespace
 
 }  // namespace data_reduction_proxy
