@@ -175,8 +175,10 @@ const struct QuicErrorMapping {
     // A crypto handshake message specified an unsupported version.
     {quic::QUIC_CRYPTO_VERSION_NOT_SUPPORTED,
      "quic.crypto.version_not_supported"},
-    // A crypto handshake message resulted in a stateless reject.
-    {quic::QUIC_CRYPTO_HANDSHAKE_STATELESS_REJECT,
+    // QUIC_CRYPTO_STATELESS_REJECT was code 72. The code has been
+    // deprecated, but to keep the assert below happy, there needs to be
+    // an entry for it, but the symbol is gone.
+    {static_cast<quic::QuicErrorCode>(72),
      "quic.crypto.handshake_stateless_reject"},
     // There was no intersection between the crypto primitives supported by the
     // peer and ourselves.
@@ -265,19 +267,16 @@ const struct QuicErrorMapping {
     {quic::QUIC_DECOMPRESSION_FAILURE, "quic.decompression_failure"},
     // Receive a RST_STREAM with offset larger than kMaxStreamLength.
     {quic::QUIC_STREAM_LENGTH_OVERFLOW, "quic.stream_length_overflow"},
-    // APPLICATION_CLOSE frame data is malformed.
-    {quic::QUIC_INVALID_APPLICATION_CLOSE_DATA,
-     "quic.invalid.application_close_data"},
     // Received a MAX DATA frame with errors.
     {quic::QUIC_INVALID_MAX_DATA_FRAME_DATA,
      "quic.invalid.max_data_frame_data"},
     // Received a MAX STREAM DATA frame with errors.
     {quic::QUIC_INVALID_MAX_STREAM_DATA_FRAME_DATA,
      "quic.invalid.max_stream_data_frame_data"},
-    // Received a MAX STREAM ID frame with bad data
-    {quic::QUIC_MAX_STREAM_ID_DATA, "quic.max_stream_id_data"},
-    // Received a STREAM ID BLOCKED frame with bad data
-    {quic::QUIC_STREAM_ID_BLOCKED_DATA, "quic.stream_id_blocked_data"},
+    // Received a MAX STREAMS frame with bad data
+    {quic::QUIC_MAX_STREAMS_DATA, "quic.max_streams_data"},
+    // Received a STREAMS BLOCKED frame with bad data
+    {quic::QUIC_STREAMS_BLOCKED_DATA, "quic.streams_blocked_data"},
     // Error deframing a STREAM BLOCKED frame.
     {quic::QUIC_INVALID_STREAM_BLOCKED_DATA,
      "quic.invalid.stream_blocked_data"},
@@ -300,16 +299,26 @@ const struct QuicErrorMapping {
      "quic.try.to.write.data.on.read.unidirectional.stream"},
     {quic::QUIC_INVALID_RETIRE_CONNECTION_ID_DATA,
      "quic.invalid.retire.connection.id.data"},
-    {quic::QUIC_STREAM_ID_BLOCKED_ERROR,
-     "quic.stream.id.in.stream_id_blocked.frame"},
-    {quic::QUIC_MAX_STREAM_ID_ERROR, "quic.stream.id.in.max_stream_id.frame"},
+    {quic::QUIC_STREAMS_BLOCKED_ERROR,
+     "quic.stream.id.in.streams_blocked.frame"},
+    {quic::QUIC_MAX_STREAMS_ERROR, "quic.stream.id.in.max_streams.frame"},
     {quic::QUIC_HTTP_DECODER_ERROR, "quic.http.decoder.error"},
+    {quic::QUIC_STALE_CONNECTION_CANCELLED, "quic.stale.connection.cancelled"},
+    {quic::QUIC_IETF_GQUIC_ERROR_MISSING, "quic.ietf.gquic.error_missing"},
+    {quic::QUIC_WINDOW_UPDATE_RECEIVED_ON_READ_UNIDIRECTIONAL_STREAM,
+     "quic.window.update.received.on.read.unidirectional.stream"},
+
+    // QUIC_INVALID_APPLICATION_CLOSE_DATA was code 101. The code has been
+    // deprecated, but to keep the assert below happy, there needs to be
+    // an entry for it, but the symbol is gone.
+    {static_cast<quic::QuicErrorCode>(101),
+     "quic.invalid.application_close_data"},
 
     // No error. Used as bound while iterating.
     {quic::QUIC_LAST_ERROR, "quic.last_error"}};
 
 // Must be updated any time a quic::QuicErrorCode is deprecated in
-// net/quic/core/quic_packets.h.
+// net/third_party/quiche/src/quic/core/quic_error_codes.h.
 const int kDeprecatedQuicErrorCount = 5;
 const int kActiveQuicErrorCount =
     quic::QUIC_LAST_ERROR - kDeprecatedQuicErrorCount;
