@@ -22,53 +22,51 @@ const struct NetErrorMapping {
   int net_error;
   const char* beacon_status;
 } net_error_map[] = {
-  { net::OK, "ok" },
-  { net::ERR_ABORTED, "aborted" },
-  { net::ERR_TIMED_OUT, "tcp.connection.timed_out" },
-  { net::ERR_CONNECTION_CLOSED, "tcp.connection.closed" },
-  { net::ERR_CONNECTION_RESET, "tcp.connection.reset" },
-  { net::ERR_CONNECTION_REFUSED, "tcp.connection.refused" },
-  { net::ERR_CONNECTION_ABORTED, "tcp.connection.aborted" },
-  { net::ERR_CONNECTION_FAILED, "tcp.connection.failed" },
-  { net::ERR_NAME_NOT_RESOLVED, "dns" },
-  { net::ERR_SSL_PROTOCOL_ERROR, "ssl.protocol.error" },
-  { net::ERR_ADDRESS_INVALID, "tcp.connection.address_invalid" },
-  { net::ERR_ADDRESS_UNREACHABLE, "tcp.connection.address_unreachable" },
-  { net::ERR_CONNECTION_TIMED_OUT, "tcp.connection.timed_out" },
-  { net::ERR_NAME_RESOLUTION_FAILED, "dns" },
-  { net::ERR_SSL_PINNED_KEY_NOT_IN_CERT_CHAIN,
-        "ssl.cert.pinned_key_not_in_cert_chain" },
-  { net::ERR_CERT_COMMON_NAME_INVALID, "ssl.cert.name_invalid" },
-  { net::ERR_CERT_DATE_INVALID, "ssl.cert.date_invalid" },
-  { net::ERR_CERT_AUTHORITY_INVALID, "ssl.cert.authority_invalid" },
-  { net::ERR_CERT_REVOKED, "ssl.cert.revoked" },
-  { net::ERR_CERT_INVALID, "ssl.cert.invalid" },
-  { net::ERR_EMPTY_RESPONSE, "http.response.empty" },
-  { net::ERR_SPDY_PING_FAILED, "spdy.ping_failed" },
-  { net::ERR_SPDY_PROTOCOL_ERROR, "spdy.protocol" },
-  { net::ERR_QUIC_PROTOCOL_ERROR, "quic.protocol" },
-  { net::ERR_DNS_MALFORMED_RESPONSE, "dns.protocol" },
-  { net::ERR_DNS_SERVER_FAILED, "dns.server" },
-  { net::ERR_DNS_TIMED_OUT, "dns.timed_out" },
-  { net::ERR_INSECURE_RESPONSE, "ssl" },
-  { net::ERR_CONTENT_LENGTH_MISMATCH, "http.response.content_length_mismatch" },
-  { net::ERR_INCOMPLETE_CHUNKED_ENCODING,
-        "http.response.incomplete_chunked_encoding" },
-  { net::ERR_SSL_VERSION_OR_CIPHER_MISMATCH,
-        "ssl.version_or_cipher_mismatch" },
-  { net::ERR_BAD_SSL_CLIENT_AUTH_CERT, "ssl.bad_client_auth_cert" },
-  { net::ERR_INVALID_CHUNKED_ENCODING,
-        "http.response.invalid_chunked_encoding" },
-  { net::ERR_RESPONSE_HEADERS_TRUNCATED, "http.response.headers.truncated" },
-  { net::ERR_REQUEST_RANGE_NOT_SATISFIABLE,
-        "http.request.range_not_satisfiable" },
-  { net::ERR_INVALID_RESPONSE, "http.response.invalid" },
-  { net::ERR_RESPONSE_HEADERS_MULTIPLE_CONTENT_DISPOSITION,
-        "http.response.headers.multiple_content_disposition" },
-  { net::ERR_RESPONSE_HEADERS_MULTIPLE_CONTENT_LENGTH,
-        "http.response.headers.multiple_content_length" },
-  { net::ERR_SSL_UNRECOGNIZED_NAME_ALERT, "ssl.unrecognized_name_alert" }
-};
+    {net::OK, "ok"},
+    {net::ERR_ABORTED, "aborted"},
+    {net::ERR_TIMED_OUT, "tcp.connection.timed_out"},
+    {net::ERR_CONNECTION_CLOSED, "tcp.connection.closed"},
+    {net::ERR_CONNECTION_RESET, "tcp.connection.reset"},
+    {net::ERR_CONNECTION_REFUSED, "tcp.connection.refused"},
+    {net::ERR_CONNECTION_ABORTED, "tcp.connection.aborted"},
+    {net::ERR_CONNECTION_FAILED, "tcp.connection.failed"},
+    {net::ERR_NAME_NOT_RESOLVED, "dns"},
+    {net::ERR_SSL_PROTOCOL_ERROR, "ssl.protocol.error"},
+    {net::ERR_ADDRESS_INVALID, "tcp.connection.address_invalid"},
+    {net::ERR_ADDRESS_UNREACHABLE, "tcp.connection.address_unreachable"},
+    {net::ERR_CONNECTION_TIMED_OUT, "tcp.connection.timed_out"},
+    {net::ERR_NAME_RESOLUTION_FAILED, "dns"},
+    {net::ERR_SSL_PINNED_KEY_NOT_IN_CERT_CHAIN,
+     "ssl.cert.pinned_key_not_in_cert_chain"},
+    {net::ERR_CERT_COMMON_NAME_INVALID, "ssl.cert.name_invalid"},
+    {net::ERR_CERT_DATE_INVALID, "ssl.cert.date_invalid"},
+    {net::ERR_CERT_AUTHORITY_INVALID, "ssl.cert.authority_invalid"},
+    {net::ERR_CERT_REVOKED, "ssl.cert.revoked"},
+    {net::ERR_CERT_INVALID, "ssl.cert.invalid"},
+    {net::ERR_EMPTY_RESPONSE, "http.response.empty"},
+    {net::ERR_HTTP2_PING_FAILED, "spdy.ping_failed"},
+    {net::ERR_HTTP2_PROTOCOL_ERROR, "spdy.protocol"},
+    {net::ERR_QUIC_PROTOCOL_ERROR, "quic.protocol"},
+    {net::ERR_DNS_MALFORMED_RESPONSE, "dns.protocol"},
+    {net::ERR_DNS_SERVER_FAILED, "dns.server"},
+    {net::ERR_DNS_TIMED_OUT, "dns.timed_out"},
+    {net::ERR_INSECURE_RESPONSE, "ssl"},
+    {net::ERR_CONTENT_LENGTH_MISMATCH, "http.response.content_length_mismatch"},
+    {net::ERR_INCOMPLETE_CHUNKED_ENCODING,
+     "http.response.incomplete_chunked_encoding"},
+    {net::ERR_SSL_VERSION_OR_CIPHER_MISMATCH, "ssl.version_or_cipher_mismatch"},
+    {net::ERR_BAD_SSL_CLIENT_AUTH_CERT, "ssl.bad_client_auth_cert"},
+    {net::ERR_INVALID_CHUNKED_ENCODING,
+     "http.response.invalid_chunked_encoding"},
+    {net::ERR_RESPONSE_HEADERS_TRUNCATED, "http.response.headers.truncated"},
+    {net::ERR_REQUEST_RANGE_NOT_SATISFIABLE,
+     "http.request.range_not_satisfiable"},
+    {net::ERR_INVALID_RESPONSE, "http.response.invalid"},
+    {net::ERR_RESPONSE_HEADERS_MULTIPLE_CONTENT_DISPOSITION,
+     "http.response.headers.multiple_content_disposition"},
+    {net::ERR_RESPONSE_HEADERS_MULTIPLE_CONTENT_LENGTH,
+     "http.response.headers.multiple_content_length"},
+    {net::ERR_SSL_UNRECOGNIZED_NAME_ALERT, "ssl.unrecognized_name_alert"}};
 
 bool CanReportFullBeaconURLToCollector(const GURL& beacon_url,
                                        const GURL& collector_url) {
@@ -134,7 +132,10 @@ std::string GetDomainReliabilityProtocol(
     case net::HttpResponseInfo::CONNECTION_INFO_QUIC_44:
     case net::HttpResponseInfo::CONNECTION_INFO_QUIC_45:
     case net::HttpResponseInfo::CONNECTION_INFO_QUIC_46:
+    case net::HttpResponseInfo::CONNECTION_INFO_QUIC_47:
+    case net::HttpResponseInfo::CONNECTION_INFO_QUIC_48:
     case net::HttpResponseInfo::CONNECTION_INFO_QUIC_99:
+    case net::HttpResponseInfo::CONNECTION_INFO_QUIC_999:
       return "QUIC";
     case net::HttpResponseInfo::NUM_OF_CONNECTION_INFOS:
       NOTREACHED();
