@@ -6,9 +6,15 @@
 
 #include "base/files/file_util.h"
 #include "base/format_macros.h"
+#include "base/i18n/file_util_icu.h"
+#include "base/metrics/field_trial_params.h"
 #include "base/rand_util.h"
+#include "base/strings/string_number_conversions.h"
 #include "base/strings/stringprintf.h"
+#include "base/strings/utf_string_conversions.h"
+#include "build/build_config.h"
 #include "components/download/public/common/download_create_info.h"
+#include "components/download/public/common/download_features.h"
 #include "components/download/public/common/download_interrupt_reasons_utils.h"
 #include "components/download/public/common/download_save_info.h"
 #include "components/download/public/common/download_stats.h"
@@ -18,10 +24,22 @@
 #include "net/http/http_request_headers.h"
 #include "net/http/http_status_code.h"
 #include "services/network/public/cpp/resource_request.h"
+#if defined(OS_ANDROID)
+#include "base/android/content_uri_utils.h"
+#include "components/download/internal/common/android/download_collection_bridge.h"
+#endif
 
 namespace download {
 
 namespace {
+// Default value for |kDownloadContentValidationLengthFinchKey|, when no
+// parameter is specified.
+const int64_t kDefaultContentValidationLength = 1024;
+
+// If the file_offset value from SaveInfo is equal to this, no content
+// validation will be performed and download stream will be written to
+// file starting at the offset from the response.
+const int64_t kInvalidFileWriteOffset = -1;
 
 void AppendExtraHeaders(net::HttpRequestHeaders* headers,
                         DownloadUrlParameters* params) {
@@ -35,22 +53,12 @@ const uint32_t DownloadItem::kInvalidId = 0;
 
 DownloadInterruptReason HandleRequestCompletionStatus(
     net::Error error_code,
-    bool has_strong_validators,
+    bool ignore_content_length_mismatch,
     net::CertStatus cert_status,
+    bool is_partial_request,
     DownloadInterruptReason abort_reason) {
-  // ERR_CONTENT_LENGTH_MISMATCH can be caused by 1 of the following reasons:
-  // 1. Server or proxy closes the connection too early.
-  // 2. The content-length header is wrong.
-  // If the download has strong validators, we can interrupt the download
-  // and let it resume automatically. Otherwise, resuming the download will
-  // cause it to restart and the download may never complete if the error was
-  // caused by reason 2. As a result, downloads without strong validators are
-  // treated as completed here.
-  // TODO(qinmin): check the metrics from downloads with strong validators,
-  // and decide whether we should interrupt downloads without strong validators
-  // rather than complete them.
   if (error_code == net::ERR_CONTENT_LENGTH_MISMATCH &&
-      !has_strong_validators) {
+      ignore_content_length_mismatch) {
     error_code = net::OK;
     RecordDownloadCount(COMPLETED_WITH_CONTENT_LENGTH_MISMATCH_COUNT);
   }
@@ -75,6 +83,12 @@ DownloadInterruptReason HandleRequestCompletionStatus(
     return abort_reason;
   }
 
+  // For some servers, a range request could cause the server to send
+  // wrongly encoded content and cause decoding failures. Restart the download
+  // in that case.
+  if (is_partial_request && error_code == net::ERR_CONTENT_DECODING_FAILED)
+    return DOWNLOAD_INTERRUPT_REASON_SERVER_NO_RANGE;
+
   return ConvertNetErrorToInterruptReason(error_code,
                                           DOWNLOAD_INTERRUPT_FROM_NETWORK);
 }
@@ -104,9 +118,9 @@ DownloadInterruptReason HandleSuccessfulServerResponse(
 
     case net::HTTP_NO_CONTENT:
     case net::HTTP_RESET_CONTENT:
-    // These two status codes don't have an entity (or rather RFC 7231
-    // requires that there be no entity). They are treated the same as the
-    // resource not being found since there is no entity to download.
+      // These two status codes don't have an entity (or rather RFC 7231
+      // requires that there be no entity). They are treated the same as the
+      // resource not being found since there is no entity to download.
 
     case net::HTTP_NOT_FOUND:
       result = DOWNLOAD_INTERRUPT_REASON_SERVER_BAD_CONTENT;
@@ -155,6 +169,7 @@ DownloadInterruptReason HandleSuccessfulServerResponse(
       // The response can be HTTP 200 or other error code when
       // |fetch_error_body| is true.
       save_info->offset = 0;
+      save_info->file_offset = kInvalidFileWriteOffset;
       save_info->hash_of_partial_file.clear();
       save_info->hash_state.reset();
       return DOWNLOAD_INTERRUPT_REASON_NONE;
@@ -171,12 +186,8 @@ DownloadInterruptReason HandleSuccessfulServerResponse(
         (save_info->length > 0 &&
          last_byte != save_info->offset + save_info->length - 1)) {
       // The server returned a different range than the one we requested. Assume
-      // the response is bad.
-      //
-      // In the future we should consider allowing offsets that are less than
-      // the offset we've requested, since in theory we can truncate the partial
-      // file at the offset and continue.
-      return DOWNLOAD_INTERRUPT_REASON_SERVER_BAD_CONTENT;
+      // the server doesn't support range request.
+      return DOWNLOAD_INTERRUPT_REASON_SERVER_NO_RANGE;
     }
 
     return DOWNLOAD_INTERRUPT_REASON_NONE;
@@ -218,10 +229,15 @@ void HandleResponseHeaders(const net::HttpResponseHeaders* headers,
   // In RFC 7233, a single part 206 partial response must generate
   // Content-Range. Accept-Range may be sent in 200 response to indicate the
   // server can handle range request, but optional in 206 response.
-  create_info->accept_range =
-      headers->HasHeaderValue("Accept-Ranges", "bytes") ||
+  if (headers->HasHeaderValue("Accept-Ranges", "bytes") ||
       (headers->HasHeader("Content-Range") &&
-       headers->response_code() == net::HTTP_PARTIAL_CONTENT);
+       headers->response_code() == net::HTTP_PARTIAL_CONTENT)) {
+    create_info->accept_range = RangeRequestSupportType::kSupport;
+  } else if (headers->HasHeaderValue("Accept-Ranges", "none")) {
+    create_info->accept_range = RangeRequestSupportType::kNoSupport;
+  } else {
+    create_info->accept_range = RangeRequestSupportType::kUnknown;
+  }
 }
 
 std::unique_ptr<network::ResourceRequest> CreateResourceRequest(
@@ -240,8 +256,11 @@ std::unique_ptr<network::ResourceRequest> CreateResourceRequest(
   request->allow_download = true;
   request->is_main_frame = true;
 
-  if (params->render_process_host_id() >= 0)
-    request->render_frame_id = params->render_frame_host_routing_id();
+  // Downloads should be treated as navigations from Fetch spec perspective.
+  // See also:
+  // - https://crbug.com/952834
+  // - https://github.com/whatwg/fetch/issues/896#issuecomment-484423278
+  request->mode = network::mojom::RequestMode::kNavigate;
 
   bool has_upload_data = false;
   if (params->post_body()) {
@@ -302,9 +321,13 @@ std::unique_ptr<net::HttpRequestHeaders> GetAdditionalRequestHeaders(
   bool has_etag = !params->etag().empty();
 
   // Strong validator(i.e. etag or last modified) is required in range requests
-  // for download resumption and parallel download.
-  DCHECK(has_etag || has_last_modified);
-  if (!has_etag && !has_last_modified) {
+  // for download resumption and parallel download, unless
+  // |kAllowDownloadResumptionWithoutStrongValidators| is enabled.
+  bool allow_resumption =
+      has_etag || has_last_modified ||
+      base::FeatureList::IsEnabled(
+          features::kAllowDownloadResumptionWithoutStrongValidators);
+  if (!allow_resumption) {
     DVLOG(1) << "Creating partial request without strong validators.";
     AppendExtraHeaders(headers.get(), params);
     return headers;
@@ -440,11 +463,11 @@ ResumeMode GetDownloadResumeMode(const GURL& url,
       break;
 
     case DOWNLOAD_INTERRUPT_REASON_SERVER_NO_RANGE:
-    // The server disagreed with the file offset that we sent.
+      // The server disagreed with the file offset that we sent.
 
     case DOWNLOAD_INTERRUPT_REASON_FILE_HASH_MISMATCH:
-    // The file on disk was found to not match the expected hash. Discard and
-    // start from beginning.
+      // The file on disk was found to not match the expected hash. Discard and
+      // start from beginning.
 
     case DOWNLOAD_INTERRUPT_REASON_FILE_TOO_SHORT:
       // The [possibly persisted] file offset disagreed with the file on disk.
@@ -532,11 +555,62 @@ bool IsDownloadDone(const GURL& url,
 
 bool DeleteDownloadedFile(const base::FilePath& path) {
   DCHECK(GetDownloadTaskRunner()->RunsTasksInCurrentSequence());
-
+#if defined(OS_ANDROID)
+  if (path.IsContentUri()) {
+    base::DeleteContentUri(path);
+    return true;
+  }
+#endif
   // Make sure we only delete files.
   if (base::DirectoryExists(path))
     return true;
   return base::DeleteFile(path, false);
 }
 
+download::DownloadItem::DownloadRenameResult RenameDownloadedFile(
+    const base::FilePath& from_path,
+    const base::FilePath& to_path) {
+  if (!base::PathExists(from_path) ||
+      !base::DirectoryExists(from_path.DirName()))
+    return DownloadItem::DownloadRenameResult::FAILURE_UNAVAILABLE;
+
+  if (!base::DirectoryExists(to_path.DirName()))
+    return DownloadItem::DownloadRenameResult::FAILURE_NAME_INVALID;
+
+  if (base::PathExists(to_path))
+    return DownloadItem::DownloadRenameResult::FAILURE_NAME_CONFLICT;
+
+  int max_path_component_length =
+      base::GetMaximumPathComponentLength(to_path.DirName());
+  if (max_path_component_length != -1) {
+    if (static_cast<int>(to_path.value().length()) >=
+        max_path_component_length) {
+      return DownloadItem::DownloadRenameResult::FAILURE_NAME_TOO_LONG;
+    }
+  }
+#if defined(OS_ANDROID)
+  if (from_path.IsContentUri()) {
+    return DownloadCollectionBridge::RenameDownloadUri(from_path,
+                                                       to_path.BaseName())
+               ? download::DownloadItem::DownloadRenameResult::SUCCESS
+               : download::DownloadItem::DownloadRenameResult::
+                     FAILURE_NAME_INVALID;
+  }
+#endif
+
+  return base::Move(from_path, to_path)
+             ? download::DownloadItem::DownloadRenameResult::SUCCESS
+             : download::DownloadItem::DownloadRenameResult::
+                   FAILURE_NAME_INVALID;
+}
+
+int64_t GetDownloadValidationLengthConfig() {
+  std::string finch_value = base::GetFieldTrialParamValueByFeature(
+      features::kAllowDownloadResumptionWithoutStrongValidators,
+      kDownloadContentValidationLengthFinchKey);
+  int64_t result;
+  return base::StringToInt64(finch_value, &result)
+             ? result
+             : kDefaultContentValidationLength;
+}
 }  // namespace download
