@@ -16,12 +16,14 @@
 #include "ash/public/cpp/immersive/immersive_fullscreen_controller.h"
 #include "ash/public/cpp/rounded_corner_decorator.h"
 #include "ash/public/cpp/shell_window_ids.h"
+#include "ash/public/cpp/window_pin_type.h"
 #include "ash/public/cpp/window_properties.h"
 #include "ash/public/cpp/window_state_type.h"
-#include "ash/public/interfaces/window_pin_type.mojom.h"
+#include "ash/root_window_controller.h"
 #include "ash/shell.h"
 #include "ash/wm/client_controlled_state.h"
 #include "ash/wm/drag_details.h"
+#include "ash/wm/splitview/split_view_controller.h"
 #include "ash/wm/toplevel_window_event_handler.h"
 #include "ash/wm/window_positioning_utils.h"
 #include "ash/wm/window_properties.h"
@@ -35,7 +37,6 @@
 #include "base/trace_event/traced_value.h"
 #include "components/exo/surface.h"
 #include "components/exo/wm_helper.h"
-#include "services/ws/public/mojom/window_tree_constants.mojom.h"
 #include "ui/aura/client/aura_constants.h"
 #include "ui/aura/window.h"
 #include "ui/aura/window_event_dispatcher.h"
@@ -74,81 +75,82 @@ Orientation SizeToOrientation(const gfx::Size& size) {
 // A ClientControlledStateDelegate that sends the state/bounds
 // change request to exo client.
 class ClientControlledStateDelegate
-    : public ash::wm::ClientControlledState::Delegate {
+    : public ash::ClientControlledState::Delegate {
  public:
   explicit ClientControlledStateDelegate(
       ClientControlledShellSurface* shell_surface)
       : shell_surface_(shell_surface) {}
   ~ClientControlledStateDelegate() override {}
 
-  // Overridden from ash::wm::ClientControlledState::Delegate:
-  void HandleWindowStateRequest(
-      ash::wm::WindowState* window_state,
-      ash::mojom::WindowStateType next_state) override {
+  // Overridden from ash::ClientControlledState::Delegate:
+  void HandleWindowStateRequest(ash::WindowState* window_state,
+                                ash::WindowStateType next_state) override {
     shell_surface_->OnWindowStateChangeEvent(window_state->GetStateType(),
                                              next_state);
   }
-  void HandleBoundsRequest(ash::wm::WindowState* window_state,
-                           ash::mojom::WindowStateType requested_state,
-                           const gfx::Rect& bounds) override {
-    gfx::Rect bounds_in_screen(bounds);
-    ::wm::ConvertRectToScreen(window_state->window()->GetRootWindow(),
-                              &bounds_in_screen);
-    int64_t display_id = display::Screen::GetScreen()
-                             ->GetDisplayNearestWindow(window_state->window())
-                             .id();
+  void HandleBoundsRequest(ash::WindowState* window_state,
+                           ash::WindowStateType requested_state,
+                           const gfx::Rect& bounds_in_display,
+                           int64_t display_id) override {
+    const display::Screen* screen = display::Screen::GetScreen();
+    display::Display target_display;
+    if (!screen->GetDisplayWithDisplayId(display_id, &target_display))
+      return;
+
+    gfx::Rect bounds_in_screen(bounds_in_display);
+    bounds_in_screen.Offset(target_display.bounds().OffsetFromOrigin());
 
     shell_surface_->OnBoundsChangeEvent(
         window_state->GetStateType(), requested_state, display_id,
         bounds_in_screen,
-        window_state->drag_details()
+        window_state->drag_details() && shell_surface_->IsDragging()
             ? window_state->drag_details()->bounds_change
             : 0);
   }
 
  private:
   ClientControlledShellSurface* shell_surface_;
+
   DISALLOW_COPY_AND_ASSIGN(ClientControlledStateDelegate);
 };
 
 // A WindowStateDelegate that implements ToggleFullscreen behavior for
 // client controlled window.
-class ClientControlledWindowStateDelegate
-    : public ash::wm::WindowStateDelegate {
+class ClientControlledWindowStateDelegate : public ash::WindowStateDelegate {
  public:
   explicit ClientControlledWindowStateDelegate(
       ClientControlledShellSurface* shell_surface,
-      ash::wm::ClientControlledState::Delegate* delegate)
+      ash::ClientControlledState::Delegate* delegate)
       : shell_surface_(shell_surface), delegate_(delegate) {}
   ~ClientControlledWindowStateDelegate() override {}
 
-  // Overridden from ash::wm::WindowStateDelegate:
-  bool ToggleFullscreen(ash::wm::WindowState* window_state) override {
-    ash::mojom::WindowStateType next_state;
+  // Overridden from ash::WindowStateDelegate:
+  bool ToggleFullscreen(ash::WindowState* window_state) override {
+    ash::WindowStateType next_state;
     aura::Window* window = window_state->window();
     switch (window_state->GetStateType()) {
-      case ash::mojom::WindowStateType::DEFAULT:
-      case ash::mojom::WindowStateType::NORMAL:
+      case ash::WindowStateType::kDefault:
+      case ash::WindowStateType::kNormal:
         window->SetProperty(aura::client::kPreFullscreenShowStateKey,
                             ui::SHOW_STATE_NORMAL);
-        next_state = ash::mojom::WindowStateType::FULLSCREEN;
+        next_state = ash::WindowStateType::kFullscreen;
         break;
-      case ash::mojom::WindowStateType::MAXIMIZED:
+      case ash::WindowStateType::kMaximized:
         window->SetProperty(aura::client::kPreFullscreenShowStateKey,
                             ui::SHOW_STATE_MAXIMIZED);
-        next_state = ash::mojom::WindowStateType::FULLSCREEN;
+        next_state = ash::WindowStateType::kFullscreen;
         break;
-      case ash::mojom::WindowStateType::FULLSCREEN:
+      case ash::WindowStateType::kFullscreen:
         switch (window->GetProperty(aura::client::kPreFullscreenShowStateKey)) {
           case ui::SHOW_STATE_DEFAULT:
           case ui::SHOW_STATE_NORMAL:
-            next_state = ash::mojom::WindowStateType::NORMAL;
+            next_state = ash::WindowStateType::kNormal;
             break;
           case ui::SHOW_STATE_MAXIMIZED:
-            next_state = ash::mojom::WindowStateType::MAXIMIZED;
+            next_state = ash::WindowStateType::kMaximized;
             break;
           case ui::SHOW_STATE_MINIMIZED:
-            next_state = ash::mojom::WindowStateType::MINIMIZED;
+            next_state = ash::WindowStateType::kMinimized;
             break;
           case ui::SHOW_STATE_FULLSCREEN:
           case ui::SHOW_STATE_INACTIVE:
@@ -159,14 +161,14 @@ class ClientControlledWindowStateDelegate
             return false;
         }
         break;
-      case ash::mojom::WindowStateType::MINIMIZED: {
+      case ash::WindowStateType::kMinimized: {
         ui::WindowShowState pre_full_state =
             window->GetProperty(aura::client::kPreMinimizedShowStateKey);
         if (pre_full_state != ui::SHOW_STATE_FULLSCREEN) {
           window->SetProperty(aura::client::kPreFullscreenShowStateKey,
                               pre_full_state);
         }
-        next_state = ash::mojom::WindowStateType::FULLSCREEN;
+        next_state = ash::WindowStateType::kFullscreen;
         break;
       }
       default:
@@ -187,12 +189,12 @@ class ClientControlledWindowStateDelegate
 
  private:
   ClientControlledShellSurface* shell_surface_;
-  ash::wm::ClientControlledState::Delegate* delegate_;
+  ash::ClientControlledState::Delegate* delegate_;
 
   DISALLOW_COPY_AND_ASSIGN(ClientControlledWindowStateDelegate);
 };
 
-bool IsPinned(const ash::wm::WindowState* window_state) {
+bool IsPinned(const ash::WindowState* window_state) {
   return window_state->IsPinned() || window_state->IsTrustedPinned();
 }
 
@@ -240,7 +242,7 @@ class EventTargetingBlocker : aura::WindowObserver {
   void Register(aura::Window* window) {
     window->AddObserver(this);
     auto policy = window->event_targeting_policy();
-    window->SetEventTargetingPolicy(ws::mojom::EventTargetingPolicy::NONE);
+    window->SetEventTargetingPolicy(aura::EventTargetingPolicy::kNone);
     policy_map_.emplace(window, policy);
     for (auto* child : window->children())
       Register(child);
@@ -261,7 +263,7 @@ class EventTargetingBlocker : aura::WindowObserver {
     window->RemoveObserver(this);
   }
 
-  std::map<aura::Window*, ws::mojom::EventTargetingPolicy> policy_map_;
+  std::map<aura::Window*, aura::EventTargetingPolicy> policy_map_;
   aura::Window* window_ = nullptr;
 
   DISALLOW_COPY_AND_ASSIGN(EventTargetingBlocker);
@@ -278,7 +280,7 @@ class ClientControlledShellSurface::ScopedSetBoundsLocally {
   ~ScopedSetBoundsLocally() { state_->set_bounds_locally(false); }
 
  private:
-  ash::wm::ClientControlledState* const state_;
+  ash::ClientControlledState* const state_;
 
   DISALLOW_COPY_AND_ASSIGN(ScopedSetBoundsLocally);
 };
@@ -303,13 +305,20 @@ class ClientControlledShellSurface::ScopedLockedToRoot {
 ClientControlledShellSurface::ClientControlledShellSurface(Surface* surface,
                                                            bool can_minimize,
                                                            int container)
-    : ShellSurfaceBase(surface, gfx::Point(), true, can_minimize, container) {
+    : ShellSurfaceBase(surface, gfx::Point(), true, can_minimize, container),
+      current_pin_(ash::WindowPinType::kNone) {
   display::Screen::GetScreen()->AddObserver(this);
 }
 
 ClientControlledShellSurface::~ClientControlledShellSurface() {
+  // Reset the window delegate here so that we won't try to do any dragging
+  // operation on a to-be-destroyed window. |widget_| can be nullptr in tests.
+  if (GetWidget())
+    GetWindowState()->SetDelegate(nullptr);
   wide_frame_.reset();
   display::Screen::GetScreen()->RemoveObserver(this);
+  if (current_pin_ != ash::WindowPinType::kNone)
+    SetPinned(ash::WindowPinType::kNone);
 }
 
 void ClientControlledShellSurface::SetBounds(int64_t display_id,
@@ -328,42 +337,42 @@ void ClientControlledShellSurface::SetBounds(int64_t display_id,
 
 void ClientControlledShellSurface::SetMaximized() {
   TRACE_EVENT0("exo", "ClientControlledShellSurface::SetMaximized");
-  pending_window_state_ = ash::mojom::WindowStateType::MAXIMIZED;
+  pending_window_state_ = ash::WindowStateType::kMaximized;
 }
 
 void ClientControlledShellSurface::SetMinimized() {
   TRACE_EVENT0("exo", "ClientControlledShellSurface::SetMinimized");
-  pending_window_state_ = ash::mojom::WindowStateType::MINIMIZED;
+  pending_window_state_ = ash::WindowStateType::kMinimized;
 }
 
 void ClientControlledShellSurface::SetRestored() {
   TRACE_EVENT0("exo", "ClientControlledShellSurface::SetRestored");
-  pending_window_state_ = ash::mojom::WindowStateType::NORMAL;
+  pending_window_state_ = ash::WindowStateType::kNormal;
 }
 
 void ClientControlledShellSurface::SetFullscreen(bool fullscreen) {
   TRACE_EVENT1("exo", "ClientControlledShellSurface::SetFullscreen",
                "fullscreen", fullscreen);
-  pending_window_state_ = fullscreen ? ash::mojom::WindowStateType::FULLSCREEN
-                                     : ash::mojom::WindowStateType::NORMAL;
+  pending_window_state_ = fullscreen ? ash::WindowStateType::kFullscreen
+                                     : ash::WindowStateType::kNormal;
 }
 
 void ClientControlledShellSurface::SetSnappedToLeft() {
   TRACE_EVENT0("exo", "ClientControlledShellSurface::SetSnappedToLeft");
-  pending_window_state_ = ash::mojom::WindowStateType::LEFT_SNAPPED;
+  pending_window_state_ = ash::WindowStateType::kLeftSnapped;
 }
 
 void ClientControlledShellSurface::SetSnappedToRight() {
   TRACE_EVENT0("exo", "ClientControlledShellSurface::SetSnappedToRight");
-  pending_window_state_ = ash::mojom::WindowStateType::RIGHT_SNAPPED;
+  pending_window_state_ = ash::WindowStateType::kRightSnapped;
 }
 
 void ClientControlledShellSurface::SetPip() {
   TRACE_EVENT0("exo", "ClientControlledShellSurface::SetPip");
-  pending_window_state_ = ash::mojom::WindowStateType::PIP;
+  pending_window_state_ = ash::WindowStateType::kPip;
 }
 
-void ClientControlledShellSurface::SetPinned(ash::mojom::WindowPinType type) {
+void ClientControlledShellSurface::SetPinned(ash::WindowPinType type) {
   TRACE_EVENT1("exo", "ClientControlledShellSurface::SetPinned", "type",
                static_cast<int>(type));
 
@@ -371,6 +380,7 @@ void ClientControlledShellSurface::SetPinned(ash::mojom::WindowPinType type) {
     CreateShellSurfaceWidget(ui::SHOW_STATE_NORMAL);
 
   widget_->GetNativeWindow()->SetProperty(ash::kWindowPinTypeKey, type);
+  current_pin_ = type;
 }
 
 void ClientControlledShellSurface::SetSystemUiVisibility(bool autohide) {
@@ -380,7 +390,7 @@ void ClientControlledShellSurface::SetSystemUiVisibility(bool autohide) {
   if (!widget_)
     CreateShellSurfaceWidget(ui::SHOW_STATE_NORMAL);
 
-  ash::wm::SetAutoHideShelf(widget_->GetNativeWindow(), autohide);
+  ash::window_util::SetAutoHideShelf(widget_->GetNativeWindow(), autohide);
 }
 
 void ClientControlledShellSurface::SetAlwaysOnTop(bool always_on_top) {
@@ -390,8 +400,21 @@ void ClientControlledShellSurface::SetAlwaysOnTop(bool always_on_top) {
   if (!widget_)
     CreateShellSurfaceWidget(ui::SHOW_STATE_NORMAL);
 
-  widget_->GetNativeWindow()->SetProperty(aura::client::kAlwaysOnTopKey,
-                                          always_on_top);
+  widget_->GetNativeWindow()->SetProperty(aura::client::kZOrderingKey,
+                                          always_on_top
+                                              ? ui::ZOrderLevel::kFloatingWindow
+                                              : ui::ZOrderLevel::kNormal);
+}
+
+void ClientControlledShellSurface::SetImeBlocked(bool ime_blocked) {
+  TRACE_EVENT1("exo", "ClientControlledShellSurface::SetImeBlocked",
+               "ime_blocked", ime_blocked);
+
+  if (!widget_)
+    CreateShellSurfaceWidget(ui::SHOW_STATE_NORMAL);
+
+  WMHelper::GetInstance()->SetImeBlocked(widget_->GetNativeWindow(),
+                                         ime_blocked);
 }
 
 void ClientControlledShellSurface::SetOrientation(Orientation orientation) {
@@ -423,6 +446,16 @@ void ClientControlledShellSurface::SetScale(double scale) {
   pending_scale_ = scale;
 }
 
+void ClientControlledShellSurface::CommitPendingScale() {
+  if (pending_scale_ != scale_) {
+    gfx::Transform transform;
+    DCHECK_NE(pending_scale_, 0.0);
+    transform.Scale(1.0 / pending_scale_, 1.0 / pending_scale_);
+    host_window()->SetTransform(transform);
+    scale_ = pending_scale_;
+  }
+}
+
 void ClientControlledShellSurface::SetTopInset(int height) {
   TRACE_EVENT1("exo", "ClientControlledShellSurface::SetTopInset", "height",
                height);
@@ -433,16 +466,16 @@ void ClientControlledShellSurface::SetTopInset(int height) {
 void ClientControlledShellSurface::SetResizeOutset(int outset) {
   TRACE_EVENT1("exo", "ClientControlledShellSurface::SetResizeOutset", "outset",
                outset);
-  if (client_controlled_move_resize_) {
-    if (root_surface())
-      root_surface()->SetInputOutset(outset);
-  }
+  // Deprecated.
+  NOTREACHED();
 }
 
 void ClientControlledShellSurface::OnWindowStateChangeEvent(
-    ash::mojom::WindowStateType current_state,
-    ash::mojom::WindowStateType next_state) {
-  if (state_changed_callback_)
+    ash::WindowStateType current_state,
+    ash::WindowStateType next_state) {
+  // Android already knows this state change. Don't send state change to Android
+  // that it is about to do anyway.
+  if (state_changed_callback_ && pending_window_state_ != next_state)
     state_changed_callback_.Run(current_state, next_state);
 }
 
@@ -451,7 +484,7 @@ void ClientControlledShellSurface::StartDrag(int component,
   TRACE_EVENT2("exo", "ClientControlledShellSurface::StartDrag", "component",
                component, "location", location.ToString());
 
-  if (!widget_ || (client_controlled_move_resize_ && component != HTCAPTION))
+  if (!widget_)
     return;
   AttemptToStartDrag(component, location);
 }
@@ -473,10 +506,14 @@ void ClientControlledShellSurface::AttemptToStartDrag(
     wm::ConvertPointFromScreen(target->GetRootWindow(), &point_in_root);
     toplevel_handler->AttemptToStartDrag(
         target, point_in_root, component,
-        ash::wm::WmToplevelWindowEventHandler::EndClosure());
+        ash::ToplevelWindowEventHandler::EndClosure());
   }
 }
 
+bool ClientControlledShellSurface::IsDragging() {
+  return in_drag_;
+}
+
 void ClientControlledShellSurface::SetCanMaximize(bool can_maximize) {
   TRACE_EVENT1("exo", "ClientControlledShellSurface::SetCanMaximize",
                "can_maximzie", can_maximize);
@@ -518,6 +555,10 @@ void ClientControlledShellSurface::SetExtraTitle(
 
   GetFrameView()->GetHeaderView()->GetFrameHeader()->SetFrameTextOverride(
       extra_title);
+  if (wide_frame_) {
+    wide_frame_->header_view()->GetFrameHeader()->SetFrameTextOverride(
+        extra_title);
+  }
 }
 
 void ClientControlledShellSurface::SetOrientationLock(
@@ -536,41 +577,63 @@ void ClientControlledShellSurface::SetOrientationLock(
 }
 
 void ClientControlledShellSurface::OnBoundsChangeEvent(
-    ash::mojom::WindowStateType current_state,
-    ash::mojom::WindowStateType requested_state,
+    ash::WindowStateType current_state,
+    ash::WindowStateType requested_state,
     int64_t display_id,
     const gfx::Rect& window_bounds,
     int bounds_change) {
+  if (ignore_bounds_change_request_)
+    return;
   // 1) Do no update the bounds unless we have geometry from client.
-  // 2) Do not update the bounds if window is minimized.
+  // 2) Do not update the bounds if window is minimized unless it
+  // exiting the minimzied state.
   // The bounds will be provided by client when unminimized.
   if (!geometry().IsEmpty() && !window_bounds.IsEmpty() &&
-      !widget_->IsMinimized() && bounds_changed_callback_) {
+      (!widget_->IsMinimized() ||
+       requested_state != ash::WindowStateType::kMinimized) &&
+      bounds_changed_callback_) {
     // Sends the client bounds, which matches the geometry
     // when frame is enabled.
     ash::NonClientFrameViewAsh* frame_view = GetFrameView();
 
-    // The client's geometry uses fullscreen in client controlled,
-    // (but the surface is placed under the frame), so just use
-    // the window bounds instead for maximixed state.
+    // Snapped window states in tablet mode do not include the caption height.
+    const bool becoming_snapped =
+        requested_state == ash::WindowStateType::kLeftSnapped ||
+        requested_state == ash::WindowStateType::kRightSnapped;
+    const bool is_tablet_mode = WMHelper::GetInstance()->InTabletMode();
     gfx::Rect client_bounds =
-        widget_->IsMaximized()
+        becoming_snapped && is_tablet_mode
             ? window_bounds
             : frame_view->GetClientBoundsForWindowBounds(window_bounds);
     gfx::Size current_size = frame_view->GetBoundsForClientView().size();
-    bool is_resize = client_bounds.size() != current_size;
+    bool is_resize = client_bounds.size() != current_size &&
+                     !widget_->IsMaximized() && !widget_->IsFullscreen();
+
     bounds_changed_callback_.Run(current_state, requested_state, display_id,
                                  client_bounds, is_resize, bounds_change);
+
+    auto* window_state = GetWindowState();
+    if (server_reparent_window_ &&
+        window_state->GetDisplay().id() != display_id) {
+      ScopedSetBoundsLocally scoped_set_bounds(this);
+      int container_id = window_state->window()->parent()->id();
+      aura::Window* new_parent =
+          ash::Shell::GetRootWindowControllerWithDisplayId(display_id)
+              ->GetContainer(container_id);
+      new_parent->AddChild(window_state->window());
+    }
   }
 }
 
 void ClientControlledShellSurface::OnDragStarted(int component) {
+  in_drag_ = true;
   if (drag_started_callback_)
     drag_started_callback_.Run(component);
 }
 
 void ClientControlledShellSurface::OnDragFinished(bool canceled,
                                                   const gfx::Point& location) {
+  in_drag_ = false;
   if (drag_finished_callback_)
     drag_finished_callback_.Run(location.x(), location.y(), canceled);
 }
@@ -630,6 +693,9 @@ void ClientControlledShellSurface::OnSetFrameColors(SkColor active_color,
 
 void ClientControlledShellSurface::OnWindowAddedToRootWindow(
     aura::Window* window) {
+  if (client_controlled_state_->set_bounds_locally())
+    return;
+
   ScopedLockedToRoot scoped_locked_to_root(widget_);
   UpdateWidgetBounds();
 }
@@ -643,8 +709,8 @@ bool ClientControlledShellSurface::CanMaximize() const {
 
 views::NonClientFrameView*
 ClientControlledShellSurface::CreateNonClientFrameView(views::Widget* widget) {
-  ash::wm::WindowState* window_state = GetWindowState();
-  std::unique_ptr<ash::wm::ClientControlledState::Delegate> delegate =
+  ash::WindowState* window_state = GetWindowState();
+  std::unique_ptr<ash::ClientControlledState::Delegate> delegate =
       GetFactoryForTesting()
           ? GetFactoryForTesting().Run()
           : std::make_unique<ClientControlledStateDelegate>(this);
@@ -652,16 +718,15 @@ ClientControlledShellSurface::CreateNonClientFrameView(views::Widget* widget) {
   auto window_delegate = std::make_unique<ClientControlledWindowStateDelegate>(
       this, delegate.get());
   auto state =
-      std::make_unique<ash::wm::ClientControlledState>(std::move(delegate));
+      std::make_unique<ash::ClientControlledState>(std::move(delegate));
   client_controlled_state_ = state.get();
   window_state->SetStateObject(std::move(state));
   window_state->SetDelegate(std::move(window_delegate));
   ash::NonClientFrameViewAsh* frame_view =
       static_cast<ash::NonClientFrameViewAsh*>(
-          ShellSurfaceBase::CreateNonClientFrameView(widget));
+          CreateNonClientFrameViewInternal(widget, /*client_controlled=*/true));
   immersive_fullscreen_controller_ =
-      std::make_unique<ash::ImmersiveFullscreenController>(
-          ash::Shell::Get()->immersive_context());
+      std::make_unique<ash::ImmersiveFullscreenController>();
   frame_view->InitImmersiveFullscreenControllerForView(
       immersive_fullscreen_controller_.get());
   return frame_view;
@@ -706,7 +771,7 @@ void ClientControlledShellSurface::OnDisplayMetricsChanged(
     const display::Display& new_display,
     uint32_t changed_metrics) {
   if (!widget_ || !widget_->IsActive() ||
-      !WMHelper::GetInstance()->IsTabletModeWindowManagerEnabled()) {
+      !WMHelper::GetInstance()->InTabletMode()) {
     return;
   }
 
@@ -746,6 +811,8 @@ void ClientControlledShellSurface::SetWidgetBounds(const gfx::Rect& bounds) {
   display::Display display;
   if (screen->GetDisplayWithDisplayId(display_id_, &display)) {
     bool is_display_stale = display_id_ != current_display.id();
+    LOG(ERROR) << "DisplayId:" << display_id_
+               << ", current:" << current_display.id();
 
     // Preserve widget bounds until client acknowledges display move.
     if (preserve_widget_bounds_ && is_display_stale)
@@ -768,7 +835,7 @@ void ClientControlledShellSurface::SetWidgetBounds(const gfx::Rect& bounds) {
   // Calculate a minimum window visibility required bounds.
   gfx::Rect adjusted_bounds = bounds;
   if (!is_display_move_pending) {
-    ash::wm::ClientControlledState::AdjustBoundsForMinimumWindowVisibility(
+    ash::ClientControlledState::AdjustBoundsForMinimumWindowVisibility(
         target_display.bounds(), &adjusted_bounds);
   }
 
@@ -777,9 +844,9 @@ void ClientControlledShellSurface::SetWidgetBounds(const gfx::Rect& bounds) {
     return;
   }
 
-  bool set_bounds_locally = !client_controlled_move_resize_ &&
-                            GetWindowState()->is_dragged() &&
-                            !is_display_move_pending;
+  bool set_bounds_locally =
+      GetWindowState()->is_dragged() && !is_display_move_pending;
+  LOG(ERROR) << "Updating Locally";
 
   if (set_bounds_locally || client_controlled_state_->set_bounds_locally()) {
     // Convert from screen to display coordinates.
@@ -794,6 +861,7 @@ void ClientControlledShellSurface::SetWidgetBounds(const gfx::Rect& bounds) {
     UpdateSurfaceBounds();
     return;
   }
+  LOG(ERROR) << "Updating Remotely";
 
   {
     ScopedSetBoundsLocally scoped_set_bounds(this);
@@ -801,6 +869,8 @@ void ClientControlledShellSurface::SetWidgetBounds(const gfx::Rect& bounds) {
   }
 
   if (bounds != adjusted_bounds || is_display_move_pending) {
+    LOG(ERROR) << "Sending Bounds:" << bounds.ToString()
+               << ", adjusted=" << adjusted_bounds.ToString();
     // Notify client that bounds were adjusted or window moved across displays.
     auto state_type = GetWindowState()->GetStateType();
     OnBoundsChangeEvent(state_type, state_type, target_display.id(),
@@ -813,7 +883,7 @@ void ClientControlledShellSurface::SetWidgetBounds(const gfx::Rect& bounds) {
 gfx::Rect ClientControlledShellSurface::GetShadowBounds() const {
   gfx::Rect shadow_bounds = ShellSurfaceBase::GetShadowBounds();
   const ash::NonClientFrameViewAsh* frame_view = GetFrameView();
-  if (frame_view->visible()) {
+  if (frame_view->GetVisible()) {
     // The client controlled geometry is only for the client
     // area. When the chrome side frame is enabled, the shadow height
     // has to include the height of the frame, and the total height is
@@ -827,7 +897,7 @@ gfx::Rect ClientControlledShellSurface::GetShadowBounds() const {
 }
 
 void ClientControlledShellSurface::InitializeWindowState(
-    ash::wm::WindowState* window_state) {
+    ash::WindowState* window_state) {
   // Allow the client to request bounds that do not fill the entire work area
   // when maximized, or the entire display when fullscreen.
   window_state->set_allow_set_bounds_direct(true);
@@ -850,13 +920,7 @@ float ClientControlledShellSurface::GetScale() const {
 base::Optional<gfx::Rect> ClientControlledShellSurface::GetWidgetBounds()
     const {
   const ash::NonClientFrameViewAsh* frame_view = GetFrameView();
-  if (frame_view->visible()) {
-    // The client's geometry uses entire display area in client
-    // controlled in maximized, and the surface is placed under the
-    // frame. Just use the visible bounds (geometry) for the widget
-    // bounds.
-    if (widget_->IsMaximized())
-      return GetVisibleBounds();
+  if (frame_view->GetVisible()) {
     return frame_view->GetWindowBoundsForClientBounds(GetVisibleBounds());
   }
 
@@ -878,14 +942,14 @@ bool ClientControlledShellSurface::OnPreWidgetCommit() {
     CreateShellSurfaceWidget(ash::ToWindowShowState(pending_window_state_));
   }
 
-  ash::wm::WindowState* window_state = GetWindowState();
-  if (window_state->GetStateType() == pending_window_state_) {
+  ash::WindowState* window_state = GetWindowState();
+  state_changed_ = window_state->GetStateType() != pending_window_state_;
+  if (!state_changed_) {
     // Animate PIP window movement unless it is being dragged.
     if (window_state->IsPip() && !window_state->is_dragged()) {
       client_controlled_state_->set_next_bounds_change_animation_type(
-          ash::wm::ClientControlledState::kAnimationAnimated);
+          ash::ClientControlledState::kAnimationAnimated);
     }
-
     return true;
   }
 
@@ -894,25 +958,25 @@ bool ClientControlledShellSurface::OnPreWidgetCommit() {
     return true;
   }
 
-  auto animation_type = ash::wm::ClientControlledState::kAnimationNone;
+  auto animation_type = ash::ClientControlledState::kAnimationNone;
   switch (pending_window_state_) {
-    case ash::mojom::WindowStateType::NORMAL:
+    case ash::WindowStateType::kNormal:
       if (widget_->IsMaximized() || widget_->IsFullscreen()) {
-        animation_type = ash::wm::ClientControlledState::kAnimationCrossFade;
+        animation_type = ash::ClientControlledState::kAnimationCrossFade;
       }
       break;
 
-    case ash::mojom::WindowStateType::MAXIMIZED:
-    case ash::mojom::WindowStateType::FULLSCREEN:
-      animation_type = ash::wm::ClientControlledState::kAnimationCrossFade;
+    case ash::WindowStateType::kMaximized:
+    case ash::WindowStateType::kFullscreen:
+      if (!window_state->IsPip())
+        animation_type = ash::ClientControlledState::kAnimationCrossFade;
       break;
 
     default:
       break;
   }
 
-  // PIP windows should not be able to be active.
-  if (pending_window_state_ == ash::mojom::WindowStateType::PIP) {
+  if (pending_window_state_ == ash::WindowStateType::kPip) {
     if (ash::features::IsPipRoundedCornersEnabled()) {
       decorator_ = std::make_unique<ash::RoundedCornerDecorator>(
           window_state->window(), host_window(), host_window()->layer(),
@@ -922,12 +986,27 @@ bool ClientControlledShellSurface::OnPreWidgetCommit() {
     decorator_.reset();  // Remove rounded corners.
   }
 
+  bool wasPip = window_state->IsPip();
+
+  // As the bounds of the widget is updated later, ensure that no bounds change
+  // happens with this state change (e.g. updatePipBounds can be triggered).
+  base::AutoReset<bool> resetter(&ignore_bounds_change_request_, true);
   if (client_controlled_state_->EnterNextState(window_state,
                                                pending_window_state_)) {
     client_controlled_state_->set_next_bounds_change_animation_type(
         animation_type);
   }
 
+  if (wasPip && !window_state->IsMinimized()) {
+    // Expanding PIP should end split-view. See crbug.com/941788.
+    ash::Shell::Get()->split_view_controller()->EndSplitView(
+        ash::SplitViewController::EndReason::kPipExpanded);
+    // As Android doesn't activate PIP tasks after they are expanded, we need
+    // to do it here explicitly.
+    // TODO(937738): Investigate if we can activate PIP windows inside commit.
+    window_state->Activate();
+  }
+
   return true;
 }
 
@@ -948,13 +1027,7 @@ void ClientControlledShellSurface::OnPostWidgetCommit() {
   }
 
   // Update surface scale.
-  if (pending_scale_ != scale_) {
-    gfx::Transform transform;
-    DCHECK_NE(pending_scale_, 0.0);
-    transform.Scale(1.0 / pending_scale_, 1.0 / pending_scale_);
-    host_window()->SetTransform(transform);
-    scale_ = pending_scale_;
-  }
+  CommitPendingScale();
 
   orientation_ = pending_orientation_;
   if (expected_orientation_ == orientation_)
@@ -972,17 +1045,25 @@ void ClientControlledShellSurface::UpdateFrame() {
           ->GetDisplayNearestWindow(widget_->GetNativeWindow())
           .work_area();
 
-  ash::wm::WindowState* window_state = GetWindowState();
-  bool enable_wide_frame = GetFrameView()->visible() &&
+  ash::WindowState* window_state = GetWindowState();
+  bool enable_wide_frame = GetFrameView()->GetVisible() &&
                            window_state->IsMaximizedOrFullscreenOrPinned() &&
                            work_area.width() != geometry().width();
-
+  bool update_frame = state_changed_;
+  state_changed_ = false;
   if (enable_wide_frame) {
     if (!wide_frame_) {
+      update_frame = true;
       wide_frame_ = std::make_unique<ash::WideFrameView>(widget_);
       ash::ImmersiveFullscreenController::EnableForWidget(widget_, false);
       wide_frame_->Init(immersive_fullscreen_controller_.get());
+      wide_frame_->header_view()->GetFrameHeader()->SetFrameTextOverride(
+          GetFrameView()
+              ->GetHeaderView()
+              ->GetFrameHeader()
+              ->frame_text_override());
       wide_frame_->GetWidget()->Show();
+
       // Restoring window targeter replaced by ImmersiveFullscreenController.
       InstallCustomWindowTargeter();
 
@@ -990,6 +1071,7 @@ void ClientControlledShellSurface::UpdateFrame() {
     }
   } else {
     if (wide_frame_) {
+      update_frame = true;
       ash::ImmersiveFullscreenController::EnableForWidget(widget_, false);
       wide_frame_.reset();
       GetFrameView()->InitImmersiveFullscreenControllerForView(
@@ -1004,7 +1086,8 @@ void ClientControlledShellSurface::UpdateFrame() {
   // The autohide should be applied when the window state is in
   // maximzied, fullscreen or pinned. Update the auto hide state
   // inside commit.
-  UpdateAutoHideFrame();
+  if (update_frame)
+    UpdateAutoHideFrame();
 }
 
 void ClientControlledShellSurface::UpdateCaptionButtonModel() {
@@ -1053,8 +1136,8 @@ void ClientControlledShellSurface::
   }
 }
 
-ash::wm::WindowState* ClientControlledShellSurface::GetWindowState() {
-  return ash::wm::GetWindowState(widget_->GetNativeWindow());
+ash::WindowState* ClientControlledShellSurface::GetWindowState() {
+  return ash::WindowState::Get(widget_->GetNativeWindow());
 }
 
 ash::NonClientFrameViewAsh* ClientControlledShellSurface::GetFrameView() {
