@@ -7,7 +7,7 @@
 #include <memory>
 #include <utility>
 
-#include "base/stl_util.h"
+#include "base/strings/strcat.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
@@ -57,15 +57,52 @@ constexpr const char* kCustomPatternsWithContext[] = {
 
     // Serial numbers
     "(?i-s)(serial\\s*(?:number)?\\s*[:=]\\s*)([0-9a-zA-Z\\-\"]+)()",
+
+    // GAIA IDs
+    R"xxx((\"?\bgaia_id\"?[=:]['\"])(\d+)(\b['\"]))xxx",
+    R"xxx((\{id: )(\d+)(, email:))xxx",
 };
 
-// Returns the number of leading bytes that may be kept unsanitized.
-std::string MaybeScrubIPv4Address(const std::string& addr) {
+bool MaybeUnmapAddress(net::IPAddress* addr) {
+  if (!addr->IsIPv4MappedIPv6())
+    return false;
+
+  *addr = net::ConvertIPv4MappedIPv6ToIPv4(*addr);
+  return true;
+}
+
+bool MaybeUntranslateAddress(net::IPAddress* addr) {
+  if (!addr->IsIPv6())
+    return false;
+
+  static const net::IPAddress kTranslated6To4(0, 0x64, 0xff, 0x9b, 0, 0, 0, 0,
+                                              0, 0, 0, 0, 0, 0, 0, 0);
+  if (!IPAddressMatchesPrefix(*addr, kTranslated6To4, 96))
+    return false;
+
+  const auto bytes = addr->bytes();
+  *addr = net::IPAddress(bytes[12], bytes[13], bytes[14], bytes[15]);
+  return true;
+}
+
+// If |addr| points to a valid IPv6 address, this function truncates it at /32.
+bool MaybeTruncateIPv6(net::IPAddress* addr) {
+  if (!addr->IsIPv6())
+    return false;
+
+  const auto bytes = addr->bytes();
+  *addr = net::IPAddress(bytes[0], bytes[1], bytes[2], bytes[3], 0, 0, 0, 0, 0,
+                         0, 0, 0, 0, 0, 0, 0);
+  return true;
+}
+
+// Returns an appropriately scrubbed version of |addr| if applicable.
+std::string MaybeScrubIPAddress(const std::string& addr) {
   struct {
     net::IPAddress ip_addr;
     int prefix_length;
     bool scrub;
-  } static const kWhitelistedIPv4Ranges[] = {
+  } static const kWhitelistedIPRanges[] = {
       // Private.
       {net::IPAddress(10, 0, 0, 0), 8, true},
       {net::IPAddress(172, 16, 0, 0), 12, true},
@@ -84,20 +121,57 @@ std::string MaybeScrubIPv4Address(const std::string& addr) {
       {net::IPAddress(224, 0, 0, 0), 4, true},
       // Link local.
       {net::IPAddress(169, 254, 0, 0), 16, true},
+      {net::IPAddress(0xfe, 0x80, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0), 10,
+       true},
       // Broadcast.
       {net::IPAddress(255, 255, 255, 255), 32, false},
+      // IPv6 loopback, unspecified and non-address strings.
+      {net::IPAddress::IPv6AllZeros(), 112, false},
+      // IPv6 multicast all nodes and routers.
+      {net::IPAddress(0xff, 0x01, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1),
+       128, false},
+      {net::IPAddress(0xff, 0x01, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2),
+       128, false},
+      {net::IPAddress(0xff, 0x02, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1),
+       128, false},
+      {net::IPAddress(0xff, 0x02, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2),
+       128, false},
+      // IPv6 other multicast (link and interface local).
+      {net::IPAddress(0xff, 0x01, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0), 16,
+       true},
+      {net::IPAddress(0xff, 0x02, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0), 16,
+       true},
+
   };
   net::IPAddress input_addr;
-  if (input_addr.AssignFromIPLiteral(addr) && input_addr.IsIPv4()) {
-    for (const auto& range : kWhitelistedIPv4Ranges) {
+  if (input_addr.AssignFromIPLiteral(addr) && input_addr.IsValid()) {
+    bool mapped = MaybeUnmapAddress(&input_addr);
+    bool translated = !mapped ? MaybeUntranslateAddress(&input_addr) : false;
+    for (const auto& range : kWhitelistedIPRanges) {
       if (IPAddressMatchesPrefix(input_addr, range.ip_addr,
                                  range.prefix_length)) {
-        return range.scrub ? base::StringPrintf(
-                                 "%s/%d", range.ip_addr.ToString().c_str(),
-                                 range.prefix_length)
-                           : addr;
+        std::string prefix;
+        std::string out_addr = addr;
+        if (mapped) {
+          prefix = "M ";
+          out_addr = input_addr.ToString();
+        } else if (translated) {
+          prefix = "T ";
+          out_addr = input_addr.ToString();
+        }
+        if (range.scrub) {
+          out_addr = base::StringPrintf(
+              "%s/%d", range.ip_addr.ToString().c_str(), range.prefix_length);
+        }
+        return base::StrCat({prefix, out_addr});
       }
     }
+    // |addr| may have been over-aggressively matched as an IPv6 address when
+    // it's really just an arbitrary part of a sentence. If the string is the
+    // same as the coarsely truncated address then keep it because even if
+    // it happens to be a real address, there is no loss of anonymity.
+    if (MaybeTruncateIPv6(&input_addr) && input_addr.ToString() == addr)
+      return addr;
   }
   return "";
 }
@@ -279,8 +353,9 @@ bool FindAndConsumeAndGetSkipped(re2::StringPiece* input,
 
 }  // namespace
 
-AnonymizerTool::AnonymizerTool()
-    : custom_patterns_with_context_(base::size(kCustomPatternsWithContext)),
+AnonymizerTool::AnonymizerTool(const char* const* first_party_extension_ids)
+    : first_party_extension_ids_(first_party_extension_ids),
+      custom_patterns_with_context_(base::size(kCustomPatternsWithContext)),
       custom_patterns_without_context_(
           base::size(kCustomPatternsWithoutContext)) {
   DETACH_FROM_SEQUENCE(sequence_checker_);
@@ -390,7 +465,7 @@ std::string AnonymizerTool::AnonymizeCustomPatternWithContext(
     std::string matched_id_as_string = matched_id.as_string();
     std::string replacement_id = (*identifier_space)[matched_id_as_string];
     if (replacement_id.empty()) {
-      replacement_id = base::IntToString(identifier_space->size());
+      replacement_id = base::NumberToString(identifier_space->size());
       (*identifier_space)[matched_id_as_string] = replacement_id;
     }
 
@@ -403,6 +478,51 @@ std::string AnonymizerTool::AnonymizeCustomPatternWithContext(
   return result;
 }
 
+// This takes a |url| argument and returns true if the URL is whitelisted and
+// does NOT need to be redacted, returns false otherwise.
+bool IsUrlWhitelisted(re2::StringPiece url,
+                      const char* const* first_party_extension_ids) {
+  // We do not whitelist anything with a query parameter.
+  if (url.contains("?"))
+    return false;
+
+  // Check for whitelisting of chrome:// URLs.
+  if (url.starts_with("chrome://")) {
+    // We allow everything in chrome://resources/.
+    if (url.starts_with("chrome://resources/"))
+      return true;
+
+    // We allow chrome://*/crisper.js.
+    if (url.ends_with("/crisper.js"))
+      return true;
+
+    return false;
+  }
+
+  // If the whitelist is null, then don't check it.
+  if (!first_party_extension_ids)
+    return false;
+
+  // Whitelist URLs of the format chrome-extension://<first-party-id>/*.js
+  if (!url.starts_with("chrome-extension://"))
+    return false;
+
+  // These must end with a .js extension.
+  if (!url.ends_with(".js"))
+    return false;
+
+  int i = 0;
+  const char* test_id = first_party_extension_ids[i];
+  const re2::StringPiece url_sub =
+      url.substr(sizeof("chrome-extension://") - 1);
+  while (test_id) {
+    if (url_sub.starts_with(test_id))
+      return true;
+    test_id = first_party_extension_ids[++i];
+  }
+  return false;
+}
+
 std::string AnonymizerTool::AnonymizeCustomPatternWithoutContext(
     const std::string& input,
     const CustomPatternWithoutContext& pattern,
@@ -418,10 +538,15 @@ std::string AnonymizerTool::AnonymizeCustomPatternWithoutContext(
   re2::StringPiece skipped;
   re2::StringPiece matched_id;
   while (FindAndConsumeAndGetSkipped(&text, *re, &skipped, &matched_id)) {
+    if (IsUrlWhitelisted(matched_id, first_party_extension_ids_)) {
+      skipped.AppendToString(&result);
+      matched_id.AppendToString(&result);
+      continue;
+    }
     std::string matched_id_as_string = matched_id.as_string();
     std::string replacement_id = (*identifier_space)[matched_id_as_string];
     if (replacement_id.empty()) {
-      replacement_id = MaybeScrubIPv4Address(matched_id_as_string);
+      replacement_id = MaybeScrubIPAddress(matched_id_as_string);
       if (replacement_id != matched_id_as_string) {
         // The weird Uint64toString trick is because Windows does not like
         // to deal with %zu and a size_t in printf, nor does it support %llu.
@@ -441,8 +566,10 @@ std::string AnonymizerTool::AnonymizeCustomPatternWithoutContext(
 }
 
 AnonymizerToolContainer::AnonymizerToolContainer(
-    scoped_refptr<base::SequencedTaskRunner> task_runner)
-    : anonymizer_(new AnonymizerTool), task_runner_(task_runner) {}
+    scoped_refptr<base::SequencedTaskRunner> task_runner,
+    const char* const* first_party_extension_ids)
+    : anonymizer_(new AnonymizerTool(first_party_extension_ids)),
+      task_runner_(task_runner) {}
 
 AnonymizerToolContainer::~AnonymizerToolContainer() {
   task_runner_->DeleteSoon(FROM_HERE, std::move(anonymizer_));
