@@ -9,7 +9,7 @@
 #include <vector>
 
 #include "base/i18n/case_conversion.h"
-#include "base/macros.h"
+#include "base/stl_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "components/history/core/browser/keyword_search_term.h"
 #include "components/url_formatter/url_formatter.h"
@@ -60,7 +60,7 @@ std::string URLDatabase::GURLToDatabaseURL(const GURL& gurl) {
 
 // Convenience to fill a URLRow. Must be in sync with the fields in
 // kURLRowFields.
-void URLDatabase::FillURLRow(sql::Statement& s, URLRow* i) {
+void URLDatabase::FillURLRow(const sql::Statement& s, URLRow* i) {
   DCHECK(i);
   i->set_id(s.ColumnInt64(0));
   i->set_url(GURL(s.ColumnString(1)));
@@ -87,18 +87,6 @@ bool URLDatabase::GetURLRow(URLID url_id, URLRow* info) {
   return false;
 }
 
-bool URLDatabase::GetAllTypedUrls(URLRows* urls) {
-  sql::Statement statement(GetDB().GetCachedStatement(SQL_FROM_HERE,
-      "SELECT" HISTORY_URL_ROW_FIELDS "FROM urls WHERE typed_count > 0"));
-
-  while (statement.Step()) {
-    URLRow info;
-    FillURLRow(statement, &info);
-    urls->push_back(info);
-  }
-  return true;
-}
-
 URLID URLDatabase::GetRowForURL(const GURL& url, URLRow* info) {
   sql::Statement statement(GetDB().GetCachedStatement(SQL_FROM_HERE,
       "SELECT" HISTORY_URL_ROW_FIELDS "FROM urls WHERE url=?"));
@@ -137,19 +125,19 @@ URLID URLDatabase::AddURLInternal(const URLRow& info, bool is_temporary) {
       " (url, title, visit_count, typed_count, "\
       "last_visit_time, hidden) "\
       "VALUES (?,?,?,?,?,?)"
-  const char* statement_name;
+  size_t statement_line;
   const char* statement_sql;
   if (is_temporary) {
-    statement_name = "AddURLTemporary";
+    statement_line = __LINE__;
     statement_sql = "INSERT INTO temp_urls" ADDURL_COMMON_SUFFIX;
   } else {
-    statement_name = "AddURL";
+    statement_line = __LINE__;
     statement_sql = "INSERT INTO urls" ADDURL_COMMON_SUFFIX;
   }
   #undef ADDURL_COMMON_SUFFIX
 
   sql::Statement statement(GetDB().GetCachedStatement(
-      sql::StatementID(statement_name), statement_sql));
+      sql::StatementID(__FILE__, statement_line), statement_sql));
   statement.BindString(0, GURLToDatabaseURL(info.url()));
   statement.BindString16(1, info.title());
   statement.BindInt(2, info.visit_count());
@@ -165,6 +153,27 @@ URLID URLDatabase::AddURLInternal(const URLRow& info, bool is_temporary) {
   return GetDB().GetLastInsertRowId();
 }
 
+bool URLDatabase::URLTableContainsAutoincrement() {
+  // sqlite_master has columns:
+  //   type - "index" or "table".
+  //   name - name of created element.
+  //   tbl_name - name of element, or target table in case of index.
+  //   rootpage - root page of the element in database file.
+  //   sql - SQL to create the element.
+  sql::Statement statement(GetDB().GetUniqueStatement(
+      "SELECT sql FROM sqlite_master WHERE type = 'table' AND name = 'urls'"));
+
+  // urls table does not exist.
+  if (!statement.Step())
+    return false;
+
+  std::string urls_schema = statement.ColumnString(0);
+  // We check if the whole schema contains "AUTOINCREMENT", since
+  // "AUTOINCREMENT" only can be used for "INTEGER PRIMARY KEY", so we assume no
+  // other columns could cantain "AUTOINCREMENT".
+  return urls_schema.find("AUTOINCREMENT") != std::string::npos;
+}
+
 bool URLDatabase::InsertOrUpdateURLRowByID(const URLRow& info) {
   // SQLite does not support INSERT OR UPDATE, however, it does have INSERT OR
   // REPLACE, which is feasible to use, because of the following.
@@ -227,9 +236,7 @@ bool URLDatabase::CommitTemporaryURLTable() {
 
   // Re-create the index over the now permanent URLs table -- this was not there
   // for the temporary table.
-  CreateMainURLIndex();
-
-  return true;
+  return CreateMainURLIndex();
 }
 
 bool URLDatabase::InitURLEnumeratorForEverything(URLEnumerator* enumerator) {
@@ -246,8 +253,9 @@ bool URLDatabase::InitURLEnumeratorForSignificant(URLEnumerator* enumerator) {
   DCHECK(!enumerator->initialized_);
   std::string sql("SELECT ");
   sql.append(kURLRowFields);
-  sql.append(" FROM urls WHERE last_visit_time >= ? OR visit_count >= ? OR "
-             "typed_count >= ?");
+  sql.append(
+      " FROM urls WHERE hidden = 0 AND "
+      "(last_visit_time >= ? OR visit_count >= ? OR typed_count >= ?)");
   sql.append(
       " ORDER BY typed_count DESC, last_visit_time DESC, visit_count "
       "DESC");
@@ -268,8 +276,9 @@ bool URLDatabase::AutocompleteForPrefix(const std::string& prefix,
   // as bookmarks is no longer part of the db we no longer include the order
   // by clause.
   results->clear();
+
   const char* sql;
-  int line;
+  size_t line;
   if (typed_only) {
     sql = "SELECT" HISTORY_URL_ROW_FIELDS "FROM urls "
         "WHERE url >= ? AND url < ? AND hidden = 0 AND typed_count > 0 "
@@ -306,19 +315,22 @@ bool URLDatabase::AutocompleteForPrefix(const std::string& prefix,
   return !results->empty();
 }
 
-bool URLDatabase::IsTypedHost(const std::string& host) {
+bool URLDatabase::IsTypedHost(const std::string& host, std::string* scheme) {
   const char* schemes[] = {
     url::kHttpScheme,
     url::kHttpsScheme,
     url::kFtpScheme
   };
   URLRows dummy;
-  for (size_t i = 0; i < arraysize(schemes); ++i) {
+  for (size_t i = 0; i < base::size(schemes); ++i) {
     std::string scheme_and_host(schemes[i]);
     scheme_and_host += url::kStandardSchemeSeparator + host;
     if (AutocompleteForPrefix(scheme_and_host + '/', 1, true, &dummy) ||
-        AutocompleteForPrefix(scheme_and_host + ':', 1, true, &dummy))
+        AutocompleteForPrefix(scheme_and_host + ':', 1, true, &dummy)) {
+      if (scheme != nullptr)
+        *scheme = schemes[i];
       return true;
+    }
   }
   return false;
 }
@@ -639,9 +651,7 @@ bool URLDatabase::RecreateURLTableWithAllContents() {
   }
 
   // Rename/commit the tmp table.
-  CommitTemporaryURLTable();
-
-  return true;
+  return CommitTemporaryURLTable();
 }
 
 const int kLowQualityMatchTypedLimit = 1;
