@@ -49,11 +49,6 @@ void IOSImageDataFetcherWrapper::FetchImageDataWebpDecoded(
       referrer_policy, NO_TRAFFIC_ANNOTATION_YET, send_cookies);
 }
 
-void IOSImageDataFetcherWrapper::SetDataUseServiceName(
-    DataUseServiceName data_use_service_name) {
-  image_data_fetcher_.SetDataUseServiceName(data_use_service_name);
-}
-
 ImageDataFetcherCallback
 IOSImageDataFetcherWrapper::CallbackForImageDataFetcher(
     ImageDataFetcherBlock callback) {
