@@ -4,17 +4,18 @@
 
 #include "components/ntp_snippets/remote/cached_image_fetcher.h"
 
+#include <map>
 #include <memory>
+#include <utility>
 
 #include "base/bind.h"
 #include "base/callback.h"
-#include "base/files/file_path.h"
-#include "base/files/scoped_temp_dir.h"
 #include "base/test/mock_callback.h"
 #include "base/test/scoped_task_environment.h"
-#include "components/image_fetcher/core/image_decoder.h"
+#include "components/image_fetcher/core/fake_image_decoder.h"
 #include "components/image_fetcher/core/image_fetcher.h"
 #include "components/image_fetcher/core/image_fetcher_impl.h"
+#include "components/leveldb_proto/testing/fake_db.h"
 #include "components/ntp_snippets/remote/proto/ntp_snippets.pb.h"
 #include "components/ntp_snippets/remote/remote_suggestions_database.h"
 #include "components/prefs/pref_registry_simple.h"
@@ -26,6 +27,7 @@
 #include "ui/gfx/image/image.h"
 #include "ui/gfx/image/image_unittest_util.h"
 
+using leveldb_proto::test::FakeDB;
 using testing::_;
 using testing::Eq;
 using testing::Property;
@@ -41,28 +43,6 @@ const ContentSuggestion::ID kSuggestionID(
     Category::FromKnownCategory(KnownCategories::ARTICLES),
     kSnippetID);
 
-// Always decodes a valid image for all non-empty input.
-class FakeImageDecoder : public image_fetcher::ImageDecoder {
- public:
-  void DecodeImage(
-      const std::string& image_data,
-      const gfx::Size& desired_image_frame_size,
-      const image_fetcher::ImageDecodedCallback& callback) override {
-    ASSERT_TRUE(enabled_);
-    gfx::Image image;
-    if (!image_data.empty()) {
-      ASSERT_EQ(kImageData, image_data);
-      image = gfx::test::CreateImage();
-    }
-    base::SequencedTaskRunnerHandle::Get()->PostTask(
-        FROM_HERE, base::BindRepeating(callback, image));
-  }
-  void SetEnabled(bool enabled) { enabled_ = enabled; }
-
- private:
-  bool enabled_ = true;
-};
-
 enum class TestType {
   kImageCallback,
   kImageDataCallback,
@@ -72,35 +52,42 @@ enum class TestType {
 
 // This test is parameterized to run all tests in the three configurations:
 // both callbacks used, only image_callback used, only image_data_callback used.
-class CachedImageFetcherTest : public testing::TestWithParam<TestType> {
+class NtpSnippetsCachedImageFetcherTest
+    : public testing::TestWithParam<TestType> {
  public:
-  CachedImageFetcherTest() {
-    EXPECT_TRUE(database_dir_.CreateUniqueTempDir());
-
+  NtpSnippetsCachedImageFetcherTest() {
     RequestThrottler::RegisterProfilePrefs(pref_service_.registry());
-    database_ =
-        std::make_unique<RemoteSuggestionsDatabase>(database_dir_.GetPath());
+
+    // Setup RemoteSuggestionsDatabase with fake ProtoDBs.
+    auto suggestion_db =
+        std::make_unique<FakeDB<SnippetProto>>(&suggestion_db_storage_);
+    auto image_db =
+        std::make_unique<FakeDB<SnippetImageProto>>(&image_db_storage_);
+    suggestion_db_ = suggestion_db.get();
+    image_db_ = image_db.get();
+    database_ = std::make_unique<RemoteSuggestionsDatabase>(
+        std::move(suggestion_db), std::move(image_db));
+    suggestion_db_->InitStatusCallback(leveldb_proto::Enums::InitStatus::kOK);
+    image_db_->InitStatusCallback(leveldb_proto::Enums::InitStatus::kOK);
 
     shared_factory_ =
         base::MakeRefCounted<network::WeakWrapperSharedURLLoaderFactory>(
             &test_url_loader_factory_);
 
-    auto decoder = std::make_unique<FakeImageDecoder>();
+    auto decoder = std::make_unique<image_fetcher::FakeImageDecoder>();
     fake_image_decoder_ = decoder.get();
+    auto image_fetcher = std::make_unique<image_fetcher::ImageFetcherImpl>(
+        std::move(decoder), shared_factory_);
+
     cached_image_fetcher_ = std::make_unique<ntp_snippets::CachedImageFetcher>(
-        std::make_unique<image_fetcher::ImageFetcherImpl>(std::move(decoder),
-                                                          shared_factory_),
-        &pref_service_, database_.get());
-    RunUntilIdle();
+        std::move(image_fetcher), &pref_service_, database_.get());
+
     EXPECT_TRUE(database_->IsInitialized());
   }
 
-  ~CachedImageFetcherTest() override {
+  ~NtpSnippetsCachedImageFetcherTest() override {
     cached_image_fetcher_.reset();
     database_.reset();
-    // We need to run until idle after deleting the database, because
-    // ProtoDatabase deletes the actual LevelDB asynchronously.
-    RunUntilIdle();
   }
 
   void Fetch(std::string expected_image_data, bool expect_image) {
@@ -115,13 +102,14 @@ class CachedImageFetcherTest : public testing::TestWithParam<TestType> {
         cached_image_fetcher()->FetchSuggestionImage(
             kSuggestionID, GURL(kImageURL), ImageDataFetchedCallback(),
             image_callback.Get());
-
+        image_db_->GetCallback(true);
       } break;
       case TestType::kImageDataCallback: {
         EXPECT_CALL(image_data_callback, Run(expected_image_data));
         cached_image_fetcher()->FetchSuggestionImage(
             kSuggestionID, GURL(kImageURL), image_data_callback.Get(),
             ImageFetchedCallback());
+        image_db_->GetCallback(true);
       } break;
       case TestType::kBothCallbacks: {
         EXPECT_CALL(image_data_callback, Run(expected_image_data));
@@ -130,47 +118,56 @@ class CachedImageFetcherTest : public testing::TestWithParam<TestType> {
         cached_image_fetcher()->FetchSuggestionImage(
             kSuggestionID, GURL(kImageURL), image_data_callback.Get(),
             image_callback.Get());
+        image_db_->GetCallback(true);
       } break;
     }
-    RunUntilIdle();
+    scoped_task_environment_.RunUntilIdle();
   }
 
-  void RunUntilIdle() { scoped_task_environment_.RunUntilIdle(); }
-
   RemoteSuggestionsDatabase* database() { return database_.get(); }
-  FakeImageDecoder* fake_image_decoder() { return fake_image_decoder_; }
+  image_fetcher::FakeImageDecoder* fake_image_decoder() {
+    return fake_image_decoder_;
+  }
   network::TestURLLoaderFactory* test_url_loader_factory() {
     return &test_url_loader_factory_;
   }
   CachedImageFetcher* cached_image_fetcher() {
     return cached_image_fetcher_.get();
   }
+  FakeDB<SnippetProto>* suggestion_db() { return suggestion_db_; }
+  FakeDB<SnippetImageProto>* image_db() { return image_db_; }
 
  private:
   network::TestURLLoaderFactory test_url_loader_factory_;
   scoped_refptr<network::SharedURLLoaderFactory> shared_factory_;
   std::unique_ptr<CachedImageFetcher> cached_image_fetcher_;
+  image_fetcher::FakeImageDecoder* fake_image_decoder_;
+
   std::unique_ptr<RemoteSuggestionsDatabase> database_;
-  base::ScopedTempDir database_dir_;
-  FakeImageDecoder* fake_image_decoder_;
+  std::map<std::string, SnippetProto> suggestion_db_storage_;
+  std::map<std::string, SnippetImageProto> image_db_storage_;
+
+  // Owned by |database_|.
+  FakeDB<SnippetProto>* suggestion_db_;
+  FakeDB<SnippetImageProto>* image_db_;
 
   TestingPrefServiceSimple pref_service_;
   base::test::ScopedTaskEnvironment scoped_task_environment_;
 
-  DISALLOW_COPY_AND_ASSIGN(CachedImageFetcherTest);
+  DISALLOW_COPY_AND_ASSIGN(NtpSnippetsCachedImageFetcherTest);
 };
 
-TEST_P(CachedImageFetcherTest, FetchImageFromCache) {
+TEST_P(NtpSnippetsCachedImageFetcherTest, FetchImageFromCache) {
   // Save the image in the database.
   database()->SaveImage(kSnippetID, kImageData);
-  RunUntilIdle();
+  image_db()->UpdateCallback(true);
 
   // Do not provide any URL responses and expect that the image is fetched (from
   // cache).
   Fetch(kImageData, true);
 }
 
-TEST_P(CachedImageFetcherTest, FetchImagePopulatesCache) {
+TEST_P(NtpSnippetsCachedImageFetcherTest, FetchImagePopulatesCache) {
   // Expect the image to be fetched by URL.
   {
     test_url_loader_factory()->AddResponse(kImageURL, kImageData);
@@ -183,7 +180,7 @@ TEST_P(CachedImageFetcherTest, FetchImagePopulatesCache) {
   }
 }
 
-TEST_P(CachedImageFetcherTest, FetchNonExistingImage) {
+TEST_P(NtpSnippetsCachedImageFetcherTest, FetchNonExistingImage) {
   const std::string kErrorResponse = "error-response";
   test_url_loader_factory()->AddResponse(kImageURL, kErrorResponse,
                                          net::HTTP_NOT_FOUND);
@@ -191,10 +188,10 @@ TEST_P(CachedImageFetcherTest, FetchNonExistingImage) {
   Fetch("", false);
 }
 
-INSTANTIATE_TEST_CASE_P(,
-                        CachedImageFetcherTest,
-                        testing::Values(TestType::kImageCallback,
-                                        TestType::kImageDataCallback,
-                                        TestType::kBothCallbacks));
+INSTANTIATE_TEST_SUITE_P(,
+                         NtpSnippetsCachedImageFetcherTest,
+                         testing::Values(TestType::kImageCallback,
+                                         TestType::kImageDataCallback,
+                                         TestType::kBothCallbacks));
 
 }  // namespace ntp_snippets
