@@ -4,7 +4,10 @@
 
 #include "components/offline_pages/content/background_loader/background_loader_contents.h"
 
+#include <utility>
+
 #include "content/public/browser/web_contents.h"
+#include "third_party/blink/public/mojom/mediastream/media_stream.mojom-shared.h"
 
 namespace background_loader {
 
@@ -64,12 +67,12 @@ bool BackgroundLoaderContents::ShouldFocusPageAfterCrash() {
 void BackgroundLoaderContents::CanDownload(
     const GURL& url,
     const std::string& request_method,
-    const base::Callback<void(bool)>& callback) {
+    base::OnceCallback<void(bool)> callback) {
   if (delegate_) {
-    delegate_->CanDownload(callback);
+    delegate_->CanDownload(std::move(callback));
   } else {
     // Do not download anything if there's no delegate.
-    callback.Run(false);
+    std::move(callback).Run(false);
   }
 }
 
@@ -117,35 +120,22 @@ void BackgroundLoaderContents::RequestMediaAccessPermission(
   // No permissions granted, act as if dismissed.
   std::move(callback).Run(
       blink::MediaStreamDevices(),
-      blink::MediaStreamRequestResult::MEDIA_DEVICE_PERMISSION_DISMISSED,
+      blink::mojom::MediaStreamRequestResult::PERMISSION_DISMISSED,
       std::unique_ptr<content::MediaStreamUI>());
 }
 
 bool BackgroundLoaderContents::CheckMediaAccessPermission(
     content::RenderFrameHost* render_frame_host,
     const GURL& security_origin,
-    blink::MediaStreamType type) {
+    blink::mojom::MediaStreamType type) {
   return false;  // No permissions granted.
 }
 
 void BackgroundLoaderContents::AdjustPreviewsStateForNavigation(
     content::WebContents* web_contents,
     content::PreviewsState* previews_state) {
-  DCHECK(previews_state);
-
-  // If previews are already disabled, do nothing.
-  if (*previews_state == content::PREVIEWS_OFF ||
-      *previews_state == content::PREVIEWS_NO_TRANSFORM) {
-    return;
-  }
-
-  if (*previews_state == content::PREVIEWS_UNSPECIFIED) {
-    *previews_state = content::PARTIAL_CONTENT_SAFE_PREVIEWS;
-  } else {
-    *previews_state &= content::PARTIAL_CONTENT_SAFE_PREVIEWS;
     if (*previews_state == 0)
       *previews_state = content::PREVIEWS_OFF;
-  }
 }
 
 bool BackgroundLoaderContents::ShouldAllowLazyLoad() {
