@@ -6,6 +6,7 @@
 
 #include <stddef.h>
 
+#include <algorithm>
 #include <map>
 #include <memory>
 #include <utility>
@@ -18,16 +19,16 @@
 #include "build/build_config.h"
 #include "components/autofill/core/browser/autofill_field.h"
 #include "components/autofill/core/browser/form_structure.h"
+#include "components/autofill/core/browser/logging/log_manager.h"
 #include "components/autofill/core/common/form_data_predictions.h"
 #include "components/autofill/core/common/password_form_field_prediction_map.h"
+#include "components/autofill/core/common/save_password_progress_logger.h"
 #include "components/password_manager/core/browser/browser_save_password_progress_logger.h"
 #include "components/password_manager/core/browser/form_saver_impl.h"
-#include "components/password_manager/core/browser/keychain_migration_status_mac.h"
-#include "components/password_manager/core/browser/log_manager.h"
 #include "components/password_manager/core/browser/new_password_form_manager.h"
 #include "components/password_manager/core/browser/password_autofill_manager.h"
 #include "components/password_manager/core/browser/password_form_manager.h"
-#include "components/password_manager/core/browser/password_generation_manager.h"
+#include "components/password_manager/core/browser/password_generation_frame_helper.h"
 #include "components/password_manager/core/browser/password_manager_client.h"
 #include "components/password_manager/core/browser/password_manager_driver.h"
 #include "components/password_manager/core/browser/password_manager_metrics_util.h"
@@ -45,6 +46,7 @@
 
 using autofill::FormData;
 using autofill::PasswordForm;
+using autofill::mojom::PasswordFormFieldPredictionType;
 #if defined(SYNC_PASSWORD_REUSE_DETECTION_ENABLED)
 using password_manager::metrics_util::SyncPasswordHashChange;
 #endif  // SYNC_PASSWORD_REUSE_DETECTION_ENABLED
@@ -57,7 +59,7 @@ const char kSpdyProxyRealm[] = "/SpdyProxy";
 
 // Shorten the name to spare line breaks. The code provides enough context
 // already.
-typedef autofill::SavePasswordProgressLogger Logger;
+using Logger = autofill::SavePasswordProgressLogger;
 
 bool URLsEqualUpToScheme(const GURL& a, const GURL& b) {
   return (a.GetContent() == b.GetContent());
@@ -111,16 +113,6 @@ bool IsPasswordFormReappeared(const PasswordForm& observed_form,
   return false;
 }
 
-// Helper UMA reporting function for differences in URLs during form submission.
-void RecordWhetherTargetDomainDiffers(const GURL& src, const GURL& target) {
-  bool target_domain_differs =
-      !net::registry_controlled_domains::SameDomainOrHost(
-          src, target,
-          net::registry_controlled_domains::INCLUDE_PRIVATE_REGISTRIES);
-  UMA_HISTOGRAM_BOOLEAN("PasswordManager.SubmitNavigatesToDifferentDomain",
-                        target_domain_differs);
-}
-
 bool IsSignupForm(const PasswordForm& form) {
   return !form.new_password_element.empty() && form.password_element.empty();
 }
@@ -132,20 +124,20 @@ bool IsSignupForm(const PasswordForm& form) {
 bool ServerPredictionsToPasswordFormPrediction(
     std::vector<autofill::AutofillQueryResponseContents::Field::FieldPrediction>
         server_field_predictions,
-    autofill::PasswordFormFieldPredictionType* type) {
+    PasswordFormFieldPredictionType* type) {
   for (auto const& server_field_prediction : server_field_predictions) {
     switch (server_field_prediction.type()) {
       case autofill::USERNAME:
       case autofill::USERNAME_AND_EMAIL_ADDRESS:
-        *type = autofill::PREDICTION_USERNAME;
+        *type = PasswordFormFieldPredictionType::kUsername;
         return true;
 
       case autofill::PASSWORD:
-        *type = autofill::PREDICTION_CURRENT_PASSWORD;
+        *type = PasswordFormFieldPredictionType::kCurrentPassword;
         return true;
 
       case autofill::ACCOUNT_CREATION_PASSWORD:
-        *type = autofill::PREDICTION_NEW_PASSWORD;
+        *type = PasswordFormFieldPredictionType::kNewPassword;
         return true;
 
       default:
@@ -160,7 +152,8 @@ bool ServerPredictionsToPasswordFormPrediction(
 bool IsPredictedTypeNotPasswordPrediction(
     autofill::ServerFieldType field_type) {
   return field_type == autofill::CREDIT_CARD_NUMBER ||
-         field_type == autofill::CREDIT_CARD_VERIFICATION_CODE;
+         field_type == autofill::CREDIT_CARD_VERIFICATION_CODE ||
+         field_type == autofill::NOT_PASSWORD;
 }
 
 bool AreAllFieldsEmpty(const PasswordForm& form) {
@@ -327,11 +320,44 @@ void RecordParsingOnSavingDifference(
   metrics_recorder->RecordParsingOnSavingDifference(result);
 }
 
+bool IsNewFormParsingForFillingEnabled() {
+#if defined(OS_IOS)
+  return true;
+#else
+  return base::FeatureList::IsEnabled(features::kNewPasswordFormParsing);
+#endif
+}
+
+bool IsNewFormParsingForSavingEnabled() {
+#if defined(OS_IOS)
+  return true;
+#else
+  return base::FeatureList::IsEnabled(
+             features::kNewPasswordFormParsingForSaving) &&
+         base::FeatureList::IsEnabled(features::kNewPasswordFormParsing);
+#endif
+}
+
+// Returns true if it is turned off using PasswordFormManager in
+// PasswordManager.
+bool IsOnlyNewParserEnabled() {
+#if defined(OS_IOS)
+  return true;
+#else
+  return base::FeatureList::IsEnabled(
+             features::kNewPasswordFormParsingForSaving) &&
+         base::FeatureList::IsEnabled(features::kNewPasswordFormParsing) &&
+         base::FeatureList::IsEnabled(features::kOnlyNewParser);
+#endif
+}
+
 }  // namespace
 
 // static
 void PasswordManager::RegisterProfilePrefs(
     user_prefs::PrefRegistrySyncable* registry) {
+  registry->RegisterBooleanPref(prefs::kBlacklistedCredentialsNormalized,
+                                false);
   registry->RegisterBooleanPref(
       prefs::kCredentialsEnableService, true,
       user_prefs::PrefRegistrySyncable::SYNCABLE_PRIORITY_PREF);
@@ -350,9 +376,8 @@ void PasswordManager::RegisterProfilePrefs(
                                0.0);
 
 #if defined(OS_MACOSX)
-  registry->RegisterIntegerPref(
-      prefs::kKeychainMigrationStatus,
-      static_cast<int>(MigrationStatus::MIGRATED_DELETED));
+  registry->RegisterIntegerPref(prefs::kKeychainMigrationStatus,
+                                4 /* MIGRATED_DELETED */);
 #endif
   registry->RegisterListPref(prefs::kPasswordHashDataList,
                              PrefRegistry::NO_REGISTRATION_FLAGS);
@@ -371,23 +396,11 @@ void PasswordManager::RegisterLocalPrefs(PrefRegistrySimple* registry) {
 }
 
 PasswordManager::PasswordManager(PasswordManagerClient* client)
-    : client_(client),
-      is_new_form_parsing_for_saving_enabled_(
-          base::FeatureList::IsEnabled(
-              features::kNewPasswordFormParsingForSaving) &&
-          base::FeatureList::IsEnabled(features::kNewPasswordFormParsing)),
-      is_only_new_parser_enabled_(
-          base::FeatureList::IsEnabled(
-              features::kNewPasswordFormParsingForSaving) &&
-          base::FeatureList::IsEnabled(features::kNewPasswordFormParsing) &&
-          base::FeatureList::IsEnabled(features::kOnlyNewParser)) {
+    : client_(client) {
   DCHECK(client_);
 }
 
-PasswordManager::~PasswordManager() {
-  for (LoginModelObserver& observer : observers_)
-    observer.OnLoginModelDestroying();
-}
+PasswordManager::~PasswordManager() = default;
 
 void PasswordManager::GenerationAvailableForForm(const PasswordForm& form) {
   DCHECK(client_->IsSavingAndFillingEnabled(form.origin));
@@ -399,18 +412,34 @@ void PasswordManager::GenerationAvailableForForm(const PasswordForm& form) {
   }
 }
 
+void PasswordManager::OnGeneratedPasswordAccepted(
+    PasswordManagerDriver* driver,
+    const autofill::FormData& form_data,
+    uint32_t generation_element_id,
+    const base::string16& password) {
+  if (IsNewFormParsingForSavingEnabled()) {
+    NewPasswordFormManager* manager = GetMatchedManager(driver, form_data);
+    if (manager) {
+      manager->OnGeneratedPasswordAccepted(form_data, generation_element_id,
+                                           password);
+    } else {
+      // OnPresaveGeneratedPassword records the histogram in all other cases.
+      UMA_HISTOGRAM_BOOLEAN("PasswordManager.GeneratedFormHasNoFormManager",
+                            true);
+    }
+  } else {
+    driver->GeneratedPasswordAccepted(password);
+  }
+}
+
 void PasswordManager::OnPresaveGeneratedPassword(PasswordManagerDriver* driver,
                                                  const PasswordForm& form) {
   DCHECK(client_->IsSavingAndFillingEnabled(form.origin));
   PasswordFormManagerInterface* form_manager = GetMatchedManager(driver, form);
-  if (form_manager) {
+  UMA_HISTOGRAM_BOOLEAN("PasswordManager.GeneratedFormHasNoFormManager",
+                        !form_manager);
+  if (form_manager)
     form_manager->PresaveGeneratedPassword(form);
-    UMA_HISTOGRAM_BOOLEAN("PasswordManager.GeneratedFormHasNoFormManager",
-                          false);
-    return;
-  }
-
-  UMA_HISTOGRAM_BOOLEAN("PasswordManager.GeneratedFormHasNoFormManager", true);
 }
 
 void PasswordManager::OnPasswordNoLongerGenerated(PasswordManagerDriver* driver,
@@ -439,7 +468,7 @@ void PasswordManager::SetGenerationElementAndReasonForForm(
   // If there is no corresponding PasswordFormManager, we create one. This is
   // not the common case, and should only happen when there is a bug in our
   // ability to detect forms. No matched |NewPasswordFormManager| is unlikely.
-  if (!is_new_form_parsing_for_saving_enabled_) {
+  if (!IsNewFormParsingForSavingEnabled()) {
     auto manager = std::make_unique<PasswordFormManager>(
         this, client_, driver->AsWeakPtr(), form,
         std::make_unique<FormSaverImpl>(client_->GetPasswordStore()), nullptr);
@@ -450,7 +479,10 @@ void PasswordManager::SetGenerationElementAndReasonForForm(
 
 void PasswordManager::ProvisionallySavePassword(
     const PasswordForm& form,
-    const password_manager::PasswordManagerDriver* driver) {
+    const PasswordManagerDriver* driver) {
+  if (IsOnlyNewParserEnabled())
+    return;
+
   // If the form was declined by some heuristics, don't show automatic bubble
   // for it, only fallback saving should be available.
   if (form.only_for_fallback)
@@ -479,10 +511,7 @@ void PasswordManager::ProvisionallySavePassword(
     return;
   }
 
-  bool should_block = ShouldBlockPasswordForSameOriginButDifferentScheme(form);
-  metrics_util::LogShouldBlockPasswordForSameOriginButDifferentScheme(
-      should_block);
-  if (should_block) {
+  if (ShouldBlockPasswordForSameOriginButDifferentScheme(form.origin)) {
     RecordProvisionalSaveFailure(
         PasswordManagerMetricsRecorder::SAVING_ON_HTTP_AFTER_HTTPS, form.origin,
         logger.get());
@@ -517,42 +546,30 @@ void PasswordManager::ProvisionallySavePassword(
   // cached and report the difference through UMA.
   main_frame_url_ = client_->GetMainFrameURL();
 
-  // Report SubmittedFormFrame metric.
-  if (driver) {
-    metrics_util::SubmittedFormFrame frame;
-    if (driver->IsMainFrame()) {
-      frame = metrics_util::SubmittedFormFrame::MAIN_FRAME;
-    } else if (form.origin == main_frame_url_) {
-      frame =
-          metrics_util::SubmittedFormFrame::IFRAME_WITH_SAME_URL_AS_MAIN_FRAME;
-    } else {
-      GURL::Replacements rep;
-      rep.SetPathStr("");
-      std::string main_frame_signon_realm =
-          main_frame_url_.ReplaceComponents(rep).spec();
-      frame =
-          (main_frame_signon_realm == form.signon_realm)
-              ? metrics_util::SubmittedFormFrame::
-                    IFRAME_WITH_DIFFERENT_URL_SAME_SIGNON_REALM_AS_MAIN_FRAME
-              : metrics_util::SubmittedFormFrame::
-                    IFRAME_WITH_DIFFERENT_SIGNON_REALM;
-    }
-    metrics_util::LogSubmittedFormFrame(frame);
-  }
+  ReportSubmittedFormFrameMetric(driver, form);
 }
 
 void PasswordManager::DidNavigateMainFrame(bool form_may_be_submitted) {
   pending_login_managers_.clear();
 
-  if (form_may_be_submitted) {
-    for (std::unique_ptr<NewPasswordFormManager>& manager : form_managers_) {
-      if (manager->is_submitted()) {
-        owned_submitted_form_manager_ = std::move(manager);
-        break;
-      }
+  if (client_->IsNewTabPage()) {
+    // On a successful Chrome sign-in the page navigates to the new tab page
+    // (ntp). OnPasswordFormsRendered is not called on ntp. That is why the
+    // standard flow for saving hash does not work. Save a password hash now
+    // since a navigation to ntp is the sign of successful sign-in.
+    PasswordFormManagerInterface* manager = GetSubmittedManager();
+    if (manager && manager->GetSubmittedForm()
+                       ->form_data.is_gaia_with_skip_save_password_form) {
+      MaybeSavePasswordHash(*manager);
     }
   }
 
+  for (std::unique_ptr<NewPasswordFormManager>& manager : form_managers_) {
+    if (form_may_be_submitted && manager->is_submitted()) {
+      owned_submitted_form_manager_ = std::move(manager);
+      break;
+    }
+  }
   form_managers_.clear();
   predictions_.clear();
   store_password_called_ = false;
@@ -608,36 +625,19 @@ void PasswordManager::DropFormManagers() {
 }
 
 bool PasswordManager::IsPasswordFieldDetectedOnPage() {
-  return !pending_login_managers_.empty();
-}
-
-void PasswordManager::AddObserverAndDeliverCredentials(
-    LoginModelObserver* observer,
-    const PasswordForm& observed_form) {
-  observers_.AddObserver(observer);
-
-  // The observers are responsible for filtering notifications by the observer
-  // signon_realm. Each notification is broadcasted to every observer.
-  observer->set_signon_realm(observed_form.signon_realm);
-
-  std::vector<PasswordForm> observed_forms;
-  observed_forms.push_back(observed_form);
-  OnPasswordFormsParsed(nullptr, observed_forms);
-}
-
-void PasswordManager::RemoveObserver(LoginModelObserver* observer) {
-  observers_.RemoveObserver(observer);
+  return !pending_login_managers_.empty() || !form_managers_.empty();
 }
 
 void PasswordManager::OnPasswordFormSubmitted(
     password_manager::PasswordManagerDriver* driver,
     const PasswordForm& password_form) {
-  if (is_new_form_parsing_for_saving_enabled_)
-    ProvisionallySaveForm(password_form.form_data, driver);
+  if (IsNewFormParsingForSavingEnabled())
+    ProvisionallySaveForm(password_form.form_data, driver, false);
 
   ProvisionallySavePassword(password_form, driver);
 }
 
+
 void PasswordManager::OnPasswordFormSubmittedNoChecks(
     password_manager::PasswordManagerDriver* driver,
     const PasswordForm& password_form) {
@@ -656,8 +656,9 @@ void PasswordManager::OnPasswordFormSubmittedNoChecks(
     return;
   }
 
-  if (is_new_form_parsing_for_saving_enabled_)
-    ProvisionallySaveForm(password_form.form_data, driver);
+  if (IsNewFormParsingForSavingEnabled()) {
+    ProvisionallySaveForm(password_form.form_data, driver, false);
+  }
 
   ProvisionallySavePassword(password_form, driver);
 
@@ -665,19 +666,28 @@ void PasswordManager::OnPasswordFormSubmittedNoChecks(
     OnLoginSuccessful();
 }
 
+void PasswordManager::OnUserModifiedNonPasswordField(
+    password_manager::PasswordManagerDriver* driver,
+    int32_t renderer_id,
+    const base::string16& value) {
+  // TODO(https://crbug.com/959776): Implemented processing |value| as possible
+  // username for username first flow.
+}
+
 void PasswordManager::ShowManualFallbackForSaving(
     password_manager::PasswordManagerDriver* driver,
     const PasswordForm& password_form) {
   if (!client_->GetPasswordStore()->IsAbleToSavePasswords() ||
       !client_->IsSavingAndFillingEnabled(password_form.origin) ||
-      ShouldBlockPasswordForSameOriginButDifferentScheme(password_form) ||
+      ShouldBlockPasswordForSameOriginButDifferentScheme(
+          password_form.origin) ||
       !client_->GetStoreResultFilter()->ShouldSave(password_form))
     return;
 
-  std::unique_ptr<PasswordFormManagerInterface> manager = nullptr;
-  if (is_new_form_parsing_for_saving_enabled_) {
+  std::unique_ptr<PasswordFormManagerInterface> manager;
+  if (IsNewFormParsingForSavingEnabled()) {
     NewPasswordFormManager* matched_manager =
-        ProvisionallySaveForm(password_form.form_data, driver);
+        ProvisionallySaveForm(password_form.form_data, driver, true);
     manager = matched_manager ? matched_manager->Clone() : nullptr;
   } else {
     manager = FindAndCloneMatchedPasswordFormManager(
@@ -714,8 +724,8 @@ void PasswordManager::OnPasswordFormsParsed(
     const std::vector<PasswordForm>& forms) {
   CreatePendingLoginManagers(driver, forms);
 
-  PasswordGenerationManager* password_generation_manager =
-      driver ? driver->GetPasswordGenerationManager() : nullptr;
+  PasswordGenerationFrameHelper* password_generation_manager =
+      driver ? driver->GetPasswordGenerationHelper() : nullptr;
   if (password_generation_manager) {
     password_generation_manager->PrefetchSpec(
         client_->GetLastCommittedEntryURL().GetOrigin());
@@ -732,9 +742,8 @@ void PasswordManager::CreatePendingLoginManagers(
     logger->LogMessage(Logger::STRING_CREATE_LOGIN_MANAGERS_METHOD);
   }
 
-  if (base::FeatureList::IsEnabled(features::kNewPasswordFormParsing)) {
+  if (IsNewFormParsingForFillingEnabled())
     CreateFormManagers(driver, forms);
-  }
 
   // Record whether or not this top-level URL has at least one password field.
   client_->AnnotateNavigationEntry(!forms.empty());
@@ -768,14 +777,7 @@ void PasswordManager::CreatePendingLoginManagers(
                       pending_login_managers_.size());
   }
 
-  bool html_scheme =
-      !forms.empty() && forms[0].scheme == PasswordForm::SCHEME_HTML;
-
-  // Create PasswordFormManager for non-html scheme even if
-  // |is_only_new_parser_enabled_|.
-  // TODO(https://crbug.com/915161) Implement support of non-html schemes in
-  // NewPasswordFormManager.
-  if (html_scheme && is_only_new_parser_enabled_)
+  if (IsOnlyNewParserEnabled())
     return;
 
   for (const PasswordForm& form : forms) {
@@ -801,18 +803,6 @@ void PasswordManager::CreatePendingLoginManagers(
     if (old_manager_found)
       continue;  // The current form is already managed.
 
-    UMA_HISTOGRAM_BOOLEAN("PasswordManager.EmptyUsernames.ParsedUsernameField",
-                          form.username_element.empty());
-
-    // Out of the forms not containing a username field, determine how many
-    // are password change forms.
-    if (form.username_element.empty()) {
-      UMA_HISTOGRAM_BOOLEAN(
-          "PasswordManager.EmptyUsernames."
-          "FormWithoutUsernameFieldIsPasswordChangeForm",
-          form.new_password_element.empty());
-    }
-
     if (logger)
       logger->LogFormSignatures(Logger::STRING_ADDING_SIGNATURE, form);
     auto manager = std::make_unique<PasswordFormManager>(
@@ -840,10 +830,11 @@ void PasswordManager::CreateFormManagers(
     // TODO(https://crbug.com/831123): Implement inside NewPasswordFormManger
     // not-filling Gaia forms that should be ignored instead of non-creating
     // NewPasswordFormManger instance.
-    if (form.is_gaia_with_skip_save_password_form)
+    if (form.form_data.is_gaia_with_skip_save_password_form)
       continue;
     if (!client_->IsFillingEnabled(form.origin))
       continue;
+
     NewPasswordFormManager* manager =
         FindMatchedManager(form.form_data, form_managers_, driver);
 
@@ -880,15 +871,17 @@ NewPasswordFormManager* PasswordManager::CreateFormManager(
 
 NewPasswordFormManager* PasswordManager::ProvisionallySaveForm(
     const FormData& submitted_form,
-    PasswordManagerDriver* driver) {
+    PasswordManagerDriver* driver,
+    bool is_manual_fallback) {
   std::unique_ptr<BrowserSavePasswordProgressLogger> logger;
   if (password_manager_util::IsLoggingActive(client_)) {
     logger.reset(
         new BrowserSavePasswordProgressLogger(client_->GetLogManager()));
+    logger->LogMessage(Logger::STRING_PROVISIONALLY_SAVE_FORM_METHOD);
   }
-  if (!client_->IsSavingAndFillingEnabled(submitted_form.origin)) {
+  if (!client_->IsSavingAndFillingEnabled(submitted_form.url)) {
     RecordProvisionalSaveFailure(
-        PasswordManagerMetricsRecorder::SAVING_DISABLED, submitted_form.origin,
+        PasswordManagerMetricsRecorder::SAVING_DISABLED, submitted_form.url,
         logger.get());
     return nullptr;
   }
@@ -900,8 +893,13 @@ NewPasswordFormManager* PasswordManager::ProvisionallySaveForm(
   // PasswordToSave in NewPasswordFormManager DCHECKs that the password is never
   // empty.
 
-  // TODO(https://crbug.com/831123): Add the
-  // ShouldBlockPasswordForSameOriginButDifferentScheme check.
+  const GURL& origin = submitted_form.url;
+  if (ShouldBlockPasswordForSameOriginButDifferentScheme(origin)) {
+    RecordProvisionalSaveFailure(
+        PasswordManagerMetricsRecorder::SAVING_ON_HTTP_AFTER_HTTPS, origin,
+        logger.get());
+    return nullptr;
+  }
 
   NewPasswordFormManager* matched_manager =
       GetMatchedManager(driver, submitted_form);
@@ -916,12 +914,20 @@ NewPasswordFormManager* PasswordManager::ProvisionallySaveForm(
 
   if (!matched_manager) {
     RecordProvisionalSaveFailure(
-        PasswordManagerMetricsRecorder::NO_MATCHING_FORM, submitted_form.origin,
+        PasswordManagerMetricsRecorder::NO_MATCHING_FORM, submitted_form.url,
         logger.get());
     matched_manager = CreateFormManager(driver, submitted_form);
   }
 
-  if (!matched_manager->ProvisionallySave(submitted_form, driver))
+  if (is_manual_fallback && matched_manager->GetFormFetcher()->GetState() ==
+                                FormFetcher::State::WAITING) {
+    // In case of manual fallback, the form manager has to be ready for saving.
+    return nullptr;
+  }
+
+  if (!matched_manager->ProvisionallySave(
+          submitted_form, driver,
+          submitted_form.is_gaia_with_skip_save_password_form))
     return nullptr;
 
   // Set all other form managers to no submission state.
@@ -930,6 +936,13 @@ NewPasswordFormManager* PasswordManager::ProvisionallySaveForm(
       manager->set_not_submitted();
   }
 
+  // Cache the user-visible URL (i.e., the one seen in the omnibox). Once the
+  // post-submit navigation concludes, we compare the landing URL against the
+  // cached and report the difference through UMA.
+  main_frame_url_ = client_->GetMainFrameURL();
+
+  ReportSubmittedFormFrameMetric(driver, *matched_manager->GetSubmittedForm());
+
   return matched_manager;
 }
 
@@ -958,6 +971,43 @@ void PasswordManager::NotifyStorePasswordCalled() {
   DropFormManagers();
 }
 
+#if defined(OS_IOS)
+void PasswordManager::PresaveGeneratedPassword(
+    PasswordManagerDriver* driver,
+    const FormData& form,
+    const base::string16& generated_password,
+    const base::string16& generation_element) {
+  NewPasswordFormManager* form_manager =
+      FindMatchedManager(form, form_managers_, driver);
+  UMA_HISTOGRAM_BOOLEAN("PasswordManager.GeneratedFormHasNoFormManager",
+                        !form_manager);
+
+  // TODO(https://crbug.com/886583): Create form manager if not found.
+  if (form_manager) {
+    form_manager->PresaveGeneratedPassword(driver, form, generated_password,
+                                           generation_element);
+  }
+}
+
+void PasswordManager::UpdateGeneratedPasswordOnUserInput(
+    const base::string16& form_identifier,
+    const base::string16& field_identifier,
+    const base::string16& field_value) {
+  for (std::unique_ptr<NewPasswordFormManager>& manager : form_managers_) {
+    if (manager->UpdateGeneratedPasswordOnUserInput(
+            form_identifier, field_identifier, field_value)) {
+      break;
+    }
+  }
+}
+
+void PasswordManager::OnPasswordNoLongerGenerated(
+    PasswordManagerDriver* driver) {
+  for (std::unique_ptr<NewPasswordFormManager>& manager : form_managers_)
+    manager->PasswordNoLongerGenerated();
+}
+#endif
+
 void PasswordManager::ProvisionallySaveManager(
     const PasswordForm& form,
     PasswordFormManager* matched_manager,
@@ -1006,12 +1056,10 @@ bool PasswordManager::IsAutomaticSavePromptAvailable() {
 }
 
 bool PasswordManager::ShouldBlockPasswordForSameOriginButDifferentScheme(
-    const PasswordForm& form) const {
+    const GURL& origin) const {
   const GURL& old_origin = main_frame_url_.GetOrigin();
-  const GURL& new_origin = form.origin.GetOrigin();
-  return old_origin.host_piece() == new_origin.host_piece() &&
-         old_origin.SchemeIsCryptographic() &&
-         !new_origin.SchemeIsCryptographic();
+  return old_origin.host_piece() == origin.host_piece() &&
+         old_origin.SchemeIsCryptographic() && !origin.SchemeIsCryptographic();
 }
 
 void PasswordManager::OnPasswordFormsRendered(
@@ -1052,12 +1100,19 @@ void PasswordManager::OnPasswordFormsRendered(
                             visible_forms.begin(),
                             visible_forms.end());
 
-  if (!did_stop_loading)
+  if (!did_stop_loading &&
+      !submitted_manager->GetSubmittedForm()
+           ->form_data.is_gaia_with_skip_save_password_form) {
+    // |form_data.is_gaia_with_skip_save_password_form| = true means that this
+    // is a Chrome sign-in page. Chrome sign-in pages are redirected to an empty
+    // pages, and for some reasons |did_stop_loading| might be false. So
+    // |did_stop_loading| is ignored for them.
     return;
+  }
 
   // If we see the login form again, then the login failed.
   if (submitted_manager->GetPendingCredentials().scheme ==
-      PasswordForm::SCHEME_HTML) {
+      PasswordForm::Scheme::kHtml) {
     for (const PasswordForm& form : all_visible_forms_) {
       if (IsPasswordFormReappeared(
               form, submitted_manager->GetPendingCredentials())) {
@@ -1078,19 +1133,19 @@ void PasswordManager::OnPasswordFormsRendered(
         return;
       }
     }
-    } else {
-      if (logger)
-        logger->LogMessage(Logger::STRING_PROVISIONALLY_SAVED_FORM_IS_NOT_HTML);
-    }
+  } else {
+    if (logger)
+      logger->LogMessage(Logger::STRING_PROVISIONALLY_SAVED_FORM_IS_NOT_HTML);
+  }
 
-    // Clear all_visible_forms_ after checking all the visible forms.
-    all_visible_forms_.clear();
+  // Clear all_visible_forms_ after checking all the visible forms.
+  all_visible_forms_.clear();
 
-    // Looks like a successful login attempt. Either show an infobar or
-    // automatically save the login data. We prompt when the user hasn't
-    // already given consent, either through previously accepting the infobar
-    // or by having the browser generate the password.
-    OnLoginSuccessful();
+  // Looks like a successful login attempt. Either show an infobar or
+  // automatically save the login data. We prompt when the user hasn't
+  // already given consent, either through previously accepting the infobar
+  // or by having the browser generate the password.
+  OnLoginSuccessful();
 }
 
 void PasswordManager::OnLoginSuccessful() {
@@ -1136,7 +1191,6 @@ void PasswordManager::OnLoginSuccessful() {
 
   submitted_manager->GetMetricsRecorder()->LogSubmitPassed();
 
-  RecordWhetherTargetDomainDiffers(main_frame_url_, client_->GetMainFrameURL());
   UMA_HISTOGRAM_BOOLEAN(
       "PasswordManager.SuccessfulLoginHappened",
       submitted_manager->GetSubmittedForm()->origin.SchemeIsCryptographic());
@@ -1146,7 +1200,7 @@ void PasswordManager::OnLoginSuccessful() {
 
   // TODO(https://crbug.com/831123): Remove logging when the old form parsing is
   // removed.
-  if (is_new_form_parsing_for_saving_enabled_) {
+  if (IsNewFormParsingForSavingEnabled()) {
     // In this case, |submitted_manager| points to a NewPasswordFormManager and
     // |provisional_save_manager_| to a PasswordFormManager. They use the new
     // and the old FormData parser, respectively. Log the differences using UKM
@@ -1159,10 +1213,6 @@ void PasswordManager::OnLoginSuccessful() {
   }
 
   if (ShouldPromptUserToSavePassword(*submitted_manager)) {
-    bool empty_password =
-        submitted_manager->GetPendingCredentials().username_value.empty();
-    UMA_HISTOGRAM_BOOLEAN("PasswordManager.EmptyUsernames.OfferedToSave",
-                          empty_password);
     if (logger)
       logger->LogMessage(Logger::STRING_DECISION_ASK);
     bool update_password = submitted_manager->IsPasswordUpdate();
@@ -1181,13 +1231,11 @@ void PasswordManager::OnLoginSuccessful() {
           submitted_manager->GetPendingCredentials());
     }
 
-    if (submitted_manager->HasGeneratedPassword()) {
+    if (submitted_manager->HasGeneratedPassword())
       client_->AutomaticPasswordSave(MoveOwnedSubmittedManager());
-    } else {
-      provisional_save_manager_.reset();
-      owned_submitted_form_manager_.reset();
-    }
   }
+  provisional_save_manager_.reset();
+  owned_submitted_form_manager_.reset();
 }
 
 void PasswordManager::MaybeSavePasswordHash(
@@ -1242,27 +1290,6 @@ void PasswordManager::MaybeSavePasswordHash(
 #endif
 }
 
-void PasswordManager::AutofillHttpAuth(
-    const std::map<base::string16, const PasswordForm*>& best_matches,
-    const PasswordForm& preferred_match) const {
-  DCHECK_NE(PasswordForm::SCHEME_HTML, preferred_match.scheme);
-
-  std::unique_ptr<BrowserSavePasswordProgressLogger> logger;
-  if (password_manager_util::IsLoggingActive(client_)) {
-    logger.reset(
-        new BrowserSavePasswordProgressLogger(client_->GetLogManager()));
-    logger->LogMessage(Logger::STRING_PASSWORDMANAGER_AUTOFILLHTTPAUTH);
-    logger->LogBoolean(Logger::STRING_LOGINMODELOBSERVER_PRESENT,
-                       observers_.might_have_observers());
-  }
-
-  for (LoginModelObserver& observer : observers_)
-    observer.OnAutofillDataAvailable(preferred_match);
-  DCHECK(!best_matches.empty());
-  client_->PasswordWasAutofilled(best_matches,
-                                 best_matches.begin()->second->origin, nullptr);
-}
-
 void PasswordManager::ProcessAutofillPredictions(
     password_manager::PasswordManagerDriver* driver,
     const std::vector<autofill::FormStructure*>& forms) {
@@ -1271,8 +1298,7 @@ void PasswordManager::ProcessAutofillPredictions(
     logger.reset(
         new BrowserSavePasswordProgressLogger(client_->GetLogManager()));
 
-  if (base::FeatureList::IsEnabled(
-          password_manager::features::kNewPasswordFormParsing)) {
+  if (IsNewFormParsingForFillingEnabled()) {
     for (const autofill::FormStructure* form : forms)
       predictions_[form->form_signature()] = ConvertToFormPredictions(*form);
     for (auto& manager : form_managers_)
@@ -1280,12 +1306,12 @@ void PasswordManager::ProcessAutofillPredictions(
   }
 
   // Leave only forms that contain fields that are useful for password manager.
-  std::map<FormData, autofill::PasswordFormFieldPredictionMap> predictions;
+  autofill::FormsPredictionsMap predictions;
   for (const autofill::FormStructure* form : forms) {
     if (logger)
       logger->LogFormStructure(Logger::STRING_SERVER_PREDICTIONS, *form);
     for (const auto& field : *form) {
-      autofill::PasswordFormFieldPredictionType prediction_type;
+      PasswordFormFieldPredictionType prediction_type;
       if (ServerPredictionsToPasswordFormPrediction(field->server_predictions(),
                                                     &prediction_type)) {
         predictions[form->ToFormData()][*field] = prediction_type;
@@ -1296,13 +1322,19 @@ void PasswordManager::ProcessAutofillPredictions(
           IsPredictedTypeNotPasswordPrediction(
               field->Type().GetStorableType())) {
         predictions[form->ToFormData()][*field] =
-            autofill::PREDICTION_NOT_PASSWORD;
+            PasswordFormFieldPredictionType::kNotPassword;
       }
     }
   }
   if (predictions.empty())
     return;
-  driver->AutofillDataReceived(predictions);
+
+  if (driver) {
+    // |driver_| is nullptr on iOS.
+    // TODO(https://crbug.com/831123): Remove propagating predictions to the
+    // renderer when the old parser is gone.
+    driver->AutofillDataReceived(predictions);
+  }
 }
 
 PasswordFormManager* PasswordManager::GetMatchingPendingManager(
@@ -1341,14 +1373,7 @@ PasswordFormManager* PasswordManager::GetMatchingPendingManager(
 }
 
 PasswordFormManagerInterface* PasswordManager::GetSubmittedManager() const {
-  // TODO(https://crbug.com/915161) Implement support of non-html schemes in
-  // NewPasswordFormManager remove using PasswordFormManager for such forms.
-  if (provisional_save_manager_ &&
-      provisional_save_manager_->GetPendingCredentials().scheme !=
-          PasswordForm::SCHEME_HTML)
-    return provisional_save_manager_.get();
-
-  if (!is_new_form_parsing_for_saving_enabled_)
+  if (!IsNewFormParsingForSavingEnabled())
     return provisional_save_manager_.get();
 
   if (owned_submitted_form_manager_)
@@ -1365,14 +1390,7 @@ PasswordFormManagerInterface* PasswordManager::GetSubmittedManager() const {
 
 std::unique_ptr<PasswordFormManagerForUI>
 PasswordManager::MoveOwnedSubmittedManager() {
-  // TODO(https://crbug.com/915161) Implement support of non-html schemes in
-  // NewPasswordFormManager remove using PasswordFormManager for such forms.
-  if (provisional_save_manager_ &&
-      provisional_save_manager_->GetPendingCredentials().scheme !=
-          PasswordForm::SCHEME_HTML)
-    return std::move(provisional_save_manager_);
-
-  if (!is_new_form_parsing_for_saving_enabled_)
+  if (!IsNewFormParsingForSavingEnabled())
     return std::move(provisional_save_manager_);
 
   if (owned_submitted_form_manager_)
@@ -1415,7 +1433,7 @@ PasswordManager::GetMetricRecorderFromNewPasswordFormManager(
 PasswordFormManagerInterface* PasswordManager::GetMatchedManager(
     const PasswordManagerDriver* driver,
     const PasswordForm& form) {
-  if (!is_new_form_parsing_for_saving_enabled_)
+  if (!IsNewFormParsingForSavingEnabled())
     return GetMatchingPendingManager(form);
 
   return GetMatchedManager(driver, form.form_data);
@@ -1431,4 +1449,29 @@ NewPasswordFormManager* PasswordManager::GetMatchedManager(
   return nullptr;
 }
 
+void PasswordManager::ReportSubmittedFormFrameMetric(
+    const PasswordManagerDriver* driver,
+    const PasswordForm& form) {
+  if (!driver)
+    return;
+  metrics_util::SubmittedFormFrame frame;
+  if (driver->IsMainFrame()) {
+    frame = metrics_util::SubmittedFormFrame::MAIN_FRAME;
+  } else if (form.origin == main_frame_url_) {
+    frame =
+        metrics_util::SubmittedFormFrame::IFRAME_WITH_SAME_URL_AS_MAIN_FRAME;
+  } else {
+    GURL::Replacements rep;
+    rep.SetPathStr("");
+    std::string main_frame_signon_realm =
+        main_frame_url_.ReplaceComponents(rep).spec();
+    frame = (main_frame_signon_realm == form.signon_realm)
+                ? metrics_util::SubmittedFormFrame::
+                      IFRAME_WITH_DIFFERENT_URL_SAME_SIGNON_REALM_AS_MAIN_FRAME
+                : metrics_util::SubmittedFormFrame::
+                      IFRAME_WITH_DIFFERENT_SIGNON_REALM;
+  }
+  metrics_util::LogSubmittedFormFrame(frame);
+}
+
 }  // namespace password_manager
