@@ -8,6 +8,7 @@
 #include <vector>
 
 #include "base/at_exit.h"
+#include "base/command_line.h"
 #include "base/i18n/icu_util.h"
 #include "base/json/json_reader.h"
 #include "base/logging.h"
@@ -30,7 +31,10 @@ extern "C" int LLVMFuzzerTestOneInput(const uint8_t* data, size_t size) {
   bool all_origins_supported;
 
   base::StringPiece json_data(reinterpret_cast<const char*>(data), size);
-  std::unique_ptr<base::Value> value = base::JSONReader::Read(json_data);
+  std::unique_ptr<base::Value> value =
+      base::JSONReader::ReadDeprecated(json_data);
+
+  base::CommandLine::Init(0, nullptr);
 
   payments::ErrorLogger log;
   log.DisableInTest();
