@@ -130,6 +130,8 @@ class WebViewPlugin : public blink::WebPlugin,
   void UpdatePluginForNewGeometry(const blink::WebRect& window_rect,
                                   const blink::WebRect& unobscured_rect);
 
+  scoped_refptr<base::SingleThreadTaskRunner> GetTaskRunner();
+
   // Manages its own lifetime.
   Delegate* delegate_;
 
@@ -166,6 +168,7 @@ class WebViewPlugin : public blink::WebPlugin,
     bool CanHandleGestureEvent() override;
     bool CanUpdateLayout() override;
     blink::WebScreenInfo GetScreenInfo() override;
+    void DidInvalidateRect(const blink::WebRect&) override;
 
     // WebWidgetClient methods:
     void SetToolTipText(const blink::WebString&,
@@ -175,7 +178,6 @@ class WebViewPlugin : public blink::WebPlugin,
                        blink::WebDragOperationsMask,
                        const SkBitmap&,
                        const gfx::Point&) override;
-    void DidInvalidateRect(const blink::WebRect&) override;
     void DidChangeCursor(const blink::WebCursorInfo& cursor) override;
     void ScheduleAnimation() override;
     std::unique_ptr<blink::WebURLLoaderFactory> CreateURLLoaderFactory()
@@ -196,7 +198,7 @@ class WebViewPlugin : public blink::WebPlugin,
   WebViewHelper web_view_helper_;
 
   // Should be invalidated when destroy() is called.
-  base::WeakPtrFactory<WebViewPlugin> weak_factory_;
+  base::WeakPtrFactory<WebViewPlugin> weak_factory_{this};
 };
 
 #endif  // COMPONENTS_PLUGINS_RENDERER_WEBVIEW_PLUGIN_H_
