@@ -20,6 +20,7 @@
 #include "base/strings/stringprintf.h"
 #include "base/time/clock.h"
 #include "components/blacklist/opt_out_blacklist/opt_out_store.h"
+#include "components/optimization_guide/optimization_guide_features.h"
 #include "components/previews/content/previews_ui_service.h"
 #include "components/previews/content/previews_user_data.h"
 #include "components/previews/core/previews_experiments.h"
@@ -45,46 +46,22 @@ void LogPreviewsEligibilityReason(PreviewsEligibilityReason status,
       ->Add(static_cast<int>(status));
 }
 
-bool AllowedOnReload(PreviewsType type) {
-  if (base::FeatureList::IsEnabled(features::kPreviewsDisallowedOnReloads))
-    return false;
-
-  switch (type) {
-    // These types return new content on refresh.
-    case PreviewsType::LITE_PAGE:
-    case PreviewsType::LITE_PAGE_REDIRECT:
-    case PreviewsType::LOFI:
-    case PreviewsType::NOSCRIPT:
-    case PreviewsType::RESOURCE_LOADING_HINTS:
-      return true;
-    // Loading these types will always be stale when refreshed.
-    case PreviewsType::OFFLINE:
-      return false;
-    case PreviewsType::NONE:
-    case PreviewsType::UNSPECIFIED:
-    case PreviewsType::DEPRECATED_AMP_REDIRECTION:
-    case PreviewsType::LAST:
-      break;
-  }
-  NOTREACHED();
-  return false;
-}
-
 bool ShouldCheckOptimizationHints(PreviewsType type) {
   switch (type) {
     // These types may have server optimization hints.
     case PreviewsType::NOSCRIPT:
     case PreviewsType::RESOURCE_LOADING_HINTS:
     case PreviewsType::LITE_PAGE_REDIRECT:
+    case PreviewsType::DEFER_ALL_SCRIPT:
       return true;
     // These types do not have server optimization hints.
     case PreviewsType::OFFLINE:
     case PreviewsType::LITE_PAGE:
-    case PreviewsType::LOFI:
       return false;
     case PreviewsType::NONE:
     case PreviewsType::UNSPECIFIED:
     case PreviewsType::DEPRECATED_AMP_REDIRECTION:
+    case PreviewsType::DEPRECATED_LOFI:
     case PreviewsType::LAST:
       break;
   }
@@ -94,12 +71,12 @@ bool ShouldCheckOptimizationHints(PreviewsType type) {
 
 // Returns true if ECT should be checked for |type| only at the commit time. If
 // true is returned, then ECT need not be checked at the navigation time.
-bool CheckForUnknownECTOnlyAtCommitTime(PreviewsType type) {
+bool CheckECTOnlyAtCommitTime(PreviewsType type) {
   switch (type) {
     case PreviewsType::NOSCRIPT:
     case PreviewsType::RESOURCE_LOADING_HINTS:
+    case PreviewsType::DEFER_ALL_SCRIPT:
       return true;
-    case PreviewsType::LOFI:
     case PreviewsType::LITE_PAGE_REDIRECT:
     case PreviewsType::OFFLINE:
     case PreviewsType::LITE_PAGE:
@@ -107,6 +84,7 @@ bool CheckForUnknownECTOnlyAtCommitTime(PreviewsType type) {
     case PreviewsType::NONE:
     case PreviewsType::UNSPECIFIED:
     case PreviewsType::DEPRECATED_AMP_REDIRECTION:
+    case PreviewsType::DEPRECATED_LOFI:
     case PreviewsType::LAST:
       break;
   }
@@ -114,11 +92,6 @@ bool CheckForUnknownECTOnlyAtCommitTime(PreviewsType type) {
   return false;
 }
 
-bool IsPreviewsBlacklistIgnoredViaFlag() {
-  return base::CommandLine::ForCurrentProcess()->HasSwitch(
-      switches::kIgnorePreviewsBlacklist);
-}
-
 // We don't care if the ECT is unknown if the slow page threshold is set to 4G
 // (i.e.: all pages).
 bool ShouldCheckForUnknownECT(net::EffectiveConnectionType ect) {
@@ -130,12 +103,10 @@ bool ShouldCheckForUnknownECT(net::EffectiveConnectionType ect) {
 
 }  // namespace
 
-PreviewsDeciderImpl::PreviewsDeciderImpl(
-    base::Clock* clock)
-    : blacklist_ignored_(IsPreviewsBlacklistIgnoredViaFlag()),
+PreviewsDeciderImpl::PreviewsDeciderImpl(base::Clock* clock)
+    : blacklist_ignored_(switches::ShouldIgnorePreviewsBlacklist()),
       clock_(clock),
-      page_id_(1u),
-      weak_factory_(this) {}
+      page_id_(1u) {}
 
 PreviewsDeciderImpl::~PreviewsDeciderImpl() {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
@@ -195,11 +166,12 @@ void PreviewsDeciderImpl::LogPreviewDecisionMade(
     base::Time time,
     PreviewsType type,
     std::vector<PreviewsEligibilityReason>&& passed_reasons,
-    uint64_t page_id) const {
+    PreviewsUserData* user_data) const {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   LogPreviewsEligibilityReason(reason, type);
+  user_data->SetEligibilityReasonForPreview(type, reason);
   previews_ui_service_->LogPreviewDecisionMade(
-      reason, url, time, type, std::move(passed_reasons), page_id);
+      reason, url, time, type, std::move(passed_reasons), user_data->page_id());
 }
 
 void PreviewsDeciderImpl::AddPreviewNavigation(const GURL& url,
@@ -209,9 +181,6 @@ void PreviewsDeciderImpl::AddPreviewNavigation(const GURL& url,
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   base::Time time =
       previews_black_list_->AddPreviewNavigation(url, opt_out, type);
-  if (opt_out) {
-    last_opt_out_time_ = time;
-  }
   LogPreviewNavigation(url, opt_out, type, time, page_id);
 }
 
@@ -220,6 +189,10 @@ void PreviewsDeciderImpl::ClearBlackList(base::Time begin_time,
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
 
   previews_black_list_->ClearBlackList(begin_time, end_time);
+
+  // Removes all fetched hints known to the owned optimization guide.
+  if (previews_opt_guide_)
+    previews_opt_guide_->ClearFetchedHints();
 }
 
 void PreviewsDeciderImpl::SetIgnorePreviewsBlacklistDecision(bool ignored) {
@@ -245,7 +218,7 @@ bool PreviewsDeciderImpl::ShouldAllowPreviewAtNavigationStart(
       DeterminePreviewEligibility(previews_data, url, is_reload, type,
                                   is_drp_server_preview, &passed_reasons);
   LogPreviewDecisionMade(eligibility, url, clock_->Now(), type,
-                         std::move(passed_reasons), previews_data->page_id());
+                         std::move(passed_reasons), previews_data);
   return eligibility == PreviewsEligibilityReason::ALLOWED;
 }
 
@@ -277,30 +250,30 @@ PreviewsEligibilityReason PreviewsDeciderImpl::DeterminePreviewEligibility(
   // Do not allow previews on any authenticated pages.
   if (url.has_username() || url.has_password())
     return PreviewsEligibilityReason::URL_HAS_BASIC_AUTH;
+  passed_reasons->push_back(PreviewsEligibilityReason::URL_HAS_BASIC_AUTH);
 
-  // Trigger the USER_RECENTLY_OPTED_OUT rule when a reload on a preview has
-  // occurred recently.
-  if (recent_preview_reload_time_ &&
-      recent_preview_reload_time_.value() + params::SingleOptOutDuration() >
-          clock_->Now()) {
-    return PreviewsEligibilityReason::USER_RECENTLY_OPTED_OUT;
-  }
+  // Do not allow previews for URL suffixes which are excluded. In practice,
+  // this is used to exclude navigations that look like media resources like
+  // navigating to http://chromium.org/video.mp4.
+  if (params::ShouldExcludeMediaSuffix(url))
+    return PreviewsEligibilityReason::EXCLUDED_BY_MEDIA_SUFFIX;
+  passed_reasons->push_back(
+      PreviewsEligibilityReason::EXCLUDED_BY_MEDIA_SUFFIX);
 
-  // In the case that the user has chosen to ignore the normal blacklist rules
-  // (flags or interventions-internals), a preview should still not be served
-  // for 5 seconds after the last opt out. This allows "show original" to
-  // function correctly as the start of that navigation will be within 5 seconds
-  // (we don't yet re-evaluate on redirects, so this is sufficient).
-  if (blacklist_ignored_) {
-    if (clock_->Now() < last_opt_out_time_ + base::TimeDelta::FromSeconds(5)) {
+  // Skip blacklist checks if the blacklist is ignored.
+  if (!blacklist_ignored_) {
+    if (!previews_black_list_)
+      return PreviewsEligibilityReason::BLACKLIST_UNAVAILABLE;
+    passed_reasons->push_back(PreviewsEligibilityReason::BLACKLIST_UNAVAILABLE);
+
+    // Trigger the USER_RECENTLY_OPTED_OUT rule when a reload on a preview has
+    // occurred recently. No need to push_back the eligibility reason as it will
+    // be added in IsLoadedAndAllowed as the first check.
+    if (recent_preview_reload_time_ &&
+        recent_preview_reload_time_.value() + params::SingleOptOutDuration() >
+            clock_->Now()) {
       return PreviewsEligibilityReason::USER_RECENTLY_OPTED_OUT;
     }
-    passed_reasons->push_back(
-        PreviewsEligibilityReason::USER_RECENTLY_OPTED_OUT);
-  } else if (!previews_black_list_) {
-    return PreviewsEligibilityReason::BLACKLIST_UNAVAILABLE;
-  } else {
-    passed_reasons->push_back(PreviewsEligibilityReason::BLACKLIST_UNAVAILABLE);
 
     // The blacklist will disallow certain hosts for periods of time based on
     // user's opting out of the preview.
@@ -322,9 +295,8 @@ PreviewsEligibilityReason PreviewsDeciderImpl::DeterminePreviewEligibility(
   // hints. This defers checking ECT for server previews because the server will
   // perform its own ECT check and for previews with hints because the hints may
   // specify variable ECT thresholds for slow page hints.
-  if (!is_drp_server_preview && !ShouldCheckOptimizationHints(type)) {
-    if (effective_connection_type_ == net::EFFECTIVE_CONNECTION_TYPE_UNKNOWN &&
-        !CheckForUnknownECTOnlyAtCommitTime(type)) {
+  if (!is_drp_server_preview && !CheckECTOnlyAtCommitTime(type)) {
+    if (effective_connection_type_ == net::EFFECTIVE_CONNECTION_TYPE_UNKNOWN) {
       return PreviewsEligibilityReason::NETWORK_QUALITY_UNAVAILABLE;
     }
     passed_reasons->push_back(
@@ -347,22 +319,22 @@ PreviewsEligibilityReason PreviewsDeciderImpl::DeterminePreviewEligibility(
     passed_reasons->push_back(PreviewsEligibilityReason::NETWORK_NOT_SLOW);
   }
 
-  // LOAD_VALIDATE_CACHE or LOAD_BYPASS_CACHE mean the user reloaded the page.
-  // If this is a query for offline previews, reloads should be disallowed.
-  if (!AllowedOnReload(type) && is_reload) {
+  if (is_reload) {
     return PreviewsEligibilityReason::RELOAD_DISALLOWED;
   }
+
   passed_reasons->push_back(PreviewsEligibilityReason::RELOAD_DISALLOWED);
 
   // Check server whitelist/blacklist, if provided.
   if (ShouldCheckOptimizationHints(type)) {
-    if (params::IsOptimizationHintsEnabled()) {
+    if (optimization_guide::features::IsOptimizationHintsEnabled()) {
       // Optimization hints are configured, so determine if those hints
       // allow the optimization type (as of start-of-navigation time anyway).
       return ShouldAllowPreviewPerOptimizationHints(previews_data, url, type,
                                                     passed_reasons);
     } else if (type == PreviewsType::RESOURCE_LOADING_HINTS ||
-               type == PreviewsType::NOSCRIPT) {
+               type == PreviewsType::NOSCRIPT ||
+               type == PreviewsType::DEFER_ALL_SCRIPT) {
       return PreviewsEligibilityReason::HOST_NOT_WHITELISTED_BY_SERVER;
     }
   }
@@ -402,7 +374,8 @@ bool PreviewsDeciderImpl::ShouldCommitPreview(PreviewsUserData* previews_data,
                                               PreviewsType type) const {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   DCHECK(PreviewsType::NOSCRIPT == type ||
-         PreviewsType::RESOURCE_LOADING_HINTS == type);
+         PreviewsType::RESOURCE_LOADING_HINTS == type ||
+         PreviewsType::DEFER_ALL_SCRIPT == type);
   if (previews_black_list_ && !blacklist_ignored_) {
     std::vector<PreviewsEligibilityReason> passed_reasons;
     // The blacklist will disallow certain hosts for periods of time based on
@@ -411,22 +384,20 @@ bool PreviewsDeciderImpl::ShouldCommitPreview(PreviewsUserData* previews_data,
         committed_url, type, false, &passed_reasons);
     if (status != PreviewsEligibilityReason::ALLOWED) {
       LogPreviewDecisionMade(status, committed_url, clock_->Now(), type,
-                             std::move(passed_reasons),
-                             previews_data->page_id());
+                             std::move(passed_reasons), previews_data);
       return false;
     }
   }
 
   // Re-check server optimization hints (if provided) on this commit-time URL.
   if (ShouldCheckOptimizationHints(type) &&
-      params::IsOptimizationHintsEnabled()) {
+      optimization_guide::features::IsOptimizationHintsEnabled()) {
     std::vector<PreviewsEligibilityReason> passed_reasons;
     PreviewsEligibilityReason status = ShouldCommitPreviewPerOptimizationHints(
         previews_data, committed_url, type, &passed_reasons);
     if (status != PreviewsEligibilityReason::ALLOWED) {
       LogPreviewDecisionMade(status, committed_url, clock_->Now(), type,
-                             std::move(passed_reasons),
-                             previews_data->page_id());
+                             std::move(passed_reasons), previews_data);
       return false;
     }
   }
@@ -443,29 +414,30 @@ PreviewsDeciderImpl::ShouldAllowPreviewPerOptimizationHints(
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   DCHECK(type == PreviewsType::LITE_PAGE_REDIRECT ||
          type == PreviewsType::NOSCRIPT ||
-         type == PreviewsType::RESOURCE_LOADING_HINTS);
-
-  // Per-PreviewsType default if no optimization guide data.
-  if (!previews_opt_guide_) {
-    if (type == PreviewsType::NOSCRIPT) {
-      return PreviewsEligibilityReason::ALLOWED;
-    } else {
-      return PreviewsEligibilityReason::HOST_NOT_WHITELISTED_BY_SERVER;
-    }
-  }
-
-  // For LitePageRedirect, ensure it is not blacklisted for this request.
+         type == PreviewsType::RESOURCE_LOADING_HINTS ||
+         type == PreviewsType::DEFER_ALL_SCRIPT);
+  // For LitePageRedirect, ensure it is not blacklisted for this request, and
+  // hints have been fully loaded.
+  //
+  // We allow all other Optimization Hint previews in the hopes that the missing
+  // state will load in before commit.
   if (type == PreviewsType::LITE_PAGE_REDIRECT) {
-    if (previews_opt_guide_->IsBlacklisted(url, type)) {
-      return PreviewsEligibilityReason::HOST_BLACKLISTED_BY_SERVER;
+    if (base::CommandLine::ForCurrentProcess()->HasSwitch(
+            switches::kIgnoreLitePageRedirectOptimizationBlacklist)) {
+      return PreviewsEligibilityReason::ALLOWED;
     }
+
+    if (!previews_opt_guide_ || !previews_opt_guide_->has_hints())
+      return PreviewsEligibilityReason::OPTIMIZATION_HINTS_NOT_AVAILABLE;
+    passed_reasons->push_back(
+        PreviewsEligibilityReason::OPTIMIZATION_HINTS_NOT_AVAILABLE);
+
+    if (previews_opt_guide_->IsBlacklisted(url, type))
+      return PreviewsEligibilityReason::HOST_BLACKLISTED_BY_SERVER;
     passed_reasons->push_back(
         PreviewsEligibilityReason::HOST_BLACKLISTED_BY_SERVER);
   }
 
-  // Note: allow NoScript and ResourceLoadingHints since the guide is available.
-  // Page hints may need to be loaded from it for commit time detail check.
-
   return PreviewsEligibilityReason::ALLOWED;
 }
 
@@ -477,12 +449,22 @@ PreviewsDeciderImpl::ShouldCommitPreviewPerOptimizationHints(
     std::vector<PreviewsEligibilityReason>* passed_reasons) const {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   DCHECK(type == PreviewsType::NOSCRIPT ||
-         type == PreviewsType::RESOURCE_LOADING_HINTS);
+         type == PreviewsType::RESOURCE_LOADING_HINTS ||
+         type == PreviewsType::DEFER_ALL_SCRIPT);
 
-  // For NoScript, if optimization guide is not present, assume that all URLs
-  // are ALLOWED.
-  if (!previews_opt_guide_ && type == PreviewsType::NOSCRIPT)
+  // If kEnableDeferAllScriptWithoutOptimizationHints switch is provided, then
+  // DEFER_ALL_SCRIPT is triggered on all pages irrespective of hints provided
+  // by optimization hints guide.
+  if (type == PreviewsType::DEFER_ALL_SCRIPT &&
+      base::CommandLine::ForCurrentProcess()->HasSwitch(
+          switches::kEnableDeferAllScriptWithoutOptimizationHints)) {
     return PreviewsEligibilityReason::ALLOWED;
+  }
+
+  if (!previews_opt_guide_ || !previews_opt_guide_->has_hints())
+    return PreviewsEligibilityReason::OPTIMIZATION_HINTS_NOT_AVAILABLE;
+  passed_reasons->push_back(
+      PreviewsEligibilityReason::OPTIMIZATION_HINTS_NOT_AVAILABLE);
 
   // Check if request URL is whitelisted by the optimization guide.
   net::EffectiveConnectionType ect_threshold =
@@ -500,7 +482,7 @@ PreviewsDeciderImpl::ShouldCommitPreviewPerOptimizationHints(
   // connectivity as null. See https://crbug.com/838969. So, we do not trigger
   // previews when |ect| is net::EFFECTIVE_CONNECTION_TYPE_OFFLINE.
   net::EffectiveConnectionType ect = previews_data->navigation_ect();
-  if (CheckForUnknownECTOnlyAtCommitTime(type) &&
+  if (CheckECTOnlyAtCommitTime(type) &&
       ect == net::EFFECTIVE_CONNECTION_TYPE_UNKNOWN) {
     // Update the |ect| to the current value.
     ect = effective_connection_type_;
