@@ -12,20 +12,16 @@ namespace features {
 
 extern const base::Feature kPreviews;
 extern const base::Feature kOfflinePreviews;
-extern const base::Feature kClientLoFi;
 extern const base::Feature kNoScriptPreviews;
 extern const base::Feature kStalePreviewsTimestamp;
-extern const base::Feature kOptimizationHints;
-extern const base::Feature kOptimizationHintsExperiments;
-constexpr char kOptimizationHintsExperimentNameParam[] = "experiment_name";
 extern const base::Feature kResourceLoadingHints;
 extern const base::Feature kLitePageServerPreviews;
-extern const base::Feature kAndroidOmniboxPreviewsBadge;
 extern const base::Feature kSlowPageTriggering;
-extern const base::Feature kPreviewsDisallowedOnReloads;
 extern const base::Feature kHTTPSServerPreviewsUsingURLLoader;
-extern const base::Feature kDataSaverLiteModeRebranding;
-extern const base::Feature kPreviewsReloadsAreSoftOptOuts;
+extern const base::Feature kOfflinePreviewsFalsePositivePrevention;
+extern const base::Feature kCoinFlipHoldback;
+extern const base::Feature kExcludedMediaSuffixes;
+extern const base::Feature kDeferAllScriptPreviews;
 
 }  // namespace features
 }  // namespace previews
