@@ -4,6 +4,7 @@
 
 #include <stddef.h>
 
+#include "base/base64.h"
 #include "base/base_paths.h"
 #include "base/command_line.h"
 #include "base/i18n/case_conversion.h"
@@ -172,7 +173,8 @@ TEST_F(TemplateURLTest, URLRefTestImageURLWithPOST) {
   const char kValidPostParamsString[] =
       "image_content={google:imageThumbnail},image_url={google:imageURL},"
       "sbisrc={google:imageSearchSource},language={language},empty_param=,"
-      "constant_param=constant,width={google:imageOriginalWidth}";
+      "constant_param=constant,width={google:imageOriginalWidth},"
+      "base64_image_content={google:imageThumbnailBase64}";
   const char KImageSearchURL[] = "http://foo.com/sbi";
 
   TemplateURLData data;
@@ -220,7 +222,7 @@ TEST_F(TemplateURLTest, URLRefTestImageURLWithPOST) {
       url.image_url_ref().replacements_;
   const TemplateURLRef::PostParams& post_params =
       url.image_url_ref().post_params_;
-  EXPECT_EQ(7U, post_params.size());
+  EXPECT_EQ(8U, post_params.size());
   for (auto i = post_params.begin(); i != post_params.end(); ++i) {
     auto j = replacements.begin();
     for (; j != replacements.end(); ++j) {
@@ -228,9 +230,9 @@ TEST_F(TemplateURLTest, URLRefTestImageURLWithPOST) {
           static_cast<size_t>(i - post_params.begin())) {
         switch (j->type) {
           case TemplateURLRef::GOOGLE_IMAGE_ORIGINAL_WIDTH:
-            ExpectPostParamIs(*i, "width",
-                              base::IntToString(
-                                   search_args.image_original_size.width()));
+            ExpectPostParamIs(
+                *i, "width",
+                base::NumberToString(search_args.image_original_size.width()));
             break;
           case TemplateURLRef::GOOGLE_IMAGE_SEARCH_SOURCE:
             ExpectPostParamIs(*i, "sbisrc",
@@ -241,6 +243,14 @@ TEST_F(TemplateURLTest, URLRefTestImageURLWithPOST) {
                               search_args.image_thumbnail_content,
                               "image/jpeg");
             break;
+          case TemplateURLRef::GOOGLE_IMAGE_THUMBNAIL_BASE64: {
+            std::string base64_image_content;
+            base::Base64Encode(search_args.image_thumbnail_content,
+                               &base64_image_content);
+            ExpectPostParamIs(*i, "base64_image_content", base64_image_content,
+                              "image/jpeg");
+            break;
+          }
           case TemplateURLRef::GOOGLE_IMAGE_URL:
             ExpectPostParamIs(*i, "image_url", search_args.image_url.spec());
             break;
@@ -722,6 +732,39 @@ TEST_F(TemplateURLTest, ReplaceInputType) {
   }
 }
 
+// Tests replacing omnibox focus type (&oft=).
+TEST_F(TemplateURLTest, ReplaceOmniboxFocusType) {
+  struct TestData {
+    const base::string16 search_term;
+    TemplateURLRef::SearchTermsArgs::OmniboxFocusType omnibox_focus_type;
+    const std::string url;
+    const std::string expected_result;
+  } test_data[] = {
+      {ASCIIToUTF16("foo"),
+       TemplateURLRef::SearchTermsArgs::OmniboxFocusType::DEFAULT,
+       "{google:baseURL}?{searchTerms}&{google:omniboxFocusType}",
+       "http://www.google.com/?foo&"},
+      {ASCIIToUTF16("foo"),
+       TemplateURLRef::SearchTermsArgs::OmniboxFocusType::ON_FOCUS,
+       "{google:baseURL}?{searchTerms}&{google:omniboxFocusType}",
+       "http://www.google.com/?foo&oft=1&"},
+  };
+  TemplateURLData data;
+  data.input_encodings.push_back("UTF-8");
+  for (size_t i = 0; i < base::size(test_data); ++i) {
+    data.SetURL(test_data[i].url);
+    TemplateURL url(data);
+    EXPECT_TRUE(url.url_ref().IsValid(search_terms_data_));
+    ASSERT_TRUE(url.url_ref().SupportsReplacement(search_terms_data_));
+    TemplateURLRef::SearchTermsArgs search_terms_args(test_data[i].search_term);
+    search_terms_args.omnibox_focus_type = test_data[i].omnibox_focus_type;
+    GURL result(url.url_ref().ReplaceSearchTerms(search_terms_args,
+                                                 search_terms_data_));
+    ASSERT_TRUE(result.is_valid());
+    EXPECT_EQ(test_data[i].expected_result, result.spec());
+  }
+}
+
 // Tests replacing currentPageUrl.
 TEST_F(TemplateURLTest, ReplaceCurrentPageUrl) {
   struct TestData {
