@@ -8,11 +8,12 @@
 #include <sstream>
 #include <string>
 
+#include "base/bind_helpers.h"
 #include "base/callback.h"
-#include "base/message_loop/message_loop_current.h"
 #include "base/run_loop.h"
 #include "base/strings/stringprintf.h"
 #include "base/test/metrics/histogram_tester.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "components/subresource_filter/content/browser/async_document_subresource_filter.h"
 #include "components/subresource_filter/content/browser/async_document_subresource_filter_test_utils.h"
 #include "components/subresource_filter/content/browser/subresource_filter_observer_test_utils.h"
@@ -93,7 +94,7 @@ class SubframeNavigationFilteringThrottleTest
     // tests, to ensure that the NavigationSimulator properly runs all necessary
     // tasks while waiting for throttle checks to finish.
     dealer_handle_ = std::make_unique<VerifiedRulesetDealer::Handle>(
-        base::MessageLoopCurrent::Get()->task_runner());
+        base::ThreadTaskRunnerHandle::Get());
     dealer_handle_->TryOpenAndSetRulesetFile(test_ruleset_pair_.indexed.path,
                                              /*expected_checksum=*/0,
                                              base::DoNothing());
@@ -184,7 +185,7 @@ TEST_F(SubframeNavigationFilteringThrottleTest, FilterOnStart) {
   SimulateStartAndExpectResult(
       content::NavigationThrottle::BLOCK_REQUEST_AND_COLLAPSE);
   EXPECT_TRUE(
-      base::ContainsValue(GetConsoleMessages(), GetFilterConsoleMessage(url)));
+      base::Contains(GetConsoleMessages(), GetFilterConsoleMessage(url)));
 }
 
 TEST_F(SubframeNavigationFilteringThrottleTest, FilterOnRedirect) {
@@ -207,7 +208,7 @@ TEST_F(SubframeNavigationFilteringThrottleTest, DryRunOnStart) {
 
   SimulateStartAndExpectResult(content::NavigationThrottle::PROCEED);
   EXPECT_FALSE(
-      base::ContainsValue(GetConsoleMessages(), GetFilterConsoleMessage(url)));
+      base::Contains(GetConsoleMessages(), GetFilterConsoleMessage(url)));
 }
 
 TEST_F(SubframeNavigationFilteringThrottleTest, DryRunOnRedirect) {
