@@ -4,6 +4,8 @@
 
 #include "components/sync_sessions/favicon_cache.h"
 
+#include <utility>
+
 #include "base/memory/ptr_util.h"
 #include "base/run_loop.h"
 #include "base/strings/string_number_conversions.h"
@@ -345,13 +347,14 @@ testing::AssertionResult SyncFaviconCacheTest::ExpectFaviconEquals(
     const std::string& page_url,
     const std::string& bytes) const {
   GURL gurl(page_url);
-  scoped_refptr<base::RefCountedMemory> favicon;
-  if (!cache_.GetSyncedFaviconForPageURL(gurl, &favicon))
+  favicon_base::FaviconRawBitmapResult favicon =
+      cache_.GetSyncedFaviconForPageURL(gurl);
+  if (!favicon.is_valid())
     return testing::AssertionFailure() << "Favicon is missing.";
-  if (favicon->size() != bytes.size())
+  if (favicon.bitmap_data->size() != bytes.size())
     return testing::AssertionFailure() << "Favicon sizes don't match.";
-  for (size_t i = 0; i < favicon->size(); ++i) {
-    if (bytes[i] != *(favicon->front() + i))
+  for (size_t i = 0; i < favicon.bitmap_data->size(); ++i) {
+    if (bytes[i] != *(favicon.bitmap_data->front() + i))
       return testing::AssertionFailure() << "Favicon data doesn't match.";
   }
   return testing::AssertionSuccess();
@@ -455,7 +458,13 @@ void SyncFaviconCacheTest::PopulateFaviconService(
 
   ON_CALL(mock_favicon_service_,
           GetFaviconForPageURL(test_data.page_url, _, _, _, _))
-      .WillByDefault(favicon::PostReply<5>(bitmap_results));
+      .WillByDefault([=](auto, auto, auto,
+                         favicon_base::FaviconResultsCallback callback,
+                         base::CancelableTaskTracker* tracker) {
+        return tracker->PostTask(
+            base::ThreadTaskRunnerHandle::Get().get(), FROM_HERE,
+            base::BindOnce(std::move(callback), bitmap_results));
+      });
 }
 
 void SyncFaviconCacheTest::TriggerSyncFaviconReceived(
@@ -468,14 +477,20 @@ void SyncFaviconCacheTest::TriggerSyncFaviconReceived(
         new base::RefCountedString());
     temp_string->data() = icon_bytes;
 
-    std::vector<favicon_base::FaviconRawBitmapResult> result;
-    result.push_back(favicon_base::FaviconRawBitmapResult());
-    result.back().icon_url = icon_url;
-    result.back().bitmap_data = temp_string;
-    result.back().pixel_size = gfx::Size(16, 16);
+    std::vector<favicon_base::FaviconRawBitmapResult> bitmap_results;
+    bitmap_results.push_back(favicon_base::FaviconRawBitmapResult());
+    bitmap_results.back().icon_url = icon_url;
+    bitmap_results.back().bitmap_data = temp_string;
+    bitmap_results.back().pixel_size = gfx::Size(16, 16);
 
     ON_CALL(mock_favicon_service_, GetFaviconForPageURL(page_url, _, _, _, _))
-        .WillByDefault(favicon::PostReply<5>(result));
+        .WillByDefault([=](auto, auto, auto,
+                           favicon_base::FaviconResultsCallback callback,
+                           base::CancelableTaskTracker* tracker) {
+          return tracker->PostTask(
+              base::ThreadTaskRunnerHandle::Get().get(), FROM_HERE,
+              base::BindOnce(std::move(callback), bitmap_results));
+        });
 
     // Mimic the icon itself having been cached long time ago.
     cache()->OnPageFaviconUpdated(page_url, base::Time::UnixEpoch());
