@@ -17,7 +17,7 @@
 #include "base/strings/utf_string_conversions.h"
 #include "base/threading/thread_local_storage.h"
 #include "build/build_config.h"
-#include "components/url_formatter/idn_spoof_checker.h"
+#include "components/url_formatter/spoof_checks/idn_spoof_checker.h"
 #include "net/base/registry_controlled_domains/registry_controlled_domain.h"
 #include "third_party/icu/source/common/unicode/uidna.h"
 #include "third_party/icu/source/common/unicode/utypes.h"
@@ -28,24 +28,16 @@ namespace url_formatter {
 
 namespace {
 
-enum IDNConversionStatus {
-  // The input has no IDN component.
-  NO_IDN,
-  // The input has an IDN component but it's unsafe to display
-  // so was not converted.
-  UNSAFE_IDN,
-  // The input was successfully converted to IDN.
-  IDN_CONVERTED,
-};
-
 IDNConversionResult IDNToUnicodeWithAdjustments(
     base::StringPiece host,
     base::OffsetAdjuster::Adjustments* adjustments);
 
-IDNConversionStatus IDNToUnicodeOneComponent(const base::char16* comp,
-                                             size_t comp_len,
-                                             bool is_tld_ascii,
-                                             base::string16* out);
+bool IDNToUnicodeOneComponent(const base::char16* comp,
+                              size_t comp_len,
+                              bool is_tld_ascii,
+                              bool enable_spoof_checks,
+                              base::string16* out,
+                              bool* has_idn_component);
 
 class AppendComponentTransform {
  public:
@@ -204,6 +196,12 @@ base::string16 FormatViewSourceUrl(
   const char kViewSource[] = "view-source:";
   const size_t kViewSourceLength = base::size(kViewSource) - 1;
 
+  // The URL embedded within view-source should never have destructive elisions
+  // applied to it. Users of view-source likely want to see the full URL.
+  format_types &= ~kFormatUrlOmitHTTPS;
+  format_types &= ~kFormatUrlOmitTrivialSubdomains;
+  format_types &= ~kFormatUrlTrimAfterHost;
+
   // Format the underlying URL and record adjustments.
   const std::string& url_str(url.possibly_invalid_spec());
   adjustments->clear();
@@ -236,11 +234,10 @@ base::string16 FormatViewSourceUrl(
 base::LazyInstance<IDNSpoofChecker>::Leaky g_idn_spoof_checker =
     LAZY_INSTANCE_INITIALIZER;
 
-// TODO(brettw): We may want to skip this step in the case of file URLs to
-// allow unicode UNC hostnames regardless of encodings.
-IDNConversionResult IDNToUnicodeWithAdjustments(
+IDNConversionResult IDNToUnicodeWithAdjustmentsImpl(
     base::StringPiece host,
-    base::OffsetAdjuster::Adjustments* adjustments) {
+    base::OffsetAdjuster::Adjustments* adjustments,
+    bool enable_spoof_checks) {
   if (adjustments)
     adjustments->clear();
   // Convert the ASCII input to a base::string16 for ICU.
@@ -271,12 +268,11 @@ IDNConversionResult IDNToUnicodeWithAdjustments(
     bool converted_idn = false;
     if (component_end > component_start) {
       // Add the substring that we just found.
-      IDNConversionStatus status =
-          IDNToUnicodeOneComponent(input16.data() + component_start,
-                                   component_length, is_tld_ascii, &out16);
-      converted_idn = (status == IDN_CONVERTED);
-      result.has_idn_component |=
-          (status == IDN_CONVERTED || status == UNSAFE_IDN);
+      bool has_idn_component = false;
+      converted_idn = IDNToUnicodeOneComponent(
+          input16.data() + component_start, component_length, is_tld_ascii,
+          enable_spoof_checks, &out16, &has_idn_component);
+      result.has_idn_component |= has_idn_component;
     }
     size_t new_component_length = out16.length() - new_component_start;
 
@@ -296,7 +292,7 @@ IDNConversionResult IDNToUnicodeWithAdjustments(
   if (result.has_idn_component) {
     result.matching_top_domain =
         g_idn_spoof_checker.Get().GetSimilarTopDomain(out16);
-    if (!result.matching_top_domain.empty()) {
+    if (enable_spoof_checks && !result.matching_top_domain.empty()) {
       if (adjustments)
         adjustments->clear();
       result.result = input16;
@@ -306,6 +302,20 @@ IDNConversionResult IDNToUnicodeWithAdjustments(
   return result;
 }
 
+// TODO(brettw): We may want to skip this step in the case of file URLs to
+// allow unicode UNC hostnames regardless of encodings.
+IDNConversionResult IDNToUnicodeWithAdjustments(
+    base::StringPiece host,
+    base::OffsetAdjuster::Adjustments* adjustments) {
+  return IDNToUnicodeWithAdjustmentsImpl(host, adjustments, true);
+}
+
+IDNConversionResult UnsafeIDNToUnicodeWithAdjustments(
+    base::StringPiece host,
+    base::OffsetAdjuster::Adjustments* adjustments) {
+  return IDNToUnicodeWithAdjustmentsImpl(host, adjustments, false);
+}
+
 // Returns true if the given Unicode host component is safe to display to the
 // user. Note that this function does not deal with pure ASCII domain labels at
 // all even though it's possible to make up look-alike labels with ASCII
@@ -340,8 +350,12 @@ struct UIDNAWrapper {
     // TODO(jungshik): Change options as different parties (browsers,
     // registrars, search engines) converge toward a consensus.
     value = uidna_openUTS46(UIDNA_CHECK_BIDI, &err);
-    if (U_FAILURE(err))
-      value = nullptr;
+    CHECK(U_SUCCESS(err)) << "failed to open UTS46 data with error: "
+                          << u_errorName(err)
+                          << ". If you see this error message in a test "
+                          << "environment your test environment likely lacks "
+                          << "the required data tables for libicu. See "
+                          << "https://crbug.com/778929.";
   }
 
   UIDNA* value;
@@ -350,61 +364,71 @@ struct UIDNAWrapper {
 base::LazyInstance<UIDNAWrapper>::Leaky g_uidna = LAZY_INSTANCE_INITIALIZER;
 
 // Converts one component (label) of a host (between dots) to Unicode if safe.
+// If |enable_spoof_checks| is false and input is valid unicode, skips spoof
+// checks and always converts to unicode.
 // The result will be APPENDED to the given output string and will be the
 // same as the input if it is not IDN in ACE/punycode or the IDN is unsafe to
 // display.
-// Returns whether any conversion was performed.
-IDNConversionStatus IDNToUnicodeOneComponent(const base::char16* comp,
-                                             size_t comp_len,
-                                             bool is_tld_ascii,
-                                             base::string16* out) {
+// Returns true if conversion was made. Sets |has_idn_component| to true if the
+// input has IDN, regardless of whether it was converted to unicode or not.
+bool IDNToUnicodeOneComponent(const base::char16* comp,
+                              size_t comp_len,
+                              bool is_tld_ascii,
+                              bool enable_spoof_checks,
+                              base::string16* out,
+                              bool* has_idn_component) {
   DCHECK(out);
+  DCHECK(has_idn_component);
+  *has_idn_component = false;
   if (comp_len == 0)
-    return NO_IDN;
+    return false;
 
-  IDNConversionStatus idn_status = NO_IDN;
-  // Only transform if the input can be an IDN component.
+  // Early return if the input cannot be an IDN component.
   static const base::char16 kIdnPrefix[] = {'x', 'n', '-', '-'};
-  if ((comp_len > base::size(kIdnPrefix)) &&
-      !memcmp(comp, kIdnPrefix, sizeof(kIdnPrefix))) {
-    UIDNA* uidna = g_uidna.Get().value;
-    DCHECK(uidna != nullptr);
-    size_t original_length = out->length();
-    int32_t output_length = 64;
-    UIDNAInfo info = UIDNA_INFO_INITIALIZER;
-    UErrorCode status;
-    do {
-      out->resize(original_length + output_length);
-      status = U_ZERO_ERROR;
-      // This returns the actual length required. If this is more than 64
-      // code units, |status| will be U_BUFFER_OVERFLOW_ERROR and we'll try
-      // the conversion again, but with a sufficiently large buffer.
-      output_length = uidna_labelToUnicode(
-          uidna, (const UChar*)comp, static_cast<int32_t>(comp_len), (UChar*)&(*out)[original_length],
-          output_length, &info, &status);
-    } while ((status == U_BUFFER_OVERFLOW_ERROR && info.errors == 0));
-
-    if (U_SUCCESS(status) && info.errors == 0) {
-      // Converted successfully. Ensure that the converted component
-      // can be safely displayed to the user.
-      out->resize(original_length + output_length);
-      if (IsIDNComponentSafe(
-              base::StringPiece16(out->data() + original_length,
-                                  base::checked_cast<size_t>(output_length)),
-              is_tld_ascii)) {
-        return IDN_CONVERTED;
-      }
-      idn_status = UNSAFE_IDN;
-    }
-
-    // Something went wrong. Revert to original string.
-    out->resize(original_length);
+  if (comp_len <= base::size(kIdnPrefix) ||
+      memcmp(comp, kIdnPrefix, sizeof(kIdnPrefix)) != 0) {
+    out->append(comp, comp_len);
+    return false;
   }
 
-  // We get here with no IDN or on error, in which case we just append the
-  // literal input.
+  UIDNA* uidna = g_uidna.Get().value;
+  DCHECK(uidna != nullptr);
+  size_t original_length = out->length();
+  int32_t output_length = 64;
+  UIDNAInfo info = UIDNA_INFO_INITIALIZER;
+  UErrorCode status;
+  do {
+    out->resize(original_length + output_length);
+    status = U_ZERO_ERROR;
+    // This returns the actual length required. If this is more than 64
+    // code units, |status| will be U_BUFFER_OVERFLOW_ERROR and we'll try
+    // the conversion again, but with a sufficiently large buffer.
+    output_length = uidna_labelToUnicode(
+        uidna, comp, static_cast<int32_t>(comp_len), &(*out)[original_length],
+        output_length, &info, &status);
+  } while ((status == U_BUFFER_OVERFLOW_ERROR && info.errors == 0));
+
+  if (U_SUCCESS(status) && info.errors == 0) {
+    *has_idn_component = true;
+    // Converted successfully. Ensure that the converted component
+    // can be safely displayed to the user.
+    out->resize(original_length + output_length);
+    if (!enable_spoof_checks) {
+      return true;
+    }
+    if (IsIDNComponentSafe(
+            base::StringPiece16(out->data() + original_length,
+                                base::checked_cast<size_t>(output_length)),
+            is_tld_ascii)) {
+      return true;
+    }
+  }
+
+  // We get here with no IDN or on error, in which case we just revert to
+  // original string and append the literal input.
+  out->resize(original_length);
   out->append(comp, comp_len);
-  return idn_status;
+  return false;
 }
 
 }  // namespace
@@ -664,8 +688,8 @@ void AppendFormattedHost(const GURL& url, base::string16* output) {
       HostComponentTransform(false), output, nullptr, nullptr);
 }
 
-IDNConversionResult IDNToUnicodeWithDetails(base::StringPiece host) {
-  return IDNToUnicodeWithAdjustments(host, nullptr);
+IDNConversionResult UnsafeIDNToUnicodeWithDetails(base::StringPiece host) {
+  return UnsafeIDNToUnicodeWithAdjustments(host, nullptr);
 }
 
 base::string16 IDNToUnicode(base::StringPiece host) {
