@@ -83,7 +83,7 @@ int CompareDomains(base::StringPiece lhs_domain, base::StringPiece rhs_domain);
 // Increase this value when introducing an incompatible change to the
 // UrlPatternIndex schema (flat/url_pattern_index.fbs). url_pattern_index
 // clients can use this as a signal to rebuild rulesets.
-constexpr int kUrlPatternIndexFormatVersion = 4;
+constexpr int kUrlPatternIndexFormatVersion = 5;
 
 // The class used to construct an index over the URL patterns of a set of URL
 // rules. The rules themselves need to be converted to FlatBuffers format by the
@@ -145,6 +145,8 @@ class UrlPatternIndexMatcher {
   // nullptr, then all requests return no match.
   explicit UrlPatternIndexMatcher(const flat::UrlPatternIndex* flat_index);
   ~UrlPatternIndexMatcher();
+  UrlPatternIndexMatcher(UrlPatternIndexMatcher&&);
+  UrlPatternIndexMatcher& operator=(UrlPatternIndexMatcher&&);
 
   // If the index contains one or more UrlRules that match the request, returns
   // one of them, depending on the |strategy|. Otherwise, returns nullptr.
