@@ -7,11 +7,13 @@
 #include <algorithm>
 #include <utility>
 
+#include "base/bind.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/stl_util.h"
 #include "base/time/time.h"
 #include "components/viz/common/frame_sinks/begin_frame_source.h"
 #include "components/viz/common/quads/compositor_frame.h"
+#include "components/viz/common/resources/bitmap_allocation.h"
 #include "components/viz/common/surfaces/surface_info.h"
 #include "components/viz/service/display/display.h"
 #include "components/viz/service/display/shared_bitmap_manager.h"
@@ -35,8 +37,7 @@ CompositorFrameSinkSupport::CompositorFrameSinkSupport(
       surface_resource_holder_(this),
       is_root_(is_root),
       needs_sync_tokens_(needs_sync_tokens),
-      allow_copy_output_requests_(is_root),
-      weak_factory_(this) {
+      allow_copy_output_requests_(is_root) {
   // This may result in SetBeginFrameSource() being called.
   frame_sink_manager_->RegisterCompositorFrameSinkSupport(frame_sink_id_, this);
 }
@@ -46,7 +47,7 @@ CompositorFrameSinkSupport::~CompositorFrameSinkSupport() {
   // BeginFrameSource does not call into |this| after it's deleted.
   callback_received_begin_frame_ = true;
   callback_received_receive_ack_ = true;
-  presentation_feedbacks_.clear();
+  frame_timing_details_.clear();
   SetNeedsBeginFrame(false);
 
   // For display root surfaces the surface is no longer going to be visible.
@@ -60,7 +61,7 @@ CompositorFrameSinkSupport::~CompositorFrameSinkSupport() {
   if (last_activated_surface_id_.is_valid())
     EvictLastActiveSurface();
   if (last_created_surface_id_.is_valid())
-    surface_manager_->DestroySurface(last_created_surface_id_);
+    surface_manager_->MarkSurfaceForDestruction(last_created_surface_id_);
   frame_sink_manager_->UnregisterCompositorFrameSinkSupport(frame_sink_id_);
 
   // The display compositor has ownership of shared memory for each
@@ -72,7 +73,15 @@ CompositorFrameSinkSupport::~CompositorFrameSinkSupport() {
   // No video capture clients should remain after calling
   // UnregisterCompositorFrameSinkSupport().
   DCHECK(capture_clients_.empty());
-  DCHECK(!added_frame_observer_);
+
+  if (begin_frame_source_ && added_frame_observer_)
+    begin_frame_source_->RemoveObserver(this);
+}
+
+FrameTimingDetailsMap CompositorFrameSinkSupport::TakeFrameTimingDetailsMap() {
+  FrameTimingDetailsMap map;
+  map.swap(frame_timing_details_);
+  return map;
 }
 
 void CompositorFrameSinkSupport::SetUpHitTest(
@@ -101,37 +110,36 @@ void CompositorFrameSinkSupport::SetBeginFrameSource(
 void CompositorFrameSinkSupport::OnSurfaceActivated(Surface* surface) {
   DCHECK(surface);
   DCHECK(surface->HasActiveFrame());
+  DCHECK(!surface->HasPendingFrame());
 
-  const LocalSurfaceId& local_surface_id =
-      surface->surface_id().local_surface_id();
-  const LocalSurfaceId& last_activated_local_surface_id =
-      last_activated_surface_id_.local_surface_id();
+  pending_surfaces_.erase(surface);
+  if (pending_surfaces_.empty())
+    UpdateNeedsBeginFramesInternal();
 
-  if (!last_activated_surface_id_.is_valid() ||
-      local_surface_id > last_activated_local_surface_id) {
-    if (last_activated_surface_id_.is_valid()) {
-      CHECK_GE(local_surface_id.parent_sequence_number(),
-               last_activated_local_surface_id.parent_sequence_number());
-      CHECK_GE(local_surface_id.child_sequence_number(),
-               last_activated_local_surface_id.child_sequence_number());
+  if (surface->surface_id() == last_activated_surface_id_)
+    return;
 
-      Surface* prev_surface =
-          surface_manager_->GetSurfaceForId(last_activated_surface_id_);
-      DCHECK(prev_surface);
-      surface->SetPreviousFrameSurface(prev_surface);
-      surface_manager_->DestroySurface(prev_surface->surface_id());
-    }
+  Surface* previous_surface =
+      surface_manager_->GetSurfaceForId(last_activated_surface_id_);
+
+  if (!previous_surface) {
     last_activated_surface_id_ = surface->surface_id();
-  } else if (surface->surface_id() < last_activated_surface_id_) {
+  } else if (previous_surface->GetActiveFrameIndex() <
+             surface->GetActiveFrameIndex()) {
+    surface_manager_->MarkSurfaceForDestruction(last_activated_surface_id_);
+    last_activated_surface_id_ = surface->surface_id();
+    // TODO(samans): Why is this not done when creating the surface?
+    surface->SetPreviousFrameSurface(previous_surface);
+  } else {
+    DCHECK_GT(previous_surface->GetActiveFrameIndex(),
+              surface->GetActiveFrameIndex());
     // We can get into a situation where a child-initiated synchronization is
     // deferred until after a parent-initiated synchronization happens resulting
     // in activations happening out of order. In that case, we simply discard
     // the stale surface.
-    surface_manager_->DestroySurface(surface->surface_id());
+    surface_manager_->MarkSurfaceForDestruction(surface->surface_id());
   }
 
-  DCHECK(surface->HasActiveFrame());
-
   // Check if this is a display root surface and the SurfaceId is changing.
   if (is_root_ && (!referenced_local_surface_id_ ||
                    *referenced_local_surface_id_ !=
@@ -142,6 +150,12 @@ void CompositorFrameSinkSupport::OnSurfaceActivated(Surface* surface) {
   MaybeEvictSurfaces();
 }
 
+void CompositorFrameSinkSupport::OnSurfaceDrawn(Surface* surface) {
+  if (last_drawn_frame_index_ >= surface->GetActiveFrameIndex())
+    return;
+  last_drawn_frame_index_ = surface->GetActiveFrameIndex();
+}
+
 void CompositorFrameSinkSupport::OnFrameTokenChanged(uint32_t frame_token) {
   frame_sink_manager_->OnFrameTokenChanged(frame_sink_id_, frame_token);
 }
@@ -170,7 +184,9 @@ void CompositorFrameSinkSupport::OnSurfaceAggregatedDamage(
   }
 }
 
-void CompositorFrameSinkSupport::OnSurfaceDiscarded(Surface* surface) {
+void CompositorFrameSinkSupport::OnSurfaceDestroyed(Surface* surface) {
+  pending_surfaces_.erase(surface);
+
   if (surface->surface_id() == last_activated_surface_id_)
     last_activated_surface_id_ = SurfaceId();
 
@@ -178,6 +194,16 @@ void CompositorFrameSinkSupport::OnSurfaceDiscarded(Surface* surface) {
     last_created_surface_id_ = SurfaceId();
 }
 
+void CompositorFrameSinkSupport::OnSurfacePresented(
+    uint32_t frame_token,
+    const gfx::PresentationFeedback& feedback) {
+  DidPresentCompositorFrame(frame_token, feedback);
+}
+
+bool CompositorFrameSinkSupport::NeedsSyncTokens() const {
+  return needs_sync_tokens_;
+}
+
 void CompositorFrameSinkSupport::RefResources(
     const std::vector<TransferableResource>& resources) {
   surface_resource_holder_.RefResources(resources);
@@ -225,22 +251,19 @@ CompositorFrameSinkSupport::TakeCopyOutputRequests(
 }
 
 void CompositorFrameSinkSupport::EvictSurface(const LocalSurfaceId& id) {
-  DCHECK_GE(id.parent_sequence_number(), last_evicted_parent_sequence_number_);
-  last_evicted_parent_sequence_number_ = id.parent_sequence_number();
+  DCHECK(id.embed_token() != last_evicted_local_surface_id_.embed_token() ||
+         id.parent_sequence_number() >=
+             last_evicted_local_surface_id_.parent_sequence_number());
+  last_evicted_local_surface_id_ = id;
   surface_manager_->DropTemporaryReference(SurfaceId(frame_sink_id_, id));
   MaybeEvictSurfaces();
 }
 
 void CompositorFrameSinkSupport::MaybeEvictSurfaces() {
-  if (last_activated_surface_id_.is_valid() &&
-      last_activated_surface_id_.local_surface_id().parent_sequence_number() <=
-          last_evicted_parent_sequence_number_) {
+  if (IsEvicted(last_activated_surface_id_.local_surface_id()))
     EvictLastActiveSurface();
-  }
-  if (last_created_surface_id_.is_valid() &&
-      last_created_surface_id_.local_surface_id().parent_sequence_number() <=
-          last_evicted_parent_sequence_number_) {
-    surface_manager_->DestroySurface(last_created_surface_id_);
+  if (IsEvicted(last_created_surface_id_.local_surface_id())) {
+    surface_manager_->MarkSurfaceForDestruction(last_created_surface_id_);
     last_created_surface_id_ = SurfaceId();
   }
 }
@@ -250,7 +273,7 @@ void CompositorFrameSinkSupport::EvictLastActiveSurface() {
   if (last_created_surface_id_ == last_activated_surface_id_)
     last_created_surface_id_ = SurfaceId();
   last_activated_surface_id_ = SurfaceId();
-  surface_manager_->DestroySurface(to_destroy_surface_id);
+  surface_manager_->MarkSurfaceForDestruction(to_destroy_surface_id);
 
   // For display root surfaces the surface is no longer going to be visible.
   // Make it unreachable from the top-level root.
@@ -275,6 +298,10 @@ bool CompositorFrameSinkSupport::WantsAnimateOnlyBeginFrames() const {
   return wants_animate_only_begin_frames_;
 }
 
+bool CompositorFrameSinkSupport::IsRoot() const {
+  return is_root_;
+}
+
 void CompositorFrameSinkSupport::DidNotProduceFrame(const BeginFrameAck& ack) {
   TRACE_EVENT2("viz", "CompositorFrameSinkSupport::DidNotProduceFrame",
                "ack.source_id", ack.source_id, "ack.sequence_number",
@@ -305,11 +332,12 @@ void CompositorFrameSinkSupport::SubmitCompositorFrame(
 }
 
 bool CompositorFrameSinkSupport::DidAllocateSharedBitmap(
-    mojo::ScopedSharedBufferHandle buffer,
+    base::ReadOnlySharedMemoryRegion region,
     const SharedBitmapId& id) {
   if (!frame_sink_manager_->shared_bitmap_manager()->ChildAllocatedSharedBitmap(
-          std::move(buffer), id))
+          region.Map(), id)) {
     return false;
+  }
 
   owned_bitmaps_.insert(id);
   return true;
@@ -327,29 +355,11 @@ SubmitResult CompositorFrameSinkSupport::MaybeSubmitCompositorFrameInternal(
     base::Optional<HitTestRegionList> hit_test_region_list,
     uint64_t submit_time,
     mojom::CompositorFrameSink::SubmitCompositorFrameSyncCallback callback) {
-  TRACE_EVENT1("viz", "CompositorFrameSinkSupport::MaybeSubmitCompositorFrame",
-               "FrameSinkId", frame_sink_id_.ToString());
-
-  TRACE_EVENT_WITH_FLOW1(
+  TRACE_EVENT_WITH_FLOW2(
       "viz,benchmark", "Graphics.Pipeline",
       TRACE_ID_GLOBAL(frame.metadata.begin_frame_ack.trace_id),
       TRACE_EVENT_FLAG_FLOW_IN | TRACE_EVENT_FLAG_FLOW_OUT, "step",
-      "ReceiveCompositorFrame");
-
-  TRACE_EVENT_FLOW_END0(TRACE_DISABLED_BY_DEFAULT("cc.debug.ipc"),
-                        "SubmitCompositorFrame", local_surface_id.hash());
-
-  bool tracing_enabled;
-  TRACE_EVENT_CATEGORY_GROUP_ENABLED(TRACE_DISABLED_BY_DEFAULT("cc.debug.ipc"),
-                                     &tracing_enabled);
-  if (tracing_enabled) {
-    base::TimeDelta elapsed = base::TimeTicks::Now().since_origin() -
-                              base::TimeDelta::FromMicroseconds(submit_time);
-    TRACE_EVENT_INSTANT1(TRACE_DISABLED_BY_DEFAULT("cc.debug.ipc"),
-                         "SubmitCompositorFrame::TimeElapsed",
-                         TRACE_EVENT_SCOPE_THREAD,
-                         "elapsed time:", elapsed.InMicroseconds());
-  }
+      "ReceiveCompositorFrame", "FrameSinkId", frame_sink_id_.ToString());
 
   DCHECK(local_surface_id.is_valid());
   DCHECK(!frame.render_pass_list.empty());
@@ -390,6 +400,9 @@ SubmitResult CompositorFrameSinkSupport::MaybeSubmitCompositorFrameInternal(
 
   if (!ui::LatencyInfo::Verify(frame.metadata.latency_info,
                                "RenderWidgetHostImpl::OnSwapCompositorFrame")) {
+    for (auto& info : frame.metadata.latency_info) {
+      info.Terminate();
+    }
     std::vector<ui::LatencyInfo>().swap(frame.metadata.latency_info);
   }
   for (ui::LatencyInfo& latency : frame.metadata.latency_info) {
@@ -398,6 +411,11 @@ SubmitResult CompositorFrameSinkSupport::MaybeSubmitCompositorFrameInternal(
     }
   }
 
+  if (frame.metadata.preferred_frame_interval) {
+    frame_sink_manager_->SetPreferredFrameIntervalForFrameSinkId(
+        frame_sink_id_, *frame.metadata.preferred_frame_interval);
+  }
+
   Surface* prev_surface =
       surface_manager_->GetSurfaceForId(last_created_surface_id_);
   Surface* current_surface = nullptr;
@@ -421,8 +439,6 @@ SubmitResult CompositorFrameSinkSupport::MaybeSubmitCompositorFrameInternal(
     // to determine the freshness of a surface at aggregation time.
     const LocalSurfaceId& last_created_local_surface_id =
         last_created_surface_id_.local_surface_id();
-    bool last_surface_has_dependent_frame =
-        prev_surface && prev_surface->HasDependentFrame();
 
     bool child_initiated_synchronization_event =
         last_created_local_surface_id.is_valid() &&
@@ -440,36 +456,31 @@ SubmitResult CompositorFrameSinkSupport::MaybeSubmitCompositorFrameInternal(
              last_created_local_surface_id.parent_sequence_number() ||
          child_initiated_synchronization_event);
 
-    if (!surface_info.is_valid() || !monotonically_increasing_id) {
-      TRACE_EVENT_INSTANT0("viz", "Surface Invariants Violation",
+    DCHECK(surface_info.is_valid());
+    if (local_surface_id.embed_token() ==
+            last_created_local_surface_id.embed_token() &&
+        !monotonically_increasing_id) {
+      TRACE_EVENT_INSTANT0("viz", "LocalSurfaceId decreased",
                            TRACE_EVENT_SCOPE_THREAD);
-      return SubmitResult::SURFACE_INVARIANTS_VIOLATION;
+      return SubmitResult::SURFACE_ID_DECREASED;
     }
 
-    // If the last Surface doesn't have a dependent frame, and this frame
-    // corresponds to a child-initiated synchronization event then defer this
-    // Surface until a dependent frame arrives. This throttles child submission
-    // of CompositorFrames to the parent's embedding rate.
-    const bool block_activation_on_parent =
-        child_initiated_synchronization_event &&
-        !last_surface_has_dependent_frame;
-
     // Don't recreate a surface that was previously evicted. Drop the
     // CompositorFrame and return all its resources.
-    if (local_surface_id.parent_sequence_number() <=
-        last_evicted_parent_sequence_number_) {
+    if (IsEvicted(local_surface_id)) {
       TRACE_EVENT_INSTANT0("viz", "Submit rejected to evicted surface",
                            TRACE_EVENT_SCOPE_THREAD);
       return SubmitResult::ACCEPTED;
     }
-    current_surface = CreateSurface(surface_info, block_activation_on_parent);
-    last_created_surface_id_ = SurfaceId(frame_sink_id_, local_surface_id);
 
+    current_surface = surface_manager_->CreateSurface(
+        weak_factory_.GetWeakPtr(), surface_info);
     if (!current_surface) {
-      TRACE_EVENT_INSTANT0("viz", "Surface Invariants Violation",
+      TRACE_EVENT_INSTANT0("viz", "Surface belongs to another client",
                            TRACE_EVENT_SCOPE_THREAD);
-      return SubmitResult::SURFACE_INVARIANTS_VIOLATION;
+      return SubmitResult::SURFACE_OWNED_BY_ANOTHER_CLIENT;
     }
+    last_created_surface_id_ = SurfaceId(frame_sink_id_, local_surface_id);
 
     surface_manager_->SurfaceDamageExpected(current_surface->surface_id(),
                                             last_begin_frame_args_);
@@ -486,13 +497,21 @@ SubmitResult CompositorFrameSinkSupport::MaybeSubmitCompositorFrameInternal(
   frame_sink_manager()->SubmitHitTestRegionList(
       last_created_surface_id_, frame_index, std::move(hit_test_region_list));
 
-  bool result = current_surface->QueueFrame(
-      std::move(frame), frame_index, std::move(frame_rejected_callback),
-      base::BindOnce(&CompositorFrameSinkSupport::DidPresentCompositorFrame,
-                     weak_factory_.GetWeakPtr(), frame.metadata.frame_token));
-  if (!result) {
-    TRACE_EVENT_INSTANT0("viz", "QueueFrame failed", TRACE_EVENT_SCOPE_THREAD);
-    return SubmitResult::SURFACE_INVARIANTS_VIOLATION;
+  Surface::QueueFrameResult result = current_surface->QueueFrame(
+      std::move(frame), frame_index, std::move(frame_rejected_callback));
+  switch (result) {
+    case Surface::QueueFrameResult::REJECTED:
+      TRACE_EVENT_INSTANT0("viz", "QueueFrame failed",
+                           TRACE_EVENT_SCOPE_THREAD);
+      return SubmitResult::SIZE_MISMATCH;
+    case Surface::QueueFrameResult::ACCEPTED_PENDING:
+      // Make sure we periodically check if the frame should activate.
+      pending_surfaces_.insert(current_surface);
+      UpdateNeedsBeginFramesInternal();
+      break;
+    case Surface::QueueFrameResult::ACCEPTED_ACTIVE:
+      // Nothing to do here.
+      break;
   }
 
   if (begin_frame_source_)
@@ -539,8 +558,9 @@ void CompositorFrameSinkSupport::DidPresentCompositorFrame(
     uint32_t presentation_token,
     const gfx::PresentationFeedback& feedback) {
   DCHECK(presentation_token);
-  DCHECK(!presentation_feedbacks_.count(presentation_token));
-  presentation_feedbacks_.emplace(presentation_token, feedback);
+  FrameTimingDetails details;
+  details.presentation_feedback = feedback;
+  frame_timing_details_.emplace(presentation_token, details);
   UpdateNeedsBeginFramesInternal();
 }
 
@@ -581,6 +601,8 @@ void CompositorFrameSinkSupport::OnBeginFrame(const BeginFrameArgs& args) {
     HandleCallback();
   }
 
+  CheckPendingSurfaces();
+
   if (client_ && ShouldSendBeginFrame(args.frame_time)) {
     BeginFrameArgs copy_args = args;
     copy_args.trace_id = ComputeTraceId();
@@ -589,8 +611,8 @@ void CompositorFrameSinkSupport::OnBeginFrame(const BeginFrameArgs& args) {
                            TRACE_EVENT_FLAG_FLOW_OUT, "step",
                            "IssueBeginFrame");
     last_frame_time_ = args.frame_time;
-    client_->OnBeginFrame(copy_args, std::move(presentation_feedbacks_));
-    presentation_feedbacks_.clear();
+    client_->OnBeginFrame(copy_args, std::move(frame_timing_details_));
+    frame_timing_details_.clear();
     UpdateNeedsBeginFramesInternal();
   }
 }
@@ -610,9 +632,10 @@ void CompositorFrameSinkSupport::UpdateNeedsBeginFramesInternal() {
     return;
 
   // We require a begin frame if there's a callback pending, or if the client
-  // requested it, or if the client needs to get some presentation feedbacks.
+  // requested it, or if the client needs to get some frame timing details.
   bool needs_begin_frame =
-      client_needs_begin_frame_ || !presentation_feedbacks_.empty() ||
+      client_needs_begin_frame_ || !frame_timing_details_.empty() ||
+      !pending_surfaces_.empty() ||
       (compositor_frame_callback_ && !callback_received_begin_frame_);
 
   if (needs_begin_frame == added_frame_observer_)
@@ -625,15 +648,6 @@ void CompositorFrameSinkSupport::UpdateNeedsBeginFramesInternal() {
     begin_frame_source_->RemoveObserver(this);
 }
 
-Surface* CompositorFrameSinkSupport::CreateSurface(
-    const SurfaceInfo& surface_info,
-    bool block_activation_on_parent) {
-  return surface_manager_->CreateSurface(
-      weak_factory_.GetWeakPtr(), surface_info,
-      frame_sink_manager_->GetPrimaryBeginFrameSource(), needs_sync_tokens_,
-      block_activation_on_parent);
-}
-
 SubmitResult CompositorFrameSinkSupport::MaybeSubmitCompositorFrame(
     const LocalSurfaceId& local_surface_id,
     CompositorFrame frame,
@@ -650,7 +664,7 @@ SubmitResult CompositorFrameSinkSupport::MaybeSubmitCompositorFrame(
 
 void CompositorFrameSinkSupport::AttachCaptureClient(
     CapturableFrameSink::Client* client) {
-  DCHECK(!base::ContainsValue(capture_clients_, client));
+  DCHECK(!base::Contains(capture_clients_, client));
   capture_clients_.push_back(client);
 }
 
@@ -715,8 +729,12 @@ const char* CompositorFrameSinkSupport::GetSubmitResultAsString(
       return "Accepted";
     case SubmitResult::COPY_OUTPUT_REQUESTS_NOT_ALLOWED:
       return "CopyOutputRequests not allowed";
-    case SubmitResult::SURFACE_INVARIANTS_VIOLATION:
-      return "Surface invariants violation";
+    case SubmitResult::SIZE_MISMATCH:
+      return "CompositorFrame size doesn't match surface size";
+    case SubmitResult::SURFACE_ID_DECREASED:
+      return "LocalSurfaceId sequence numbers decreased";
+    case SubmitResult::SURFACE_OWNED_BY_ANOTHER_CLIENT:
+      return "Surface belongs to another client";
   }
   NOTREACHED();
   return nullptr;
@@ -733,9 +751,9 @@ int64_t CompositorFrameSinkSupport::ComputeTraceId() {
 
 bool CompositorFrameSinkSupport::ShouldSendBeginFrame(
     base::TimeTicks frame_time) {
-  // If there are pending presentation feedbacks from the previous frame(s),
+  // If there are pending timing details from the previous frame(s),
   // then the client needs to receive the begin-frame.
-  if (!presentation_feedbacks_.empty())
+  if (!frame_timing_details_.empty())
     return true;
 
   if (!client_needs_begin_frame_)
@@ -744,20 +762,49 @@ bool CompositorFrameSinkSupport::ShouldSendBeginFrame(
   if (!last_activated_surface_id_.is_valid())
     return true;
 
-  Surface* surface =
-      surface_manager_->GetSurfaceForId(last_activated_surface_id_);
-  // If client has not submitted any frames, or the first frame submitted is
-  // yet to be embedded, then allow the begin-frame to be dispatched to the
-  // client.
-  if (!surface || !surface->seen_first_surface_embedding())
+  // We should never throttle BeginFrames if there is another client waiting for
+  // this client to submit a frame.
+  if (surface_manager_->HasBlockedEmbedder(frame_sink_id_))
     return true;
 
-  // Send begin-frames if the client has requested for it, and the previously
-  // submitted frame has already been drawn, or if the previous begin-frame was
-  // sent more than 1 second ago.
+  Surface* surface =
+      surface_manager_->GetSurfaceForId(last_activated_surface_id_);
+
+  DCHECK(surface);
+  DCHECK(surface->HasActiveFrame());
+
+  uint64_t active_frame_index = surface->GetActiveFrameIndex();
+
+  // Since we have an active frame, and frame indexes strictly increase during
+  // the lifetime of the CompositorFrameSinkSupport, our active frame index
+  // must be at least as large as our last drawn frame index.
+  DCHECK_GE(active_frame_index, last_drawn_frame_index_);
+
+  uint64_t num_undrawn_frames = active_frame_index - last_drawn_frame_index_;
+  if (num_undrawn_frames <= kUndrawnFrameLimit)
+    return true;
+
+  // Send begin-frames if the previous begin-frame was sent more than 1 second
+  // ago.
   constexpr base::TimeDelta throttled_rate = base::TimeDelta::FromSeconds(1);
-  return !surface->HasUndrawnActiveFrame() ||
-         (frame_time - last_frame_time_) >= throttled_rate;
+  return (frame_time - last_frame_time_) >= throttled_rate;
+}
+
+bool CompositorFrameSinkSupport::IsEvicted(
+    const LocalSurfaceId& local_surface_id) const {
+  return local_surface_id.embed_token() ==
+             last_evicted_local_surface_id_.embed_token() &&
+         local_surface_id.parent_sequence_number() <=
+             last_evicted_local_surface_id_.parent_sequence_number();
+}
+
+void CompositorFrameSinkSupport::CheckPendingSurfaces() {
+  if (pending_surfaces_.empty())
+    return;
+  base::flat_set<Surface*> pending_surfaces(pending_surfaces_);
+  for (Surface* surface : pending_surfaces) {
+    surface->ActivateIfDeadlinePassed();
+  }
 }
 
 }  // namespace viz
