@@ -7,10 +7,10 @@
 #include <utility>
 
 #include "base/atomic_sequence_num.h"
-#include "content/public/common/service_manager_connection.h"
-#include "jni/NfcHost_jni.h"
-#include "services/device/public/interfaces/constants.mojom.h"
-#include "services/device/public/interfaces/nfc.mojom.h"
+#include "content/public/android/content_jni_headers/NfcHost_jni.h"
+#include "content/public/browser/system_connector.h"
+#include "services/device/public/mojom/constants.mojom.h"
+#include "services/device/public/mojom/nfc.mojom.h"
 #include "services/service_manager/public/cpp/connector.h"
 
 namespace content {
@@ -28,11 +28,10 @@ NFCHost::NFCHost(WebContents* web_contents)
   JNIEnv* env = base::android::AttachCurrentThread();
 
   java_nfc_host_.Reset(
-      Java_NfcHost_create(env, web_contents_->GetJavaWebContents().obj(), id_));
+      Java_NfcHost_create(env, web_contents_->GetJavaWebContents(), id_));
 
-  if (ServiceManagerConnection::GetForProcess()) {
-    service_manager::Connector* connector =
-        ServiceManagerConnection::GetForProcess()->GetConnector();
+  service_manager::Connector* connector = content::GetSystemConnector();
+  if (connector) {
     connector->BindInterface(device::mojom::kServiceName,
                              mojo::MakeRequest(&nfc_provider_));
   }
