@@ -7,6 +7,7 @@
 #include <utility>
 
 #include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/command_line.h"
 #include "content/browser/appcache/appcache.h"
 #include "content/browser/appcache/appcache_backend_impl.h"
@@ -18,13 +19,21 @@
 #include "content/browser/appcache/appcache_url_loader_job.h"
 #include "content/browser/appcache/appcache_url_loader_request.h"
 #include "content/browser/appcache/appcache_url_request_job.h"
-#include "content/browser/service_worker/service_worker_request_handler.h"
-#include "content/public/common/content_features.h"
+#include "content/browser/navigation_subresource_loader_params.h"
 #include "net/url_request/url_request.h"
 #include "net/url_request/url_request_job.h"
+#include "services/network/public/cpp/features.h"
+#include "third_party/blink/public/common/features.h"
+#include "third_party/blink/public/mojom/appcache/appcache_info.mojom.h"
 
 namespace content {
 
+namespace {
+
+bool g_running_in_tests = false;
+
+}  // namespace
+
 AppCacheRequestHandler::AppCacheRequestHandler(
     AppCacheHost* host,
     ResourceType resource_type,
@@ -40,9 +49,7 @@ AppCacheRequestHandler::AppCacheRequestHandler(
       cache_entry_not_found_(false),
       is_delivering_network_response_(false),
       maybe_load_resource_executed_(false),
-      old_process_id_(0),
-      old_host_id_(kAppCacheNoHostId),
-      cache_id_(kAppCacheNoCacheId),
+      cache_id_(blink::mojom::kAppCacheNoCacheId),
       service_(host_->service()),
       request_(std::move(request)) {
   DCHECK(host_);
@@ -58,6 +65,9 @@ AppCacheRequestHandler::~AppCacheRequestHandler() {
   }
   if (service_)
     service_->RemoveObserver(this);
+
+  if (job_ && job_->AsURLLoaderJob())
+    job_->AsURLLoaderJob()->DeleteIfNeeded();
 }
 
 AppCacheStorage* AppCacheRequestHandler::storage() const {
@@ -71,7 +81,7 @@ AppCacheJob* AppCacheRequestHandler::MaybeLoadResource(
   if (!host_ ||
       !AppCacheRequest::IsSchemeAndMethodSupportedForAppCache(request_.get()) ||
       cache_entry_not_found_) {
-    return NULL;
+    return nullptr;
   }
 
   // This method can get called multiple times over the life
@@ -83,14 +93,14 @@ AppCacheJob* AppCacheRequestHandler::MaybeLoadResource(
   // This time through, we return NULL so the request hits the wire.
   if (is_delivering_network_response_) {
     is_delivering_network_response_ = false;
-    return NULL;
+    return nullptr;
   }
 
   // Clear out our 'found' fields since we're starting a request for a
   // new resource, any values in those fields are no longer valid.
   found_entry_ = AppCacheEntry();
   found_fallback_entry_ = AppCacheEntry();
-  found_cache_id_ = kAppCacheNoCacheId;
+  found_cache_id_ = blink::mojom::kAppCacheNoCacheId;
   found_manifest_url_ = GURL();
   found_network_namespace_ = false;
 
@@ -103,9 +113,14 @@ AppCacheJob* AppCacheRequestHandler::MaybeLoadResource(
   // If its been setup to deliver a network response, we can just delete
   // it now and return NULL instead to achieve that since it couldn't
   // have been started yet.
-  if (job && job->IsDeliveringNetworkResponse() && !job->AsURLLoaderJob()) {
+  if (job && job->IsDeliveringNetworkResponse()) {
     DCHECK(!job->IsStarted());
-    job.reset();
+    if (job->AsURLLoaderJob()) {
+      job.release()->AsURLLoaderJob()->DeleteIfNeeded();
+      job_ = nullptr;
+    } else {
+      job.reset();
+    }
   }
 
   return job.release();
@@ -117,34 +132,33 @@ AppCacheJob* AppCacheRequestHandler::MaybeLoadFallbackForRedirect(
   if (!host_ ||
       !AppCacheRequest::IsSchemeAndMethodSupportedForAppCache(request_.get()) ||
       cache_entry_not_found_)
-    return NULL;
+    return nullptr;
   if (is_main_resource())
-    return NULL;
-  // TODO(vabr) This is a temporary fix (see crbug/141114). We should get rid of
-  // it once a more general solution to crbug/121325 is in place.
+    return nullptr;
+  // If MaybeLoadResourceExecuted did not run, this might be, e.g., a redirect
+  // caused by the Web Request API late in the loading progress. AppCache might
+  // misinterpret the rules for the new target and cause unnecessary
+  // fallbacks/errors, therefore it is better to give up on app-caching in this
+  // case. More information in https://crbug.com/141114 and the discussion at
+  // https://chromiumcodereview.appspot.com/10829356.
   if (!maybe_load_resource_executed_)
-    return NULL;
+    return nullptr;
   if (request_->GetURL().GetOrigin() == location.GetOrigin())
-    return NULL;
+    return nullptr;
 
-  // In network service land, the existing job initiates a fallback request.
-  if (!base::FeatureList::IsEnabled(features::kNetworkService)) {
-    DCHECK(!job_.get());  // our jobs never generate redirects
-  } else {
-    DCHECK(job_.get());
-  }
+  DCHECK(!job_.get());  // our jobs never generate redirects
 
   std::unique_ptr<AppCacheJob> job;
   if (found_fallback_entry_.has_response_id()) {
-    job = MaybeCreateJobForFallback(network_delegate);
     // 6.9.6, step 4: If this results in a redirect to another origin,
     // get the resource of the fallback entry.
+    job = CreateJob(network_delegate);
     DeliverAppCachedResponse(found_fallback_entry_, found_cache_id_,
                              found_manifest_url_, true,
                              found_namespace_entry_url_);
   } else if (!found_network_namespace_) {
     // 6.9.6, step 6: Fail the resource load.
-    job = MaybeCreateJobForFallback(network_delegate);
+    job = CreateJob(network_delegate);
     DeliverErrorResponse();
   } else {
     // 6.9.6 step 3 and 5: Fetch the resource normally.
@@ -158,25 +172,30 @@ AppCacheJob* AppCacheRequestHandler::MaybeLoadFallbackForResponse(
   if (!host_ ||
       !AppCacheRequest::IsSchemeAndMethodSupportedForAppCache(request_.get()) ||
       cache_entry_not_found_)
-    return NULL;
+    return nullptr;
   if (!found_fallback_entry_.has_response_id())
-    return NULL;
+    return nullptr;
 
   if (request_->IsCancelled()) {
     // 6.9.6, step 4: But not if the user canceled the download.
-    return NULL;
+    return nullptr;
   }
 
   // We don't fallback for responses that we delivered.
-  if (job_.get() && !base::FeatureList::IsEnabled(features::kNetworkService)) {
-    DCHECK(!job_->IsDeliveringNetworkResponse());
-    return NULL;
+  if (job_.get()) {
+    if (!base::FeatureList::IsEnabled(network::features::kNetworkService)) {
+      DCHECK(!job_->IsDeliveringNetworkResponse());
+      return nullptr;
+    } else if (job_->IsDeliveringAppCacheResponse() ||
+               job_->IsDeliveringErrorResponse()) {
+      return nullptr;
+    }
   }
 
   if (request_->IsSuccess()) {
     int code_major = request_->GetResponseCode() / 100;
     if (code_major !=4 && code_major != 5)
-      return NULL;
+      return nullptr;
 
     // Servers can override the fallback behavior with a response header.
     const std::string kFallbackOverrideHeader(
@@ -186,16 +205,13 @@ AppCacheJob* AppCacheRequestHandler::MaybeLoadFallbackForResponse(
     std::string header_value;
     header_value = request_->GetResponseHeaderByName(kFallbackOverrideHeader);
     if (header_value == kFallbackOverrideValue)
-      return NULL;
+      return nullptr;
   }
 
   // 6.9.6, step 4: If this results in a 4xx or 5xx status code
   // or there were network errors, get the resource of the fallback entry.
 
-  // In network service land, the job initiates a fallback request. We reuse
-  // the existing job to deliver the fallback response.
-  std::unique_ptr<AppCacheJob> job =
-      MaybeCreateJobForFallback(network_delegate);
+  std::unique_ptr<AppCacheJob> job = CreateJob(network_delegate);
 
   DeliverAppCachedResponse(found_fallback_entry_, found_cache_id_,
                            found_manifest_url_, true,
@@ -209,65 +225,38 @@ void AppCacheRequestHandler::GetExtraResponseInfo(int64_t* cache_id,
   *manifest_url = manifest_url_;
 }
 
-void AppCacheRequestHandler::PrepareForCrossSiteTransfer(int old_process_id) {
-  if (!host_)
-    return;
-  AppCacheBackendImpl* backend = host_->service()->GetBackend(old_process_id);
-  DCHECK(backend) << "appcache detected likely storage partition mismatch";
-  old_process_id_ = old_process_id;
-  old_host_id_ = host_->host_id();
-  host_for_cross_site_transfer_ = backend->TransferHostOut(host_->host_id());
-  DCHECK_EQ(host_, host_for_cross_site_transfer_.get());
-}
-
-void AppCacheRequestHandler::CompleteCrossSiteTransfer(
-    int new_process_id, int new_host_id) {
-  if (!host_for_cross_site_transfer_.get())
-    return;
-  DCHECK_EQ(host_, host_for_cross_site_transfer_.get());
-  AppCacheBackendImpl* backend = host_->service()->GetBackend(new_process_id);
-  DCHECK(backend) << "appcache detected likely storage partition mismatch";
-  backend->TransferHostIn(new_host_id,
-                          std::move(host_for_cross_site_transfer_));
-}
-
-void AppCacheRequestHandler::MaybeCompleteCrossSiteTransferInOldProcess(
-    int old_process_id) {
-  if (!host_ || !host_for_cross_site_transfer_.get() ||
-      old_process_id != old_process_id_) {
-    return;
-  }
-  CompleteCrossSiteTransfer(old_process_id_, old_host_id_);
-}
-
 // static
 std::unique_ptr<AppCacheRequestHandler>
-AppCacheRequestHandler::InitializeForNavigationNetworkService(
-    const ResourceRequest& request,
-    AppCacheNavigationHandleCore* appcache_handle_core,
-    URLLoaderFactoryGetter* url_loader_factory_getter) {
+AppCacheRequestHandler::InitializeForMainResourceNetworkService(
+    const network::ResourceRequest& request,
+    base::WeakPtr<AppCacheHost> appcache_host) {
   std::unique_ptr<AppCacheRequestHandler> handler =
-      appcache_handle_core->host()->CreateRequestHandler(
-          AppCacheURLLoaderRequest::Create(request), request.resource_type,
+      appcache_host->CreateRequestHandler(
+          std::make_unique<AppCacheURLLoaderRequest>(request),
+          static_cast<ResourceType>(request.resource_type),
           request.should_reset_appcache);
-  handler->network_url_loader_factory_getter_ = url_loader_factory_getter;
-  handler->appcache_host_ = appcache_handle_core->host()->GetWeakPtr();
+  handler->appcache_host_ = std::move(appcache_host);
   return handler;
 }
 
-void AppCacheRequestHandler::SetSubresourceRequestLoadInfo(
-    std::unique_ptr<SubresourceLoadInfo> subresource_load_info) {
-  subresource_load_info_ = std::move(subresource_load_info);
+// static
+bool AppCacheRequestHandler::IsMainResourceType(ResourceType type) {
+  // When PlzDedicatedWorker is enabled, a dedicated worker script is considered
+  // to be a main resource.
+  if (type == ResourceType::kWorker)
+    return blink::features::IsPlzDedicatedWorkerEnabled();
+  return IsResourceTypeFrame(type) || type == ResourceType::kSharedWorker;
 }
 
 void AppCacheRequestHandler::OnDestructionImminent(AppCacheHost* host) {
   storage()->CancelDelegateCallbacks(this);
-  host_ = NULL;  // no need to RemoveObserver, the host is being deleted
+  host_ = nullptr;  // no need to RemoveObserver, the host is being deleted
 
   // Since the host is being deleted, we don't have to complete any job
   // that is current running. It's destined for the bit bucket anyway.
   if (job_.get()) {
-    job_->Kill();
+    if (job_->AsURLRequestJob())
+      job_->AsURLRequestJob()->Kill();
     job_.reset();
   }
 }
@@ -276,13 +265,11 @@ void AppCacheRequestHandler::OnServiceDestructionImminent(
     AppCacheServiceImpl* service) {
   service_ = nullptr;
   if (!host_) {
-    DCHECK(!host_for_cross_site_transfer_);
     DCHECK(!job_);
     return;
   }
   host_->RemoveObserver(this);
   OnDestructionImminent(host_);
-  host_for_cross_site_transfer_.reset();
 }
 
 void AppCacheRequestHandler::DeliverAppCachedResponse(
@@ -306,23 +293,24 @@ void AppCacheRequestHandler::DeliverAppCachedResponse(
 
 void AppCacheRequestHandler::DeliverErrorResponse() {
   DCHECK(job_.get() && job_->IsWaiting());
-  DCHECK_EQ(kAppCacheNoCacheId, cache_id_);
+  DCHECK_EQ(blink::mojom::kAppCacheNoCacheId, cache_id_);
   DCHECK(manifest_url_.is_empty());
   job_->DeliverErrorResponse();
 }
 
 void AppCacheRequestHandler::DeliverNetworkResponse() {
   DCHECK(job_.get() && job_->IsWaiting());
-  DCHECK_EQ(kAppCacheNoCacheId, cache_id_);
+  DCHECK_EQ(blink::mojom::kAppCacheNoCacheId, cache_id_);
   DCHECK(manifest_url_.is_empty());
   job_->DeliverNetworkResponse();
 }
 
-void AppCacheRequestHandler::OnPrepareToRestart() {
+void AppCacheRequestHandler::OnPrepareToRestartURLRequest() {
+  DCHECK(job_->AsURLRequestJob());
   DCHECK(job_->IsDeliveringNetworkResponse() || job_->IsCacheEntryNotFound());
 
   // Any information about the source of the response is no longer relevant.
-  cache_id_ = kAppCacheNoCacheId;
+  cache_id_ = blink::mojom::kAppCacheNoCacheId;
   manifest_url_ = GURL();
 
   cache_entry_not_found_ = job_->IsCacheEntryNotFound();
@@ -335,33 +323,21 @@ void AppCacheRequestHandler::OnPrepareToRestart() {
 
 std::unique_ptr<AppCacheJob> AppCacheRequestHandler::CreateJob(
     net::NetworkDelegate* network_delegate) {
-  std::unique_ptr<AppCacheJob> job = AppCacheJob::Create(
-      is_main_resource(), host_, storage(), request_.get(), network_delegate,
-      base::Bind(&AppCacheRequestHandler::OnPrepareToRestart,
-                 base::Unretained(this)));
-  job_ = job->GetWeakPtr();
-  if (!is_main_resource() &&
-      base::FeatureList::IsEnabled(features::kNetworkService)) {
-    AppCacheURLLoaderJob* loader_job = job_->AsURLLoaderJob();
-
-    loader_job->SetSubresourceLoadInfo(
-        std::move(subresource_load_info_),
-        network_url_loader_factory_getter_.get());
+  std::unique_ptr<AppCacheJob> job;
+  if (base::FeatureList::IsEnabled(network::features::kNetworkService)) {
+    job = std::make_unique<AppCacheURLLoaderJob>(
+        request_->AsURLLoaderRequest(), storage(), std::move(loader_callback_));
+  } else {
+    job = std::make_unique<AppCacheURLRequestJob>(
+        request_->GetURLRequest(), network_delegate, storage(), host_,
+        is_main_resource(),
+        base::BindOnce(&AppCacheRequestHandler::OnPrepareToRestartURLRequest,
+                       base::Unretained(this)));
   }
-
+  job_ = job->GetWeakPtr();
   return job;
 }
 
-std::unique_ptr<AppCacheJob> AppCacheRequestHandler::MaybeCreateJobForFallback(
-    net::NetworkDelegate* network_delegate) {
-  if (!base::FeatureList::IsEnabled(features::kNetworkService))
-    return CreateJob(network_delegate);
-  // In network service land, the job initiates a fallback request. We reuse
-  // the existing job to deliver the fallback response.
-  DCHECK(job_.get());
-  return std::unique_ptr<AppCacheJob>(job_.get());
-}
-
 // Main-resource handling ----------------------------------------------
 
 std::unique_ptr<AppCacheJob> AppCacheRequestHandler::MaybeLoadMainResource(
@@ -369,30 +345,18 @@ std::unique_ptr<AppCacheJob> AppCacheRequestHandler::MaybeLoadMainResource(
   DCHECK(!job_.get());
   DCHECK(host_);
 
-  // If a page falls into the scope of a ServiceWorker, any matching AppCaches
-  // should be ignored. This depends on the ServiceWorker handler being invoked
-  // prior to the AppCache handler.
-  // TODO(ananta)
-  // We need to handle this for AppCache requests initiated for the network
-  // service
-  if (request_->GetURLRequest() &&
-      ServiceWorkerRequestHandler::IsControlledByServiceWorker(
-          request_->GetURLRequest())) {
-    host_->enable_cache_selection(false);
-    return nullptr;
-  }
-
   if (storage()->IsInitialized() &&
-      service_->storage()->usage_map()->find(request_->GetURL().GetOrigin()) ==
-          service_->storage()->usage_map()->end()) {
+      !base::Contains(service_->storage()->usage_map(),
+                      url::Origin::Create(request_->GetURL()))) {
     return nullptr;
   }
 
   host_->enable_cache_selection(true);
 
   const AppCacheHost* spawning_host =
-      (resource_type_ == RESOURCE_TYPE_SHARED_WORKER) ?
-      host_ : host_->GetSpawningHost();
+      (resource_type_ == ResourceType::kSharedWorker)
+          ? host_
+          : host_->GetSpawningHost();
   GURL preferred_manifest_url = spawning_host ?
       spawning_host->preferred_manifest_url() : GURL();
 
@@ -431,21 +395,22 @@ void AppCacheRequestHandler::OnMainResponseFound(
     if (IsResourceTypeFrame(resource_type_)) {
       host_->NotifyMainResourceBlocked(manifest_url);
     } else {
-      DCHECK_EQ(resource_type_, RESOURCE_TYPE_SHARED_WORKER);
-      host_->frontend()->OnContentBlocked(host_->host_id(), manifest_url);
+      DCHECK_EQ(resource_type_, ResourceType::kSharedWorker);
+      host_->OnContentBlocked(manifest_url);
     }
     DeliverNetworkResponse();
     return;
   }
 
   if (should_reset_appcache_ && !manifest_url.is_empty()) {
-    host_->service()->DeleteAppCacheGroup(
-        manifest_url, net::CompletionCallback());
+    host_->service()->DeleteAppCacheGroup(manifest_url,
+                                          net::CompletionOnceCallback());
     DeliverNetworkResponse();
     return;
   }
 
-  if (IsMainResourceType(resource_type_) && cache_id != kAppCacheNoCacheId) {
+  if (IsMainResourceType(resource_type_) &&
+      cache_id != blink::mojom::kAppCacheNoCacheId) {
     // AppCacheHost loads and holds a reference to the main resource cache
     // for two reasons, firstly to preload the cache into the working set
     // in advance of subresource loads happening, secondly to prevent the
@@ -473,6 +438,18 @@ void AppCacheRequestHandler::OnMainResponseFound(
   }
 }
 
+// NetworkService loading:
+void AppCacheRequestHandler::RunLoaderCallbackForMainResource(
+    LoaderCallback callback,
+    SingleRequestURLLoaderFactory::RequestHandler handler) {
+  // For now let |this| always also return the subresource loader
+  // if (and only if) this returns a non-null |start_loader_callback|
+  // for handling the main resource.
+  if (handler)
+    should_create_subresource_loader_ = true;
+  std::move(callback).Run(std::move(handler));
+}
+
 // Sub-resource handling ----------------------------------------------
 
 std::unique_ptr<AppCacheJob> AppCacheRequestHandler::MaybeLoadSubResource(
@@ -504,7 +481,7 @@ void AppCacheRequestHandler::ContinueMaybeLoadSubResource() {
   DCHECK(job_.get());
   DCHECK(host_->associated_cache() && host_->associated_cache()->is_complete());
 
-  const GURL& url = job_->GetURL();
+  const GURL& url = request_->GetURL();
   AppCache* cache = host_->associated_cache();
   storage()->FindResponseForSubRequest(
       host_->associated_cache(), url,
@@ -570,29 +547,143 @@ void AppCacheRequestHandler::OnCacheSelectionComplete(AppCacheHost* host) {
 }
 
 void AppCacheRequestHandler::MaybeCreateLoader(
-    const ResourceRequest& resource_request,
+    const network::ResourceRequest& tentative_resource_request,
+    BrowserContext* browser_context,
     ResourceContext* resource_context,
-    LoaderCallback callback) {
-  // MaybeLoadMainResource will invoke navigation_request_job's methods
-  // asynchronously via AppCacheStorage::Delegate.
-  navigation_request_job_ = MaybeLoadMainResource(nullptr);
-  if (!navigation_request_job_.get()) {
-    std::move(callback).Run(StartLoaderCallback());
-    return;
-  }
-  navigation_request_job_->AsURLLoaderJob()->set_main_resource_loader_callback(
-      std::move(callback));
+    LoaderCallback callback,
+    FallbackCallback fallback_callback) {
+  loader_callback_ =
+      base::BindOnce(&AppCacheRequestHandler::RunLoaderCallbackForMainResource,
+                     weak_factory_.GetWeakPtr(), std::move(callback));
+
+  // TODO(crbug.com/876531): Figure out how AppCache interception should
+  // interact with URLLoaderThrottles. It might be incorrect to store
+  // |tentative_resource_request| here, since throttles can rewrite headers
+  // between now and when the request handler passed to |loader_callback_| is
+  // invoked.
+  request_->AsURLLoaderRequest()->set_request(tentative_resource_request);
+
+  MaybeLoadResource(nullptr);
+  // If a job is created, the job assumes ownership of the callback and
+  // the responsibility to call it. If no job is created, we call it with
+  // a null callback to let our client know we have no loader for this request.
+  if (loader_callback_)
+    std::move(loader_callback_).Run({});
 }
 
-mojom::URLLoaderFactoryPtr
-AppCacheRequestHandler::MaybeCreateSubresourceFactory() {
-  mojom::URLLoaderFactoryPtr factory_ptr = nullptr;
+bool AppCacheRequestHandler::MaybeCreateLoaderForResponse(
+    const network::ResourceRequest& request,
+    const network::ResourceResponseHead& response,
+    mojo::ScopedDataPipeConsumerHandle* response_body,
+    network::mojom::URLLoaderPtr* loader,
+    network::mojom::URLLoaderClientRequest* client_request,
+    ThrottlingURLLoader* url_loader,
+    bool* skip_other_interceptors) {
+  // The sync interface of this method is inherited from the
+  // NavigationLoaderInterceptor class. The LoaderCallback created here is
+  // invoked synchronously in fallback cases, and only when there really is
+  // a loader to start.
+  bool was_called = false;
+  loader_callback_ = base::BindOnce(
+      [](const network::ResourceRequest& resource_request,
+         network::mojom::URLLoaderPtr* loader,
+         network::mojom::URLLoaderClientRequest* client_request,
+         bool* was_called,
+         SingleRequestURLLoaderFactory::RequestHandler handler) {
+        *was_called = true;
+        network::mojom::URLLoaderClientPtr client;
+        *client_request = mojo::MakeRequest(&client);
+        std::move(handler).Run(resource_request, mojo::MakeRequest(loader),
+                               std::move(client));
+      },
+      *(request_->AsURLLoaderRequest()->GetResourceRequest()), loader,
+      client_request, &was_called);
+  request_->AsURLLoaderRequest()->set_response(response);
+  if (!MaybeLoadFallbackForResponse(nullptr)) {
+    DCHECK(!was_called);
+    loader_callback_.Reset();
+    return false;
+  }
+  DCHECK(was_called);
+  if (IsMainResourceType(resource_type_))
+    should_create_subresource_loader_ = true;
+  return true;
+}
+
+base::Optional<SubresourceLoaderParams>
+AppCacheRequestHandler::MaybeCreateSubresourceLoaderParams() {
+  if (!should_create_subresource_loader_)
+    return base::nullopt;
 
   // The factory is destroyed when the renderer drops the connection.
-  AppCacheSubresourceURLFactory::CreateURLLoaderFactory(
-      network_url_loader_factory_getter_.get(), appcache_host_, &factory_ptr);
+  // TODO(crbug.com/955171): Replace this with Remote.
+  network::mojom::URLLoaderFactoryPtr factory_ptr;
 
-  return factory_ptr;
+  AppCacheSubresourceURLFactory::CreateURLLoaderFactory(appcache_host_,
+                                                        &factory_ptr);
+
+  SubresourceLoaderParams params;
+  params.pending_appcache_loader_factory = factory_ptr.PassInterface();
+  return base::Optional<SubresourceLoaderParams>(std::move(params));
+}
+
+void AppCacheRequestHandler::MaybeCreateSubresourceLoader(
+    const network::ResourceRequest& resource_request,
+    LoaderCallback loader_callback) {
+  DCHECK(!job_);
+  DCHECK(!is_main_resource());
+  // AppCache doesn't use the fallback_callback.
+  FallbackCallback fallback_callback = base::DoNothing();
+
+  // Subresource loads start out just like a main resource loads, but they go
+  // down different branches along the way to completion.
+  MaybeCreateLoader(resource_request, nullptr, nullptr,
+                    std::move(loader_callback), std::move(fallback_callback));
+}
+
+void AppCacheRequestHandler::MaybeFallbackForSubresourceResponse(
+    const network::ResourceResponseHead& response,
+    LoaderCallback loader_callback) {
+  DCHECK(!job_);
+  DCHECK(!is_main_resource());
+  loader_callback_ = std::move(loader_callback);
+  request_->AsURLLoaderRequest()->set_response(response);
+  MaybeLoadFallbackForResponse(nullptr);
+  if (loader_callback_)
+    std::move(loader_callback_).Run({});
+}
+
+void AppCacheRequestHandler::MaybeFallbackForSubresourceRedirect(
+    const net::RedirectInfo& redirect_info,
+    LoaderCallback loader_callback) {
+  DCHECK(!job_);
+  DCHECK(!is_main_resource());
+  loader_callback_ = std::move(loader_callback);
+  MaybeLoadFallbackForRedirect(nullptr, redirect_info.new_url);
+  if (loader_callback_)
+    std::move(loader_callback_).Run({});
+}
+
+void AppCacheRequestHandler::MaybeFollowSubresourceRedirect(
+    const net::RedirectInfo& redirect_info,
+    LoaderCallback loader_callback) {
+  DCHECK(!job_);
+  DCHECK(!is_main_resource());
+  loader_callback_ = std::move(loader_callback);
+  request_->AsURLLoaderRequest()->UpdateWithRedirectInfo(redirect_info);
+  MaybeLoadResource(nullptr);
+  if (loader_callback_)
+    std::move(loader_callback_).Run({});
+}
+
+// static
+void AppCacheRequestHandler::SetRunningInTests(bool in_tests) {
+  g_running_in_tests = in_tests;
+}
+
+// static
+bool AppCacheRequestHandler::IsRunningInTests() {
+  return g_running_in_tests;
 }
 
 }  // namespace content
