@@ -23,48 +23,31 @@
 #include "content/browser/service_worker/service_worker_object_host.h"
 #include "content/browser/service_worker/service_worker_registration.h"
 #include "content/common/content_export.h"
-#include "content/common/service_worker/service_worker_types.h"
 #include "content/public/common/resource_type.h"
 #include "mojo/public/cpp/bindings/associated_binding.h"
 #include "mojo/public/cpp/bindings/binding.h"
 #include "mojo/public/cpp/bindings/binding_set.h"
+#include "mojo/public/cpp/bindings/pending_receiver.h"
+#include "mojo/public/cpp/bindings/remote.h"
 #include "services/network/public/mojom/fetch_api.mojom.h"
-#include "services/network/public/mojom/request_context_frame_type.mojom.h"
 #include "third_party/blink/public/mojom/fetch/fetch_api_request.mojom.h"
 #include "third_party/blink/public/mojom/service_worker/service_worker_container.mojom.h"
 #include "third_party/blink/public/mojom/service_worker/service_worker_provider.mojom.h"
 #include "third_party/blink/public/mojom/service_worker/service_worker_provider_type.mojom.h"
 #include "third_party/blink/public/mojom/service_worker/service_worker_registration.mojom.h"
-#include "third_party/blink/public/platform/web_feature.mojom.h"
-
-namespace network {
-class ResourceRequestBody;
-}
+#include "third_party/blink/public/mojom/web_feature/web_feature.mojom.h"
 
 namespace service_worker_object_host_unittest {
 class ServiceWorkerObjectHostTest;
 }
 
-namespace storage {
-class BlobStorageContext;
-}
-
 namespace content {
 
 class ServiceWorkerContextCore;
 class ServiceWorkerRegistrationObjectHost;
-class ServiceWorkerRequestHandler;
 class ServiceWorkerVersion;
 class WebContents;
 
-namespace service_worker_dispatcher_host_unittest {
-class ServiceWorkerDispatcherHostTest;
-FORWARD_DECLARE_TEST(ServiceWorkerDispatcherHostTest,
-                     DispatchExtendableMessageEvent);
-FORWARD_DECLARE_TEST(ServiceWorkerDispatcherHostTest,
-                     DispatchExtendableMessageEvent_Fail);
-}  // namespace service_worker_dispatcher_host_unittest
-
 // ServiceWorkerProviderHost is the browser-process representation of a
 // renderer-process entity that can involve service workers. Currently, these
 // entities are frames or workers. So basically, one ServiceWorkerProviderHost
@@ -108,20 +91,12 @@ FORWARD_DECLARE_TEST(ServiceWorkerDispatcherHostTest,
 //
 // 1) For a client created for a navigation (for both top-level and
 // non-top-level frames), the provider host for the resulting document is
-// pre-created by the browser process. Upon navigation commit, the provider is
-// created on the renderer, which sends an OnProviderCreated IPC to establish
-// the Mojo connection.
+// pre-created by the browser process and the provider info is sent in the
+// navigation commit IPC.
 //
-// 2) For clients created by the renderer not due to navigations (shared workers
-// in the non-S13nServiceWorker case, and about:blank iframes), the provider
-// host is created and the Mojo connection is established when the provider is
-// created by the renderer process and sends an OnProviderCreated IPC.
-//
-// 3) For shared workers in the S13nServiceWorker case and for service workers,
-// the provider host is pre-created by the browser process, and information
-// about the host is sent in the start worker IPC message. The Mojo connection
-// is established when renderer process receives the start message and creates
-// the provider.
+// 2) For shared workers and for service workers, the provider host is
+// pre-created by the browser process and the provider info is sent in the start
+// worker IPC message.
 class CONTENT_EXPORT ServiceWorkerProviderHost
     : public ServiceWorkerRegistration::Listener,
       public base::SupportsWeakPtr<ServiceWorkerProviderHost>,
@@ -132,20 +107,21 @@ class CONTENT_EXPORT ServiceWorkerProviderHost
   using ExecutionReadyCallback = base::OnceClosure;
 
   // Used to pre-create a ServiceWorkerProviderHost for a navigation. The
-  // ServiceWorkerNetworkProvider will later be created in the renderer, should
+  // ServiceWorkerProviderContext will later be created in the renderer, should
   // the navigation succeed. |are_ancestors_secure| should be true for main
   // frames. Otherwise it is true iff all ancestor frames of this frame have a
-  // secure origin. |web_contents_getter| indicates the tab where the navigation
-  // is occurring.
+  // secure origin. |frame_tree_node_id| is FrameTreeNode
+  // id. |web_contents_getter| indicates the tab where the navigation is
+  // occurring.
   //
-  // The returned host is owned by |context|. Upon successful navigation, the
-  // caller should remove it from |context| and re-add it after calling
-  // CompleteNavigationInitialized() to update it with the correct process id.
-  // If navigation fails, the caller should remove it from |context|.
+  // The returned host stays alive as long as the corresponding host ptr for
+  // |host_request| stays alive.
   static base::WeakPtr<ServiceWorkerProviderHost> PreCreateNavigationHost(
       base::WeakPtr<ServiceWorkerContextCore> context,
       bool are_ancestors_secure,
-      WebContentsGetter web_contents_getter);
+      int frame_tree_node_id,
+      blink::mojom::ServiceWorkerContainerHostAssociatedRequest host_request,
+      blink::mojom::ServiceWorkerContainerAssociatedPtrInfo client_ptr_info);
 
   // Used for starting a service worker. Returns a provider host for the service
   // worker and partially fills |out_provider_info|.  The host stays alive as
@@ -159,24 +135,15 @@ class CONTENT_EXPORT ServiceWorkerProviderHost
       blink::mojom::ServiceWorkerProviderInfoForStartWorkerPtr*
           out_provider_info);
 
-  // S13nServiceWorker:
-  // Used for starting a shared worker. Returns a provider host for the shared
-  // worker and fills |out_provider_info| with info to send to the renderer to
-  // connect to the host. The host stays alive as long as this info stays alive
-  // (namely, as long as |out_provider_info->host_ptr_info| stays alive).
-  static base::WeakPtr<ServiceWorkerProviderHost> PreCreateForSharedWorker(
+  // Used for starting a web worker (dedicated worker or shared worker). Returns
+  // a provider host for the worker. The host stays alive as long as the
+  // corresponding host for |host_request| stays alive.
+  static base::WeakPtr<ServiceWorkerProviderHost> PreCreateForWebWorker(
       base::WeakPtr<ServiceWorkerContextCore> context,
       int process_id,
-      blink::mojom::ServiceWorkerProviderInfoForSharedWorkerPtr*
-          out_provider_info);
-
-  // Used to create a ServiceWorkerProviderHost when the renderer-side provider
-  // is created. This ProviderHost will be created for the process specified by
-  // |process_id|.
-  static std::unique_ptr<ServiceWorkerProviderHost> Create(
-      int process_id,
-      blink::mojom::ServiceWorkerProviderHostInfoPtr info,
-      base::WeakPtr<ServiceWorkerContextCore> context);
+      blink::mojom::ServiceWorkerProviderType provider_type,
+      blink::mojom::ServiceWorkerContainerHostAssociatedRequest host_request,
+      blink::mojom::ServiceWorkerContainerAssociatedPtrInfo client_ptr_info);
 
   ~ServiceWorkerProviderHost() override;
 
@@ -186,21 +153,21 @@ class CONTENT_EXPORT ServiceWorkerProviderHost
   }
   base::TimeTicks create_time() const { return create_time_; }
   int process_id() const { return render_process_id_; }
-  int provider_id() const { return info_->provider_id; }
-  int frame_id() const;
-  int route_id() const { return info_->route_id; }
+  int provider_id() const { return provider_id_; }
+  int frame_id() const { return frame_id_; }
   const WebContentsGetter& web_contents_getter() const {
     return web_contents_getter_;
   }
+  int frame_tree_node_id() const { return frame_tree_node_id_; }
 
-  bool is_parent_frame_secure() const { return info_->is_parent_frame_secure; }
+  bool is_parent_frame_secure() const { return is_parent_frame_secure_; }
 
   // Returns whether this provider host is secure enough to have a service
   // worker controller.
-  // Analogous to Blink's Document::isSecureContext. Because of how service
+  // Analogous to Blink's Document::IsSecureContext. Because of how service
   // worker intercepts main resource requests, this check must be done
   // browser-side once the URL is known (see comments in
-  // ServiceWorkerNetworkProvider::CreateForNavigation). This function uses
+  // ServiceWorkerNetworkProviderForFrame::Create). This function uses
   // |url_| and |is_parent_frame_secure_| to determine context security, so they
   // must be set properly before calling this function.
   bool IsContextSecureForServiceWorker() const;
@@ -210,17 +177,11 @@ class CONTENT_EXPORT ServiceWorkerProviderHost
   blink::mojom::ControllerServiceWorkerMode GetControllerMode() const;
 
   // For service worker clients. Returns this client's controller.
-  ServiceWorkerVersion* controller() const {
-#if DCHECK_IS_ON()
-    CheckControllerConsistency();
-#endif  // DCHECK_IS_ON()
-
-    return controller_.get();
-  }
+  ServiceWorkerVersion* controller() const;
 
   ServiceWorkerRegistration* controller_registration() const {
 #if DCHECK_IS_ON()
-    CheckControllerConsistency();
+    CheckControllerConsistency(false);
 #endif  // DCHECK_IS_ON()
 
     return controller_registration_.get();
@@ -231,12 +192,10 @@ class CONTENT_EXPORT ServiceWorkerProviderHost
   // CompleteStartWorkerPreparation() is called).
   ServiceWorkerVersion* running_hosted_version() const {
     DCHECK(!running_hosted_version_ ||
-           info_->type ==
-               blink::mojom::ServiceWorkerProviderType::kForServiceWorker);
+           type_ == blink::mojom::ServiceWorkerProviderType::kForServiceWorker);
     return running_hosted_version_.get();
   }
 
-  // S13nServiceWorker:
   // For service worker clients. Similar to EnsureControllerServiceWorker, but
   // this returns a bound Mojo ptr which is supposed to be sent to clients. The
   // controller ptr passed to the clients will be used to intercept requests
@@ -270,7 +229,10 @@ class CONTENT_EXPORT ServiceWorkerProviderHost
   //
   // TODO(kinuko): revisit this if we start to use the ControllerServiceWorker
   // for posting messages.
-  blink::mojom::ControllerServiceWorkerPtr GetControllerServiceWorkerPtr();
+  // TODO(hayato): Return PendingRemote, instead of Remote. Binding to Remote
+  // as late as possible is more idiomatic for new Mojo types.
+  mojo::Remote<blink::mojom::ControllerServiceWorker>
+  GetRemoteControllerServiceWorker();
 
   // For service worker clients. Sets |url_| and |site_for_cookies_| and updates
   // the client uuid if it's a cross-origin transition.
@@ -296,7 +258,7 @@ class CONTENT_EXPORT ServiceWorkerProviderHost
   const GURL& site_for_cookies() const;
 
   blink::mojom::ServiceWorkerProviderType provider_type() const {
-    return info_->type;
+    return type_;
   }
   bool IsProviderForServiceWorker() const;
   bool IsProviderForClient() const;
@@ -311,37 +273,6 @@ class CONTENT_EXPORT ServiceWorkerProviderHost
       scoped_refptr<ServiceWorkerRegistration> controller_registration,
       bool notify_controllerchange);
 
-  // For use by the ServiceWorkerControlleeRequestHandler to disallow a
-  // registration claiming this host while its main resource request is
-  // occurring.
-  //
-  // TODO(crbug.com/866353): This should be unneccessary: registration code
-  // already avoids claiming clients that are not execution ready. However
-  // there may be edge cases with shared workers (pre-NetS13nServiceWorker) and
-  // about:blank iframes, since |is_execution_ready()| is initialized true for
-  // them. Try to remove this after S13nServiceWorker.
-  void AllowSetControllerRegistration(bool allow) {
-    allow_set_controller_registration_ = allow;
-  }
-  bool IsSetControllerRegistrationAllowed() {
-    return allow_set_controller_registration_;
-  }
-
-  // Returns a handler for a request. May return nullptr if the request doesn't
-  // require special handling.
-  std::unique_ptr<ServiceWorkerRequestHandler> CreateRequestHandler(
-      network::mojom::FetchRequestMode request_mode,
-      network::mojom::FetchCredentialsMode credentials_mode,
-      network::mojom::FetchRedirectMode redirect_mode,
-      const std::string& integrity,
-      bool keepalive,
-      ResourceType resource_type,
-      blink::mojom::RequestContextType request_context_type,
-      network::mojom::RequestContextFrameType frame_type,
-      base::WeakPtr<storage::BlobStorageContext> blob_storage_context,
-      scoped_refptr<network::ResourceRequestBody> body,
-      bool skip_service_worker);
-
   // Returns an object info representing |registration|. The object info holds a
   // Mojo connection to the ServiceWorkerRegistrationObjectHost for the
   // |registration| to ensure the host stays alive while the object info is
@@ -363,6 +294,9 @@ class CONTENT_EXPORT ServiceWorkerProviderHost
   base::WeakPtr<ServiceWorkerObjectHost> GetOrCreateServiceWorkerObjectHost(
       scoped_refptr<ServiceWorkerVersion> version);
 
+  // May return nullptr if the context has shut down.
+  base::WeakPtr<ServiceWorkerContextCore> context() { return context_; }
+
   // Returns true if the context referred to by this host (i.e. |context_|) is
   // still alive.
   bool IsContextAlive();
@@ -379,36 +313,23 @@ class CONTENT_EXPORT ServiceWorkerProviderHost
   void ClaimedByRegistration(
       scoped_refptr<ServiceWorkerRegistration> registration);
 
-  // For service worker clients. Completes initialization of
-  // provider hosts used for navigation requests.
-  void CompleteNavigationInitialized(
-      int process_id,
-      blink::mojom::ServiceWorkerProviderHostInfoPtr info);
+  // For service worker window clients. Called when the navigation is ready to
+  // commit in the browser process to complete the initialization for the
+  // pre-created instance.
+  void OnBeginNavigationCommit(int render_process_id, int render_frame_id);
 
   // For service worker execution contexts. Completes initialization of this
   // provider host. It is called once a renderer process has been found to host
-  // the worker. Returns the info needed for creating a provider on the renderer
-  // which will be connected to this provider host.
-  //
-  // |provider_info| should be the info returned by PreCreateForController(),
-  // which is partially filled out. This function returns it after
-  // filling it out completely.
-  //
-  // S13nServiceWorker:
-  // |loader_factory| is the factory to use for "network" requests for the
-  // service worker main script and import scripts. It is possibly not the
-  // simple direct network factory, since service worker scripts can have
-  // non-NetworkService schemes, e.g., chrome-extension:// URLs.
-  blink::mojom::ServiceWorkerProviderInfoForStartWorkerPtr
-  CompleteStartWorkerPreparation(
+  // the worker.
+  void CompleteStartWorkerPreparation(
       int process_id,
-      scoped_refptr<network::SharedURLLoaderFactory> loader_factory,
-      blink::mojom::ServiceWorkerProviderInfoForStartWorkerPtr provider_info);
+      service_manager::mojom::InterfaceProviderRequest
+          interface_provider_request);
 
-  // Called when the shared worker main script resource has finished loading.
+  // Called when the web worker main script resource has finished loading.
   // After this is called, is_response_committed() and is_execution_ready()
   // return true.
-  void CompleteSharedWorkerPreparation();
+  void CompleteWebWorkerPreparation();
 
   // For service worker clients. The host keeps track of all the prospective
   // longest-matching registrations, in order to resolve .ready or respond to
@@ -438,14 +359,14 @@ class CONTENT_EXPORT ServiceWorkerProviderHost
   // settings allows service workers to run at |scope|. If this provider is for
   // a window client, the check involves the topmost frame url as well as
   // |scope|, and may display tab-level UI.
-  bool AllowServiceWorker(const GURL& scope);
+  // If non-empty, |script_url| is the script the service worker will run.
+  bool AllowServiceWorker(const GURL& scope, const GURL& script_url);
 
   // Called when our controller has been terminated and doomed due to an
   // exceptional condition like it could no longer be read from the script
   // cache.
   void NotifyControllerLost();
 
-  // S13nServiceWorker:
   // For service worker clients. Called when |version| is the active worker upon
   // the main resource request for this client. Remembers |version| as needing
   // a Soft Update. To avoid affecting page load performance, the update occurs
@@ -459,10 +380,6 @@ class CONTENT_EXPORT ServiceWorkerProviderHost
   //
   // This can be called multiple times due to redirects during a main resource
   // load. All service workers are updated.
-  //
-  // For non-S13nServiceWorker: The update logic is controlled entirely by
-  // ServiceWorkerControlleeRequestHandler, which sees all resource request
-  // activity and schedules an update at a convenient time.
   void AddServiceWorkerToUpdate(scoped_refptr<ServiceWorkerVersion> version);
 
   // For service worker clients. |callback| is called when this client becomes
@@ -475,8 +392,8 @@ class CONTENT_EXPORT ServiceWorkerProviderHost
   bool is_response_committed() const;
 
   // For service worker clients. True if the client is execution ready and
-  // therefore can be exposed to JavaScript. Execution ready implies connected
-  // to renderer.
+  // therefore can be exposed to JavaScript. Execution ready implies response
+  // committed.
   // https://html.spec.whatwg.org/multipage/webappapis.html#concept-environment-execution-ready-flag
   bool is_execution_ready() const;
 
@@ -494,7 +411,6 @@ class CONTENT_EXPORT ServiceWorkerProviderHost
   friend class LinkHeaderServiceWorkerTest;
   friend class ServiceWorkerProviderHostTest;
   friend class ServiceWorkerWriteToCacheJobTest;
-  friend class ServiceWorkerContextRequestHandlerTest;
   friend class service_worker_controllee_request_handler_unittest::
       ServiceWorkerControlleeRequestHandlerTest;
   friend class service_worker_object_host_unittest::ServiceWorkerObjectHostTest;
@@ -507,23 +423,25 @@ class CONTENT_EXPORT ServiceWorkerProviderHost
                            Update_ElongatedScript);
   FRIEND_TEST_ALL_PREFIXES(ServiceWorkerWriteToCacheJobTest,
                            Update_EmptyScript);
-  FRIEND_TEST_ALL_PREFIXES(
-      service_worker_dispatcher_host_unittest::ServiceWorkerDispatcherHostTest,
-      DispatchExtendableMessageEvent);
-  FRIEND_TEST_ALL_PREFIXES(
-      service_worker_dispatcher_host_unittest::ServiceWorkerDispatcherHostTest,
-      DispatchExtendableMessageEvent_Fail);
   FRIEND_TEST_ALL_PREFIXES(ServiceWorkerProviderHostTest, ContextSecurity);
   FRIEND_TEST_ALL_PREFIXES(ServiceWorkerJobTest, Unregister);
   FRIEND_TEST_ALL_PREFIXES(ServiceWorkerJobTest, RegisterDuplicateScript);
-  FRIEND_TEST_ALL_PREFIXES(ServiceWorkerJobTest,
+  FRIEND_TEST_ALL_PREFIXES(ServiceWorkerUpdateJobTest,
                            RegisterWithDifferentUpdateViaCache);
   FRIEND_TEST_ALL_PREFIXES(BackgroundSyncManagerTest,
                            RegisterWithoutLiveSWRegistration);
 
-  ServiceWorkerProviderHost(int process_id,
-                            blink::mojom::ServiceWorkerProviderHostInfoPtr info,
-                            base::WeakPtr<ServiceWorkerContextCore> context);
+  static void RegisterToContextCore(
+      base::WeakPtr<ServiceWorkerContextCore> context,
+      std::unique_ptr<ServiceWorkerProviderHost> host);
+
+  ServiceWorkerProviderHost(
+      blink::mojom::ServiceWorkerProviderType type,
+      bool is_parent_frame_secure,
+      int frame_tree_node_id,
+      blink::mojom::ServiceWorkerContainerHostAssociatedRequest host_request,
+      blink::mojom::ServiceWorkerContainerAssociatedPtrInfo client_ptr_info,
+      base::WeakPtr<ServiceWorkerContextCore> context);
 
   // ServiceWorkerRegistration::Listener overrides.
   void OnVersionAttributesChanged(
@@ -566,7 +484,7 @@ class CONTENT_EXPORT ServiceWorkerProviderHost
   bool IsControllerDecided() const;
 
 #if DCHECK_IS_ON()
-  void CheckControllerConsistency() const;
+  void CheckControllerConsistency(bool should_crash) const;
 #endif  // DCHECK_IS_ON()
 
   // Implements blink::mojom::ServiceWorkerContainerHost.
@@ -579,7 +497,7 @@ class CONTENT_EXPORT ServiceWorkerProviderHost
   void GetRegistrationForReady(
       GetRegistrationForReadyCallback callback) override;
   void EnsureControllerServiceWorker(
-      blink::mojom::ControllerServiceWorkerRequest controller_request,
+      mojo::PendingReceiver<blink::mojom::ControllerServiceWorker> receiver,
       blink::mojom::ControllerServiceWorkerPurpose purpose) override;
   void CloneContainerHost(blink::mojom::ServiceWorkerContainerHostRequest
                               container_host_request) override;
@@ -610,7 +528,7 @@ class CONTENT_EXPORT ServiceWorkerProviderHost
 
   // Callback for ServiceWorkerVersion::RunAfterStartWorker()
   void StartControllerComplete(
-      blink::mojom::ControllerServiceWorkerRequest controller_request,
+      mojo::PendingReceiver<blink::mojom::ControllerServiceWorker> receiver,
       blink::ServiceWorkerStatusCode status);
 
   bool IsValidGetRegistrationMessage(const GURL& client_url,
@@ -626,12 +544,15 @@ class CONTENT_EXPORT ServiceWorkerProviderHost
   // Perform common checks that need to run before ContainerHost methods that
   // come from a child process are handled.
   // |scope| is checked if it is allowed to run a service worker.
+  // If non-empty, |script_url| is the script associated with the service
+  // worker.
   // Returns true if all checks have passed.
   // If anything looks wrong |callback| will run with an error
   // message prefixed by |error_prefix| and |args|, and false is returned.
   template <typename CallbackType, typename... Args>
   bool CanServeContainerHostMethods(CallbackType* callback,
                                     const GURL& scope,
+                                    const GURL& script_url,
                                     const char* error_prefix,
                                     Args... args);
 
@@ -642,6 +563,13 @@ class CONTENT_EXPORT ServiceWorkerProviderHost
 
   void TransitionToClientPhase(ClientPhase new_phase);
 
+  void SetRenderProcessId(int process_id);
+
+  // Unique among all provider hosts.
+  const int provider_id_;
+
+  const blink::mojom::ServiceWorkerProviderType type_;
+
   // A GUID that is web-exposed as FetchEvent.clientId.
   std::string client_uuid_;
 
@@ -658,16 +586,27 @@ class CONTENT_EXPORT ServiceWorkerProviderHost
   const base::TimeTicks create_time_;
   int render_process_id_;
 
-  // For service worker execution contexts, the id of the service worker thread
-  // or |kInvalidEmbeddedWorkerThreadId| before the service worker starts up.
-  // Otherwise, |kDocumentMainThreadId|.
-  int render_thread_id_;
+  // The window's RenderFrame id, if this is a service worker window client.
+  // Otherwise, |MSG_ROUTING_NONE|.
+  int frame_id_;
 
-  blink::mojom::ServiceWorkerProviderHostInfoPtr info_;
+  // |is_parent_frame_secure_| is false if the provider host is created for a
+  // document whose parent frame is not secure. This doesn't mean the document
+  // is necessarily an insecure context, because the document may have a URL
+  // whose scheme is granted an exception that allows bypassing the ancestor
+  // secure context check. If the provider is not created for a document, or the
+  // document does not have a parent frame, is_parent_frame_secure_| is true.
+  // TODO(leonhsl): make it be const, currently only some test code wants to
+  // change its value.
+  bool is_parent_frame_secure_;
+
+  // FrameTreeNode id if this is a service worker window client.
+  // Otherwise, |FrameTreeNode::kFrameTreeNodeInvalidId|.
+  const int frame_tree_node_id_;
 
   // Only set when this object is pre-created for a navigation. It indicates the
-  // tab where the navigation occurs.
-  WebContentsGetter web_contents_getter_;
+  // tab where the navigation occurs. Otherwise, a null callback.
+  const WebContentsGetter web_contents_getter_;
 
   // For service worker clients. See comments for the getter functions.
   GURL url_;
@@ -715,12 +654,16 @@ class CONTENT_EXPORT ServiceWorkerProviderHost
   // the provider host's controller is updated to match it.
   scoped_refptr<ServiceWorkerVersion> controller_;
   scoped_refptr<ServiceWorkerRegistration> controller_registration_;
-  bool allow_set_controller_registration_ = true;
 
   // For service worker execution contexts. The ServiceWorkerVersion of the
   // service worker this is a provider for.
   scoped_refptr<ServiceWorkerVersion> running_hosted_version_;
 
+  // |context_| owns |this| but if the context is destroyed and a new one is
+  // created, the provider host becomes owned by the new context, while this
+  // |context_| is reset to null.
+  // TODO(https://crbug.com/877356): Don't support copying context, so this can
+  // just be a raw ptr that is never null.
   base::WeakPtr<ServiceWorkerContextCore> context_;
 
   // |container_| is the Mojo endpoint to the renderer-side
