@@ -155,9 +155,11 @@ class MAYBE_WebRtcInternalsBrowserTest: public ContentBrowserTest {
   MAYBE_WebRtcInternalsBrowserTest() {}
   ~MAYBE_WebRtcInternalsBrowserTest() override {}
 
+  void SetUpCommandLine(base::CommandLine* command_line) override {
+    command_line->AppendSwitch(switches::kUseFakeUIForMediaStream);
+  }
+
   void SetUpOnMainThread() override {
-    base::CommandLine::ForCurrentProcess()->AppendSwitch(
-        switches::kUseFakeUIForMediaStream);
     ASSERT_TRUE(base::CommandLine::ForCurrentProcess()->HasSwitch(
         switches::kUseFakeDeviceForMediaStream));
   }
@@ -237,16 +239,16 @@ class MAYBE_WebRtcInternalsBrowserTest: public ContentBrowserTest {
                                       "    JSON.stringify(userMediaRequests));",
                                       &json_requests));
     std::unique_ptr<base::Value> value_requests =
-        base::JSONReader::Read(json_requests);
+        base::JSONReader::ReadDeprecated(json_requests);
 
-    EXPECT_EQ(base::Value::Type::LIST, value_requests->GetType());
+    EXPECT_EQ(base::Value::Type::LIST, value_requests->type());
 
     base::ListValue* list_request =
         static_cast<base::ListValue*>(value_requests.get());
     EXPECT_EQ(requests.size(), list_request->GetSize());
 
     for (size_t i = 0; i < requests.size(); ++i) {
-      base::DictionaryValue* dict = NULL;
+      base::DictionaryValue* dict = nullptr;
       ASSERT_TRUE(list_request->GetDictionary(i, &dict));
       int pid, rid;
       std::string origin, audio, video;
@@ -325,11 +327,13 @@ class MAYBE_WebRtcInternalsBrowserTest: public ContentBrowserTest {
       pc.stats_[id][iter->first].push_back(iter->second);
     }
     std::stringstream ss;
-    ss << "{pid:" << pc.pid_ << ", lid:" << pc.lid_ << ","
-           "reports:[" << "{id:'" << id << "', type:'" << type << "', "
-                           "stats:" << stats.GetString() << "}]}";
-
-    ASSERT_TRUE(ExecuteJavascript("addStats(" + ss.str() + ")"));
+    ss << "(() => {\n";
+    ss << "  currentGetStatsMethod = OPTION_GETSTATS_LEGACY;\n";
+    ss << "  addLegacyStats({pid:" << pc.pid_ << ", lid:" << pc.lid_
+       << ", reports:[{id:'" << id << "', type:'" << type
+       << "', stats:" << stats.GetString() << "}]});\n";
+    ss << "})()";
+    ASSERT_TRUE(ExecuteJavascript(ss.str()));
     VerifyStatsTable(pc, entry);
   }
 
@@ -438,8 +442,8 @@ class MAYBE_WebRtcInternalsBrowserTest: public ContentBrowserTest {
                                int peer_connection_number,
                                int update_number,
                                int stats_number) {
-    EXPECT_NE((base::Value*)NULL, dump);
-    EXPECT_EQ(base::Value::Type::DICTIONARY, dump->GetType());
+    EXPECT_NE((base::Value*)nullptr, dump);
+    EXPECT_EQ(base::Value::Type::DICTIONARY, dump->type());
 
     base::DictionaryValue* dict_dump =
         static_cast<base::DictionaryValue*>(dump);
@@ -447,9 +451,9 @@ class MAYBE_WebRtcInternalsBrowserTest: public ContentBrowserTest {
 
     base::DictionaryValue::Iterator it(*dict_dump);
     for (; !it.IsAtEnd(); it.Advance()) {
-      base::Value* value = NULL;
+      base::Value* value = nullptr;
       dict_dump->Get(it.key(), &value);
-      EXPECT_EQ(base::Value::Type::DICTIONARY, value->GetType());
+      EXPECT_EQ(base::Value::Type::DICTIONARY, value->type());
       base::DictionaryValue* pc_dump =
           static_cast<base::DictionaryValue*>(value);
       EXPECT_TRUE(pc_dump->HasKey("updateLog"));
@@ -457,13 +461,13 @@ class MAYBE_WebRtcInternalsBrowserTest: public ContentBrowserTest {
 
       // Verifies the number of updates.
       pc_dump->Get("updateLog", &value);
-      EXPECT_EQ(base::Value::Type::LIST, value->GetType());
+      EXPECT_EQ(base::Value::Type::LIST, value->type());
       base::ListValue* list = static_cast<base::ListValue*>(value);
       EXPECT_EQ((size_t) update_number, list->GetSize());
 
       // Verifies the number of stats tables.
       pc_dump->Get("stats", &value);
-      EXPECT_EQ(base::Value::Type::DICTIONARY, value->GetType());
+      EXPECT_EQ(base::Value::Type::DICTIONARY, value->type());
       base::DictionaryValue* dict = static_cast<base::DictionaryValue*>(value);
       EXPECT_EQ((size_t) stats_number, dict->size());
     }
@@ -476,19 +480,19 @@ class MAYBE_WebRtcInternalsBrowserTest: public ContentBrowserTest {
                        const string& report_type,
                        const string& report_id,
                        const StatsUnit& stats) {
-    EXPECT_NE((base::Value*)NULL, dump);
-    EXPECT_EQ(base::Value::Type::DICTIONARY, dump->GetType());
+    EXPECT_NE((base::Value*)nullptr, dump);
+    EXPECT_EQ(base::Value::Type::DICTIONARY, dump->type());
 
     base::DictionaryValue* dict_dump =
         static_cast<base::DictionaryValue*>(dump);
-    base::Value* value = NULL;
+    base::Value* value = nullptr;
     dict_dump->Get(pc.getIdString(), &value);
     base::DictionaryValue* pc_dump = static_cast<base::DictionaryValue*>(value);
 
     // Verifies there is one data series per stats name.
-    value = NULL;
+    value = nullptr;
     pc_dump->Get("stats", &value);
-    EXPECT_EQ(base::Value::Type::DICTIONARY, value->GetType());
+    EXPECT_EQ(base::Value::Type::DICTIONARY, value->type());
 
     base::DictionaryValue* dataSeries =
         static_cast<base::DictionaryValue*>(value);
@@ -785,7 +789,8 @@ IN_PROC_BROWSER_TEST_F(MAYBE_WebRtcInternalsBrowserTest, CreatePageDump) {
       "window.domAutomationController.send("
       "    JSON.stringify(peerConnectionDataStore));",
       &dump_json));
-  std::unique_ptr<base::Value> dump = base::JSONReader::Read(dump_json);
+  std::unique_ptr<base::Value> dump =
+      base::JSONReader::ReadDeprecated(dump_json);
   VerifyPageDumpStructure(dump.get(),
                           2 /*peer_connection_number*/,
                           2 /*update_number*/,
@@ -804,7 +809,7 @@ IN_PROC_BROWSER_TEST_F(MAYBE_WebRtcInternalsBrowserTest, CreatePageDump) {
       "window.domAutomationController.send("
       "    JSON.stringify(peerConnectionDataStore));",
       &dump_json));
-  dump = base::JSONReader::Read(dump_json);
+  dump = base::JSONReader::ReadDeprecated(dump_json);
   VerifyStatsDump(dump.get(), pc_0, type, id, stats);
 }
 
