@@ -7,24 +7,23 @@
 #include <memory>
 #include <utility>
 
-#include "base/strings/string_util.h"
+#include "base/bind.h"
 #include "base/values.h"
-#include "content/common/view_messages.h"
-#include "content/public/child/v8_value_converter.h"
+#include "content/common/frame_messages.h"
 #include "content/public/common/bindings_policy.h"
 #include "content/public/common/url_constants.h"
 #include "content/public/renderer/chrome_object_extensions_utils.h"
 #include "content/public/renderer/render_frame.h"
 #include "content/public/renderer/render_thread.h"
 #include "content/public/renderer/render_view.h"
+#include "content/public/renderer/v8_value_converter.h"
 #include "content/renderer/web_ui_extension_data.h"
 #include "gin/arguments.h"
 #include "gin/function_template.h"
-#include "third_party/WebKit/public/web/WebDocument.h"
-#include "third_party/WebKit/public/web/WebKit.h"
-#include "third_party/WebKit/public/web/WebLocalFrame.h"
-#include "third_party/WebKit/public/web/WebUserGestureIndicator.h"
-#include "third_party/WebKit/public/web/WebView.h"
+#include "third_party/blink/public/web/blink.h"
+#include "third_party/blink/public/web/web_document.h"
+#include "third_party/blink/public/web/web_local_frame.h"
+#include "third_party/blink/public/web/web_view.h"
 #include "url/gurl.h"
 #include "v8/include/v8.h"
 
@@ -33,15 +32,11 @@ namespace content {
 namespace {
 
 bool ShouldRespondToRequest(blink::WebLocalFrame** frame_ptr,
-                            RenderView** render_view_ptr) {
+                            RenderFrame** render_frame_ptr) {
   blink::WebLocalFrame* frame = blink::WebLocalFrame::FrameForCurrentContext();
   if (!frame || !frame->View())
     return false;
 
-  RenderView* render_view = RenderView::FromWebView(frame->View());
-  if (!render_view)
-    return false;
-
   GURL frame_url = frame->GetDocument().Url();
 
   RenderFrame* render_frame = RenderFrame::FromWebFrame(frame);
@@ -57,7 +52,7 @@ bool ShouldRespondToRequest(blink::WebLocalFrame** frame_ptr,
     return false;
 
   *frame_ptr = frame;
-  *render_view_ptr = render_view;
+  *render_frame_ptr = render_frame;
   return true;
 }
 
@@ -78,22 +73,28 @@ void WebUIExtension::Install(blink::WebLocalFrame* frame) {
 
   v8::Context::Scope context_scope(context);
 
-  v8::Local<v8::Object> chrome = GetOrCreateChromeObject(isolate,
-                                                          context->Global());
-  chrome->Set(gin::StringToSymbol(isolate, "send"),
-              gin::CreateFunctionTemplate(
-                  isolate, base::Bind(&WebUIExtension::Send))->GetFunction());
-  chrome->Set(gin::StringToSymbol(isolate, "getVariableValue"),
-              gin::CreateFunctionTemplate(
-                  isolate, base::Bind(&WebUIExtension::GetVariableValue))
-                  ->GetFunction());
+  v8::Local<v8::Object> chrome = GetOrCreateChromeObject(isolate, context);
+  chrome
+      ->Set(context, gin::StringToSymbol(isolate, "send"),
+            gin::CreateFunctionTemplate(
+                isolate, base::BindRepeating(&WebUIExtension::Send))
+                ->GetFunction(context)
+                .ToLocalChecked())
+      .Check();
+  chrome
+      ->Set(context, gin::StringToSymbol(isolate, "getVariableValue"),
+            gin::CreateFunctionTemplate(
+                isolate, base::BindRepeating(&WebUIExtension::GetVariableValue))
+                ->GetFunction(context)
+                .ToLocalChecked())
+      .Check();
 }
 
 // static
 void WebUIExtension::Send(gin::Arguments* args) {
   blink::WebLocalFrame* frame;
-  RenderView* render_view;
-  if (!ShouldRespondToRequest(&frame, &render_view))
+  RenderFrame* render_frame;
+  if (!ShouldRespondToRequest(&frame, &render_frame))
     return;
 
   std::string message;
@@ -102,13 +103,6 @@ void WebUIExtension::Send(gin::Arguments* args) {
     return;
   }
 
-  if (base::EndsWith(message, "RequiringGesture",
-                     base::CompareCase::SENSITIVE) &&
-      !blink::WebUserGestureIndicator::IsProcessingUserGesture()) {
-    NOTREACHED();
-    return;
-  }
-
   // If they've provided an optional message parameter, convert that into a
   // Value to send to the browser process.
   std::unique_ptr<base::ListValue> content;
@@ -134,19 +128,18 @@ void WebUIExtension::Send(gin::Arguments* args) {
   }
 
   // Send the message up to the browser.
-  render_view->Send(new ViewHostMsg_WebUISend(render_view->GetRoutingID(),
-                                              frame->GetDocument().Url(),
-                                              message, *content));
+  render_frame->Send(new FrameHostMsg_WebUISend(render_frame->GetRoutingID(),
+                                                message, *content));
 }
 
 // static
 std::string WebUIExtension::GetVariableValue(const std::string& name) {
   blink::WebLocalFrame* frame;
-  RenderView* render_view;
-  if (!ShouldRespondToRequest(&frame, &render_view))
+  RenderFrame* render_frame;
+  if (!ShouldRespondToRequest(&frame, &render_frame))
     return std::string();
 
-  return WebUIExtensionData::Get(render_view)->GetValue(name);
+  return WebUIExtensionData::Get(render_frame->GetRenderView())->GetValue(name);
 }
 
 }  // namespace content
