@@ -11,7 +11,7 @@
 #include "base/bind.h"
 #include "base/json/json_writer.h"
 #include "base/lazy_instance.h"
-#include "base/message_loop/message_loop.h"
+#include "base/macros.h"
 #include "base/time/clock.h"
 #include "base/time/default_clock.h"
 #include "base/time/time.h"
@@ -20,6 +20,7 @@
 #include "extensions/browser/api/alarms/alarms_api_constants.h"
 #include "extensions/browser/event_router.h"
 #include "extensions/browser/extension_registry.h"
+#include "extensions/browser/extension_registry_factory.h"
 #include "extensions/browser/extension_system.h"
 #include "extensions/browser/state_store.h"
 #include "extensions/common/api/alarms.h"
@@ -76,8 +77,11 @@ AlarmManager::AlarmList AlarmsFromValue(const std::string extension_id,
     if (list->GetDictionary(i, &alarm_dict) &&
         alarms::Alarm::Populate(*alarm_dict, alarm->js_alarm.get())) {
       const base::Value* time_value = nullptr;
-      if (alarm_dict->Get(kAlarmGranularity, &time_value))
-        base::GetValueAsTimeDelta(*time_value, &alarm->granularity);
+      if (alarm_dict->Get(kAlarmGranularity, &time_value)) {
+        // It's okay to ignore the failure since we have minimum granularity.
+        ignore_result(
+            base::GetValueAsTimeDelta(*time_value, &alarm->granularity));
+      }
       alarm->minimum_granularity = base::TimeDelta::FromSecondsD(
           (is_unpacked ? alarms_api_constants::kDevDelayMinimum
                        : alarms_api_constants::kReleaseDelayMinimum) *
@@ -96,8 +100,8 @@ std::unique_ptr<base::ListValue> AlarmsToValue(
   for (size_t i = 0; i < alarms.size(); ++i) {
     std::unique_ptr<base::DictionaryValue> alarm =
         alarms[i]->js_alarm->ToValue();
-    alarm->Set(kAlarmGranularity,
-               base::CreateTimeDeltaValue(alarms[i]->granularity));
+    alarm->SetKey(kAlarmGranularity,
+                  base::CreateTimeDeltaValue(alarms[i]->granularity));
     list->Append(std::move(alarm));
   }
   return list;
@@ -109,7 +113,7 @@ std::unique_ptr<base::ListValue> AlarmsToValue(
 
 AlarmManager::AlarmManager(content::BrowserContext* context)
     : browser_context_(context),
-      clock_(new base::DefaultClock()),
+      clock_(base::DefaultClock::GetInstance()),
       delegate_(new DefaultAlarmDelegate(context)),
       extension_registry_observer_(this) {
   extension_registry_observer_.Add(ExtensionRegistry::Get(browser_context_));
@@ -124,77 +128,81 @@ AlarmManager::~AlarmManager() {
 
 void AlarmManager::AddAlarm(const std::string& extension_id,
                             std::unique_ptr<Alarm> alarm,
-                            const AddAlarmCallback& callback) {
+                            AddAlarmCallback callback) {
   RunWhenReady(extension_id,
-               base::Bind(&AlarmManager::AddAlarmWhenReady, AsWeakPtr(),
-                          base::Passed(std::move(alarm)), callback));
+               base::BindOnce(&AlarmManager::AddAlarmWhenReady, AsWeakPtr(),
+                              std::move(alarm), std::move(callback)));
 }
 
 void AlarmManager::GetAlarm(const std::string& extension_id,
                             const std::string& name,
-                            const GetAlarmCallback& callback) {
-  RunWhenReady(extension_id, base::Bind(&AlarmManager::GetAlarmWhenReady,
-                                        AsWeakPtr(), name, callback));
+                            GetAlarmCallback callback) {
+  RunWhenReady(extension_id,
+               base::BindOnce(&AlarmManager::GetAlarmWhenReady, AsWeakPtr(),
+                              name, std::move(callback)));
 }
 
 void AlarmManager::GetAllAlarms(const std::string& extension_id,
-                                const GetAllAlarmsCallback& callback) {
-  RunWhenReady(extension_id, base::Bind(&AlarmManager::GetAllAlarmsWhenReady,
-                                        AsWeakPtr(), callback));
+                                GetAllAlarmsCallback callback) {
+  RunWhenReady(extension_id,
+               base::BindOnce(&AlarmManager::GetAllAlarmsWhenReady, AsWeakPtr(),
+                              std::move(callback)));
 }
 
 void AlarmManager::RemoveAlarm(const std::string& extension_id,
                                const std::string& name,
-                               const RemoveAlarmCallback& callback) {
-  RunWhenReady(extension_id, base::Bind(&AlarmManager::RemoveAlarmWhenReady,
-                                        AsWeakPtr(), name, callback));
+                               RemoveAlarmCallback callback) {
+  RunWhenReady(extension_id,
+               base::BindOnce(&AlarmManager::RemoveAlarmWhenReady, AsWeakPtr(),
+                              name, std::move(callback)));
 }
 
 void AlarmManager::RemoveAllAlarms(const std::string& extension_id,
-                                   const RemoveAllAlarmsCallback& callback) {
-  RunWhenReady(extension_id, base::Bind(&AlarmManager::RemoveAllAlarmsWhenReady,
-                                        AsWeakPtr(), callback));
+                                   RemoveAllAlarmsCallback callback) {
+  RunWhenReady(extension_id,
+               base::BindOnce(&AlarmManager::RemoveAllAlarmsWhenReady,
+                              AsWeakPtr(), std::move(callback)));
 }
 
 void AlarmManager::AddAlarmWhenReady(std::unique_ptr<Alarm> alarm,
-                                     const AddAlarmCallback& callback,
+                                     AddAlarmCallback callback,
                                      const std::string& extension_id) {
   AddAlarmImpl(extension_id, std::move(alarm));
   WriteToStorage(extension_id);
-  callback.Run();
+  std::move(callback).Run();
 }
 
 void AlarmManager::GetAlarmWhenReady(const std::string& name,
-                                     const GetAlarmCallback& callback,
+                                     GetAlarmCallback callback,
                                      const std::string& extension_id) {
   AlarmIterator it = GetAlarmIterator(extension_id, name);
-  callback.Run(it.first != alarms_.end() ? it.second->get() : nullptr);
+  std::move(callback).Run(it.first != alarms_.end() ? it.second->get()
+                                                    : nullptr);
 }
 
-void AlarmManager::GetAllAlarmsWhenReady(const GetAllAlarmsCallback& callback,
+void AlarmManager::GetAllAlarmsWhenReady(GetAllAlarmsCallback callback,
                                          const std::string& extension_id) {
-  AlarmMap::iterator list = alarms_.find(extension_id);
-  callback.Run(list != alarms_.end() ? &list->second : NULL);
+  auto list = alarms_.find(extension_id);
+  std::move(callback).Run(list != alarms_.end() ? &list->second : NULL);
 }
 
 void AlarmManager::RemoveAlarmWhenReady(const std::string& name,
-                                        const RemoveAlarmCallback& callback,
+                                        RemoveAlarmCallback callback,
                                         const std::string& extension_id) {
   AlarmIterator it = GetAlarmIterator(extension_id, name);
   if (it.first == alarms_.end()) {
-    callback.Run(false);
+    std::move(callback).Run(false);
     return;
   }
 
   RemoveAlarmIterator(it);
   WriteToStorage(extension_id);
-  callback.Run(true);
+  std::move(callback).Run(true);
 }
 
-void AlarmManager::RemoveAllAlarmsWhenReady(
-    const RemoveAllAlarmsCallback& callback,
-    const std::string& extension_id) {
-  AlarmMap::iterator list = alarms_.find(extension_id);
+void AlarmManager::RemoveAllAlarmsWhenReady(RemoveAllAlarmsCallback callback,
+                                            const std::string& extension_id) {
+  auto list = alarms_.find(extension_id);
   if (list != alarms_.end()) {
     // Note: I'm using indices rather than iterators here because
     // RemoveAlarmIterator will delete the list when it becomes empty.
@@ -204,18 +212,17 @@ void AlarmManager::RemoveAllAlarmsWhenReady(
     CHECK(alarms_.find(extension_id) == alarms_.end());
     WriteToStorage(extension_id);
   }
-  callback.Run();
+  std::move(callback).Run();
 }
 
 AlarmManager::AlarmIterator AlarmManager::GetAlarmIterator(
     const std::string& extension_id,
     const std::string& name) {
-  AlarmMap::iterator list = alarms_.find(extension_id);
+  auto list = alarms_.find(extension_id);
   if (list == alarms_.end())
     return make_pair(alarms_.end(), AlarmList::iterator());
 
-  for (AlarmList::iterator it = list->second.begin(); it != list->second.end();
-       ++it) {
+  for (auto it = list->second.begin(); it != list->second.end(); ++it) {
     if (it->get()->js_alarm->name == name)
       return make_pair(list, it);
   }
@@ -224,13 +231,18 @@ AlarmManager::AlarmIterator AlarmManager::GetAlarmIterator(
 }
 
 void AlarmManager::SetClockForTesting(base::Clock* clock) {
-  clock_.reset(clock);
+  clock_ = clock;
 }
 
 static base::LazyInstance<
     BrowserContextKeyedAPIFactory<AlarmManager>>::DestructorAtExit g_factory =
     LAZY_INSTANCE_INITIALIZER;
 
+template <>
+void BrowserContextKeyedAPIFactory<AlarmManager>::DeclareFactoryDependencies() {
+  DependsOn(ExtensionRegistryFactory::GetInstance());
+}
+
 // static
 BrowserContextKeyedAPIFactory<AlarmManager>*
 AlarmManager::GetFactoryInstance() {
@@ -306,7 +318,7 @@ void AlarmManager::WriteToStorage(const std::string& extension_id) {
     return;
 
   std::unique_ptr<base::Value> alarms;
-  AlarmMap::iterator list = alarms_.find(extension_id);
+  auto list = alarms_.find(extension_id);
   if (list != alarms_.end())
     alarms = AlarmsToValue(list->second);
   else
@@ -327,7 +339,7 @@ void AlarmManager::ReadFromStorage(const std::string& extension_id,
 
   ReadyQueue& extension_ready_queue = ready_actions_[extension_id];
   while (!extension_ready_queue.empty()) {
-    extension_ready_queue.front().Run(extension_id);
+    std::move(extension_ready_queue.front()).Run(extension_id);
     extension_ready_queue.pop();
   }
   ready_actions_.erase(extension_id);
@@ -356,8 +368,8 @@ void AlarmManager::ScheduleNextPoll() {
   base::TimeDelta min_granularity = kDefaultMinPollPeriod();
   for (AlarmMap::const_iterator m_it = alarms_.begin(), m_end = alarms_.end();
        m_it != m_end; ++m_it) {
-    for (AlarmList::const_iterator l_it = m_it->second.begin();
-         l_it != m_it->second.end(); ++l_it) {
+    for (auto l_it = m_it->second.cbegin(); l_it != m_it->second.cend();
+         ++l_it) {
       base::Time cur_alarm_time =
           base::Time::FromJsTime(l_it->get()->js_alarm->scheduled_time);
       if (cur_alarm_time < soonest_alarm_time)
@@ -390,16 +402,15 @@ void AlarmManager::PollAlarms() {
   // Run any alarms scheduled in the past. OnAlarm uses vector::erase to remove
   // elements from the AlarmList, and map::erase to remove AlarmLists from the
   // AlarmMap.
-  for (AlarmMap::iterator m_it = alarms_.begin(), m_end = alarms_.end();
-       m_it != m_end;) {
-    AlarmMap::iterator cur_extension = m_it++;
+  for (auto m_it = alarms_.begin(), m_end = alarms_.end(); m_it != m_end;) {
+    auto cur_extension = m_it++;
 
     // Iterate (a) backwards so that removing elements doesn't affect
     // upcoming iterations, and (b) with indices so that if the last
     // iteration destroys the AlarmList, I'm not about to use the end
     // iterator that the destruction invalidates.
     for (size_t i = cur_extension->second.size(); i > 0; --i) {
-      AlarmList::iterator cur_alarm = cur_extension->second.begin() + i - 1;
+      auto cur_alarm = cur_extension->second.begin() + i - 1;
       if (base::Time::FromJsTime(cur_alarm->get()->js_alarm->scheduled_time) <=
           last_poll_time_) {
         OnAlarm(make_pair(cur_extension, cur_alarm));
@@ -414,13 +425,13 @@ static void RemoveAllOnUninstallCallback() {
 }
 
 void AlarmManager::RunWhenReady(const std::string& extension_id,
-                                const ReadyAction& action) {
-  ReadyMap::iterator it = ready_actions_.find(extension_id);
+                                ReadyAction action) {
+  auto it = ready_actions_.find(extension_id);
 
   if (it == ready_actions_.end())
-    action.Run(extension_id);
+    std::move(action).Run(extension_id);
   else
-    it->second.push(action);
+    it->second.push(std::move(action));
 }
 
 void AlarmManager::OnExtensionLoaded(content::BrowserContext* browser_context,
