@@ -7,14 +7,15 @@
 #include <utility>
 
 #include "base/logging.h"
-#include "base/memory/ptr_util.h"
+#include "base/numerics/safe_conversions.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
-#include "base/sys_info.h"
+#include "base/system/sys_info.h"
 #include "base/values.h"
-#include "gpu/config/gpu_info.h"
+#include "build/build_config.h"
+#include "gpu/config/gpu_util.h"
 #include "third_party/re2/src/re2/re2.h"
 
 namespace gpu {
@@ -35,7 +36,7 @@ bool ProcessVersionString(const std::string& version_string,
   // we split it into the order of "yyyy", "mm", "dd".
   if (splitter == '-') {
     std::string year = version->back();
-    for (int i = version->size() - 1; i > 0; --i) {
+    for (size_t i = version->size() - 1; i > 0; --i) {
       (*version)[i] = (*version)[i - 1];
     }
     (*version)[0] = year;
@@ -43,8 +44,10 @@ bool ProcessVersionString(const std::string& version_string,
   bool all_zero = true;
   for (size_t i = 0; i < version->size(); ++i) {
     unsigned num = 0;
-    if (!base::StringToUint((*version)[i], &num))
-      return false;
+    if (!base::StringToUint((*version)[i], &num)) {
+      version->resize(i);
+      break;
+    }
     if (num)
       all_zero = false;
   }
@@ -232,15 +235,16 @@ void GpuControlList::Entry::LogControlListMatch(
 }
 
 bool GpuControlList::DriverInfo::Contains(const GPUInfo& gpu_info) const {
-  if (StringMismatch(gpu_info.driver_vendor, driver_vendor)) {
+  const GPUInfo::GPUDevice& active_gpu = gpu_info.active_gpu();
+  if (StringMismatch(active_gpu.driver_vendor, driver_vendor)) {
     return false;
   }
-  if (driver_version.IsSpecified() && !gpu_info.driver_version.empty() &&
-      !driver_version.Contains(gpu_info.driver_version)) {
+  if (driver_version.IsSpecified() && !active_gpu.driver_version.empty() &&
+      !driver_version.Contains(active_gpu.driver_version)) {
     return false;
   }
-  if (driver_date.IsSpecified() && !gpu_info.driver_date.empty() &&
-      !driver_date.Contains(gpu_info.driver_date, '-')) {
+  if (driver_date.IsSpecified() && !active_gpu.driver_date.empty() &&
+      !driver_date.Contains(active_gpu.driver_date, '-')) {
     return false;
   }
   return true;
@@ -296,7 +300,8 @@ bool GpuControlList::More::Contains(const GPUInfo& gpu_info) const {
       return false;
     }
   }
-  if (!direct_rendering && gpu_info.direct_rendering) {
+  if (direct_rendering_version.IsSpecified() &&
+      !direct_rendering_version.Contains(gpu_info.direct_rendering_version)) {
     return false;
   }
   if (in_process_gpu && !gpu_info.in_process_gpu) {
@@ -306,6 +311,22 @@ bool GpuControlList::More::Contains(const GPUInfo& gpu_info) const {
       !pixel_shader_version.Contains(gpu_info.pixel_shader_version)) {
     return false;
   }
+  switch (hardware_overlay) {
+    case kDontCare:
+      break;
+    case kSupported:
+#if defined(OS_WIN)
+      if (!gpu_info.supports_overlays)
+        return false;
+#endif  // OS_WIN
+      break;
+    case kUnsupported:
+#if defined(OS_WIN)
+      if (gpu_info.supports_overlays)
+        return false;
+#endif  // OS_WIN
+      break;
+  }
   return true;
 }
 
@@ -319,7 +340,7 @@ bool GpuControlList::Conditions::Contains(OsType target_os_type,
     if (os_version.IsSpecified() && !os_version.Contains(target_os_version))
       return false;
   }
-  if (vendor_id != 0) {
+  if (vendor_id != 0 || gpu_series_list_size > 0) {
     std::vector<GPUInfo::GPUDevice> candidates;
     switch (multi_gpu_category) {
       case kMultiGpuCategoryPrimary:
@@ -345,26 +366,41 @@ bool GpuControlList::Conditions::Contains(OsType target_os_type,
           candidates.push_back(gpu_info.gpu);
     }
 
-    GPUInfo::GPUDevice gpu;
-    gpu.vendor_id = vendor_id;
     bool found = false;
-    if (device_id_size == 0) {
-      for (size_t ii = 0; ii < candidates.size(); ++ii) {
-        if (gpu.vendor_id == candidates[ii].vendor_id) {
-          found = true;
-          break;
+    if (gpu_series_list_size > 0) {
+      for (size_t ii = 0; !found && ii < candidates.size(); ++ii) {
+        GpuSeriesType candidate_series = GetGpuSeriesType(
+            candidates[ii].vendor_id, candidates[ii].device_id);
+        if (candidate_series == GpuSeriesType::kUnknown)
+          continue;
+        for (size_t jj = 0; jj < gpu_series_list_size; ++jj) {
+          if (candidate_series == gpu_series_list[jj]) {
+            found = true;
+            break;
+          }
         }
       }
     } else {
-      for (size_t ii = 0; ii < device_id_size; ++ii) {
-        gpu.device_id = device_ids[ii];
-        for (size_t jj = 0; jj < candidates.size(); ++jj) {
-          if (gpu.vendor_id == candidates[jj].vendor_id &&
-              gpu.device_id == candidates[jj].device_id) {
+      GPUInfo::GPUDevice gpu;
+      gpu.vendor_id = vendor_id;
+      if (device_id_size == 0) {
+        for (size_t ii = 0; ii < candidates.size(); ++ii) {
+          if (gpu.vendor_id == candidates[ii].vendor_id) {
             found = true;
             break;
           }
         }
+      } else {
+        for (size_t ii = 0; ii < device_id_size; ++ii) {
+          gpu.device_id = device_ids[ii];
+          for (size_t jj = 0; jj < candidates.size(); ++jj) {
+            if (gpu.vendor_id == candidates[jj].vendor_id &&
+                gpu.device_id == candidates[jj].device_id) {
+              found = true;
+              break;
+            }
+          }
+        }
       }
     }
     if (!found)
@@ -428,17 +464,27 @@ bool GpuControlList::Entry::Contains(OsType target_os_type,
   return true;
 }
 
+bool GpuControlList::Entry::AppliesToTestGroup(
+    uint32_t target_test_group) const {
+  // If an entry specifies non-zero test group, then the entry only applies
+  // if that test group is enabled (as specified in |target_test_group|).
+  if (conditions.more && conditions.more->test_group)
+    return conditions.more->test_group == target_test_group;
+  return true;
+}
+
 bool GpuControlList::Conditions::NeedsMoreInfo(const GPUInfo& gpu_info) const {
   // We only check for missing info that might be collected with a gl context.
   // If certain info is missing due to some error, say, we fail to collect
   // vendor_id/device_id, then even if we launch GPU process and create a gl
   // context, we won't gather such missing info, so we still return false.
+  const GPUInfo::GPUDevice& active_gpu = gpu_info.active_gpu();
   if (driver_info) {
-    if (driver_info->driver_vendor && gpu_info.driver_vendor.empty()) {
+    if (driver_info->driver_vendor && active_gpu.driver_vendor.empty()) {
       return true;
     }
     if (driver_info->driver_version.IsSpecified() &&
-        gpu_info.driver_version.empty()) {
+        active_gpu.driver_version.empty()) {
       return true;
     }
   }
@@ -480,11 +526,15 @@ void GpuControlList::Entry::GetFeatureNames(
     DCHECK(iter != feature_map.end());
     feature_names->AppendString(iter->second);
   }
+  for (size_t ii = 0; ii < disabled_extension_size; ++ii) {
+    std::string name =
+        base::StringPrintf("disable(%s)", disabled_extensions[ii]);
+    feature_names->AppendString(name);
+  }
 }
 
 GpuControlList::GpuControlList(const GpuControlListData& data)
-    : version_(data.version),
-      entry_count_(data.entry_count),
+    : entry_count_(data.entry_count),
       entries_(data.entries),
       max_entry_id_(0),
       needs_more_info_(false),
@@ -494,22 +544,28 @@ GpuControlList::GpuControlList(const GpuControlListData& data)
   max_entry_id_ = entries_[entry_count_ - 1].id;
 }
 
-GpuControlList::~GpuControlList() {
+GpuControlList::~GpuControlList() = default;
+
+std::set<int32_t> GpuControlList::MakeDecision(GpuControlList::OsType os,
+                                               const std::string& os_version,
+                                               const GPUInfo& gpu_info) {
+  return MakeDecision(os, os_version, gpu_info, 0);
 }
 
-std::set<int> GpuControlList::MakeDecision(GpuControlList::OsType os,
-                                           const std::string& os_version,
-                                           const GPUInfo& gpu_info) {
+std::set<int32_t> GpuControlList::MakeDecision(GpuControlList::OsType os,
+                                               const std::string& os_version,
+                                               const GPUInfo& gpu_info,
+                                               uint32_t target_test_group) {
   active_entries_.clear();
   std::set<int> features;
 
   needs_more_info_ = false;
   // Has all features permanently in the list without any possibility of
   // removal in the future (subset of "features" set).
-  std::set<int> permanent_features;
+  std::set<int32_t> permanent_features;
   // Has all features absent from "features" set that could potentially be
   // included later with more information.
-  std::set<int> potential_features;
+  std::set<int32_t> potential_features;
 
   if (os == kOsAny)
     os = GetOsType();
@@ -525,6 +581,8 @@ std::set<int> GpuControlList::MakeDecision(GpuControlList::OsType os,
   for (size_t ii = 0; ii < entry_count_; ++ii) {
     const Entry& entry = entries_[ii];
     DCHECK_NE(0u, entry.id);
+    if (!entry.AppliesToTestGroup(target_test_group))
+      continue;
     if (entry.Contains(os, processed_os_version, gpu_info)) {
       bool needs_more_info_main = entry.NeedsMoreInfo(gpu_info, false);
       bool needs_more_info_exception = entry.NeedsMoreInfo(gpu_info, true);
@@ -535,7 +593,7 @@ std::set<int> GpuControlList::MakeDecision(GpuControlList::OsType os,
       // set. If we don't have enough info for an exception, it's safer if we
       // just ignore the exception and assume the exception doesn't apply.
       for (size_t jj = 0; jj < entry.feature_size; ++jj) {
-        int feature = entry.features[jj];
+        int32_t feature = entry.features[jj];
         if (needs_more_info_main) {
           if (!features.count(feature))
             potential_features.insert(feature);
@@ -548,7 +606,7 @@ std::set<int> GpuControlList::MakeDecision(GpuControlList::OsType os,
       }
 
       if (!needs_more_info_main)
-        active_entries_.push_back(ii);
+        active_entries_.push_back(base::checked_cast<uint32_t>(ii));
     }
   }
 
@@ -557,14 +615,18 @@ std::set<int> GpuControlList::MakeDecision(GpuControlList::OsType os,
   return features;
 }
 
-void GpuControlList::GetDecisionEntries(
-    std::vector<uint32_t>* entry_ids) const {
-  DCHECK(entry_ids);
-  entry_ids->clear();
-  for (auto index : active_entries_) {
+const std::vector<uint32_t>& GpuControlList::GetActiveEntries() const {
+  return active_entries_;
+}
+
+std::vector<uint32_t> GpuControlList::GetEntryIDsFromIndices(
+    const std::vector<uint32_t>& entry_indices) const {
+  std::vector<uint32_t> ids;
+  for (auto index : entry_indices) {
     DCHECK_LT(index, entry_count_);
-    entry_ids->push_back(entries_[index].id);
+    ids.push_back(entries_[index].id);
   }
+  return ids;
 }
 
 std::vector<std::string> GpuControlList::GetDisabledExtensions() {
@@ -580,21 +642,36 @@ std::vector<std::string> GpuControlList::GetDisabledExtensions() {
                                   disabled_extensions.end());
 }
 
-void GpuControlList::GetReasons(base::ListValue* problem_list,
-                                const std::string& tag) const {
-  DCHECK(problem_list);
+std::vector<std::string> GpuControlList::GetDisabledWebGLExtensions() {
+  std::set<std::string> disabled_webgl_extensions;
   for (auto index : active_entries_) {
+    DCHECK_LT(index, entry_count_);
     const Entry& entry = entries_[index];
-    auto problem = base::MakeUnique<base::DictionaryValue>();
+    for (size_t ii = 0; ii < entry.disabled_webgl_extension_size; ++ii) {
+      disabled_webgl_extensions.insert(entry.disabled_webgl_extensions[ii]);
+    }
+  }
+  return std::vector<std::string>(disabled_webgl_extensions.begin(),
+                                  disabled_webgl_extensions.end());
+}
+
+void GpuControlList::GetReasons(base::ListValue* problem_list,
+                                const std::string& tag,
+                                const std::vector<uint32_t>& entries) const {
+  DCHECK(problem_list);
+  for (auto index : entries) {
+    DCHECK_LT(index, entry_count_);
+    const Entry& entry = entries_[index];
+    auto problem = std::make_unique<base::DictionaryValue>();
 
     problem->SetString("description", entry.description);
 
-    auto cr_bugs = base::MakeUnique<base::ListValue>();
+    auto cr_bugs = std::make_unique<base::ListValue>();
     for (size_t jj = 0; jj < entry.cr_bug_size; ++jj)
       cr_bugs->AppendInteger(entry.cr_bugs[jj]);
     problem->Set("crBugs", std::move(cr_bugs));
 
-    auto features = base::MakeUnique<base::ListValue>();
+    auto features = std::make_unique<base::ListValue>();
     entry.GetFeatureNames(features.get(), feature_map_);
     problem->Set("affectedGpuSettings", std::move(features));
 
@@ -613,10 +690,6 @@ uint32_t GpuControlList::max_entry_id() const {
   return max_entry_id_;
 }
 
-std::string GpuControlList::version() const {
-  return version_;
-}
-
 // static
 GpuControlList::OsType GpuControlList::GetOsType() {
 #if defined(OS_CHROMEOS)
@@ -625,6 +698,8 @@ GpuControlList::OsType GpuControlList::GetOsType() {
   return kOsWin;
 #elif defined(OS_ANDROID)
   return kOsAndroid;
+#elif defined(OS_FUCHSIA)
+  return kOsFuchsia;
 #elif defined(OS_LINUX) || defined(OS_OPENBSD)
   return kOsLinux;
 #elif defined(OS_MACOSX)
@@ -639,4 +714,15 @@ void GpuControlList::AddSupportedFeature(
   feature_map_[feature_id] = feature_name;
 }
 
+// static
+bool GpuControlList::AreEntryIndicesValid(
+    const std::vector<uint32_t>& entry_indices,
+    size_t total_entries) {
+  for (auto index : entry_indices) {
+    if (index >= total_entries)
+      return false;
+  }
+  return true;
+}
+
 }  // namespace gpu
