@@ -14,7 +14,6 @@
 #include "base/location.h"
 #include "base/memory/ptr_util.h"
 #include "base/memory/ref_counted.h"
-#include "base/profiler/scoped_tracker.h"
 #include "base/single_thread_task_runner.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "build/build_config.h"
@@ -31,8 +30,9 @@ namespace IPC {
 
 ChannelProxy::Context::Context(
     Listener* listener,
-    const scoped_refptr<base::SingleThreadTaskRunner>& ipc_task_runner)
-    : listener_task_runner_(base::ThreadTaskRunnerHandle::Get()),
+    const scoped_refptr<base::SingleThreadTaskRunner>& ipc_task_runner,
+    const scoped_refptr<base::SingleThreadTaskRunner>& listener_task_runner)
+    : default_listener_task_runner_(listener_task_runner),
       listener_(listener),
       ipc_task_runner_(ipc_task_runner),
       channel_connected_called_(false),
@@ -47,11 +47,11 @@ ChannelProxy::Context::Context(
   // Note, we currently make an exception for a NULL listener. That usage
   // basically works, but is outside the intent of ChannelProxy. This support
   // will disappear, so please don't rely on it. See crbug.com/364241
-  DCHECK(!listener || (ipc_task_runner_.get() != listener_task_runner_.get()));
+  DCHECK(!listener ||
+         (ipc_task_runner_.get() != default_listener_task_runner_.get()));
 }
 
-ChannelProxy::Context::~Context() {
-}
+ChannelProxy::Context::~Context() = default;
 
 void ChannelProxy::Context::ClearIPCTaskRunner() {
   ipc_task_runner_ = NULL;
@@ -86,8 +86,9 @@ bool ChannelProxy::Context::TryFilters(const Message& message) {
 
   if (message_filter_router_->TryFilters(message)) {
     if (message.dispatch_error()) {
-      listener_task_runner_->PostTask(
-          FROM_HERE, base::Bind(&Context::OnDispatchBadMessage, this, message));
+      GetTaskRunner(message.routing_id())
+          ->PostTask(FROM_HERE, base::BindOnce(&Context::OnDispatchBadMessage,
+                                               this, message));
     }
 #if BUILDFLAG(IPC_MESSAGE_LOG_ENABLED)
     if (logger->Enabled())
@@ -126,8 +127,9 @@ bool ChannelProxy::Context::OnMessageReceived(const Message& message) {
 
 // Called on the IPC::Channel thread
 bool ChannelProxy::Context::OnMessageReceivedNoFilter(const Message& message) {
-  listener_task_runner_->PostTask(
-      FROM_HERE, base::Bind(&Context::OnDispatchMessage, this, message));
+  GetTaskRunner(message.routing_id())
+      ->PostTask(FROM_HERE,
+                 base::BindOnce(&Context::OnDispatchMessage, this, message));
   return true;
 }
 
@@ -145,9 +147,9 @@ void ChannelProxy::Context::OnChannelConnected(int32_t peer_pid) {
   // the filter is run on the IO thread.
   OnAddFilter();
 
-  // See above comment about using listener_task_runner_ here.
-  listener_task_runner_->PostTask(
-      FROM_HERE, base::Bind(&Context::OnDispatchConnected, this));
+  // See above comment about using default_listener_task_runner_ here.
+  default_listener_task_runner_->PostTask(
+      FROM_HERE, base::BindOnce(&Context::OnDispatchConnected, this));
 }
 
 // Called on the IPC::Channel thread
@@ -155,18 +157,18 @@ void ChannelProxy::Context::OnChannelError() {
   for (size_t i = 0; i < filters_.size(); ++i)
     filters_[i]->OnChannelError();
 
-  // See above comment about using listener_task_runner_ here.
-  listener_task_runner_->PostTask(
-      FROM_HERE, base::Bind(&Context::OnDispatchError, this));
+  // See above comment about using default_listener_task_runner_ here.
+  default_listener_task_runner_->PostTask(
+      FROM_HERE, base::BindOnce(&Context::OnDispatchError, this));
 }
 
 // Called on the IPC::Channel thread
 void ChannelProxy::Context::OnAssociatedInterfaceRequest(
     const std::string& interface_name,
     mojo::ScopedInterfaceEndpointHandle handle) {
-  listener_task_runner_->PostTask(
-      FROM_HERE, base::Bind(&Context::OnDispatchAssociatedInterfaceRequest,
-                            this, interface_name, base::Passed(&handle)));
+  default_listener_task_runner_->PostTask(
+      FROM_HERE, base::BindOnce(&Context::OnDispatchAssociatedInterfaceRequest,
+                                this, interface_name, std::move(handle)));
 }
 
 // Called on the IPC::Channel thread
@@ -188,10 +190,6 @@ void ChannelProxy::Context::OnChannelOpened() {
 
 // Called on the IPC::Channel thread
 void ChannelProxy::Context::OnChannelClosed() {
-  // TODO(pkasting): Remove ScopedTracker below once crbug.com/477117 is fixed.
-  tracked_objects::ScopedTracker tracking_profile(
-      FROM_HERE_WITH_EXPLICIT_FUNCTION(
-          "477117 ChannelProxy::Context::OnChannelClosed"));
   // It's okay for IPC::ChannelProxy::Close to be called more than once, which
   // would result in this branch being taken.
   if (!channel_)
@@ -226,10 +224,6 @@ void ChannelProxy::Context::Clear() {
 
 // Called on the IPC::Channel thread
 void ChannelProxy::Context::OnSendMessage(std::unique_ptr<Message> message) {
-  // TODO(pkasting): Remove ScopedTracker below once crbug.com/477117 is fixed.
-  tracked_objects::ScopedTracker tracking_profile(
-      FROM_HERE_WITH_EXPLICIT_FUNCTION(
-          "477117 ChannelProxy::Context::OnSendMessage"));
   if (!channel_) {
     OnChannelClosed();
     return;
@@ -303,9 +297,9 @@ void ChannelProxy::Context::OnRemoveFilter(MessageFilter* filter) {
 // Called on the listener's thread
 void ChannelProxy::Context::AddFilter(MessageFilter* filter) {
   base::AutoLock auto_lock(pending_filters_lock_);
-  pending_filters_.push_back(make_scoped_refptr(filter));
-  ipc_task_runner_->PostTask(
-      FROM_HERE, base::Bind(&Context::OnAddFilter, this));
+  pending_filters_.push_back(base::WrapRefCounted(filter));
+  ipc_task_runner_->PostTask(FROM_HERE,
+                             base::BindOnce(&Context::OnAddFilter, this));
 }
 
 // Called on the listener's thread
@@ -336,6 +330,39 @@ void ChannelProxy::Context::OnDispatchMessage(const Message& message) {
 #endif
 }
 
+// Called on the listener's thread.
+void ChannelProxy::Context::AddListenerTaskRunner(
+    int32_t routing_id,
+    scoped_refptr<base::SingleThreadTaskRunner> task_runner) {
+  DCHECK(default_listener_task_runner_->BelongsToCurrentThread());
+  DCHECK(task_runner);
+  base::AutoLock lock(listener_thread_task_runners_lock_);
+  if (!base::Contains(listener_thread_task_runners_, routing_id))
+    listener_thread_task_runners_.insert({routing_id, std::move(task_runner)});
+}
+
+// Called on the listener's thread.
+void ChannelProxy::Context::RemoveListenerTaskRunner(int32_t routing_id) {
+  DCHECK(default_listener_task_runner_->BelongsToCurrentThread());
+  base::AutoLock lock(listener_thread_task_runners_lock_);
+  listener_thread_task_runners_.erase(routing_id);
+}
+
+// Called on the IPC::Channel thread.
+scoped_refptr<base::SingleThreadTaskRunner>
+ChannelProxy::Context::GetTaskRunner(int32_t routing_id) {
+  DCHECK(ipc_task_runner_->BelongsToCurrentThread());
+  if (routing_id == MSG_ROUTING_NONE)
+    return default_listener_task_runner_;
+
+  base::AutoLock lock(listener_thread_task_runners_lock_);
+  auto task_runner = listener_thread_task_runners_.find(routing_id);
+  if (task_runner == listener_thread_task_runners_.end())
+    return default_listener_task_runner_;
+  DCHECK(task_runner->second);
+  return task_runner->second;
+}
+
 // Called on the listener's thread
 void ChannelProxy::Context::OnDispatchConnected() {
   if (channel_connected_called_)
@@ -393,8 +420,8 @@ void ChannelProxy::Context::AddGenericAssociatedInterfaceForIOThread(
 
 void ChannelProxy::Context::Send(Message* message) {
   ipc_task_runner()->PostTask(
-      FROM_HERE, base::Bind(&ChannelProxy::Context::OnSendMessage, this,
-                            base::Passed(base::WrapUnique(message))));
+      FROM_HERE, base::BindOnce(&ChannelProxy::Context::OnSendMessage, this,
+                                base::WrapUnique(message)));
 }
 
 //-----------------------------------------------------------------------------
@@ -404,9 +431,10 @@ std::unique_ptr<ChannelProxy> ChannelProxy::Create(
     const IPC::ChannelHandle& channel_handle,
     Channel::Mode mode,
     Listener* listener,
-    const scoped_refptr<base::SingleThreadTaskRunner>& ipc_task_runner) {
+    const scoped_refptr<base::SingleThreadTaskRunner>& ipc_task_runner,
+    const scoped_refptr<base::SingleThreadTaskRunner>& listener_task_runner) {
   std::unique_ptr<ChannelProxy> channel(
-      new ChannelProxy(listener, ipc_task_runner));
+      new ChannelProxy(listener, ipc_task_runner, listener_task_runner));
   channel->Init(channel_handle, mode, true);
   return channel;
 }
@@ -415,9 +443,10 @@ std::unique_ptr<ChannelProxy> ChannelProxy::Create(
 std::unique_ptr<ChannelProxy> ChannelProxy::Create(
     std::unique_ptr<ChannelFactory> factory,
     Listener* listener,
-    const scoped_refptr<base::SingleThreadTaskRunner>& ipc_task_runner) {
+    const scoped_refptr<base::SingleThreadTaskRunner>& ipc_task_runner,
+    const scoped_refptr<base::SingleThreadTaskRunner>& listener_task_runner) {
   std::unique_ptr<ChannelProxy> channel(
-      new ChannelProxy(listener, ipc_task_runner));
+      new ChannelProxy(listener, ipc_task_runner, listener_task_runner));
   channel->Init(std::move(factory), true);
   return channel;
 }
@@ -431,8 +460,10 @@ ChannelProxy::ChannelProxy(Context* context)
 
 ChannelProxy::ChannelProxy(
     Listener* listener,
-    const scoped_refptr<base::SingleThreadTaskRunner>& ipc_task_runner)
-    : context_(new Context(listener, ipc_task_runner)), did_init_(false) {
+    const scoped_refptr<base::SingleThreadTaskRunner>& ipc_task_runner,
+    const scoped_refptr<base::SingleThreadTaskRunner>& listener_task_runner)
+    : context_(new Context(listener, ipc_task_runner, listener_task_runner)),
+      did_init_(false) {
 #if defined(ENABLE_IPC_FUZZER)
   outgoing_message_filter_ = NULL;
 #endif
@@ -447,7 +478,7 @@ ChannelProxy::~ChannelProxy() {
 void ChannelProxy::Init(const IPC::ChannelHandle& channel_handle,
                         Channel::Mode mode,
                         bool create_pipe_now) {
-#if defined(OS_POSIX)
+#if defined(OS_POSIX) || defined(OS_FUCHSIA)
   // When we are creating a server on POSIX, we need its file descriptor
   // to be created immediately so that it can be accessed and passed
   // to other processes. Forcing it to be created immediately avoids
@@ -455,7 +486,7 @@ void ChannelProxy::Init(const IPC::ChannelHandle& channel_handle,
   if (mode & Channel::MODE_SERVER_FLAG) {
     create_pipe_now = true;
   }
-#endif  // defined(OS_POSIX)
+#endif  // defined(OS_POSIX) || defined(OS_FUCHSIA)
   Init(
       ChannelFactory::Create(channel_handle, mode, context_->ipc_task_runner()),
       create_pipe_now);
@@ -474,14 +505,13 @@ void ChannelProxy::Init(std::unique_ptr<ChannelFactory> factory,
     context_->CreateChannel(std::move(factory));
   } else {
     context_->ipc_task_runner()->PostTask(
-        FROM_HERE, base::Bind(&Context::CreateChannel, context_,
-                              base::Passed(&factory)));
+        FROM_HERE,
+        base::BindOnce(&Context::CreateChannel, context_, std::move(factory)));
   }
 
   // complete initialization on the background thread
   context_->ipc_task_runner()->PostTask(
-      FROM_HERE,
-      base::Bind(&Context::OnChannelOpened, context_));
+      FROM_HERE, base::BindOnce(&Context::OnChannelOpened, context_));
 
   did_init_ = true;
   OnChannelInit();
@@ -489,17 +519,17 @@ void ChannelProxy::Init(std::unique_ptr<ChannelFactory> factory,
 
 void ChannelProxy::Pause() {
   context_->ipc_task_runner()->PostTask(
-      FROM_HERE, base::Bind(&Context::PauseChannel, context_));
+      FROM_HERE, base::BindOnce(&Context::PauseChannel, context_));
 }
 
 void ChannelProxy::Unpause(bool flush) {
   context_->ipc_task_runner()->PostTask(
-      FROM_HERE, base::Bind(&Context::UnpauseChannel, context_, flush));
+      FROM_HERE, base::BindOnce(&Context::UnpauseChannel, context_, flush));
 }
 
 void ChannelProxy::Flush() {
   context_->ipc_task_runner()->PostTask(
-      FROM_HERE, base::Bind(&Context::FlushChannel, context_));
+      FROM_HERE, base::BindOnce(&Context::FlushChannel, context_));
 }
 
 void ChannelProxy::Close() {
@@ -512,7 +542,7 @@ void ChannelProxy::Close() {
 
   if (context_->ipc_task_runner()) {
     context_->ipc_task_runner()->PostTask(
-        FROM_HERE, base::Bind(&Context::OnChannelClosed, context_));
+        FROM_HERE, base::BindOnce(&Context::OnChannelClosed, context_));
   }
 }
 
@@ -553,8 +583,8 @@ void ChannelProxy::RemoveFilter(MessageFilter* filter) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
 
   context_->ipc_task_runner()->PostTask(
-      FROM_HERE, base::Bind(&Context::OnRemoveFilter, context_,
-                            base::RetainedRef(filter)));
+      FROM_HERE, base::BindOnce(&Context::OnRemoveFilter, context_,
+                                base::RetainedRef(filter)));
 }
 
 void ChannelProxy::AddGenericAssociatedInterfaceForIOThread(
